/*
 * This file implements a data structure that is built and maintained by the
 * preprocessor.
 *
 * All types that can be handled for host variable declarations has to
 * be handled eventually.
 *
 * src/interfaces/ecux/include/ecuxtype.h
 */

/*
 * Here are all the types that we are to handle. Note that it is the type
 * that is registered and that has nothing whatsoever to do with the storage
 * class.
 *
 * Simple types
 * integers: char, short, int, long (signed and unsigned)
 * floats: float, double
 *
 * Complex types:
 * VARCHAR, VARCHAR2 - Strings with length (maxlen is given in the declaration)
 * Arrays of simple types and of VARCHAR, VARCHAR2 (size given in declaration)
 * Records build of simple types, arrays and other structs.
 *
 * Complicating things:
 * typedefs and struct names!
 *
 * Conclusion:
 * This is a typically recursive definition. A structure of typed list elements
 * would probably work fine:
 */

#ifndef _ECUXTYPE_H
#define _ECUXTYPE_H

#ifdef __cplusplus
extern "C"
{
#endif

enum ECUXttype
{
	ECUXt_char = 1, ECUXt_unsigned_char, ECUXt_short, ECUXt_unsigned_short,
	ECUXt_int, ECUXt_unsigned_int, ECUXt_long, ECUXt_unsigned_long,
	ECUXt_long_long, ECUXt_unsigned_long_long,
	ECUXt_bool,
	ECUXt_float, ECUXt_double,
	ECUXt_varchar, ECUXt_varchar2,
	ECUXt_numeric,				/* this is a decimal that stores its digits in
								 * a malloced array */
	ECUXt_decimal,				/* this is a decimal that stores its digits in
								 * a fixed array */
	ECUXt_date,
	ECUXt_timestamp,
	ECUXt_interval,
	ECUXt_array,
	ECUXt_struct,
	ECUXt_union,
	ECUXt_descriptor,			/* sql descriptor, no C variable */
	ECUXt_char_variable,
	ECUXt_const,				/* a constant is needed sometimes */
	ECUXt_EOIT,					/* End of insert types. */
	ECUXt_EORT,					/* End of result types. */
	ECUXt_NO_INDICATOR,			/* no indicator */
	ECUXt_string,				/* trimmed (char *) type */
	ECUXt_sqlda					/* C struct descriptor */
};

 /* descriptor items */
enum ECUXdtype
{
	ECUXd_count = 1,
	ECUXd_data,
	ECUXd_di_code,
	ECUXd_di_precision,
	ECUXd_indicator,
	ECUXd_key_member,
	ECUXd_length,
	ECUXd_name,
	ECUXd_nullable,
	ECUXd_octet,
	ECUXd_precision,
	ECUXd_ret_length,
	ECUXd_ret_octet,
	ECUXd_scale,
	ECUXd_type,
	ECUXd_EODT,					/* End of descriptor types. */
	ECUXd_cardinality
};

#define IS_SIMPLE_TYPE(type) (((type) >= ECUXt_char && (type) <= ECUXt_interval) || ((type) == ECUXt_string))

/* we also have to handle different statement types */
enum ECUX_statement_type
{
	ECUXst_normal,
	ECUXst_execute,
	ECUXst_exec_immediate,
	ECUXst_prepnormal
};

#ifdef __cplusplus
}
#endif

#endif							/* _ECUXTYPE_H */
