/*-------------------------------------------------------------------------
 *
 * libuxsql-int.h
 *	  This file contains internal definitions meant to be used only by
 *	  the frontend libuxsql library, not by applications that call it.
 *
 *	  An application can include this file if it wants to bypass the
 *	  official API defined by libuxsql-fe.h, but code that does so is much
 *	  more likely to break across UXsinoDB releases than code that uses
 *	  only the official API.
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/interfaces/libuxsql/libuxsql-int.h
 *
 *-------------------------------------------------------------------------
 */

#ifndef LIBUXSQL_INT_H
#define LIBUXSQL_INT_H

/* We assume libuxsql-fe.h has already been included. */
#include "libuxsql-events.h"

#include <time.h>
#ifndef WIN32
#include <sys/time.h>
#endif

#ifdef ENABLE_THREAD_SAFETY
#ifdef WIN32
#include "pthread-win32.h"
#else
#include <pthread.h>
#endif
#include <signal.h>
#endif

/* include stuff common to fe and be */
#include "getaddrinfo.h"
#include "libuxsql/uxsqlcomm.h"
/* include stuff found in fe only */
#include "uxsqlexpbuffer.h"

#ifdef ENABLE_GSS
#if defined(HAVE_GSSAPI_H)
#include <gssapi.h>
#else
#include <gssapi/gssapi.h>
#endif
#endif

#ifdef ENABLE_SSPI
#define SECURITY_WIN32
#if defined(WIN32) && !defined(_MSC_VER)
#include <ntsecapi.h>
#endif
#include <security.h>
#undef SECURITY_WIN32

#ifndef ENABLE_GSS
/*
 * Define a fake structure compatible with GSSAPI on Unix.
 */
typedef struct
{
	void	   *value;
	int			length;
} gss_buffer_desc;
#endif
#endif							/* ENABLE_SSPI */

#ifdef USE_OPENSSL
#include <openssl/ssl.h>
#include <openssl/err.h>

#ifndef OPENSSL_NO_ENGINE
#define USE_SSL_ENGINE
#endif
#endif							/* USE_OPENSSL */

/*
 * UXDB backend dependent Constants.
 */
#define CMDSTATUS_LEN 64		/* should match COMPLETION_TAG_BUFSIZE */

/*
 * UXresult and the subsidiary types UXresAttDesc, UXresAttValue
 * represent the result of a query (or more precisely, of a single SQL
 * command --- a query string given to UXSQLexec can contain multiple commands).
 * Note we assume that a single command can return at most one tuple group,
 * hence there is no need for multiple descriptor sets.
 */

/* Subsidiary-storage management structure for UXresult.
 * See space management routines in fe-exec.c for details.
 * Note that space[k] refers to the k'th byte starting from the physical
 * head of the block --- it's a union, not a struct!
 */
typedef union uxresult_data UXresult_data;

union uxresult_data
{
	UXresult_data *next;		/* link to next block, or NULL */
	char		space[1];		/* dummy for accessing block as bytes */
};

/* Data about a single parameter of a prepared statement */
typedef struct uxresParamDesc
{
	Oid			typid;			/* type id */
} UXresParamDesc;

/*
 * Data for a single attribute of a single tuple
 *
 * We use char* for Attribute values.
 *
 * The value pointer always points to a null-terminated area; we add a
 * null (zero) byte after whatever the backend sends us.  This is only
 * particularly useful for text values ... with a binary value, the
 * value might have embedded nulls, so the application can't use C string
 * operators on it.  But we add a null anyway for consistency.
 * Note that the value itself does not contain a length word.
 *
 * A NULL attribute is a special case in two ways: its len field is NULL_LEN
 * and its value field points to null_field in the owning UXresult.  All the
 * NULL attributes in a query result point to the same place (there's no need
 * to store a null string separately for each one).
 */

#define NULL_LEN		(-1)	/* ux_result len for NULL value */

typedef struct uxresAttValue
{
	int			len;			/* length in bytes of the value */
	char	   *value;			/* actual value, plus terminating zero byte */
} UXresAttValue;

/* Typedef for message-field list entries */
typedef struct uxMessageField
{
	struct uxMessageField *next;	/* list link */
	char		code;			/* field code */
	char		contents[FLEXIBLE_ARRAY_MEMBER];	/* value, nul-terminated */
} UXMessageField;

/* Fields needed for notice handling */
typedef struct
{
	UXSQLnoticeReceiver noticeRec; /* notice message receiver */
	void	   *noticeRecArg;
	UXSQLnoticeProcessor noticeProc;	/* notice message processor */
	void	   *noticeProcArg;
} UXNoticeHooks;

typedef struct UXEvent
{
	UXEventProc proc;			/* the function to call on events */
	char	   *name;			/* used only for error messages */
	void	   *passThrough;	/* pointer supplied at registration time */
	void	   *data;			/* optional state (instance) data */
	bool		resultInitialized;	/* T if RESULTCREATE/COPY succeeded */
} UXEvent;

struct ux_result
{
	int			ntups;
	int			numAttributes;
	UXresAttDesc *attDescs;
	UXresAttValue **tuples;		/* each UXresTuple is an array of
								 * UXresAttValue's */
	int			tupArrSize;		/* allocated size of tuples array */
	int			numParameters;
	UXresParamDesc *paramDescs;
	ExecStatusType resultStatus;
	char		cmdStatus[CMDSTATUS_LEN];	/* cmd status from the query */
	int			binary;			/* binary tuple values if binary == 1,
								 * otherwise text */

	/*
	 * These fields are copied from the originating UXconn, so that operations
	 * on the UXresult don't have to reference the UXconn.
	 */
	UXNoticeHooks noticeHooks;
	UXEvent    *events;
	int			nEvents;
	int			client_encoding;	/* encoding id */

	/*
	 * Error information (all NULL if not an error result).  errMsg is the
	 * "overall" error message returned by UXSQLresultErrorMessage.  If we have
	 * per-field info then it is stored in a linked list.
	 */
	char	   *errMsg;			/* error message, or NULL if no error */
	UXMessageField *errFields;	/* message broken into fields */
	char	   *errQuery;		/* text of triggering query, if available */

	/* All NULL attributes in the query result point to this null string */
	char		null_field[1];

	/*
	 * Space management information.  Note that attDescs and error stuff, if
	 * not null, point into allocated blocks.  But tuples points to a
	 * separately malloc'd block, so that we can realloc it.
	 */
	UXresult_data *curBlock;	/* most recently allocated block */
	int			curOffset;		/* start offset of free space in block */
	int			spaceLeft;		/* number of free bytes remaining in block */
};

/* UXAsyncStatusType defines the state of the query-execution state machine */
typedef enum
{
	UXASYNC_IDLE,				/* nothing's happening, dude */
	UXASYNC_BUSY,				/* query in progress */
	UXASYNC_READY,				/* result ready for UXSQLgetResult */
	UXASYNC_COPY_IN,			/* Copy In data transfer in progress */
	UXASYNC_COPY_OUT,			/* Copy Out data transfer in progress */
	UXASYNC_COPY_BOTH			/* Copy In/Out data transfer in progress */
} UXAsyncStatusType;

/* UXQueryClass tracks which query protocol we are now executing */
typedef enum
{
	UXQUERY_SIMPLE,				/* simple Query protocol (UXSQLexec) */
	UXQUERY_EXTENDED,			/* full Extended protocol (UXSQLexecParams) */
	UXQUERY_PREPARE,			/* Parse only (UXSQLprepare) */
	UXQUERY_DESCRIBE			/* Describe Statement or Portal */
} UXQueryClass;

/* UXSetenvStatusType defines the state of the UXSQLSetenv state machine */
/* (this is used only for 2.0-protocol connections) */
typedef enum
{
	SETENV_STATE_CLIENT_ENCODING_SEND,	/* About to send an Environment Option */
	SETENV_STATE_CLIENT_ENCODING_WAIT,	/* Waiting for above send to complete */
	SETENV_STATE_OPTION_SEND,	/* About to send an Environment Option */
	SETENV_STATE_OPTION_WAIT,	/* Waiting for above send to complete */
	SETENV_STATE_QUERY1_SEND,	/* About to send a status query */
	SETENV_STATE_QUERY1_WAIT,	/* Waiting for query to complete */
	SETENV_STATE_QUERY2_SEND,	/* About to send a status query */
	SETENV_STATE_QUERY2_WAIT,	/* Waiting for query to complete */
	SETENV_STATE_IDLE
} UXSetenvStatusType;

/* Typedef for the EnvironmentOptions[] array */
typedef struct UXSQLEnvironmentOption
{
	const char *envName,		/* name of an environment variable */
			   *uxName;			/* name of corresponding SET variable */
} UXSQLEnvironmentOption;

/* Typedef for parameter-status list entries */
typedef struct uxParameterStatus
{
	struct uxParameterStatus *next; /* list link */
	char	   *name;			/* parameter name */
	char	   *value;			/* parameter value */
	/* Note: name and value are stored in same malloc block as struct is */
} uxParameterStatus;

/* large-object-access data ... allocated only if large-object code is used. */
typedef struct uxLobjfuncs
{
	Oid			fn_lo_open;		/* OID of backend function lo_open		*/
	Oid			fn_lo_close;	/* OID of backend function lo_close		*/
	Oid			fn_lo_creat;	/* OID of backend function lo_creat		*/
	Oid			fn_lo_create;	/* OID of backend function lo_create	*/
	Oid			fn_lo_unlink;	/* OID of backend function lo_unlink	*/
	Oid			fn_lo_lseek;	/* OID of backend function lo_lseek		*/
	Oid			fn_lo_lseek64;	/* OID of backend function lo_lseek64	*/
	Oid			fn_lo_tell;		/* OID of backend function lo_tell		*/
	Oid			fn_lo_tell64;	/* OID of backend function lo_tell64	*/
	Oid			fn_lo_truncate; /* OID of backend function lo_truncate	*/
	Oid			fn_lo_truncate64;	/* OID of function lo_truncate64 */
	Oid			fn_lo_read;		/* OID of backend function LOread		*/
	Oid			fn_lo_write;	/* OID of backend function LOwrite		*/
} UXlobjfuncs;

/* UXdataValue represents a data field value being passed to a row processor.
 * It could be either text or binary data; text data is not zero-terminated.
 * A SQL NULL is represented by len < 0; then value is still valid but there
 * are no data bytes there.
 */
typedef struct uxDataValue
{
	int			len;			/* data length in bytes, or <0 if NULL */
	const char *value;			/* data value, without zero-termination */
} UXdataValue;

typedef enum ux_conn_host_type
{
	CHT_HOST_NAME,
	CHT_HOST_ADDRESS,
	CHT_UNIX_SOCKET
} ux_conn_host_type;

/*
 * ux_conn_host stores all information about one of possibly several hosts
 * mentioned in the connection string.  Derived by splitting the uxhost
 * on the comma character and then parsing each segment.
 */
typedef struct ux_conn_host
{
	ux_conn_host_type type;		/* type of host */
	char	   *host;			/* host name or socket path */
	char	   *hostaddr;		/* host address */
	char	   *port;			/* port number for this host; if not NULL,
								 * overrides the UXConn's uxport */
	char	   *password;		/* password for this host, read from the
								 * password file.  only set if the UXconn's
								 * uxpass field is NULL. */
	struct addrinfo *addrlist;	/* list of possible backend addresses */
} ux_conn_host;

/*
 * UXconn stores all the state data associated with a single connection
 * to a backend.
 */
struct ux_conn
{
	/* Saved values of connection options */
	char	   *uxhost;			/* the machine on which the server is running,
								 * or a path to a UNIX-domain socket, or a
								 * comma-separated list of machines and/or
								 * paths, optionally with port suffixes; if
								 * NULL, use DEFAULT_UXSOCKET_DIR */
	char	   *uxhostaddr;		/* the numeric IP address of the machine on
								 * which the server is running.  Takes
								 * precedence over above. */
	char	   *uxport;			/* the server's communication port number */
	char	   *uxtty;			/* tty on which the backend messages is
								 * displayed (OBSOLETE, NOT USED) */
	char	   *connect_timeout;	/* connection timeout (numeric string) */
	char	   *client_encoding_initial;	/* encoding to use */
	char	   *uxoptions;		/* options to start the backend with */
	char	   *appname;		/* application name */
	char	   *fbappname;		/* fallback application name */
	char	   *dbName;			/* database name */
	char	   *replication;	/* connect as the replication standby? */
	char	   *uxuser;			/* Uxdb username and password, if any */
	char	   *uxpass;
	char	   *uxpassfile;		/* path to a file containing password(s) */
	char	   *keepalives;		/* use TCP keepalives? */
	char	   *keepalives_idle;	/* time between TCP keepalives */
	char	   *keepalives_interval;	/* time between TCP keepalive
										 * retransmits */
	char	   *keepalives_count;	/* maximum number of TCP keepalive
									 * retransmits */
	char	   *sslmode;		/* SSL mode (require,prefer,allow,disable) */
	char	   *sslcompression; /* SSL compression (0 or 1) */
	char	   *sslkey;			/* client key filename */
	char	   *sslcert;		/* client certificate filename */
	char	   *sslrootcert;	/* root certificate filename */
	char	   *sslcrl;			/* certificate revocation list filename */
	char	   *requirepeer;	/* required peer credentials for local sockets */

#if defined(ENABLE_GSS) || defined(ENABLE_SSPI)
	char	   *krbsrvname;		/* Kerberos service name */
#endif

	/* Type of connection to make.  Possible values: any, read-write. */
	char	   *target_session_attrs;

	/* Optional file to write trace info to */
	FILE	   *Pfdebug;

	/* Callback procedures for notice message processing */
	UXNoticeHooks noticeHooks;

	/* Event procs registered via UXSQLregisterEventProc */
	UXEvent    *events;			/* expandable array of event data */
	int			nEvents;		/* number of active events */
	int			eventArraySize; /* allocated array size */

	/* Status indicators */
	ConnStatusType status;
	UXAsyncStatusType asyncStatus;
	UXTransactionStatusType xactStatus; /* never changes to ACTIVE */
	UXQueryClass queryclass;
	char	   *last_query;		/* last SQL command, or NULL if unknown */
	char		last_sqlstate[6];	/* last reported SQLSTATE */
	bool		options_valid;	/* true if OK to attempt connection */
	bool		nonblocking;	/* whether this connection is using nonblock
								 * sending semantics */
	bool		singleRowMode;	/* return current query result row-by-row? */
	char		copy_is_binary; /* 1 = copy binary, 0 = copy text */
	int			copy_already_done;	/* # bytes already returned in COPY OUT */
	UXnotify   *notifyHead;		/* oldest unreported Notify msg */
	UXnotify   *notifyTail;		/* newest unreported Notify msg */

	/* Support for multiple hosts in connection string */
	int			nconnhost;		/* # of possible hosts */
	int			whichhost;		/* host we're currently considering */
	ux_conn_host *connhost;		/* details about each possible host */

	/* Connection data */
	uxsocket	sock;			/* FD for socket, UXINVALID_SOCKET if
								 * unconnected */
	SockAddr	laddr;			/* Local address */
	SockAddr	raddr;			/* Remote address */
	ProtocolVersion pversion;	/* FE/BE protocol version in use */
	int			sversion;		/* server version, e.g. 70401 for 7.4.1 */
	bool		auth_req_received;	/* true if any type of auth req received */
	bool		password_needed;	/* true if server demanded a password */
	bool		uxpassfile_used;	/* true if password is from uxpassfile */
	bool		sigpipe_so;		/* have we masked SIGPIPE via SO_NOSIGPIPE? */
	bool		sigpipe_flag;	/* can we mask SIGPIPE via MSG_NOSIGNAL? */

	/* Transient state needed while establishing connection */
	struct addrinfo *addr_cur;	/* backend address currently being tried */
	UXSetenvStatusType setenv_state;	/* for 2.0 protocol only */
	const UXSQLEnvironmentOption *next_eo;
	bool		send_appname;	/* okay to send application_name? */

	/* Miscellaneous stuff */
	int			be_pid;			/* PID of backend --- needed for cancels */
	int			be_key;			/* key of backend --- needed for cancels */
	uxParameterStatus *pstatus; /* ParameterStatus data */
	int			client_encoding;	/* encoding id */
	bool		std_strings;	/* standard_conforming_strings */
	UXVerbosity verbosity;		/* error/notice message verbosity */
	UXContextVisibility show_context;	/* whether to show CONTEXT field */
	UXlobjfuncs *lobjfuncs;		/* private state for large-object access fns */

	/* Buffer for data received from backend and not yet processed */
	char	   *inBuffer;		/* currently allocated buffer */
	int			inBufSize;		/* allocated size of buffer */
	int			inStart;		/* offset to first unconsumed data in buffer */
	int			inCursor;		/* next byte to tentatively consume */
	int			inEnd;			/* offset to first position after avail data */

	/* Buffer for data not yet sent to backend */
	char	   *outBuffer;		/* currently allocated buffer */
	int			outBufSize;		/* allocated size of buffer */
	int			outCount;		/* number of chars waiting in buffer */

	/* State for constructing messages in outBuffer */
	int			outMsgStart;	/* offset to msg start (length word); if -1,
								 * msg has no length word */
	int			outMsgEnd;		/* offset to msg end (so far) */

	/* Row processor interface workspace */
	UXdataValue *rowBuf;		/* array for passing values to rowProcessor */
	int			rowBufLen;		/* number of entries allocated in rowBuf */

	/* Status for asynchronous result construction */
	UXresult   *result;			/* result being constructed */
	UXresult   *next_result;	/* next result (used in single-row mode) */

	/* Assorted state for SASL, SSL, GSS, etc */
	void	   *sasl_state;

#ifdef USE_SSL
	bool		allow_ssl_try;	/* Allowed to try SSL negotiation */
	bool		wait_ssl_try;	/* Delay SSL negotiation until after
								 * attempting normal connection */
	bool		ssl_in_use;
#ifdef USE_OPENSSL
	SSL		   *ssl;			/* SSL status, if have SSL connection */
	X509	   *peer;			/* X509 cert of server */
#ifdef USE_SSL_ENGINE
	ENGINE	   *engine;			/* SSL engine, if any */
#else
	void	   *engine;			/* dummy field to keep struct the same if
								 * OpenSSL version changes */
#endif
#endif							/* USE_OPENSSL */
#endif							/* USE_SSL */

#ifdef ENABLE_GSS
	gss_ctx_id_t gctx;			/* GSS context */
	gss_name_t	gtarg_nam;		/* GSS target name */
#endif

#ifdef ENABLE_SSPI
#ifdef ENABLE_GSS
	char	   *gsslib;			/* What GSS library to use ("gssapi" or
								 * "sspi") */
#endif
	CredHandle *sspicred;		/* SSPI credentials handle */
	CtxtHandle *sspictx;		/* SSPI context */
	char	   *sspitarget;		/* SSPI target name */
	int			usesspi;		/* Indicate if SSPI is in use on the
								 * connection */
#endif

	/* Buffer for current error message */
	UXSQLExpBufferData errorMessage;	/* expansible string */

	/* Buffer for receiving various parts of messages */
	UXSQLExpBufferData workBuffer; /* expansible string */
};

/* UXcancel stores all data necessary to cancel a connection. A copy of this
 * data is required to safely cancel a connection running on a different
 * thread.
 */
struct ux_cancel
{
	SockAddr	raddr;			/* Remote address */
	int			be_pid;			/* PID of backend --- needed for cancels */
	int			be_key;			/* key of backend --- needed for cancels */
};


/* String descriptions of the ExecStatusTypes.
 * direct use of this array is deprecated; call UXSQLresStatus() instead.
 */
extern char *const uxresStatus[];


#ifdef USE_SSL

#ifndef WIN32
#define USER_CERT_FILE		".uxsinodb/uxsinodb.crt"
#define USER_KEY_FILE		".uxsinodb/uxsinodb.key"
#define ROOT_CERT_FILE		".uxsinodb/root.crt"
#define ROOT_CRL_FILE		".uxsinodb/root.crl"
#else
/* On Windows, the "home" directory is already UXsinoDB-specific */
#define USER_CERT_FILE		"uxsinodb.crt"
#define USER_KEY_FILE		"uxsinodb.key"
#define ROOT_CERT_FILE		"root.crt"
#define ROOT_CRL_FILE		"root.crl"
#endif

#endif							/* USE_SSL */

/* ----------------
 * Internal functions of libuxsql
 * Functions declared here need to be visible across files of libuxsql,
 * but are not intended to be called by applications.  We use the
 * convention "uxsqlXXX" for internal functions, vs. the "UXSQLxxx" names
 * used for application-visible routines.
 * ----------------
 */

/* === in fe-connect.c === */

extern void uxsqlDropConnection(UXconn *conn, bool flushInput);
extern int uxsqlPacketSend(UXconn *conn, char pack_type,
			 const void *buf, size_t buf_len);
extern bool uxsqlGetHomeDirectory(char *buf, int bufsize);

#ifdef ENABLE_THREAD_SAFETY
extern uxthreadlock_t ux_g_threadlock;

#define UXTHREAD_ERROR(msg) \
	do { \
		fprintf(stderr, "%s\n", msg); \
		abort(); \
	} while (0)


#define uxlock_thread()		ux_g_threadlock(true)
#define uxunlock_thread()	ux_g_threadlock(false)
#else
#define uxlock_thread()		((void) 0)
#define uxunlock_thread()	((void) 0)
#endif

/* === in fe-exec.c === */

extern void uxsqlSetResultError(UXresult *res, const char *msg);
extern void uxsqlCatenateResultError(UXresult *res, const char *msg);
extern void *uxsqlResultAlloc(UXresult *res, size_t nBytes, bool isBinary);
extern char *uxsqlResultStrdup(UXresult *res, const char *str);
extern void uxsqlClearAsyncResult(UXconn *conn);
extern void uxsqlSaveErrorResult(UXconn *conn);
extern UXresult *uxsqlPrepareAsyncResult(UXconn *conn);
extern void uxsqlInternalNotice(const UXNoticeHooks *hooks, const char *fmt,...) ux_attribute_printf(2, 3);
extern void uxsqlSaveMessageField(UXresult *res, char code,
				   const char *value);
extern void uxsqlSaveParameterStatus(UXconn *conn, const char *name,
					  const char *value);
extern int	uxsqlRowProcessor(UXconn *conn, const char **errmsgp);
extern void uxsqlHandleSendFailure(UXconn *conn);

/* === in fe-protocol2.c === */

extern UxdbPollingStatusType uxsqlSetenvPoll(UXconn *conn);

extern char *uxsqlBuildStartupPacket2(UXconn *conn, int *packetlen,
					  const UXSQLEnvironmentOption *options);
extern void uxsqlParseInput2(UXconn *conn);
extern int	uxsqlGetCopyData2(UXconn *conn, char **buffer, int async);
extern int	uxsqlGetline2(UXconn *conn, char *s, int maxlen);
extern int	uxsqlGetlineAsync2(UXconn *conn, char *buffer, int bufsize);
extern int	uxsqlEndcopy2(UXconn *conn);
extern UXresult *uxsqlFunctionCall2(UXconn *conn, Oid fnid,
				int *result_buf, int *actual_result_len,
				int result_is_int,
				const UXSQLArgBlock *args, int nargs);

/* === in fe-protocol3.c === */

extern char *uxsqlBuildStartupPacket3(UXconn *conn, int *packetlen,
					  const UXSQLEnvironmentOption *options);
extern void uxsqlParseInput3(UXconn *conn);
extern int	uxsqlGetErrorNotice3(UXconn *conn, bool isError);
extern void uxsqlBuildErrorMessage3(UXSQLExpBuffer msg, const UXresult *res,
					 UXVerbosity verbosity, UXContextVisibility show_context);
extern int	uxsqlGetCopyData3(UXconn *conn, char **buffer, int async);
extern int	uxsqlGetline3(UXconn *conn, char *s, int maxlen);
extern int	uxsqlGetlineAsync3(UXconn *conn, char *buffer, int bufsize);
extern int	uxsqlEndcopy3(UXconn *conn);
extern UXresult *uxsqlFunctionCall3(UXconn *conn, Oid fnid,
				int *result_buf, int *actual_result_len,
				int result_is_int,
				const UXSQLArgBlock *args, int nargs);

/* === in fe-misc.c === */

 /*
  * "Get" and "Put" routines return 0 if successful, EOF if not. Note that for
  * Get, EOF merely means the buffer is exhausted, not that there is
  * necessarily any error.
  */
extern int	uxsqlCheckOutBufferSpace(size_t bytes_needed, UXconn *conn);
extern int	uxsqlCheckInBufferSpace(size_t bytes_needed, UXconn *conn);
extern int	uxsqlGetc(char *result, UXconn *conn);
extern int	uxsqlPutc(char c, UXconn *conn);
extern int	uxsqlGets(UXSQLExpBuffer buf, UXconn *conn);
extern int	uxsqlGets_append(UXSQLExpBuffer buf, UXconn *conn);
extern int	uxsqlPuts(const char *s, UXconn *conn);
extern int	uxsqlGetnchar(char *s, size_t len, UXconn *conn);
extern int	uxsqlSkipnchar(size_t len, UXconn *conn);
extern int	uxsqlPutnchar(const char *s, size_t len, UXconn *conn);
extern int	uxsqlGetInt(int *result, size_t bytes, UXconn *conn);
extern int	uxsqlPutInt(int value, size_t bytes, UXconn *conn);
extern int	uxsqlPutMsgStart(char msg_type, bool force_len, UXconn *conn);
extern int	uxsqlPutMsgEnd(UXconn *conn);
extern int	uxsqlReadData(UXconn *conn);
extern int	uxsqlFlush(UXconn *conn);
extern int	uxsqlWait(int forRead, int forWrite, UXconn *conn);
extern int uxsqlWaitTimed(int forRead, int forWrite, UXconn *conn,
			time_t finish_time);
extern int	uxsqlReadReady(UXconn *conn);
extern int	uxsqlWriteReady(UXconn *conn);

/* === in fe-secure.c === */

extern int	uxsqlsecure_initialize(UXconn *);
extern void uxsqlsecure_destroy(void);
extern UxdbPollingStatusType uxsqlsecure_open_client(UXconn *);
extern void uxsqlsecure_close(UXconn *);
extern ssize_t uxsqlsecure_read(UXconn *, void *ptr, size_t len);
extern ssize_t uxsqlsecure_write(UXconn *, const void *ptr, size_t len);
extern ssize_t uxsqlsecure_raw_read(UXconn *, void *ptr, size_t len);
extern ssize_t uxsqlsecure_raw_write(UXconn *, const void *ptr, size_t len);

#if defined(ENABLE_THREAD_SAFETY) && !defined(WIN32)
extern int	uxsql_block_sigpipe(sigset_t *osigset, bool *sigpipe_pending);
extern void uxsql_reset_sigpipe(sigset_t *osigset, bool sigpipe_pending,
				 bool got_epipe);
#endif

/*
 * The SSL implementation provides these functions (fe-secure-openssl.c)
 */
extern void uxtls_init_library(bool do_ssl, int do_crypto);
extern int	uxtls_init(UXconn *conn);
extern UxdbPollingStatusType uxtls_open_client(UXconn *conn);
extern void uxtls_close(UXconn *conn);
extern ssize_t uxtls_read(UXconn *conn, void *ptr, size_t len);
extern bool uxtls_read_pending(UXconn *conn);
extern ssize_t uxtls_write(UXconn *conn, const void *ptr, size_t len);

/*
 * this is so that we can check if a connection is non-blocking internally
 * without the overhead of a function call
 */
#define uxsqlIsnonblocking(conn)	((conn)->nonblocking)

#ifdef ENABLE_NLS
extern char *libuxsql_gettext(const char *msgid) ux_attribute_format_arg(1);
extern char *libuxsql_ngettext(const char *msgid, const char *msgid_plural, unsigned long n) ux_attribute_format_arg(1) ux_attribute_format_arg(2);
#else
#define libuxsql_gettext(x) (x)
#define libuxsql_ngettext(s, p, n) ((n) == 1 ? (s) : (p))
#endif

/*
 * These macros are needed to let error-handling code be portable between
 * Unix and Windows.  (ugh)
 */
#ifdef WIN32
#define SOCK_ERRNO (WSAGetLastError())
#define SOCK_STRERROR winsock_strerror
#define SOCK_ERRNO_SET(e) WSASetLastError(e)
#else
#define SOCK_ERRNO errno
#define SOCK_STRERROR uxsqlStrerror
#define SOCK_ERRNO_SET(e) (errno = (e))
#endif

#endif							/* LIBUXSQL_INT_H */
