/*-------------------------------------------------------------------------
 *
 * uxsqlexpbuffer.h
 *	  Declarations/definitions for "UXSQLExpBuffer" functions.
 *
 * UXSQLExpBuffer provides an indefinitely-extensible string data type.
 * It can be used to buffer either ordinary C strings (null-terminated text)
 * or arbitrary binary data.  All storage is allocated with malloc().
 *
 * This module is essentially the same as the backend's StringInfo data type,
 * but it is intended for use in frontend libuxsql and client applications.
 * Thus, it does not rely on palloc() nor elog().
 *
 * It does rely on vsnprintf(); if configure finds that libc doesn't provide
 * a usable vsnprintf(), then a copy of our own implementation of it will
 * be linked into libuxsql.
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/interfaces/libuxsql/uxsqlexpbuffer.h
 *
 *-------------------------------------------------------------------------
 */
#ifndef UXSQLEXPBUFFER_H
#define UXSQLEXPBUFFER_H

/*-------------------------
 * UXSQLExpBufferData holds information about an extensible string.
 *		data	is the current buffer for the string (allocated with malloc).
 *		len		is the current string length.  There is guaranteed to be
 *				a terminating '\0' at data[len], although this is not very
 *				useful when the string holds binary data rather than text.
 *		maxlen	is the allocated size in bytes of 'data', i.e. the maximum
 *				string size (including the terminating '\0' char) that we can
 *				currently store in 'data' without having to reallocate
 *				more space.  We must always have maxlen > len.
 *
 * An exception occurs if we failed to allocate enough memory for the string
 * buffer.  In that case data points to a statically allocated empty string,
 * and len = maxlen = 0.
 *-------------------------
 */
typedef struct UXSQLExpBufferData
{
	char	   *data;
	size_t		len;
	size_t		maxlen;
} UXSQLExpBufferData;

typedef UXSQLExpBufferData *UXSQLExpBuffer;

/*------------------------
 * Test for a broken (out of memory) UXSQLExpBuffer.
 * When a buffer is "broken", all operations except resetting or deleting it
 * are no-ops.
 *------------------------
 */
#define UXSQLExpBufferBroken(str)	\
	((str) == NULL || (str)->maxlen == 0)

/*------------------------
 * Same, but for use when using a static or local UXSQLExpBufferData struct.
 * For that, a null-pointer test is useless and may draw compiler warnings.
 *------------------------
 */
#define UXSQLExpBufferDataBroken(buf)	\
	((buf).maxlen == 0)

/*------------------------
 * Initial size of the data buffer in a UXSQLExpBuffer.
 * NB: this must be large enough to hold error messages that might
 * be returned by UXSQLrequestCancel().
 *------------------------
 */
#define INITIAL_EXPBUFFER_SIZE	256

/*------------------------
 * There are two ways to create a UXSQLExpBuffer object initially:
 *
 * UXSQLExpBuffer stringptr = createUXSQLExpBuffer();
 *		Both the UXSQLExpBufferData and the data buffer are malloc'd.
 *
 * UXSQLExpBufferData string;
 * initUXSQLExpBuffer(&string);
 *		The data buffer is malloc'd but the UXSQLExpBufferData is presupplied.
 *		This is appropriate if the UXSQLExpBufferData is a field of another
 *		struct.
 *-------------------------
 */

/*------------------------
 * createUXSQLExpBuffer
 * Create an empty 'UXSQLExpBufferData' & return a pointer to it.
 */
extern UXSQLExpBuffer createUXSQLExpBuffer(void);

/*------------------------
 * initUXSQLExpBuffer
 * Initialize a UXSQLExpBufferData struct (with previously undefined contents)
 * to describe an empty string.
 */
extern void initUXSQLExpBuffer(UXSQLExpBuffer str);

/*------------------------
 * To destroy a UXSQLExpBuffer, use either:
 *
 * destroyUXSQLExpBuffer(str);
 *		free()s both the data buffer and the UXSQLExpBufferData.
 *		This is the inverse of createUXSQLExpBuffer().
 *
 * termUXSQLExpBuffer(str)
 *		free()s the data buffer but not the UXSQLExpBufferData itself.
 *		This is the inverse of initUXSQLExpBuffer().
 *
 * NOTE: some routines build up a string using UXSQLExpBuffer, and then
 * release the UXSQLExpBufferData but return the data string itself to their
 * caller.  At that point the data string looks like a plain malloc'd
 * string.
 */
extern void destroyUXSQLExpBuffer(UXSQLExpBuffer str);
extern void termUXSQLExpBuffer(UXSQLExpBuffer str);

/*------------------------
 * resetUXSQLExpBuffer
 *		Reset a UXSQLExpBuffer to empty
 *
 * Note: if possible, a "broken" UXSQLExpBuffer is returned to normal.
 */
extern void resetUXSQLExpBuffer(UXSQLExpBuffer str);

/*------------------------
 * enlargeUXSQLExpBuffer
 * Make sure there is enough space for 'needed' more bytes in the buffer
 * ('needed' does not include the terminating null).
 *
 * Returns 1 if OK, 0 if failed to enlarge buffer.  (In the latter case
 * the buffer is left in "broken" state.)
 */
extern int	enlargeUXSQLExpBuffer(UXSQLExpBuffer str, size_t needed);

/*------------------------
 * printfUXSQLExpBuffer
 * Format text data under the control of fmt (an sprintf-like format string)
 * and insert it into str.  More space is allocated to str if necessary.
 * This is a convenience routine that does the same thing as
 * resetUXSQLExpBuffer() followed by appendUXSQLExpBuffer().
 */
extern void printfUXSQLExpBuffer(UXSQLExpBuffer str, const char *fmt,...) ux_attribute_printf(2, 3);

/*------------------------
 * appendUXSQLExpBuffer
 * Format text data under the control of fmt (an sprintf-like format string)
 * and append it to whatever is already in str.  More space is allocated
 * to str if necessary.  This is sort of like a combination of sprintf and
 * strcat.
 */
extern void appendUXSQLExpBuffer(UXSQLExpBuffer str, const char *fmt,...) ux_attribute_printf(2, 3);

/*------------------------
 * appendUXSQLExpBufferStr
 * Append the given string to a UXSQLExpBuffer, allocating more space
 * if necessary.
 */
extern void appendUXSQLExpBufferStr(UXSQLExpBuffer str, const char *data);

/*------------------------
 * appendUXSQLExpBufferChar
 * Append a single byte to str.
 * Like appendUXSQLExpBuffer(str, "%c", ch) but much faster.
 */
extern void appendUXSQLExpBufferChar(UXSQLExpBuffer str, char ch);

/*------------------------
 * appendBinaryUXSQLExpBuffer
 * Append arbitrary binary data to a UXSQLExpBuffer, allocating more space
 * if necessary.
 */
extern void appendBinaryUXSQLExpBuffer(UXSQLExpBuffer str,
						const char *data, size_t datalen);

#endif							/* UXSQLEXPBUFFER_H */
