/*-------------------------------------------------------------------------
 *
 * libuxsql-fe.h
 *	  This file contains definitions for structures and
 *	  externs for functions used by frontend uxdb applications.
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/interfaces/libuxsql/libuxsql-fe.h
 *
 *-------------------------------------------------------------------------
 */

#ifndef LIBUXSQL_FE_H
#define LIBUXSQL_FE_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdio.h>

/*
 * uxdb_ext.h defines the backend's externally visible types,
 * such as Oid.
 */
#include "uxdb_ext.h"

/*
 * Option flags for UXSQLcopyResult
 */
#define UX_COPYRES_ATTRS		  0x01
#define UX_COPYRES_TUPLES		  0x02	/* Implies UX_COPYRES_ATTRS */
#define UX_COPYRES_EVENTS		  0x04
#define UX_COPYRES_NOTICEHOOKS	  0x08

/* Application-visible enum types */

/*
 * Although it is okay to add to these lists, values which become unused
 * should never be removed, nor should constants be redefined - that would
 * break compatibility with existing code.
 */

typedef enum
{
	CONNECTION_OK,
	CONNECTION_BAD,
	/* Non-blocking mode only below here */

	/*
	 * The existence of these should never be relied upon - they should only
	 * be used for user feedback or similar purposes.
	 */
	CONNECTION_STARTED,			/* Waiting for connection to be made.  */
	CONNECTION_MADE,			/* Connection OK; waiting to send.     */
	CONNECTION_AWAITING_RESPONSE,	/* Waiting for a response from the
									 * uxmaster.        */
	CONNECTION_AUTH_OK,			/* Received authentication; waiting for
								 * backend startup. */
	CONNECTION_SETENV,			/* Negotiating environment. */
	CONNECTION_SSL_STARTUP,		/* Negotiating SSL. */
	CONNECTION_NEEDED,			/* Internal state: connect() needed */
	CONNECTION_CHECK_WRITABLE,	/* Check if we could make a writable
								 * connection. */
	CONNECTION_CONSUME			/* Wait for any pending message and consume
								 * them. */
} ConnStatusType;

typedef enum
{
	UXRES_POLLING_FAILED = 0,
	UXRES_POLLING_READING,		/* These two indicate that one may	  */
	UXRES_POLLING_WRITING,		/* use select before polling again.   */
	UXRES_POLLING_OK,
	UXRES_POLLING_ACTIVE		/* unused; keep for awhile for backwards
								 * compatibility */
} UxdbPollingStatusType;

typedef enum
{
	UXRES_EMPTY_QUERY = 0,		/* empty query string was executed */
	UXRES_COMMAND_OK,			/* a query command that doesn't return
								 * anything was executed properly by the
								 * backend */
	UXRES_TUPLES_OK,			/* a query command that returns tuples was
								 * executed properly by the backend, UXresult
								 * contains the result tuples */
	UXRES_COPY_OUT,				/* Copy Out data transfer in progress */
	UXRES_COPY_IN,				/* Copy In data transfer in progress */
	UXRES_BAD_RESPONSE,			/* an unexpected response was recv'd from the
								 * backend */
	UXRES_NONFATAL_ERROR,		/* notice or warning message */
	UXRES_FATAL_ERROR,			/* query failed */
	UXRES_COPY_BOTH,			/* Copy In/Out data transfer in progress */
	UXRES_SINGLE_TUPLE			/* single tuple from larger resultset */
} ExecStatusType;

typedef enum
{
	UXSQLTRANS_IDLE,				/* connection idle */
	UXSQLTRANS_ACTIVE,				/* command in progress */
	UXSQLTRANS_INTRANS,			/* idle, within transaction block */
	UXSQLTRANS_INERROR,			/* idle, within failed transaction */
	UXSQLTRANS_UNKNOWN				/* cannot determine status */
} UXTransactionStatusType;

typedef enum
{
	UXSQLERRORS_TERSE,				/* single-line error messages */
	UXSQLERRORS_DEFAULT,			/* recommended style */
	UXSQLERRORS_VERBOSE			/* all the facts, ma'am */
} UXVerbosity;

typedef enum
{
	UXSQLSHOW_CONTEXT_NEVER,		/* never show CONTEXT field */
	UXSQLSHOW_CONTEXT_ERRORS,		/* show CONTEXT for errors only (default) */
	UXSQLSHOW_CONTEXT_ALWAYS		/* always show CONTEXT field */
} UXContextVisibility;

/*
 * UXPing - The ordering of this enum should not be altered because the
 * values are exposed externally via ux_isready.
 */

typedef enum
{
	UXSQLPING_OK,					/* server is accepting connections */
	UXSQLPING_REJECT,				/* server is alive but rejecting connections */
	UXSQLPING_NO_RESPONSE,			/* could not establish connection */
	UXSQLPING_NO_ATTEMPT			/* connection not attempted (bad params) */
} UXPing;

/* UXconn encapsulates a connection to the backend.
 * The contents of this struct are not supposed to be known to applications.
 */
typedef struct ux_conn UXconn;

/* UXresult encapsulates the result of a query (or more precisely, of a single
 * SQL command --- a query string given to UXSQLsendQuery can contain multiple
 * commands and thus return multiple UXresult objects).
 * The contents of this struct are not supposed to be known to applications.
 */
typedef struct ux_result UXresult;

/* UXcancel encapsulates the information needed to cancel a running
 * query on an existing connection.
 * The contents of this struct are not supposed to be known to applications.
 */
typedef struct ux_cancel UXcancel;

/* UXnotify represents the occurrence of a NOTIFY message.
 * Ideally this would be an opaque typedef, but it's so simple that it's
 * unlikely to change.
 * NOTE: in Uxdb 6.4 and later, the be_pid is the notifying backend's,
 * whereas in earlier versions it was always your own backend's PID.
 */
typedef struct uxNotify
{
	char	   *relname;		/* notification condition name */
	int			be_pid;			/* process ID of notifying server process */
	char	   *extra;			/* notification parameter */
	/* Fields below here are private to libuxsql; apps should not use 'em */
	struct uxNotify *next;		/* list link */
} UXnotify;

/* Function types for notice-handling callbacks */
typedef void (*UXSQLnoticeReceiver) (void *arg, const UXresult *res);
typedef void (*UXSQLnoticeProcessor) (void *arg, const char *message);

/* Print options for UXSQLprint() */
typedef char uxsqlbool;

typedef struct _UXSQLprintOpt
{
	uxsqlbool		header;			/* print output field headings and row count */
	uxsqlbool		align;			/* fill align the fields */
	uxsqlbool		standard;		/* old brain dead format */
	uxsqlbool		html3;			/* output html tables */
	uxsqlbool		expanded;		/* expand tables */
	uxsqlbool		pager;			/* use pager for output if needed */
	char	   *fieldSep;		/* field separator */
	char	   *tableOpt;		/* insert to HTML <table ...> */
	char	   *caption;		/* HTML <caption> */
	char	  **fieldName;		/* null terminated array of replacement field
								 * names */
} UXSQLprintOpt;

/* ----------------
 * Structure for the conninfo parameter definitions returned by UXSQLconndefaults
 * or UXSQLconninfoParse.
 *
 * All fields except "val" point at static strings which must not be altered.
 * "val" is either NULL or a malloc'd current-value string.  UXSQLconninfoFree()
 * will release both the val strings and the UXSQLconninfoOption array itself.
 * ----------------
 */
typedef struct _UXSQLconninfoOption
{
	char	   *keyword;		/* The keyword of the option			*/
	char	   *envvar;			/* Fallback environment variable name	*/
	char	   *compiled;		/* Fallback compiled in default value	*/
	char	   *val;			/* Option's current value, or NULL		 */
	char	   *label;			/* Label for field in connect dialog	*/
	char	   *dispchar;		/* Indicates how to display this field in a
								 * connect dialog. Values are: "" Display
								 * entered value as is "*" Password field -
								 * hide value "D"  Debug option - don't show
								 * by default */
	int			dispsize;		/* Field size in characters for dialog	*/
} UXSQLconninfoOption;

/* ----------------
 * UXSQLArgBlock -- structure for UXSQLfn() arguments
 * ----------------
 */
typedef struct
{
	int			len;
	int			isint;
	union
	{
		int		   *ptr;		/* can't use void (dec compiler barfs)	 */
		int			integer;
	}			u;
} UXSQLArgBlock;

/* ----------------
 * UXresAttDesc -- Data about a single attribute (column) of a query result
 * ----------------
 */
typedef struct uxresAttDesc
{
	char	   *name;			/* column name */
	Oid			tableid;		/* source table, if known */
	int			columnid;		/* source column, if known */
	int			format;			/* format code for value (text/binary) */
	Oid			typid;			/* type id */
	int			typlen;			/* type size */
	int			atttypmod;		/* type-specific modifier info */
} UXresAttDesc;

/* ----------------
 * Exported functions of libuxsql
 * ----------------
 */

/* ===	in fe-connect.c === */

/* make a new client connection to the backend */
/* Asynchronous (non-blocking) */
extern UXconn *UXSQLconnectStart(const char *conninfo);
extern UXconn *UXSQLconnectStartParams(const char *const *keywords,
					 const char *const *values, int expand_dbname);
extern UxdbPollingStatusType UXSQLconnectPoll(UXconn *conn);

/* Synchronous (blocking) */
extern UXconn *UXSQLconnectdb(const char *conninfo);
extern UXconn *UXSQLconnectdbParams(const char *const *keywords,
				  const char *const *values, int expand_dbname);
extern UXconn *UXSQLsetdbLogin(const char *uxhost, const char *uxport,
			 const char *uxoptions, const char *uxtty,
			 const char *dbName,
			 const char *login, const char *pwd);

#define UXSQLsetdb(M_UXHOST,M_UXPORT,M_UXOPT,M_UXTTY,M_DBNAME)  \
	UXSQLsetdbLogin(M_UXHOST, M_UXPORT, M_UXOPT, M_UXTTY, M_DBNAME, NULL, NULL)

/* close the current connection and free the UXconn data structure */
extern void UXSQLfinish(UXconn *conn);

/* get info about connection options known to UXSQLconnectdb */
extern UXSQLconninfoOption *UXSQLconndefaults(void);

/* parse connection options in same way as UXSQLconnectdb */
extern UXSQLconninfoOption *UXSQLconninfoParse(const char *conninfo, char **errmsg);

/* return the connection options used by a live connection */
extern UXSQLconninfoOption *UXSQLconninfo(UXconn *conn);

/* free the data structure returned by UXSQLconndefaults() or UXSQLconninfoParse() */
extern void UXSQLconninfoFree(UXSQLconninfoOption *connOptions);

/*
 * close the current connection and restablish a new one with the same
 * parameters
 */
/* Asynchronous (non-blocking) */
extern int	UXSQLresetStart(UXconn *conn);
extern UxdbPollingStatusType UXSQLresetPoll(UXconn *conn);

/* Synchronous (blocking) */
extern void UXSQLreset(UXconn *conn);

/* request a cancel structure */
extern UXcancel *UXSQLgetCancel(UXconn *conn);

/* free a cancel structure */
extern void UXSQLfreeCancel(UXcancel *cancel);

/* issue a cancel request */
extern int	UXSQLcancel(UXcancel *cancel, char *errbuf, int errbufsize);

/* backwards compatible version of UXSQLcancel; not thread-safe */
extern int	UXSQLrequestCancel(UXconn *conn);

/* Accessor functions for UXconn objects */
extern char *UXSQLdb(const UXconn *conn);
extern char *UXSQLuser(const UXconn *conn);
extern char *UXSQLpass(const UXconn *conn);
extern char *UXSQLhost(const UXconn *conn);
extern char *UXSQLport(const UXconn *conn);
extern char *UXSQLtty(const UXconn *conn);
extern char *UXSQLoptions(const UXconn *conn);
extern ConnStatusType UXSQLstatus(const UXconn *conn);
extern UXTransactionStatusType UXSQLtransactionStatus(const UXconn *conn);
extern const char *UXSQLparameterStatus(const UXconn *conn,
				  const char *paramName);
extern int	UXSQLprotocolVersion(const UXconn *conn);
extern int	UXSQLserverVersion(const UXconn *conn);
extern char *UXSQLerrorMessage(const UXconn *conn);
extern int	UXSQLsocket(const UXconn *conn);
extern int	UXSQLbackendPID(const UXconn *conn);
extern int	UXSQLconnectionNeedsPassword(const UXconn *conn);
extern int	UXSQLconnectionUsedPassword(const UXconn *conn);
extern int	UXSQLclientEncoding(const UXconn *conn);
extern int	UXSQLsetClientEncoding(UXconn *conn, const char *encoding);

/* SSL information functions */
extern int	UXSQLsslInUse(UXconn *conn);
extern void *UXSQLsslStruct(UXconn *conn, const char *struct_name);
extern const char *UXSQLsslAttribute(UXconn *conn, const char *attribute_name);
extern const char *const *UXSQLsslAttributeNames(UXconn *conn);

/* Get the OpenSSL structure associated with a connection. Returns NULL for
 * unencrypted connections or if any other TLS library is in use. */
extern void *UXSQLgetssl(UXconn *conn);

/* Tell libuxsql whether it needs to initialize OpenSSL */
extern void UXSQLinitSSL(int do_init);

/* More detailed way to tell libuxsql whether it needs to initialize OpenSSL */
extern void UXSQLinitOpenSSL(int do_ssl, int do_crypto);

/* Set verbosity for UXSQLerrorMessage and UXSQLresultErrorMessage */
extern UXVerbosity UXSQLsetErrorVerbosity(UXconn *conn, UXVerbosity verbosity);

/* Set CONTEXT visibility for UXSQLerrorMessage and UXSQLresultErrorMessage */
extern UXContextVisibility UXSQLsetErrorContextVisibility(UXconn *conn,
							UXContextVisibility show_context);

/* Enable/disable tracing */
extern void UXSQLtrace(UXconn *conn, FILE *debug_port);
extern void UXSQLuntrace(UXconn *conn);

/* Override default notice handling routines */
extern UXSQLnoticeReceiver UXSQLsetNoticeReceiver(UXconn *conn,
					UXSQLnoticeReceiver proc,
					void *arg);
extern UXSQLnoticeProcessor UXSQLsetNoticeProcessor(UXconn *conn,
					 UXSQLnoticeProcessor proc,
					 void *arg);

/*
 *	   Used to set callback that prevents concurrent access to
 *	   non-thread safe functions that libuxsql needs.
 *	   The default implementation uses a libuxsql internal mutex.
 *	   Only required for multithreaded apps that use kerberos
 *	   both within their app and for uxsinodb connections.
 */
typedef void (*uxthreadlock_t) (int acquire);

extern uxthreadlock_t UXSQLregisterThreadLock(uxthreadlock_t newhandler);

/* === in fe-exec.c === */

/* Simple synchronous query */
extern UXresult *UXSQLexec(UXconn *conn, const char *query);
extern UXresult *UXSQLexecParams(UXconn *conn,
			 const char *command,
			 int nParams,
			 const Oid *paramTypes,
			 const char *const *paramValues,
			 const int *paramLengths,
			 const int *paramFormats,
			 int resultFormat);
extern UXresult *UXSQLprepare(UXconn *conn, const char *stmtName,
		  const char *query, int nParams,
		  const Oid *paramTypes);
extern UXresult *UXSQLexecPrepared(UXconn *conn,
			   const char *stmtName,
			   int nParams,
			   const char *const *paramValues,
			   const int *paramLengths,
			   const int *paramFormats,
			   int resultFormat);

/* Interface for multiple-result or asynchronous queries */
extern int	UXSQLsendQuery(UXconn *conn, const char *query);
extern int UXSQLsendQueryParams(UXconn *conn,
				  const char *command,
				  int nParams,
				  const Oid *paramTypes,
				  const char *const *paramValues,
				  const int *paramLengths,
				  const int *paramFormats,
				  int resultFormat);
extern int UXSQLsendPrepare(UXconn *conn, const char *stmtName,
			  const char *query, int nParams,
			  const Oid *paramTypes);
extern int UXSQLsendQueryPrepared(UXconn *conn,
					const char *stmtName,
					int nParams,
					const char *const *paramValues,
					const int *paramLengths,
					const int *paramFormats,
					int resultFormat);
extern int	UXSQLsetSingleRowMode(UXconn *conn);
extern UXresult *UXSQLgetResult(UXconn *conn);

/* Routines for managing an asynchronous query */
extern int	UXSQLisBusy(UXconn *conn);
extern int	UXSQLconsumeInput(UXconn *conn);

/* LISTEN/NOTIFY support */
extern UXnotify *UXSQLnotifies(UXconn *conn);

/* Routines for copy in/out */
extern int	UXSQLputCopyData(UXconn *conn, const char *buffer, int nbytes);
extern int	UXSQLputCopyEnd(UXconn *conn, const char *errormsg);
extern int	UXSQLgetCopyData(UXconn *conn, char **buffer, int async);

/* Deprecated routines for copy in/out */
extern int	UXSQLgetline(UXconn *conn, char *string, int length);
extern int	UXSQLputline(UXconn *conn, const char *string);
extern int	UXSQLgetlineAsync(UXconn *conn, char *buffer, int bufsize);
extern int	UXSQLputnbytes(UXconn *conn, const char *buffer, int nbytes);
extern int	UXSQLendcopy(UXconn *conn);

/* Set blocking/nonblocking connection to the backend */
extern int	UXSQLsetnonblocking(UXconn *conn, int arg);
extern int	UXSQLisnonblocking(const UXconn *conn);
extern int	UXSQListhreadsafe(void);
extern UXPing UXSQLping(const char *conninfo);
extern UXPing UXSQLpingParams(const char *const *keywords,
			 const char *const *values, int expand_dbname);

/* Force the write buffer to be written (or at least try) */
extern int	UXSQLflush(UXconn *conn);

/*
 * "Fast path" interface --- not really recommended for application
 * use
 */
extern UXresult *UXSQLfn(UXconn *conn,
	 int fnid,
	 int *result_buf,
	 int *result_len,
	 int result_is_int,
	 const UXSQLArgBlock *args,
	 int nargs);

/* Accessor functions for UXresult objects */
extern ExecStatusType UXSQLresultStatus(const UXresult *res);
extern char *UXSQLresStatus(ExecStatusType status);
extern char *UXSQLresultErrorMessage(const UXresult *res);
extern char *UXSQLresultVerboseErrorMessage(const UXresult *res,
							UXVerbosity verbosity,
							UXContextVisibility show_context);
extern char *UXSQLresultErrorField(const UXresult *res, int fieldcode);
extern int	UXSQLntuples(const UXresult *res);
extern int	UXSQLnfields(const UXresult *res);
extern int	UXSQLbinaryTuples(const UXresult *res);
extern char *UXSQLfname(const UXresult *res, int field_num);
extern int	UXSQLfnumber(const UXresult *res, const char *field_name);
extern Oid	UXSQLftable(const UXresult *res, int field_num);
extern int	UXSQLftablecol(const UXresult *res, int field_num);
extern int	UXSQLfformat(const UXresult *res, int field_num);
extern Oid	UXSQLftype(const UXresult *res, int field_num);
extern int	UXSQLfsize(const UXresult *res, int field_num);
extern int	UXSQLfmod(const UXresult *res, int field_num);
extern char *UXSQLcmdStatus(UXresult *res);
extern char *UXSQLoidStatus(const UXresult *res);	/* old and ugly */
extern Oid	UXSQLoidValue(const UXresult *res);	/* new and improved */
extern char *UXSQLcmdTuples(UXresult *res);
extern char *UXSQLgetvalue(const UXresult *res, int tup_num, int field_num);
extern int	UXSQLgetlength(const UXresult *res, int tup_num, int field_num);
extern int	UXSQLgetisnull(const UXresult *res, int tup_num, int field_num);
extern int	UXSQLnparams(const UXresult *res);
extern Oid	UXSQLparamtype(const UXresult *res, int param_num);

/* Describe prepared statements and portals */
extern UXresult *UXSQLdescribePrepared(UXconn *conn, const char *stmt);
extern UXresult *UXSQLdescribePortal(UXconn *conn, const char *portal);
extern int	UXSQLsendDescribePrepared(UXconn *conn, const char *stmt);
extern int	UXSQLsendDescribePortal(UXconn *conn, const char *portal);

/* Delete a UXresult */
extern void UXSQLclear(UXresult *res);

/* For freeing other alloc'd results, such as UXnotify structs */
extern void UXSQLfreemem(void *ptr);

/* Exists for backward compatibility.  bjm 2003-03-24 */
#define UXSQLfreeNotify(ptr) UXSQLfreemem(ptr)

/* Error when no password was given. */
/* Note: depending on this is deprecated; use UXSQLconnectionNeedsPassword(). */
#define UXSQLnoPasswordSupplied	"fe_sendauth: no password supplied\n"

/* Create and manipulate UXresults */
extern UXresult *UXSQLmakeEmptyUXresult(UXconn *conn, ExecStatusType status);
extern UXresult *UXSQLcopyResult(const UXresult *src, int flags);
extern int	UXSQLsetResultAttrs(UXresult *res, int numAttributes, UXresAttDesc *attDescs);
extern void *UXSQLresultAlloc(UXresult *res, size_t nBytes);
extern int	UXSQLsetvalue(UXresult *res, int tup_num, int field_num, char *value, int len);

/* Quoting strings before inclusion in queries. */
extern size_t UXSQLescapeStringConn(UXconn *conn,
				   char *to, const char *from, size_t length,
				   int *error);
extern char *UXSQLescapeLiteral(UXconn *conn, const char *str, size_t len);
extern char *UXSQLescapeIdentifier(UXconn *conn, const char *str, size_t len);
extern unsigned char *UXSQLescapeByteaConn(UXconn *conn,
				  const unsigned char *from, size_t from_length,
				  size_t *to_length);
extern unsigned char *UXSQLunescapeBytea(const unsigned char *strtext,
				size_t *retbuflen);

/* These forms are deprecated! */
extern size_t UXSQLescapeString(char *to, const char *from, size_t length);
extern unsigned char *UXSQLescapeBytea(const unsigned char *from, size_t from_length,
			  size_t *to_length);



/* === in fe-print.c === */

extern void UXSQLprint(FILE *fout,				/* output stream */
		const UXresult *res,
		const UXSQLprintOpt *ps);	/* option structure */

/*
 * really old printing routines
 */
extern void UXSQLdisplayTuples(const UXresult *res,
				FILE *fp,		/* where to send the output */
				int fillAlign,	/* pad the fields with spaces */
				const char *fieldSep,	/* field separator */
				int printHeader,	/* display headers? */
				int quiet);

extern void UXSQLprintTuples(const UXresult *res,
			  FILE *fout,		/* output stream */
			  int printAttName, /* print attribute names */
			  int terseOutput,	/* delimiter bars */
			  int width);		/* width of column, if 0, use variable width */


/* === in fe-lobj.c === */

/* Large-object access routines */
extern int	lo_open(UXconn *conn, Oid lobjId, int mode);
extern int	lo_close(UXconn *conn, int fd);
extern int	lo_read(UXconn *conn, int fd, char *buf, size_t len);
extern int	lo_write(UXconn *conn, int fd, const char *buf, size_t len);
extern int	lo_lseek(UXconn *conn, int fd, int offset, int whence);
extern ux_int64 lo_lseek64(UXconn *conn, int fd, ux_int64 offset, int whence);
extern Oid	lo_creat(UXconn *conn, int mode);
extern Oid	lo_create(UXconn *conn, Oid lobjId);
extern int	lo_tell(UXconn *conn, int fd);
extern ux_int64 lo_tell64(UXconn *conn, int fd);
extern int	lo_truncate(UXconn *conn, int fd, size_t len);
extern int	lo_truncate64(UXconn *conn, int fd, ux_int64 len);
extern int	lo_unlink(UXconn *conn, Oid lobjId);
extern Oid	lo_import(UXconn *conn, const char *filename);
extern Oid	lo_import_with_oid(UXconn *conn, const char *filename, Oid lobjId);
extern int	lo_export(UXconn *conn, Oid lobjId, const char *filename);

/* === in fe-misc.c === */

/* Get the version of the libuxsql library in use */
extern int	UXSQLlibVersion(void);

/* Determine length of multibyte encoded char at *s */
extern int	UXSQLmblen(const char *s, int encoding);

/* Determine display length of multibyte encoded char at *s */
extern int	UXSQLdsplen(const char *s, int encoding);

/* Get encoding id from environment variable UXCLIENTENCODING */
extern int	UXSQLenv2encoding(void);

/* === in fe-auth.c === */

extern char *UXSQLencryptPassword(const char *passwd, const char *user);
extern char *UXSQLencryptPasswordConn(UXconn *conn, const char *passwd, const char *user, const char *algorithm);

/* === in encnames.c === */

extern int	ux_char_to_encoding(const char *name);
extern const char *ux_encoding_to_char(int encoding);
extern int	ux_valid_server_encoding_id(int encoding);

#ifdef __cplusplus
}
#endif

#endif							/* LIBUXSQL_FE_H */
