/*-------------------------------------------------------------------------
 *
 * indexing.h
 *	  This file provides some definitions to support indexing
 *	  on system catalogs
 *
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/include/catalog/indexing.h
 *
 *-------------------------------------------------------------------------
 */
#ifndef INDEXING_H
#define INDEXING_H

#include "access/htup.h"
#include "utils/relcache.h"

/*
 * The state object used by CatalogOpenIndexes and friends is actually the
 * same as the executor's ResultRelInfo, but we give it another type name
 * to decouple callers from that fact.
 */
typedef struct ResultRelInfo *CatalogIndexState;

/*
 * indexing.c prototypes
 */
extern CatalogIndexState CatalogOpenIndexes(Relation heapRel);
extern void CatalogCloseIndexes(CatalogIndexState indstate);
extern Oid	CatalogTupleInsert(Relation heapRel, HeapTuple tup);
extern Oid CatalogTupleInsertWithInfo(Relation heapRel, HeapTuple tup,
						   CatalogIndexState indstate);
extern void CatalogTupleUpdate(Relation heapRel, ItemPointer otid,
				   HeapTuple tup);
extern void CatalogTupleUpdateWithInfo(Relation heapRel,
						   ItemPointer otid, HeapTuple tup,
						   CatalogIndexState indstate);
extern void CatalogTupleDelete(Relation heapRel, ItemPointer tid);


/*
 * These macros are just to keep the C compiler from spitting up on the
 * upcoming commands for genbki.pl.
 */
#define DECLARE_INDEX(name,oid,decl) extern int no_such_variable
#define DECLARE_UNIQUE_INDEX(name,oid,decl) extern int no_such_variable
#define BUILD_INDICES


/*
 * What follows are lines processed by genbki.pl to create the statements
 * the bootstrap parser will turn into DefineIndex calls.
 *
 * The keyword is DECLARE_INDEX or DECLARE_UNIQUE_INDEX.  The first two
 * arguments are the index name and OID, the rest is much like a standard
 * 'create index' SQL command.
 *
 * For each index, we also provide a #define for its OID.  References to
 * the index in the C code should always use these #defines, not the actual
 * index name (much less the numeric OID).
 */

DECLARE_UNIQUE_INDEX(ux_aggregate_fnoid_index, 2650, on ux_aggregate using btree(aggfnoid oid_ops));
#define AggregateFnoidIndexId  2650

DECLARE_UNIQUE_INDEX(ux_am_name_index, 2651, on ux_am using btree(amname name_ops));
#define AmNameIndexId  2651
DECLARE_UNIQUE_INDEX(ux_am_oid_index, 2652, on ux_am using btree(oid oid_ops));
#define AmOidIndexId  2652

DECLARE_UNIQUE_INDEX(ux_amop_fam_strat_index, 2653, on ux_amop using btree(amopfamily oid_ops, amoplefttype oid_ops, amoprighttype oid_ops, amopstrategy int2_ops));
#define AccessMethodStrategyIndexId  2653
DECLARE_UNIQUE_INDEX(ux_amop_opr_fam_index, 2654, on ux_amop using btree(amopopr oid_ops, amoppurpose char_ops, amopfamily oid_ops));
#define AccessMethodOperatorIndexId  2654
DECLARE_UNIQUE_INDEX(ux_amop_oid_index, 2756, on ux_amop using btree(oid oid_ops));
#define AccessMethodOperatorOidIndexId	2756

DECLARE_UNIQUE_INDEX(ux_amproc_fam_proc_index, 2655, on ux_amproc using btree(amprocfamily oid_ops, amproclefttype oid_ops, amprocrighttype oid_ops, amprocnum int2_ops));
#define AccessMethodProcedureIndexId  2655
DECLARE_UNIQUE_INDEX(ux_amproc_oid_index, 2757, on ux_amproc using btree(oid oid_ops));
#define AccessMethodProcedureOidIndexId  2757

DECLARE_UNIQUE_INDEX(ux_attrdef_adrelid_adnum_index, 2656, on ux_attrdef using btree(adrelid oid_ops, adnum int2_ops));
#define AttrDefaultIndexId	2656
DECLARE_UNIQUE_INDEX(ux_attrdef_oid_index, 2657, on ux_attrdef using btree(oid oid_ops));
#define AttrDefaultOidIndexId  2657

DECLARE_UNIQUE_INDEX(ux_attribute_relid_attnam_index, 2658, on ux_attribute using btree(attrelid oid_ops, attname name_ops));
#define AttributeRelidNameIndexId  2658
DECLARE_UNIQUE_INDEX(ux_attribute_relid_attnum_index, 2659, on ux_attribute using btree(attrelid oid_ops, attnum int2_ops));
#define AttributeRelidNumIndexId  2659

DECLARE_UNIQUE_INDEX(ux_authid_rolname_index, 2676, on ux_authid using btree(rolname name_ops));
#define AuthIdRolnameIndexId	2676
DECLARE_UNIQUE_INDEX(ux_authid_oid_index, 2677, on ux_authid using btree(oid oid_ops));
#define AuthIdOidIndexId	2677

DECLARE_UNIQUE_INDEX(ux_auth_members_role_member_index, 2694, on ux_auth_members using btree(roleid oid_ops, member oid_ops));
#define AuthMemRoleMemIndexId	2694
DECLARE_UNIQUE_INDEX(ux_auth_members_member_role_index, 2695, on ux_auth_members using btree(member oid_ops, roleid oid_ops));
#define AuthMemMemRoleIndexId	2695

DECLARE_UNIQUE_INDEX(ux_cast_oid_index, 2660, on ux_cast using btree(oid oid_ops));
#define CastOidIndexId	2660
DECLARE_UNIQUE_INDEX(ux_cast_source_target_index, 2661, on ux_cast using btree(castsource oid_ops, casttarget oid_ops));
#define CastSourceTargetIndexId  2661

DECLARE_UNIQUE_INDEX(ux_class_oid_index, 2662, on ux_class using btree(oid oid_ops));
#define ClassOidIndexId  2662
DECLARE_UNIQUE_INDEX(ux_class_relname_nsp_index, 2663, on ux_class using btree(relname name_ops, relnamespace oid_ops));
#define ClassNameNspIndexId  2663
DECLARE_INDEX(ux_class_tblspc_relfilenode_index, 3455, on ux_class using btree(reltablespace oid_ops, relfilenode oid_ops));
#define ClassTblspcRelfilenodeIndexId  3455

DECLARE_UNIQUE_INDEX(ux_collation_name_enc_nsp_index, 3164, on ux_collation using btree(collname name_ops, collencoding int4_ops, collnamespace oid_ops));
#define CollationNameEncNspIndexId 3164
DECLARE_UNIQUE_INDEX(ux_collation_oid_index, 3085, on ux_collation using btree(oid oid_ops));
#define CollationOidIndexId  3085

DECLARE_INDEX(ux_constraint_conname_nsp_index, 2664, on ux_constraint using btree(conname name_ops, connamespace oid_ops));
#define ConstraintNameNspIndexId  2664
DECLARE_INDEX(ux_constraint_conrelid_index, 2665, on ux_constraint using btree(conrelid oid_ops));
#define ConstraintRelidIndexId	2665
DECLARE_INDEX(ux_constraint_contypid_index, 2666, on ux_constraint using btree(contypid oid_ops));
#define ConstraintTypidIndexId	2666
DECLARE_UNIQUE_INDEX(ux_constraint_oid_index, 2667, on ux_constraint using btree(oid oid_ops));
#define ConstraintOidIndexId  2667

DECLARE_UNIQUE_INDEX(ux_conversion_default_index, 2668, on ux_conversion using btree(connamespace oid_ops, conforencoding int4_ops, contoencoding int4_ops, oid oid_ops));
#define ConversionDefaultIndexId  2668
DECLARE_UNIQUE_INDEX(ux_conversion_name_nsp_index, 2669, on ux_conversion using btree(conname name_ops, connamespace oid_ops));
#define ConversionNameNspIndexId  2669
DECLARE_UNIQUE_INDEX(ux_conversion_oid_index, 2670, on ux_conversion using btree(oid oid_ops));
#define ConversionOidIndexId  2670

DECLARE_UNIQUE_INDEX(ux_database_datname_index, 2671, on ux_database using btree(datname name_ops));
#define DatabaseNameIndexId  2671
DECLARE_UNIQUE_INDEX(ux_database_oid_index, 2672, on ux_database using btree(oid oid_ops));
#define DatabaseOidIndexId	2672

DECLARE_INDEX(ux_depend_depender_index, 2673, on ux_depend using btree(classid oid_ops, objid oid_ops, objsubid int4_ops));
#define DependDependerIndexId  2673
DECLARE_INDEX(ux_depend_reference_index, 2674, on ux_depend using btree(refclassid oid_ops, refobjid oid_ops, refobjsubid int4_ops));
#define DependReferenceIndexId	2674

DECLARE_UNIQUE_INDEX(ux_description_o_c_o_index, 2675, on ux_description using btree(objoid oid_ops, classoid oid_ops, objsubid int4_ops));
#define DescriptionObjIndexId  2675
DECLARE_UNIQUE_INDEX(ux_shdescription_o_c_index, 2397, on ux_shdescription using btree(objoid oid_ops, classoid oid_ops));
#define SharedDescriptionObjIndexId 2397

DECLARE_UNIQUE_INDEX(ux_enum_oid_index, 3502, on ux_enum using btree(oid oid_ops));
#define EnumOidIndexId	3502
DECLARE_UNIQUE_INDEX(ux_enum_typid_label_index, 3503, on ux_enum using btree(enumtypid oid_ops, enumlabel name_ops));
#define EnumTypIdLabelIndexId 3503
DECLARE_UNIQUE_INDEX(ux_enum_typid_sortorder_index, 3534, on ux_enum using btree(enumtypid oid_ops, enumsortorder float4_ops));
#define EnumTypIdSortOrderIndexId 3534

DECLARE_INDEX(ux_index_indrelid_index, 2678, on ux_index using btree(indrelid oid_ops));
#define IndexIndrelidIndexId  2678
DECLARE_UNIQUE_INDEX(ux_index_indexrelid_index, 2679, on ux_index using btree(indexrelid oid_ops));
#define IndexRelidIndexId  2679

DECLARE_UNIQUE_INDEX(ux_inherits_relid_seqno_index, 2680, on ux_inherits using btree(inhrelid oid_ops, inhseqno int4_ops));
#define InheritsRelidSeqnoIndexId  2680
DECLARE_INDEX(ux_inherits_parent_index, 2187, on ux_inherits using btree(inhparent oid_ops));
#define InheritsParentIndexId  2187

DECLARE_UNIQUE_INDEX(ux_init_privs_o_c_o_index, 3395, on ux_init_privs using btree(objoid oid_ops, classoid oid_ops, objsubid int4_ops));
#define InitPrivsObjIndexId  3395

DECLARE_UNIQUE_INDEX(ux_language_name_index, 2681, on ux_language using btree(lanname name_ops));
#define LanguageNameIndexId  2681
DECLARE_UNIQUE_INDEX(ux_language_oid_index, 2682, on ux_language using btree(oid oid_ops));
#define LanguageOidIndexId	2682

DECLARE_UNIQUE_INDEX(ux_largeobject_loid_pn_index, 2683, on ux_largeobject using btree(loid oid_ops, pageno int4_ops));
#define LargeObjectLOidPNIndexId  2683

DECLARE_UNIQUE_INDEX(ux_largeobject_metadata_oid_index, 2996, on ux_largeobject_metadata using btree(oid oid_ops));
#define LargeObjectMetadataOidIndexId	2996

DECLARE_UNIQUE_INDEX(ux_statistic_ext_oid_index, 3380, on ux_statistic_ext using btree(oid oid_ops));
#define StatisticExtOidIndexId	3380
DECLARE_UNIQUE_INDEX(ux_statistic_ext_name_index, 3997, on ux_statistic_ext using btree(stxname name_ops, stxnamespace oid_ops));
#define StatisticExtNameIndexId 3997
DECLARE_INDEX(ux_statistic_ext_relid_index, 3379, on ux_statistic_ext using btree(stxrelid oid_ops));
#define StatisticExtRelidIndexId 3379

DECLARE_UNIQUE_INDEX(ux_namespace_nspname_index, 2684, on ux_namespace using btree(nspname name_ops));
#define NamespaceNameIndexId  2684
DECLARE_UNIQUE_INDEX(ux_namespace_oid_index, 2685, on ux_namespace using btree(oid oid_ops));
#define NamespaceOidIndexId  2685

DECLARE_UNIQUE_INDEX(ux_opclass_am_name_nsp_index, 2686, on ux_opclass using btree(opcmethod oid_ops, opcname name_ops, opcnamespace oid_ops));
#define OpclassAmNameNspIndexId  2686
DECLARE_UNIQUE_INDEX(ux_opclass_oid_index, 2687, on ux_opclass using btree(oid oid_ops));
#define OpclassOidIndexId  2687

DECLARE_UNIQUE_INDEX(ux_operator_oid_index, 2688, on ux_operator using btree(oid oid_ops));
#define OperatorOidIndexId	2688
DECLARE_UNIQUE_INDEX(ux_operator_oprname_l_r_n_index, 2689, on ux_operator using btree(oprname name_ops, oprleft oid_ops, oprright oid_ops, oprnamespace oid_ops));
#define OperatorNameNspIndexId	2689

DECLARE_UNIQUE_INDEX(ux_opfamily_am_name_nsp_index, 2754, on ux_opfamily using btree(opfmethod oid_ops, opfname name_ops, opfnamespace oid_ops));
#define OpfamilyAmNameNspIndexId  2754
DECLARE_UNIQUE_INDEX(ux_opfamily_oid_index, 2755, on ux_opfamily using btree(oid oid_ops));
#define OpfamilyOidIndexId	2755

DECLARE_UNIQUE_INDEX(ux_pltemplate_name_index, 1137, on ux_pltemplate using btree(tmplname name_ops));
#define PLTemplateNameIndexId  1137

DECLARE_UNIQUE_INDEX(ux_proc_oid_index, 2690, on ux_proc using btree(oid oid_ops));
#define ProcedureOidIndexId  2690
DECLARE_UNIQUE_INDEX(ux_proc_proname_args_nsp_index, 2691, on ux_proc using btree(proname name_ops, proargtypes oidvector_ops, pronamespace oid_ops));
#define ProcedureNameArgsNspIndexId  2691

DECLARE_UNIQUE_INDEX(ux_rewrite_oid_index, 2692, on ux_rewrite using btree(oid oid_ops));
#define RewriteOidIndexId  2692
DECLARE_UNIQUE_INDEX(ux_rewrite_rel_rulename_index, 2693, on ux_rewrite using btree(ev_class oid_ops, rulename name_ops));
#define RewriteRelRulenameIndexId  2693

DECLARE_UNIQUE_INDEX(ux_sequence_seqrelid_index, 5002, on ux_sequence using btree(seqrelid oid_ops));
#define SequenceRelidIndexId	5002

DECLARE_INDEX(ux_shdepend_depender_index, 1232, on ux_shdepend using btree(dbid oid_ops, classid oid_ops, objid oid_ops, objsubid int4_ops));
#define SharedDependDependerIndexId		1232
DECLARE_INDEX(ux_shdepend_reference_index, 1233, on ux_shdepend using btree(refclassid oid_ops, refobjid oid_ops));
#define SharedDependReferenceIndexId	1233

DECLARE_UNIQUE_INDEX(ux_statistic_relid_att_inh_index, 2696, on ux_statistic using btree(starelid oid_ops, staattnum int2_ops, stainherit bool_ops));
#define StatisticRelidAttnumInhIndexId	2696

DECLARE_UNIQUE_INDEX(ux_tablespace_oid_index, 2697, on ux_tablespace using btree(oid oid_ops));
#define TablespaceOidIndexId  2697
DECLARE_UNIQUE_INDEX(ux_tablespace_spcname_index, 2698, on ux_tablespace using btree(spcname name_ops));
#define TablespaceNameIndexId  2698

DECLARE_UNIQUE_INDEX(ux_transform_oid_index, 3574, on ux_transform using btree(oid oid_ops));
#define TransformOidIndexId 3574
DECLARE_UNIQUE_INDEX(ux_transform_type_lang_index, 3575, on ux_transform using btree(trftype oid_ops, trflang oid_ops));
#define TransformTypeLangIndexId  3575

DECLARE_INDEX(ux_trigger_tgconstraint_index, 2699, on ux_trigger using btree(tgconstraint oid_ops));
#define TriggerConstraintIndexId  2699
DECLARE_UNIQUE_INDEX(ux_trigger_tgrelid_tgname_index, 2701, on ux_trigger using btree(tgrelid oid_ops, tgname name_ops));
#define TriggerRelidNameIndexId  2701
DECLARE_UNIQUE_INDEX(ux_trigger_oid_index, 2702, on ux_trigger using btree(oid oid_ops));
#define TriggerOidIndexId  2702

DECLARE_UNIQUE_INDEX(ux_event_trigger_evtname_index, 3467, on ux_event_trigger using btree(evtname name_ops));
#define EventTriggerNameIndexId  3467
DECLARE_UNIQUE_INDEX(ux_event_trigger_oid_index, 3468, on ux_event_trigger using btree(oid oid_ops));
#define EventTriggerOidIndexId	3468

DECLARE_UNIQUE_INDEX(ux_ts_config_cfgname_index, 3608, on ux_ts_config using btree(cfgname name_ops, cfgnamespace oid_ops));
#define TSConfigNameNspIndexId	3608
DECLARE_UNIQUE_INDEX(ux_ts_config_oid_index, 3712, on ux_ts_config using btree(oid oid_ops));
#define TSConfigOidIndexId	3712

DECLARE_UNIQUE_INDEX(ux_ts_config_map_index, 3609, on ux_ts_config_map using btree(mapcfg oid_ops, maptokentype int4_ops, mapseqno int4_ops));
#define TSConfigMapIndexId	3609

DECLARE_UNIQUE_INDEX(ux_ts_dict_dictname_index, 3604, on ux_ts_dict using btree(dictname name_ops, dictnamespace oid_ops));
#define TSDictionaryNameNspIndexId	3604
DECLARE_UNIQUE_INDEX(ux_ts_dict_oid_index, 3605, on ux_ts_dict using btree(oid oid_ops));
#define TSDictionaryOidIndexId	3605

DECLARE_UNIQUE_INDEX(ux_ts_parser_prsname_index, 3606, on ux_ts_parser using btree(prsname name_ops, prsnamespace oid_ops));
#define TSParserNameNspIndexId	3606
DECLARE_UNIQUE_INDEX(ux_ts_parser_oid_index, 3607, on ux_ts_parser using btree(oid oid_ops));
#define TSParserOidIndexId	3607

DECLARE_UNIQUE_INDEX(ux_ts_template_tmplname_index, 3766, on ux_ts_template using btree(tmplname name_ops, tmplnamespace oid_ops));
#define TSTemplateNameNspIndexId	3766
DECLARE_UNIQUE_INDEX(ux_ts_template_oid_index, 3767, on ux_ts_template using btree(oid oid_ops));
#define TSTemplateOidIndexId	3767

DECLARE_UNIQUE_INDEX(ux_type_oid_index, 2703, on ux_type using btree(oid oid_ops));
#define TypeOidIndexId	2703
DECLARE_UNIQUE_INDEX(ux_type_typname_nsp_index, 2704, on ux_type using btree(typname name_ops, typnamespace oid_ops));
#define TypeNameNspIndexId	2704

DECLARE_UNIQUE_INDEX(ux_foreign_data_wrapper_oid_index, 112, on ux_foreign_data_wrapper using btree(oid oid_ops));
#define ForeignDataWrapperOidIndexId	112
DECLARE_UNIQUE_INDEX(ux_foreign_data_wrapper_name_index, 548, on ux_foreign_data_wrapper using btree(fdwname name_ops));
#define ForeignDataWrapperNameIndexId	548

DECLARE_UNIQUE_INDEX(ux_foreign_server_oid_index, 113, on ux_foreign_server using btree(oid oid_ops));
#define ForeignServerOidIndexId 113
DECLARE_UNIQUE_INDEX(ux_foreign_server_name_index, 549, on ux_foreign_server using btree(srvname name_ops));
#define ForeignServerNameIndexId	549

DECLARE_UNIQUE_INDEX(ux_user_mapping_oid_index, 174, on ux_user_mapping using btree(oid oid_ops));
#define UserMappingOidIndexId	174
DECLARE_UNIQUE_INDEX(ux_user_mapping_user_server_index, 175, on ux_user_mapping using btree(umuser oid_ops, umserver oid_ops));
#define UserMappingUserServerIndexId	175

DECLARE_UNIQUE_INDEX(ux_foreign_table_relid_index, 3119, on ux_foreign_table using btree(ftrelid oid_ops));
#define ForeignTableRelidIndexId 3119

DECLARE_UNIQUE_INDEX(ux_default_acl_role_nsp_obj_index, 827, on ux_default_acl using btree(defaclrole oid_ops, defaclnamespace oid_ops, defaclobjtype char_ops));
#define DefaultAclRoleNspObjIndexId 827
DECLARE_UNIQUE_INDEX(ux_default_acl_oid_index, 828, on ux_default_acl using btree(oid oid_ops));
#define DefaultAclOidIndexId	828

DECLARE_UNIQUE_INDEX(ux_db_role_setting_databaseid_rol_index, 2965, on ux_db_role_setting using btree(setdatabase oid_ops, setrole oid_ops));
#define DbRoleSettingDatidRolidIndexId	2965

DECLARE_UNIQUE_INDEX(ux_seclabel_object_index, 3597, on ux_seclabel using btree(objoid oid_ops, classoid oid_ops, objsubid int4_ops, provider text_pattern_ops));
#define SecLabelObjectIndexId				3597

DECLARE_UNIQUE_INDEX(ux_shseclabel_object_index, 3593, on ux_shseclabel using btree(objoid oid_ops, classoid oid_ops, provider text_pattern_ops));
#define SharedSecLabelObjectIndexId			3593

DECLARE_UNIQUE_INDEX(ux_extension_oid_index, 3080, on ux_extension using btree(oid oid_ops));
#define ExtensionOidIndexId 3080
DECLARE_UNIQUE_INDEX(ux_extension_name_index, 3081, on ux_extension using btree(extname name_ops));
#define ExtensionNameIndexId 3081

DECLARE_UNIQUE_INDEX(ux_range_rngtypid_index, 3542, on ux_range using btree(rngtypid oid_ops));
#define RangeTypidIndexId					3542

DECLARE_UNIQUE_INDEX(ux_policy_oid_index, 3257, on ux_policy using btree(oid oid_ops));
#define PolicyOidIndexId				3257

DECLARE_UNIQUE_INDEX(ux_policy_polrelid_polname_index, 3258, on ux_policy using btree(polrelid oid_ops, polname name_ops));
#define PolicyPolrelidPolnameIndexId				3258

DECLARE_UNIQUE_INDEX(ux_replication_origin_roiident_index, 6001, on ux_replication_origin using btree(roident oid_ops));
#define ReplicationOriginIdentIndex 6001

DECLARE_UNIQUE_INDEX(ux_replication_origin_roname_index, 6002, on ux_replication_origin using btree(roname text_pattern_ops));
#define ReplicationOriginNameIndex 6002

DECLARE_UNIQUE_INDEX(ux_partitioned_table_partrelid_index, 3351, on ux_partitioned_table using btree(partrelid oid_ops));
#define PartitionedRelidIndexId			 3351

DECLARE_UNIQUE_INDEX(ux_publication_oid_index, 6110, on ux_publication using btree(oid oid_ops));
#define PublicationObjectIndexId 6110

DECLARE_UNIQUE_INDEX(ux_publication_pubname_index, 6111, on ux_publication using btree(pubname name_ops));
#define PublicationNameIndexId 6111

DECLARE_UNIQUE_INDEX(ux_publication_rel_oid_index, 6112, on ux_publication_rel using btree(oid oid_ops));
#define PublicationRelObjectIndexId 6112

DECLARE_UNIQUE_INDEX(ux_publication_rel_prrelid_prpubid_index, 6113, on ux_publication_rel using btree(prrelid oid_ops, prpubid oid_ops));
#define PublicationRelPrrelidPrpubidIndexId 6113

DECLARE_UNIQUE_INDEX(ux_subscription_oid_index, 6114, on ux_subscription using btree(oid oid_ops));
#define SubscriptionObjectIndexId 6114

DECLARE_UNIQUE_INDEX(ux_subscription_subname_index, 6115, on ux_subscription using btree(subdbid oid_ops, subname name_ops));
#define SubscriptionNameIndexId 6115

DECLARE_UNIQUE_INDEX(ux_subscription_rel_srrelid_srsubid_index, 6117, on ux_subscription_rel using btree(srrelid oid_ops, srsubid oid_ops));
#define SubscriptionRelSrrelidSrsubidIndexId 6117

/* last step of initialization script: build the indexes declared above */
BUILD_INDICES

#endif							/* INDEXING_H */
