/*-------------------------------------------------------------------------
 *
 * pluxsql.h		- Definitions for the PL/uxSQL
 *			  procedural language
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 *
 * IDENTIFICATION
 *	  src/pl/pluxsql/src/pluxsql.h
 *
 *-------------------------------------------------------------------------
 */

#ifndef PLUXSQL_H
#define PLUXSQL_H

#include "access/xact.h"
#include "commands/event_trigger.h"
#include "commands/trigger.h"
#include "executor/spi.h"

/**********************************************************************
 * Definitions
 **********************************************************************/

/* define our text domain for translations */
#undef TEXTDOMAIN
#define TEXTDOMAIN UX_TEXTDOMAIN("pluxsql")

#undef _
#define _(x) dgettext(TEXTDOMAIN, x)

/*
 * Compiler's namespace item types
 */
typedef enum PLuxSQL_nsitem_type
{
	PLUXSQL_NSTYPE_LABEL,
	PLUXSQL_NSTYPE_VAR,
	PLUXSQL_NSTYPE_ROW,
	PLUXSQL_NSTYPE_REC
} PLuxSQL_nsitem_type;

/*
 * A PLUXSQL_NSTYPE_LABEL stack entry must be one of these types
 */
typedef enum PLuxSQL_label_type
{
	PLUXSQL_LABEL_BLOCK,		/* DECLARE/BEGIN block */
	PLUXSQL_LABEL_LOOP,			/* looping construct */
	PLUXSQL_LABEL_OTHER			/* anything else */
} PLuxSQL_label_type;

/*
 * Datum array node types
 */
typedef enum PLuxSQL_datum_type
{
	PLUXSQL_DTYPE_VAR,
	PLUXSQL_DTYPE_ROW,
	PLUXSQL_DTYPE_REC,
	PLUXSQL_DTYPE_RECFIELD,
	PLUXSQL_DTYPE_ARRAYELEM,
	PLUXSQL_DTYPE_EXPR
} PLuxSQL_datum_type;

/*
 * Variants distinguished in PLuxSQL_type structs
 */
typedef enum PLuxSQL_type_type
{
	PLUXSQL_TTYPE_SCALAR,		/* scalar types and domains */
	PLUXSQL_TTYPE_ROW,			/* composite types */
	PLUXSQL_TTYPE_REC,			/* RECORD pseudotype */
	PLUXSQL_TTYPE_PSEUDO		/* other pseudotypes */
} PLuxSQL_type_type;

/*
 * Execution tree node types
 */
typedef enum PLuxSQL_stmt_type
{
	PLUXSQL_STMT_BLOCK,
	PLUXSQL_STMT_ASSIGN,
	PLUXSQL_STMT_IF,
	PLUXSQL_STMT_CASE,
	PLUXSQL_STMT_LOOP,
	PLUXSQL_STMT_WHILE,
	PLUXSQL_STMT_FORI,
	PLUXSQL_STMT_FORS,
	PLUXSQL_STMT_FORC,
	PLUXSQL_STMT_FOREACH_A,
	PLUXSQL_STMT_EXIT,
	PLUXSQL_STMT_RETURN,
	PLUXSQL_STMT_RETURN_NEXT,
	PLUXSQL_STMT_RETURN_QUERY,
	PLUXSQL_STMT_RAISE,
	PLUXSQL_STMT_ASSERT,
	PLUXSQL_STMT_EXECSQL,
	PLUXSQL_STMT_DYNEXECUTE,
	PLUXSQL_STMT_DYNFORS,
	PLUXSQL_STMT_GETDIAG,
	PLUXSQL_STMT_OPEN,
	PLUXSQL_STMT_FETCH,
	PLUXSQL_STMT_CLOSE,
	PLUXSQL_STMT_PERFORM
} PLuxSQL_stmt_type;

/*
 * Execution node return codes
 */
enum
{
	PLUXSQL_RC_OK,
	PLUXSQL_RC_EXIT,
	PLUXSQL_RC_RETURN,
	PLUXSQL_RC_CONTINUE
};

/*
 * GET DIAGNOSTICS information items
 */
typedef enum PLuxSQL_getdiag_kind
{
	PLUXSQL_GETDIAG_ROW_COUNT,
	PLUXSQL_GETDIAG_RESULT_OID,
	PLUXSQL_GETDIAG_CONTEXT,
	PLUXSQL_GETDIAG_ERROR_CONTEXT,
	PLUXSQL_GETDIAG_ERROR_DETAIL,
	PLUXSQL_GETDIAG_ERROR_HINT,
	PLUXSQL_GETDIAG_RETURNED_SQLSTATE,
	PLUXSQL_GETDIAG_COLUMN_NAME,
	PLUXSQL_GETDIAG_CONSTRAINT_NAME,
	PLUXSQL_GETDIAG_DATATYPE_NAME,
	PLUXSQL_GETDIAG_MESSAGE_TEXT,
	PLUXSQL_GETDIAG_TABLE_NAME,
	PLUXSQL_GETDIAG_SCHEMA_NAME
} PLuxSQL_getdiag_kind;

/*
 * RAISE statement options
 */
typedef enum PLuxSQL_raise_option_type
{
	PLUXSQL_RAISEOPTION_ERRCODE,
	PLUXSQL_RAISEOPTION_MESSAGE,
	PLUXSQL_RAISEOPTION_DETAIL,
	PLUXSQL_RAISEOPTION_HINT,
	PLUXSQL_RAISEOPTION_COLUMN,
	PLUXSQL_RAISEOPTION_CONSTRAINT,
	PLUXSQL_RAISEOPTION_DATATYPE,
	PLUXSQL_RAISEOPTION_TABLE,
	PLUXSQL_RAISEOPTION_SCHEMA
} PLuxSQL_raise_option_type;

/*
 * Behavioral modes for pluxsql variable resolution
 */
typedef enum PLuxSQL_resolve_option
{
	PLUXSQL_RESOLVE_ERROR,		/* throw error if ambiguous */
	PLUXSQL_RESOLVE_VARIABLE,	/* prefer pluxsql var to table column */
	PLUXSQL_RESOLVE_COLUMN		/* prefer table column to pluxsql var */
} PLuxSQL_resolve_option;


/**********************************************************************
 * Node and structure definitions
 **********************************************************************/

/*
 * Uxdb data type
 */
typedef struct PLuxSQL_type
{
	char	   *typname;		/* (simple) name of the type */
	Oid			typoid;			/* OID of the data type */
	PLuxSQL_type_type ttype;	/* PLUXSQL_TTYPE_ code */
	int16		typlen;			/* stuff copied from its ux_type entry */
	bool		typbyval;
	char		typtype;
	Oid			typrelid;
	Oid			collation;		/* from ux_type, but can be overridden */
	bool		typisarray;		/* is "true" array, or domain over one */
	int32		atttypmod;		/* typmod (taken from someplace else) */
} PLuxSQL_type;

/*
 * Generic datum array item
 *
 * PLuxSQL_datum is the common supertype for PLuxSQL_expr, PLuxSQL_var,
 * PLuxSQL_row, PLuxSQL_rec, PLuxSQL_recfield, and PLuxSQL_arrayelem
 */
typedef struct PLuxSQL_datum
{
	PLuxSQL_datum_type dtype;
	int			dno;
} PLuxSQL_datum;

/*
 * Scalar or composite variable
 *
 * The variants PLuxSQL_var, PLuxSQL_row, and PLuxSQL_rec share these
 * fields
 */
typedef struct PLuxSQL_variable
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *refname;
	int			lineno;
} PLuxSQL_variable;

/*
 * SQL Query to plan and execute
 */
typedef struct PLuxSQL_expr
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *query;
	SPIPlanPtr	plan;
	Bitmapset  *paramnos;		/* all dnos referenced by this query */
	int			rwparam;		/* dno of read/write param, or -1 if none */

	/* function containing this expr (not set until we first parse query) */
	struct PLuxSQL_function *func;

	/* namespace chain visible to this expr */
	struct PLuxSQL_nsitem *ns;

	/* fields for "simple expression" fast-path execution: */
	Expr	   *expr_simple_expr;	/* NULL means not a simple expr */
	int			expr_simple_generation; /* plancache generation we checked */
	Oid			expr_simple_type;	/* result type Oid, if simple */
	int32		expr_simple_typmod; /* result typmod, if simple */

	/*
	 * if expr is simple AND prepared in current transaction,
	 * expr_simple_state and expr_simple_in_use are valid. Test validity by
	 * seeing if expr_simple_lxid matches current LXID.  (If not,
	 * expr_simple_state probably points at garbage!)
	 */
	ExprState  *expr_simple_state;	/* eval tree for expr_simple_expr */
	bool		expr_simple_in_use; /* true if eval tree is active */
	LocalTransactionId expr_simple_lxid;
} PLuxSQL_expr;

/*
 * Scalar variable
 */
typedef struct PLuxSQL_var
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *refname;
	int			lineno;

	PLuxSQL_type *datatype;
	int			isconst;
	int			notnull;
	PLuxSQL_expr *default_val;
	PLuxSQL_expr *cursor_explicit_expr;
	int			cursor_explicit_argrow;
	int			cursor_options;

	Datum		value;
	bool		isnull;
	bool		freeval;
} PLuxSQL_var;

/*
 * Row variable
 */
typedef struct PLuxSQL_row
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *refname;
	int			lineno;

	/* Note: TupleDesc is only set up for named rowtypes, else it is NULL. */
	TupleDesc	rowtupdesc;

	/*
	 * Note: if the underlying rowtype contains a dropped column, the
	 * corresponding fieldnames[] entry will be NULL, and there is no
	 * corresponding var (varnos[] will be -1).
	 */
	int			nfields;
	char	  **fieldnames;
	int		   *varnos;
} PLuxSQL_row;

/*
 * Record variable (non-fixed structure)
 */
typedef struct PLuxSQL_rec
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *refname;
	int			lineno;

	HeapTuple	tup;
	TupleDesc	tupdesc;
	bool		freetup;
	bool		freetupdesc;
} PLuxSQL_rec;

/*
 * Field in record
 */
typedef struct PLuxSQL_recfield
{
	PLuxSQL_datum_type dtype;
	int			dno;
	char	   *fieldname;
	int			recparentno;	/* dno of parent record */
} PLuxSQL_recfield;

/*
 * Element of array variable
 */
typedef struct PLuxSQL_arrayelem
{
	PLuxSQL_datum_type dtype;
	int			dno;
	PLuxSQL_expr *subscript;
	int			arrayparentno;	/* dno of parent array variable */

	/* Remaining fields are cached info about the array variable's type */
	Oid			parenttypoid;	/* type of array variable; 0 if not yet set */
	int32		parenttypmod;	/* typmod of array variable */
	Oid			arraytypoid;	/* OID of actual array type */
	int32		arraytypmod;	/* typmod of array (and its elements too) */
	int16		arraytyplen;	/* typlen of array type */
	Oid			elemtypoid;		/* OID of array element type */
	int16		elemtyplen;		/* typlen of element type */
	bool		elemtypbyval;	/* element type is pass-by-value? */
	char		elemtypalign;	/* typalign of element type */
} PLuxSQL_arrayelem;

/*
 * Item in the compilers namespace tree
 */
typedef struct PLuxSQL_nsitem
{
	PLuxSQL_nsitem_type itemtype;

	/*
	 * For labels, itemno is a value of enum PLuxSQL_label_type. For other
	 * itemtypes, itemno is the associated PLuxSQL_datum's dno.
	 */
	int			itemno;
	struct PLuxSQL_nsitem *prev;
	char		name[FLEXIBLE_ARRAY_MEMBER];	/* nul-terminated string */
} PLuxSQL_nsitem;

/*
 * Generic execution node
 */
typedef struct PLuxSQL_stmt
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
} PLuxSQL_stmt;

/*
 * One EXCEPTION condition name
 */
typedef struct PLuxSQL_condition
{
	int			sqlerrstate;	/* SQLSTATE code */
	char	   *condname;		/* condition name (for debugging) */
	struct PLuxSQL_condition *next;
} PLuxSQL_condition;

/*
 * EXCEPTION block
 */
typedef struct PLuxSQL_exception_block
{
	int			sqlstate_varno;
	int			sqlerrm_varno;
	List	   *exc_list;		/* List of WHEN clauses */
} PLuxSQL_exception_block;

/*
 * One EXCEPTION ... WHEN clause
 */
typedef struct PLuxSQL_exception
{
	int			lineno;
	PLuxSQL_condition *conditions;
	List	   *action;			/* List of statements */
} PLuxSQL_exception;

/*
 * Block of statements
 */
typedef struct PLuxSQL_stmt_block
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	List	   *body;			/* List of statements */
	int			n_initvars;
	int		   *initvarnos;
	PLuxSQL_exception_block *exceptions;
} PLuxSQL_stmt_block;

/*
 * Assign statement
 */
typedef struct PLuxSQL_stmt_assign
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	int			varno;
	PLuxSQL_expr *expr;
} PLuxSQL_stmt_assign;

/*
 * PERFORM statement
 */
typedef struct PLuxSQL_stmt_perform
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *expr;
} PLuxSQL_stmt_perform;

/*
 * GET DIAGNOSTICS item
 */
typedef struct PLuxSQL_diag_item
{
	PLuxSQL_getdiag_kind kind;	/* id for diagnostic value desired */
	int			target;			/* where to assign it */
} PLuxSQL_diag_item;

/*
 * GET DIAGNOSTICS statement
 */
typedef struct PLuxSQL_stmt_getdiag
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	bool		is_stacked;		/* STACKED or CURRENT diagnostics area? */
	List	   *diag_items;		/* List of PLuxSQL_diag_item */
} PLuxSQL_stmt_getdiag;

/*
 * IF statement
 */
typedef struct PLuxSQL_stmt_if
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *cond;			/* boolean expression for THEN */
	List	   *then_body;		/* List of statements */
	List	   *elsif_list;		/* List of PLuxSQL_if_elsif structs */
	List	   *else_body;		/* List of statements */
} PLuxSQL_stmt_if;

/*
 * one ELSIF arm of IF statement
 */
typedef struct PLuxSQL_if_elsif
{
	int			lineno;
	PLuxSQL_expr *cond;			/* boolean expression for this case */
	List	   *stmts;			/* List of statements */
} PLuxSQL_if_elsif;

/*
 * CASE statement
 */
typedef struct PLuxSQL_stmt_case
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *t_expr;		/* test expression, or NULL if none */
	int			t_varno;		/* var to store test expression value into */
	List	   *case_when_list; /* List of PLuxSQL_case_when structs */
	bool		have_else;		/* flag needed because list could be empty */
	List	   *else_stmts;		/* List of statements */
} PLuxSQL_stmt_case;

/*
 * one arm of CASE statement
 */
typedef struct PLuxSQL_case_when
{
	int			lineno;
	PLuxSQL_expr *expr;			/* boolean expression for this case */
	List	   *stmts;			/* List of statements */
} PLuxSQL_case_when;

/*
 * Unconditional LOOP statement
 */
typedef struct PLuxSQL_stmt_loop
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	List	   *body;			/* List of statements */
} PLuxSQL_stmt_loop;

/*
 * WHILE cond LOOP statement
 */
typedef struct PLuxSQL_stmt_while
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_expr *cond;
	List	   *body;			/* List of statements */
} PLuxSQL_stmt_while;

/*
 * FOR statement with integer loopvar
 */
typedef struct PLuxSQL_stmt_fori
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_var *var;
	PLuxSQL_expr *lower;
	PLuxSQL_expr *upper;
	PLuxSQL_expr *step;			/* NULL means default (ie, BY 1) */
	int			reverse;
	List	   *body;			/* List of statements */
} PLuxSQL_stmt_fori;

/*
 * PLuxSQL_stmt_forq represents a FOR statement running over a SQL query.
 * It is the common supertype of PLuxSQL_stmt_fors, PLuxSQL_stmt_forc
 * and PLuxSQL_dynfors.
 */
typedef struct PLuxSQL_stmt_forq
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_rec *rec;
	PLuxSQL_row *row;
	List	   *body;			/* List of statements */
} PLuxSQL_stmt_forq;

/*
 * FOR statement running over SELECT
 */
typedef struct PLuxSQL_stmt_fors
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_rec *rec;
	PLuxSQL_row *row;
	List	   *body;			/* List of statements */
	/* end of fields that must match PLuxSQL_stmt_forq */
	PLuxSQL_expr *query;
} PLuxSQL_stmt_fors;

/*
 * FOR statement running over cursor
 */
typedef struct PLuxSQL_stmt_forc
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_rec *rec;
	PLuxSQL_row *row;
	List	   *body;			/* List of statements */
	/* end of fields that must match PLuxSQL_stmt_forq */
	int			curvar;
	PLuxSQL_expr *argquery;		/* cursor arguments if any */
} PLuxSQL_stmt_forc;

/*
 * FOR statement running over EXECUTE
 */
typedef struct PLuxSQL_stmt_dynfors
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	PLuxSQL_rec *rec;
	PLuxSQL_row *row;
	List	   *body;			/* List of statements */
	/* end of fields that must match PLuxSQL_stmt_forq */
	PLuxSQL_expr *query;
	List	   *params;			/* USING expressions */
} PLuxSQL_stmt_dynfors;

/*
 * FOREACH item in array loop
 */
typedef struct PLuxSQL_stmt_foreach_a
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	char	   *label;
	int			varno;			/* loop target variable */
	int			slice;			/* slice dimension, or 0 */
	PLuxSQL_expr *expr;			/* array expression */
	List	   *body;			/* List of statements */
} PLuxSQL_stmt_foreach_a;

/*
 * OPEN a curvar
 */
typedef struct PLuxSQL_stmt_open
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	int			curvar;
	int			cursor_options;
	PLuxSQL_row *returntype;
	PLuxSQL_expr *argquery;
	PLuxSQL_expr *query;
	PLuxSQL_expr *dynquery;
	List	   *params;			/* USING expressions */
} PLuxSQL_stmt_open;

/*
 * FETCH or MOVE statement
 */
typedef struct PLuxSQL_stmt_fetch
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_rec *rec;			/* target, as record or row */
	PLuxSQL_row *row;
	int			curvar;			/* cursor variable to fetch from */
	FetchDirection direction;	/* fetch direction */
	long		how_many;		/* count, if constant (expr is NULL) */
	PLuxSQL_expr *expr;			/* count, if expression */
	bool		is_move;		/* is this a fetch or move? */
	bool		returns_multiple_rows;	/* can return more than one row? */
} PLuxSQL_stmt_fetch;

/*
 * CLOSE curvar
 */
typedef struct PLuxSQL_stmt_close
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	int			curvar;
} PLuxSQL_stmt_close;

/*
 * EXIT or CONTINUE statement
 */
typedef struct PLuxSQL_stmt_exit
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	bool		is_exit;		/* Is this an exit or a continue? */
	char	   *label;			/* NULL if it's an unlabelled EXIT/CONTINUE */
	PLuxSQL_expr *cond;
} PLuxSQL_stmt_exit;

/*
 * RETURN statement
 */
typedef struct PLuxSQL_stmt_return
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *expr;
	int			retvarno;
} PLuxSQL_stmt_return;

/*
 * RETURN NEXT statement
 */
typedef struct PLuxSQL_stmt_return_next
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *expr;
	int			retvarno;
} PLuxSQL_stmt_return_next;

/*
 * RETURN QUERY statement
 */
typedef struct PLuxSQL_stmt_return_query
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *query;		/* if static query */
	PLuxSQL_expr *dynquery;		/* if dynamic query (RETURN QUERY EXECUTE) */
	List	   *params;			/* USING arguments for dynamic query */
} PLuxSQL_stmt_return_query;

/*
 * RAISE statement
 */
typedef struct PLuxSQL_stmt_raise
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	int			elog_level;
	char	   *condname;		/* condition name, SQLSTATE, or NULL */
	char	   *message;		/* old-style message format literal, or NULL */
	List	   *params;			/* list of expressions for old-style message */
	List	   *options;		/* list of PLuxSQL_raise_option */
} PLuxSQL_stmt_raise;

/*
 * RAISE statement option
 */
typedef struct PLuxSQL_raise_option
{
	PLuxSQL_raise_option_type opt_type;
	PLuxSQL_expr *expr;
} PLuxSQL_raise_option;

/*
 * ASSERT statement
 */
typedef struct PLuxSQL_stmt_assert
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *cond;
	PLuxSQL_expr *message;
} PLuxSQL_stmt_assert;

/*
 * Generic SQL statement to execute
 */
typedef struct PLuxSQL_stmt_execsql
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *sqlstmt;
	bool		mod_stmt;		/* is the stmt INSERT/UPDATE/DELETE?  Note:
								 * mod_stmt is set when we plan the query */
	bool		into;			/* INTO supplied? */
	bool		strict;			/* INTO STRICT flag */
	PLuxSQL_rec *rec;			/* INTO target, if record */
	PLuxSQL_row *row;			/* INTO target, if row */
} PLuxSQL_stmt_execsql;

/*
 * Dynamic SQL string to execute
 */
typedef struct PLuxSQL_stmt_dynexecute
{
	PLuxSQL_stmt_type cmd_type;
	int			lineno;
	PLuxSQL_expr *query;		/* string expression */
	bool		into;			/* INTO supplied? */
	bool		strict;			/* INTO STRICT flag */
	PLuxSQL_rec *rec;			/* INTO target, if record */
	PLuxSQL_row *row;			/* INTO target, if row */
	List	   *params;			/* USING expressions */
} PLuxSQL_stmt_dynexecute;

/*
 * Hash lookup key for functions
 */
typedef struct PLuxSQL_func_hashkey
{
	Oid			funcOid;

	bool		isTrigger;		/* true if called as a trigger */

	/* be careful that pad bytes in this struct get zeroed! */

	/*
	 * For a trigger function, the OID of the trigger is part of the hash key
	 * --- we want to compile the trigger function separately for each trigger
	 * it is used with, in case the rowtype or transition table names are
	 * different.  Zero if not called as a trigger.
	 */
	Oid			trigOid;

	/*
	 * We must include the input collation as part of the hash key too,
	 * because we have to generate different plans (with different Param
	 * collations) for different collation settings.
	 */
	Oid			inputCollation;

	/*
	 * We include actual argument types in the hash key to support polymorphic
	 * PLuxSQL functions.  Be careful that extra positions are zeroed!
	 */
	Oid			argtypes[FUNC_MAX_ARGS];
} PLuxSQL_func_hashkey;

/*
 * Trigger type
 */
typedef enum PLuxSQL_trigtype
{
	PLUXSQL_DML_TRIGGER,
	PLUXSQL_EVENT_TRIGGER,
	PLUXSQL_NOT_TRIGGER
} PLuxSQL_trigtype;

/*
 * Complete compiled function
 */
typedef struct PLuxSQL_function
{
	char	   *fn_signature;
	Oid			fn_oid;
	TransactionId fn_xmin;
	ItemPointerData fn_tid;
	PLuxSQL_trigtype fn_is_trigger;
	Oid			fn_input_collation;
	PLuxSQL_func_hashkey *fn_hashkey;	/* back-link to hashtable key */
	MemoryContext fn_cxt;

	Oid			fn_rettype;
	int			fn_rettyplen;
	bool		fn_retbyval;
	bool		fn_retistuple;
	bool		fn_retset;
	bool		fn_readonly;

	int			fn_nargs;
	int			fn_argvarnos[FUNC_MAX_ARGS];
	int			out_param_varno;
	int			found_varno;
	int			new_varno;
	int			old_varno;
	int			tg_name_varno;
	int			tg_when_varno;
	int			tg_level_varno;
	int			tg_op_varno;
	int			tg_relid_varno;
	int			tg_relname_varno;
	int			tg_table_name_varno;
	int			tg_table_schema_varno;
	int			tg_nargs_varno;
	int			tg_argv_varno;

	/* for event triggers */
	int			tg_event_varno;
	int			tg_tag_varno;

	PLuxSQL_resolve_option resolve_option;

	bool		print_strict_params;

	/* extra checks */
	int			extra_warnings;
	int			extra_errors;

	/* the datums representing the function's local variables */
	int			ndatums;
	PLuxSQL_datum **datums;
	Bitmapset  *resettable_datums;	/* dnos of non-simple vars */

	/* function body parsetree */
	PLuxSQL_stmt_block *action;

	/* these fields change when the function is used */
	struct PLuxSQL_execstate *cur_estate;
	unsigned long use_count;
} PLuxSQL_function;

/*
 * Runtime execution data
 */
typedef struct PLuxSQL_execstate
{
	PLuxSQL_function *func;		/* function being executed */

	Datum		retval;
	bool		retisnull;
	Oid			rettype;		/* type of current retval */

	Oid			fn_rettype;		/* info about declared function rettype */
	bool		retistuple;
	bool		retisset;

	bool		readonly_func;

	TupleDesc	rettupdesc;
	char	   *exitlabel;		/* the "target" label of the current EXIT or
								 * CONTINUE stmt, if any */
	ErrorData  *cur_error;		/* current exception handler's error */

	Tuplestorestate *tuple_store;	/* SRFs accumulate results here */
	MemoryContext tuple_store_cxt;
	ResourceOwner tuple_store_owner;
	ReturnSetInfo *rsi;

	/* the datums representing the function's local variables */
	int			found_varno;
	int			ndatums;
	PLuxSQL_datum **datums;

	/* we pass datums[i] to the executor, when needed, in paramLI->params[i] */
	ParamListInfo paramLI;
	bool		params_dirty;	/* T if any resettable datum has been passed */

	/* EState to use for "simple" expression evaluation */
	EState	   *simple_eval_estate;

	/* lookup table to use for executing type casts */
	HTAB	   *cast_hash;
	MemoryContext cast_hash_context;

	/* memory context for statement-lifespan temporary values */
	MemoryContext stmt_mcontext;	/* current stmt context, or NULL if none */
	MemoryContext stmt_mcontext_parent; /* parent of current context */

	/* temporary state for results from evaluation of query or expr */
	SPITupleTable *eval_tuptable;
	uint64		eval_processed;
	Oid			eval_lastoid;
	ExprContext *eval_econtext; /* for executing simple expressions */

	/* status information for error context reporting */
	PLuxSQL_stmt *err_stmt;		/* current stmt */
	const char *err_text;		/* additional state info */

	void	   *plugin_info;	/* reserved for use by optional plugin */
} PLuxSQL_execstate;

/*
 * A PLuxSQL_plugin structure represents an instrumentation plugin.
 * To instrument PL/uxSQL, a plugin library must access the rendezvous
 * variable "PLuxSQL_plugin" and set it to point to a PLuxSQL_plugin struct.
 * Typically the struct could just be static data in the plugin library.
 * We expect that a plugin would do this at library load time (_UX_init()).
 * It must also be careful to set the rendezvous variable back to NULL
 * if it is unloaded (_UX_fini()).
 *
 * This structure is basically a collection of function pointers --- at
 * various interesting points in pl_exec.c, we call these functions
 * (if the pointers are non-NULL) to give the plugin a chance to watch
 * what we are doing.
 *
 * func_setup is called when we start a function, before we've initialized
 * the local variables defined by the function.
 *
 * func_beg is called when we start a function, after we've initialized
 * the local variables.
 *
 * func_end is called at the end of a function.
 *
 * stmt_beg and stmt_end are called before and after (respectively) each
 * statement.
 *
 * Also, immediately before any call to func_setup, PL/uxSQL fills in the
 * error_callback and assign_expr fields with pointers to its own
 * pluxsql_exec_error_callback and exec_assign_expr functions.  This is
 * a somewhat ad-hoc expedient to simplify life for debugger plugins.
 */
typedef struct PLuxSQL_plugin
{
	/* Function pointers set up by the plugin */
	void		(*func_setup) (PLuxSQL_execstate *estate, PLuxSQL_function *func);
	void		(*func_beg) (PLuxSQL_execstate *estate, PLuxSQL_function *func);
	void		(*func_end) (PLuxSQL_execstate *estate, PLuxSQL_function *func);
	void		(*stmt_beg) (PLuxSQL_execstate *estate, PLuxSQL_stmt *stmt);
	void		(*stmt_end) (PLuxSQL_execstate *estate, PLuxSQL_stmt *stmt);

	/* Function pointers set by PL/uxSQL itself */
	void		(*error_callback) (void *arg);
	void		(*assign_expr) (PLuxSQL_execstate *estate, PLuxSQL_datum *target,
								PLuxSQL_expr *expr);
} PLuxSQL_plugin;

/*
 * Struct types used during parsing
 */

typedef struct PLword
{
	char	   *ident;			/* palloc'd converted identifier */
	bool		quoted;			/* Was it double-quoted? */
} PLword;

typedef struct PLcword
{
	List	   *idents;			/* composite identifiers (list of String) */
} PLcword;

typedef struct PLwdatum
{
	PLuxSQL_datum *datum;		/* referenced variable */
	char	   *ident;			/* valid if simple name */
	bool		quoted;
	List	   *idents;			/* valid if composite name */
} PLwdatum;

/**********************************************************************
 * Global variable declarations
 **********************************************************************/

typedef enum
{
	IDENTIFIER_LOOKUP_NORMAL,	/* normal processing of var names */
	IDENTIFIER_LOOKUP_DECLARE,	/* In DECLARE --- don't look up names */
	IDENTIFIER_LOOKUP_EXPR		/* In SQL expression --- special case */
} IdentifierLookup;

extern IdentifierLookup pluxsql_IdentifierLookup;

extern int	pluxsql_variable_conflict;

extern bool pluxsql_print_strict_params;

extern bool pluxsql_check_asserts;

/* extra compile-time checks */
#define PLUXSQL_XCHECK_NONE			0
#define PLUXSQL_XCHECK_SHADOWVAR	1
#define PLUXSQL_XCHECK_ALL			((int) ~0)

extern int	pluxsql_extra_warnings;
extern int	pluxsql_extra_errors;

extern bool pluxsql_check_syntax;
extern bool pluxsql_DumpExecTree;

extern PLuxSQL_stmt_block *pluxsql_parse_result;

extern int	pluxsql_nDatums;
extern PLuxSQL_datum **pluxsql_Datums;

extern char *pluxsql_error_funcname;

extern PLuxSQL_function *pluxsql_curr_compile;
extern MemoryContext pluxsql_compile_tmp_cxt;

extern PLuxSQL_plugin **pluxsql_plugin_ptr;

/**********************************************************************
 * Function declarations
 **********************************************************************/

/*
 * Functions in pl_comp.c
 */
extern PLuxSQL_function *pluxsql_compile(FunctionCallInfo fcinfo,
				bool forValidator);
extern PLuxSQL_function *pluxsql_compile_inline(char *proc_source);
extern void pluxsql_parser_setup(struct ParseState *pstate,
					 PLuxSQL_expr *expr);
extern bool pluxsql_parse_word(char *word1, const char *yytxt,
				   PLwdatum *wdatum, PLword *word);
extern bool pluxsql_parse_dblword(char *word1, char *word2,
					  PLwdatum *wdatum, PLcword *cword);
extern bool pluxsql_parse_tripword(char *word1, char *word2, char *word3,
					   PLwdatum *wdatum, PLcword *cword);
extern PLuxSQL_type *pluxsql_parse_wordtype(char *ident);
extern PLuxSQL_type *pluxsql_parse_cwordtype(List *idents);
extern PLuxSQL_type *pluxsql_parse_wordrowtype(char *ident);
extern PLuxSQL_type *pluxsql_parse_cwordrowtype(List *idents);
extern PLuxSQL_type *pluxsql_build_datatype(Oid typeOid, int32 typmod,
					   Oid collation);
extern PLuxSQL_variable *pluxsql_build_variable(const char *refname, int lineno,
					   PLuxSQL_type *dtype,
					   bool add2namespace);
extern PLuxSQL_rec *pluxsql_build_record(const char *refname, int lineno,
					 bool add2namespace);
extern int pluxsql_recognize_err_condition(const char *condname,
								bool allow_sqlstate);
extern PLuxSQL_condition *pluxsql_parse_err_condition(char *condname);
extern void pluxsql_adddatum(PLuxSQL_datum *new);
extern int	pluxsql_add_initdatums(int **varnos);
extern void pluxsql_HashTableInit(void);

/*
 * Functions in pl_handler.c
 */
extern void _UX_init(void);

/*
 * Functions in pl_exec.c
 */
extern Datum pluxsql_exec_function(PLuxSQL_function *func,
					  FunctionCallInfo fcinfo,
					  EState *simple_eval_estate);
extern HeapTuple pluxsql_exec_trigger(PLuxSQL_function *func,
					 TriggerData *trigdata);
extern void pluxsql_exec_event_trigger(PLuxSQL_function *func,
						   EventTriggerData *trigdata);
extern void pluxsql_xact_cb(XactEvent event, void *arg);
extern void pluxsql_subxact_cb(SubXactEvent event, SubTransactionId mySubid,
				   SubTransactionId parentSubid, void *arg);
extern Oid pluxsql_exec_get_datum_type(PLuxSQL_execstate *estate,
							PLuxSQL_datum *datum);
extern void pluxsql_exec_get_datum_type_info(PLuxSQL_execstate *estate,
								 PLuxSQL_datum *datum,
								 Oid *typeid, int32 *typmod, Oid *collation);

/*
 * Functions for namespace handling in pl_funcs.c
 */
extern void pluxsql_ns_init(void);
extern void pluxsql_ns_push(const char *label,
				PLuxSQL_label_type label_type);
extern void pluxsql_ns_pop(void);
extern PLuxSQL_nsitem *pluxsql_ns_top(void);
extern void pluxsql_ns_additem(PLuxSQL_nsitem_type itemtype, int itemno, const char *name);
extern PLuxSQL_nsitem *pluxsql_ns_lookup(PLuxSQL_nsitem *ns_cur, bool localmode,
				  const char *name1, const char *name2,
				  const char *name3, int *names_used);
extern PLuxSQL_nsitem *pluxsql_ns_lookup_label(PLuxSQL_nsitem *ns_cur,
						const char *name);
extern PLuxSQL_nsitem *pluxsql_ns_find_nearest_loop(PLuxSQL_nsitem *ns_cur);

/*
 * Other functions in pl_funcs.c
 */
extern const char *pluxsql_stmt_typename(PLuxSQL_stmt *stmt);
extern const char *pluxsql_getdiag_kindname(PLuxSQL_getdiag_kind kind);
extern void pluxsql_free_function_memory(PLuxSQL_function *func);
extern void pluxsql_dumptree(PLuxSQL_function *func);

/*
 * Scanner functions in pl_scanner.c
 */
extern int	pluxsql_base_yylex(void);
extern int	pluxsql_yylex(void);
extern void pluxsql_push_back_token(int token);
extern bool pluxsql_token_is_unreserved_keyword(int token);
extern void pluxsql_append_source_text(StringInfo buf,
						   int startlocation, int endlocation);
extern int	pluxsql_peek(void);
extern void pluxsql_peek2(int *tok1_p, int *tok2_p, int *tok1_loc,
			  int *tok2_loc);
extern int	pluxsql_scanner_errposition(int location);
extern void pluxsql_yyerror(const char *message) ux_attribute_noreturn();
extern int	pluxsql_location_to_lineno(int location);
extern int	pluxsql_latest_lineno(void);
extern void pluxsql_scanner_init(const char *str);
extern void pluxsql_scanner_finish(void);

/*
 * Externs in gram.y
 */
extern int	pluxsql_yyparse(void);

#endif							/* PLUXSQL_H */
