/*-------------------------------------------------------------------------
 *
 * ux_sema.h
 *	  Platform-independent API for semaphores.
 *
 * UXsinoDB requires counting semaphores (the kind that keep track of
 * multiple unlock operations, and will allow an equal number of subsequent
 * lock operations before blocking).  The underlying implementation is
 * not the same on every platform.  This file defines the API that must
 * be provided by each port.
 *
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/include/storage/ux_sema.h
 *
 *-------------------------------------------------------------------------
 */
#ifndef UX_SEMA_H
#define UX_SEMA_H

/*
 * struct UXSemaphoreData and pointer type UXSemaphore are the data structure
 * representing an individual semaphore.  The contents of UXSemaphoreData vary
 * across implementations and must never be touched by platform-independent
 * code; hence, UXSemaphoreData is declared as an opaque struct here.
 *
 * However, Windows is sufficiently unlike our other ports that it doesn't
 * seem worth insisting on ABI compatibility for Windows too.  Hence, on
 * that platform just define UXSemaphore as HANDLE.
 */
#ifndef USE_WIN32_SEMAPHORES
typedef struct UXSemaphoreData *UXSemaphore;
#else
typedef HANDLE UXSemaphore;
#endif


/* Report amount of shared memory needed */
extern Size UXSemaphoreShmemSize(int maxSemas);

/* Module initialization (called during uxmaster start or shmem reinit) */
extern void UXReserveSemaphores(int maxSemas, int port);

/* Allocate a UXSemaphore structure with initial count 1 */
extern UXSemaphore UXSemaphoreCreate(void);

/* Reset a previously-initialized UXSemaphore to have count 0 */
extern void UXSemaphoreReset(UXSemaphore sema);

/* Lock a semaphore (decrement count), blocking if count would be < 0 */
extern void UXSemaphoreLock(UXSemaphore sema);

/* Unlock a semaphore (increment count) */
extern void UXSemaphoreUnlock(UXSemaphore sema);

/* Lock a semaphore only if able to do so without blocking */
extern bool UXSemaphoreTryLock(UXSemaphore sema);

#endif							/* UX_SEMA_H */
