/* ----------
 *	uxstat.h
 *
 *	Definitions for the UXsinoDB statistics collector daemon.
 *
 *	Copyright (c) 2001-2017, UXsinoDB Global Development Group
 *
 *	src/include/uxstat.h
 * ----------
 */
#ifndef UXSTAT_H
#define UXSTAT_H

#include "datatype/timestamp.h"
#include "fmgr.h"
#include "libuxsql/uxsqlcomm.h"
#include "port/atomics.h"
#include "portability/instr_time.h"
#include "uxmaster/uxarch.h"
#include "storage/proc.h"
#include "utils/hsearch.h"
#include "utils/relcache.h"


/* ----------
 * Paths for the statistics files (relative to installation's $UXDATA).
 * ----------
 */
#define UXSTAT_STAT_PERMANENT_DIRECTORY		"ux_stat"
#define UXSTAT_STAT_PERMANENT_FILENAME		"ux_stat/global.stat"
#define UXSTAT_STAT_PERMANENT_TMPFILE		"ux_stat/global.tmp"

/* Default directory to store temporary statistics data in */
#define UX_STAT_TMP_DIR		"ux_stat_tmp"

/* Values for track_functions GUC variable --- order is significant! */
typedef enum TrackFunctionsLevel
{
	TRACK_FUNC_OFF,
	TRACK_FUNC_PL,
	TRACK_FUNC_ALL
}			TrackFunctionsLevel;

/* ----------
 * The types of backend -> collector messages
 * ----------
 */
typedef enum StatMsgType
{
	UXSTAT_MTYPE_DUMMY,
	UXSTAT_MTYPE_INQUIRY,
	UXSTAT_MTYPE_TABSTAT,
	UXSTAT_MTYPE_TABPURGE,
	UXSTAT_MTYPE_DROPDB,
	UXSTAT_MTYPE_RESETCOUNTER,
	UXSTAT_MTYPE_RESETSHAREDCOUNTER,
	UXSTAT_MTYPE_RESETSINGLECOUNTER,
	UXSTAT_MTYPE_AUTOVAC_START,
	UXSTAT_MTYPE_VACUUM,
	UXSTAT_MTYPE_ANALYZE,
	UXSTAT_MTYPE_ARCHIVER,
	UXSTAT_MTYPE_BGWRITER,
	UXSTAT_MTYPE_FUNCSTAT,
	UXSTAT_MTYPE_FUNCPURGE,
	UXSTAT_MTYPE_RECOVERYCONFLICT,
	UXSTAT_MTYPE_TEMPFILE,
	UXSTAT_MTYPE_DEADLOCK
} StatMsgType;

/* ----------
 * The data type used for counters.
 * ----------
 */
typedef int64 UxStat_Counter;

/* ----------
 * UxStat_TableCounts			The actual per-table counts kept by a backend
 *
 * This struct should contain only actual event counters, because we memcmp
 * it against zeroes to detect whether there are any counts to transmit.
 * It is a component of UxStat_TableStatus (within-backend state) and
 * UxStat_TableEntry (the transmitted message format).
 *
 * Note: for a table, tuples_returned is the number of tuples successfully
 * fetched by heap_getnext, while tuples_fetched is the number of tuples
 * successfully fetched by heap_fetch under the control of bitmap indexscans.
 * For an index, tuples_returned is the number of index entries returned by
 * the index AM, while tuples_fetched is the number of tuples successfully
 * fetched by heap_fetch under the control of simple indexscans for this index.
 *
 * tuples_inserted/updated/deleted/hot_updated count attempted actions,
 * regardless of whether the transaction committed.  delta_live_tuples,
 * delta_dead_tuples, and changed_tuples are set depending on commit or abort.
 * Note that delta_live_tuples and delta_dead_tuples can be negative!
 * ----------
 */
typedef struct UxStat_TableCounts
{
	UxStat_Counter t_numscans;

	UxStat_Counter t_tuples_returned;
	UxStat_Counter t_tuples_fetched;

	UxStat_Counter t_tuples_inserted;
	UxStat_Counter t_tuples_updated;
	UxStat_Counter t_tuples_deleted;
	UxStat_Counter t_tuples_hot_updated;
	bool		t_truncated;

	UxStat_Counter t_delta_live_tuples;
	UxStat_Counter t_delta_dead_tuples;
	UxStat_Counter t_changed_tuples;

	UxStat_Counter t_blocks_fetched;
	UxStat_Counter t_blocks_hit;
} UxStat_TableCounts;

/* Possible targets for resetting cluster-wide shared values */
typedef enum UxStat_Shared_Reset_Target
{
	RESET_ARCHIVER,
	RESET_BGWRITER
} UxStat_Shared_Reset_Target;

/* Possible object types for resetting single counters */
typedef enum UxStat_Single_Reset_Type
{
	RESET_TABLE,
	RESET_FUNCTION
} UxStat_Single_Reset_Type;

/* ------------------------------------------------------------
 * Structures kept in backend local memory while accumulating counts
 * ------------------------------------------------------------
 */


/* ----------
 * UxStat_TableStatus			Per-table status within a backend
 *
 * Many of the event counters are nontransactional, ie, we count events
 * in committed and aborted transactions alike.  For these, we just count
 * directly in the UxStat_TableStatus.  However, delta_live_tuples,
 * delta_dead_tuples, and changed_tuples must be derived from event counts
 * with awareness of whether the transaction or subtransaction committed or
 * aborted.  Hence, we also keep a stack of per-(sub)transaction status
 * records for every table modified in the current transaction.  At commit
 * or abort, we propagate tuples_inserted/updated/deleted up to the
 * parent subtransaction level, or out to the parent UxStat_TableStatus,
 * as appropriate.
 * ----------
 */
typedef struct UxStat_TableStatus
{
	Oid			t_id;			/* table's OID */
	bool		t_shared;		/* is it a shared catalog? */
	struct UxStat_TableXactStatus *trans;	/* lowest subxact's counts */
	UxStat_TableCounts t_counts;	/* event counts to be sent */
} UxStat_TableStatus;

/* ----------
 * UxStat_TableXactStatus		Per-table, per-subtransaction status
 * ----------
 */
typedef struct UxStat_TableXactStatus
{
	UxStat_Counter tuples_inserted; /* tuples inserted in (sub)xact */
	UxStat_Counter tuples_updated;	/* tuples updated in (sub)xact */
	UxStat_Counter tuples_deleted;	/* tuples deleted in (sub)xact */
	bool		truncated;		/* relation truncated in this (sub)xact */
	UxStat_Counter inserted_pre_trunc;	/* tuples inserted prior to truncate */
	UxStat_Counter updated_pre_trunc;	/* tuples updated prior to truncate */
	UxStat_Counter deleted_pre_trunc;	/* tuples deleted prior to truncate */
	int			nest_level;		/* subtransaction nest level */
	/* links to other structs for same relation: */
	struct UxStat_TableXactStatus *upper;	/* next higher subxact if any */
	UxStat_TableStatus *parent; /* per-table status */
	/* structs of same subxact level are linked here: */
	struct UxStat_TableXactStatus *next;	/* next of same subxact */
} UxStat_TableXactStatus;


/* ------------------------------------------------------------
 * Message formats follow
 * ------------------------------------------------------------
 */


/* ----------
 * UxStat_MsgHdr				The common message header
 * ----------
 */
typedef struct UxStat_MsgHdr
{
	StatMsgType m_type;
	int			m_size;
} UxStat_MsgHdr;

/* ----------
 * Space available in a message.  This will keep the UDP packets below 1K,
 * which should fit unfragmented into the MTU of the loopback interface.
 * (Larger values of UXSTAT_MAX_MSG_SIZE would work for that on most
 * platforms, but we're being conservative here.)
 * ----------
 */
#define UXSTAT_MAX_MSG_SIZE 1000
#define UXSTAT_MSG_PAYLOAD	(UXSTAT_MAX_MSG_SIZE - sizeof(UxStat_MsgHdr))


/* ----------
 * UxStat_MsgDummy				A dummy message, ignored by the collector
 * ----------
 */
typedef struct UxStat_MsgDummy
{
	UxStat_MsgHdr m_hdr;
} UxStat_MsgDummy;


/* ----------
 * UxStat_MsgInquiry			Sent by a backend to ask the collector
 *								to write the stats file(s).
 *
 * Ordinarily, an inquiry message prompts writing of the global stats file,
 * the stats file for shared catalogs, and the stats file for the specified
 * database.  If databaseid is InvalidOid, only the first two are written.
 *
 * New file(s) will be written only if the existing file has a timestamp
 * older than the specified cutoff_time; this prevents duplicated effort
 * when multiple requests arrive at nearly the same time, assuming that
 * backends send requests with cutoff_times a little bit in the past.
 *
 * clock_time should be the requestor's current local time; the collector
 * uses this to check for the system clock going backward, but it has no
 * effect unless that occurs.  We assume clock_time >= cutoff_time, though.
 * ----------
 */

typedef struct UxStat_MsgInquiry
{
	UxStat_MsgHdr m_hdr;
	TimestampTz clock_time;		/* observed local clock time */
	TimestampTz cutoff_time;	/* minimum acceptable file timestamp */
	Oid			databaseid;		/* requested DB (InvalidOid => shared only) */
} UxStat_MsgInquiry;


/* ----------
 * UxStat_TableEntry			Per-table info in a MsgTabstat
 * ----------
 */
typedef struct UxStat_TableEntry
{
	Oid			t_id;
	UxStat_TableCounts t_counts;
} UxStat_TableEntry;

/* ----------
 * UxStat_MsgTabstat			Sent by the backend to report table
 *								and buffer access statistics.
 * ----------
 */
#define UXSTAT_NUM_TABENTRIES  \
	((UXSTAT_MSG_PAYLOAD - sizeof(Oid) - 3 * sizeof(int) - 2 * sizeof(UxStat_Counter))	\
	 / sizeof(UxStat_TableEntry))

typedef struct UxStat_MsgTabstat
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	int			m_nentries;
	int			m_xact_commit;
	int			m_xact_rollback;
	UxStat_Counter m_block_read_time;	/* times in microseconds */
	UxStat_Counter m_block_write_time;
	UxStat_TableEntry m_entry[UXSTAT_NUM_TABENTRIES];
} UxStat_MsgTabstat;


/* ----------
 * UxStat_MsgTabpurge			Sent by the backend to tell the collector
 *								about dead tables.
 * ----------
 */
#define UXSTAT_NUM_TABPURGE  \
	((UXSTAT_MSG_PAYLOAD - sizeof(Oid) - sizeof(int))  \
	 / sizeof(Oid))

typedef struct UxStat_MsgTabpurge
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	int			m_nentries;
	Oid			m_tableid[UXSTAT_NUM_TABPURGE];
} UxStat_MsgTabpurge;


/* ----------
 * UxStat_MsgDropdb				Sent by the backend to tell the collector
 *								about a dropped database
 * ----------
 */
typedef struct UxStat_MsgDropdb
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
} UxStat_MsgDropdb;


/* ----------
 * UxStat_MsgResetcounter		Sent by the backend to tell the collector
 *								to reset counters
 * ----------
 */
typedef struct UxStat_MsgResetcounter
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
} UxStat_MsgResetcounter;

/* ----------
 * UxStat_MsgResetsharedcounter Sent by the backend to tell the collector
 *								to reset a shared counter
 * ----------
 */
typedef struct UxStat_MsgResetsharedcounter
{
	UxStat_MsgHdr m_hdr;
	UxStat_Shared_Reset_Target m_resettarget;
} UxStat_MsgResetsharedcounter;

/* ----------
 * UxStat_MsgResetsinglecounter Sent by the backend to tell the collector
 *								to reset a single counter
 * ----------
 */
typedef struct UxStat_MsgResetsinglecounter
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	UxStat_Single_Reset_Type m_resettype;
	Oid			m_objectid;
} UxStat_MsgResetsinglecounter;

/* ----------
 * UxStat_MsgAutovacStart		Sent by the autovacuum daemon to signal
 *								that a database is going to be processed
 * ----------
 */
typedef struct UxStat_MsgAutovacStart
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	TimestampTz m_start_time;
} UxStat_MsgAutovacStart;


/* ----------
 * UxStat_MsgVacuum				Sent by the backend or autovacuum daemon
 *								after VACUUM
 * ----------
 */
typedef struct UxStat_MsgVacuum
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	Oid			m_tableoid;
	bool		m_autovacuum;
	TimestampTz m_vacuumtime;
	UxStat_Counter m_live_tuples;
	UxStat_Counter m_dead_tuples;
} UxStat_MsgVacuum;


/* ----------
 * UxStat_MsgAnalyze			Sent by the backend or autovacuum daemon
 *								after ANALYZE
 * ----------
 */
typedef struct UxStat_MsgAnalyze
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	Oid			m_tableoid;
	bool		m_autovacuum;
	bool		m_resetcounter;
	TimestampTz m_analyzetime;
	UxStat_Counter m_live_tuples;
	UxStat_Counter m_dead_tuples;
} UxStat_MsgAnalyze;


/* ----------
 * UxStat_MsgArchiver			Sent by the archiver to update statistics.
 * ----------
 */
typedef struct UxStat_MsgArchiver
{
	UxStat_MsgHdr m_hdr;
	bool		m_failed;		/* Failed attempt */
	char		m_xlog[MAX_XFN_CHARS + 1];
	TimestampTz m_timestamp;
} UxStat_MsgArchiver;

/* ----------
 * UxStat_MsgBgWriter			Sent by the bgwriter to update statistics.
 * ----------
 */
typedef struct UxStat_MsgBgWriter
{
	UxStat_MsgHdr m_hdr;

	UxStat_Counter m_timed_checkpoints;
	UxStat_Counter m_requested_checkpoints;
	UxStat_Counter m_buf_written_checkpoints;
	UxStat_Counter m_buf_written_clean;
	UxStat_Counter m_maxwritten_clean;
	UxStat_Counter m_buf_written_backend;
	UxStat_Counter m_buf_fsync_backend;
	UxStat_Counter m_buf_alloc;
	UxStat_Counter m_checkpoint_write_time; /* times in milliseconds */
	UxStat_Counter m_checkpoint_sync_time;
} UxStat_MsgBgWriter;

/* ----------
 * UxStat_MsgRecoveryConflict	Sent by the backend upon recovery conflict
 * ----------
 */
typedef struct UxStat_MsgRecoveryConflict
{
	UxStat_MsgHdr m_hdr;

	Oid			m_databaseid;
	int			m_reason;
} UxStat_MsgRecoveryConflict;

/* ----------
 * UxStat_MsgTempFile	Sent by the backend upon creating a temp file
 * ----------
 */
typedef struct UxStat_MsgTempFile
{
	UxStat_MsgHdr m_hdr;

	Oid			m_databaseid;
	size_t		m_filesize;
} UxStat_MsgTempFile;

/* ----------
 * UxStat_FunctionCounts	The actual per-function counts kept by a backend
 *
 * This struct should contain only actual event counters, because we memcmp
 * it against zeroes to detect whether there are any counts to transmit.
 *
 * Note that the time counters are in instr_time format here.  We convert to
 * microseconds in UxStat_Counter format when transmitting to the collector.
 * ----------
 */
typedef struct UxStat_FunctionCounts
{
	UxStat_Counter f_numcalls;
	instr_time	f_total_time;
	instr_time	f_self_time;
} UxStat_FunctionCounts;

/* ----------
 * UxStat_BackendFunctionEntry	Entry in backend's per-function hash table
 * ----------
 */
typedef struct UxStat_BackendFunctionEntry
{
	Oid			f_id;
	UxStat_FunctionCounts f_counts;
} UxStat_BackendFunctionEntry;

/* ----------
 * UxStat_FunctionEntry			Per-function info in a MsgFuncstat
 * ----------
 */
typedef struct UxStat_FunctionEntry
{
	Oid			f_id;
	UxStat_Counter f_numcalls;
	UxStat_Counter f_total_time;	/* times in microseconds */
	UxStat_Counter f_self_time;
} UxStat_FunctionEntry;

/* ----------
 * UxStat_MsgFuncstat			Sent by the backend to report function
 *								usage statistics.
 * ----------
 */
#define UXSTAT_NUM_FUNCENTRIES	\
	((UXSTAT_MSG_PAYLOAD - sizeof(Oid) - sizeof(int))  \
	 / sizeof(UxStat_FunctionEntry))

typedef struct UxStat_MsgFuncstat
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	int			m_nentries;
	UxStat_FunctionEntry m_entry[UXSTAT_NUM_FUNCENTRIES];
} UxStat_MsgFuncstat;

/* ----------
 * UxStat_MsgFuncpurge			Sent by the backend to tell the collector
 *								about dead functions.
 * ----------
 */
#define UXSTAT_NUM_FUNCPURGE  \
	((UXSTAT_MSG_PAYLOAD - sizeof(Oid) - sizeof(int))  \
	 / sizeof(Oid))

typedef struct UxStat_MsgFuncpurge
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
	int			m_nentries;
	Oid			m_functionid[UXSTAT_NUM_FUNCPURGE];
} UxStat_MsgFuncpurge;

/* ----------
 * UxStat_MsgDeadlock			Sent by the backend to tell the collector
 *								about a deadlock that occurred.
 * ----------
 */
typedef struct UxStat_MsgDeadlock
{
	UxStat_MsgHdr m_hdr;
	Oid			m_databaseid;
} UxStat_MsgDeadlock;


/* ----------
 * UxStat_Msg					Union over all possible messages.
 * ----------
 */
typedef union UxStat_Msg
{
	UxStat_MsgHdr msg_hdr;
	UxStat_MsgDummy msg_dummy;
	UxStat_MsgInquiry msg_inquiry;
	UxStat_MsgTabstat msg_tabstat;
	UxStat_MsgTabpurge msg_tabpurge;
	UxStat_MsgDropdb msg_dropdb;
	UxStat_MsgResetcounter msg_resetcounter;
	UxStat_MsgResetsharedcounter msg_resetsharedcounter;
	UxStat_MsgResetsinglecounter msg_resetsinglecounter;
	UxStat_MsgAutovacStart msg_autovacuum;
	UxStat_MsgVacuum msg_vacuum;
	UxStat_MsgAnalyze msg_analyze;
	UxStat_MsgArchiver msg_archiver;
	UxStat_MsgBgWriter msg_bgwriter;
	UxStat_MsgFuncstat msg_funcstat;
	UxStat_MsgFuncpurge msg_funcpurge;
	UxStat_MsgRecoveryConflict msg_recoveryconflict;
	UxStat_MsgDeadlock msg_deadlock;
} UxStat_Msg;


/* ------------------------------------------------------------
 * Statistic collector data structures follow
 *
 * UXSTAT_FILE_FORMAT_ID should be changed whenever any of these
 * data structures change.
 * ------------------------------------------------------------
 */

#define UXSTAT_FILE_FORMAT_ID	0x01A5BC9D

/* ----------
 * UxStat_StatDBEntry			The collector's data per database
 * ----------
 */
typedef struct UxStat_StatDBEntry
{
	Oid			databaseid;
	UxStat_Counter n_xact_commit;
	UxStat_Counter n_xact_rollback;
	UxStat_Counter n_blocks_fetched;
	UxStat_Counter n_blocks_hit;
	UxStat_Counter n_tuples_returned;
	UxStat_Counter n_tuples_fetched;
	UxStat_Counter n_tuples_inserted;
	UxStat_Counter n_tuples_updated;
	UxStat_Counter n_tuples_deleted;
	TimestampTz last_autovac_time;
	UxStat_Counter n_conflict_tablespace;
	UxStat_Counter n_conflict_lock;
	UxStat_Counter n_conflict_snapshot;
	UxStat_Counter n_conflict_bufferpin;
	UxStat_Counter n_conflict_startup_deadlock;
	UxStat_Counter n_temp_files;
	UxStat_Counter n_temp_bytes;
	UxStat_Counter n_deadlocks;
	UxStat_Counter n_block_read_time;	/* times in microseconds */
	UxStat_Counter n_block_write_time;

	TimestampTz stat_reset_timestamp;
	TimestampTz stats_timestamp;	/* time of db stats file update */

	/*
	 * tables and functions must be last in the struct, because we don't write
	 * the pointers out to the stats file.
	 */
	HTAB	   *tables;
	HTAB	   *functions;
} UxStat_StatDBEntry;


/* ----------
 * UxStat_StatTabEntry			The collector's data per table (or index)
 * ----------
 */
typedef struct UxStat_StatTabEntry
{
	Oid			tableid;

	UxStat_Counter numscans;

	UxStat_Counter tuples_returned;
	UxStat_Counter tuples_fetched;

	UxStat_Counter tuples_inserted;
	UxStat_Counter tuples_updated;
	UxStat_Counter tuples_deleted;
	UxStat_Counter tuples_hot_updated;

	UxStat_Counter n_live_tuples;
	UxStat_Counter n_dead_tuples;
	UxStat_Counter changes_since_analyze;

	UxStat_Counter blocks_fetched;
	UxStat_Counter blocks_hit;

	TimestampTz vacuum_timestamp;	/* user initiated vacuum */
	UxStat_Counter vacuum_count;
	TimestampTz autovac_vacuum_timestamp;	/* autovacuum initiated */
	UxStat_Counter autovac_vacuum_count;
	TimestampTz analyze_timestamp;	/* user initiated */
	UxStat_Counter analyze_count;
	TimestampTz autovac_analyze_timestamp;	/* autovacuum initiated */
	UxStat_Counter autovac_analyze_count;
} UxStat_StatTabEntry;


/* ----------
 * UxStat_StatFuncEntry			The collector's data per function
 * ----------
 */
typedef struct UxStat_StatFuncEntry
{
	Oid			functionid;

	UxStat_Counter f_numcalls;

	UxStat_Counter f_total_time;	/* times in microseconds */
	UxStat_Counter f_self_time;
} UxStat_StatFuncEntry;


/*
 * Archiver statistics kept in the stats collector
 */
typedef struct UxStat_ArchiverStats
{
	UxStat_Counter archived_count;	/* archival successes */
	char		last_archived_wal[MAX_XFN_CHARS + 1];	/* last WAL file
														 * archived */
	TimestampTz last_archived_timestamp;	/* last archival success time */
	UxStat_Counter failed_count;	/* failed archival attempts */
	char		last_failed_wal[MAX_XFN_CHARS + 1]; /* WAL file involved in
													 * last failure */
	TimestampTz last_failed_timestamp;	/* last archival failure time */
	TimestampTz stat_reset_timestamp;
} UxStat_ArchiverStats;

/*
 * Global statistics kept in the stats collector
 */
typedef struct UxStat_GlobalStats
{
	TimestampTz stats_timestamp;	/* time of stats file update */
	UxStat_Counter timed_checkpoints;
	UxStat_Counter requested_checkpoints;
	UxStat_Counter checkpoint_write_time;	/* times in milliseconds */
	UxStat_Counter checkpoint_sync_time;
	UxStat_Counter buf_written_checkpoints;
	UxStat_Counter buf_written_clean;
	UxStat_Counter maxwritten_clean;
	UxStat_Counter buf_written_backend;
	UxStat_Counter buf_fsync_backend;
	UxStat_Counter buf_alloc;
	TimestampTz stat_reset_timestamp;
} UxStat_GlobalStats;


/* ----------
 * Backend types
 * ----------
 */
typedef enum BackendType
{
	B_AUTOVAC_LAUNCHER,
	B_AUTOVAC_WORKER,
	B_BACKEND,
	B_BG_WORKER,
	B_BG_WRITER,
	B_CHECKPOINTER,
	B_STARTUP,
	B_WAL_RECEIVER,
	B_WAL_SENDER,
	B_WAL_WRITER
} BackendType;


/* ----------
 * Backend states
 * ----------
 */
typedef enum BackendState
{
	STATE_UNDEFINED,
	STATE_IDLE,
	STATE_RUNNING,
	STATE_IDLEINTRANSACTION,
	STATE_FASTPATH,
	STATE_IDLEINTRANSACTION_ABORTED,
	STATE_DISABLED
} BackendState;


/* ----------
 * Wait Classes
 * ----------
 */
#define UX_WAIT_LWLOCK				0x01000000U
#define UX_WAIT_LOCK				0x03000000U
#define UX_WAIT_BUFFER_PIN			0x04000000U
#define UX_WAIT_ACTIVITY			0x05000000U
#define UX_WAIT_CLIENT				0x06000000U
#define UX_WAIT_EXTENSION			0x07000000U
#define UX_WAIT_IPC					0x08000000U
#define UX_WAIT_TIMEOUT				0x09000000U
#define UX_WAIT_IO					0x0A000000U

/* ----------
 * Wait Events - Activity
 *
 * Use this category when a process is waiting because it has no work to do,
 * unless the "Client" or "Timeout" category describes the situation better.
 * Typically, this should only be used for background processes.
 * ----------
 */
typedef enum
{
	WAIT_EVENT_ARCHIVER_MAIN = UX_WAIT_ACTIVITY,
	WAIT_EVENT_AUTOVACUUM_MAIN,
	WAIT_EVENT_BGWRITER_HIBERNATE,
	WAIT_EVENT_BGWRITER_MAIN,
	WAIT_EVENT_CHECKPOINTER_MAIN,
	WAIT_EVENT_LOGICAL_LAUNCHER_MAIN,
	WAIT_EVENT_LOGICAL_APPLY_MAIN,
	WAIT_EVENT_UXSTAT_MAIN,
	WAIT_EVENT_RECOVERY_WAL_ALL,
	WAIT_EVENT_RECOVERY_WAL_STREAM,
	WAIT_EVENT_SYSLOGGER_MAIN,
	WAIT_EVENT_WAL_RECEIVER_MAIN,
	WAIT_EVENT_WAL_SENDER_MAIN,
	WAIT_EVENT_WAL_WRITER_MAIN
} WaitEventActivity;

/* ----------
 * Wait Events - Client
 *
 * Use this category when a process is waiting to send data to or receive data
 * from the frontend process to which it is connected.  This is never used for
 * a background process, which has no client connection.
 * ----------
 */
typedef enum
{
	WAIT_EVENT_CLIENT_READ = UX_WAIT_CLIENT,
	WAIT_EVENT_CLIENT_WRITE,
	WAIT_EVENT_LIBUXSQLWALRECEIVER_CONNECT,
	WAIT_EVENT_LIBUXSQLWALRECEIVER_RECEIVE,
	WAIT_EVENT_SSL_OPEN_SERVER,
	WAIT_EVENT_WAL_RECEIVER_WAIT_START,
	WAIT_EVENT_WAL_SENDER_WAIT_WAL,
	WAIT_EVENT_WAL_SENDER_WRITE_DATA
} WaitEventClient;

/* ----------
 * Wait Events - IPC
 *
 * Use this category when a process cannot complete the work it is doing because
 * it is waiting for a notification from another process.
 * ----------
 */
typedef enum
{
	WAIT_EVENT_BGWORKER_SHUTDOWN = UX_WAIT_IPC,
	WAIT_EVENT_BGWORKER_STARTUP,
	WAIT_EVENT_BTREE_PAGE,
	WAIT_EVENT_EXECUTE_GATHER,
	WAIT_EVENT_LOGICAL_SYNC_DATA,
	WAIT_EVENT_LOGICAL_SYNC_STATE_CHANGE,
	WAIT_EVENT_MQ_INTERNAL,
	WAIT_EVENT_MQ_PUT_MESSAGE,
	WAIT_EVENT_MQ_RECEIVE,
	WAIT_EVENT_MQ_SEND,
	WAIT_EVENT_PARALLEL_FINISH,
	WAIT_EVENT_PARALLEL_BITMAP_SCAN,
	WAIT_EVENT_PROCARRAY_GROUP_UPDATE,
	WAIT_EVENT_REPLICATION_ORIGIN_DROP,
	WAIT_EVENT_REPLICATION_SLOT_DROP,
	WAIT_EVENT_SAFE_SNAPSHOT,
	WAIT_EVENT_SYNC_REP
} WaitEventIPC;

/* ----------
 * Wait Events - Timeout
 *
 * Use this category when a process is waiting for a timeout to expire.
 * ----------
 */
typedef enum
{
	WAIT_EVENT_BASE_BACKUP_THROTTLE = UX_WAIT_TIMEOUT,
	WAIT_EVENT_UX_SLEEP,
	WAIT_EVENT_RECOVERY_APPLY_DELAY
} WaitEventTimeout;

/* ----------
 * Wait Events - IO
 *
 * Use this category when a process is waiting for a IO.
 * ----------
 */
typedef enum
{
	WAIT_EVENT_BUFFILE_READ = UX_WAIT_IO,
	WAIT_EVENT_BUFFILE_WRITE,
	WAIT_EVENT_CONTROL_FILE_READ,
	WAIT_EVENT_CONTROL_FILE_SYNC,
	WAIT_EVENT_CONTROL_FILE_SYNC_UPDATE,
	WAIT_EVENT_CONTROL_FILE_WRITE,
	WAIT_EVENT_CONTROL_FILE_WRITE_UPDATE,
	WAIT_EVENT_COPY_FILE_READ,
	WAIT_EVENT_COPY_FILE_WRITE,
	WAIT_EVENT_DATA_FILE_EXTEND,
	WAIT_EVENT_DATA_FILE_FLUSH,
	WAIT_EVENT_DATA_FILE_IMMEDIATE_SYNC,
	WAIT_EVENT_DATA_FILE_PREFETCH,
	WAIT_EVENT_DATA_FILE_READ,
	WAIT_EVENT_DATA_FILE_SYNC,
	WAIT_EVENT_DATA_FILE_TRUNCATE,
	WAIT_EVENT_DATA_FILE_WRITE,
	WAIT_EVENT_DSM_FILL_ZERO_WRITE,
	WAIT_EVENT_LOCK_FILE_ADDTODATADIR_READ,
	WAIT_EVENT_LOCK_FILE_ADDTODATADIR_SYNC,
	WAIT_EVENT_LOCK_FILE_ADDTODATADIR_WRITE,
	WAIT_EVENT_LOCK_FILE_CREATE_READ,
	WAIT_EVENT_LOCK_FILE_CREATE_SYNC,
	WAIT_EVENT_LOCK_FILE_CREATE_WRITE,
	WAIT_EVENT_LOCK_FILE_RECHECKDATADIR_READ,
	WAIT_EVENT_LOGICAL_REWRITE_CHECKPOINT_SYNC,
	WAIT_EVENT_LOGICAL_REWRITE_MAPPING_SYNC,
	WAIT_EVENT_LOGICAL_REWRITE_MAPPING_WRITE,
	WAIT_EVENT_LOGICAL_REWRITE_SYNC,
	WAIT_EVENT_LOGICAL_REWRITE_TRUNCATE,
	WAIT_EVENT_LOGICAL_REWRITE_WRITE,
	WAIT_EVENT_RELATION_MAP_READ,
	WAIT_EVENT_RELATION_MAP_SYNC,
	WAIT_EVENT_RELATION_MAP_WRITE,
	WAIT_EVENT_REORDER_BUFFER_READ,
	WAIT_EVENT_REORDER_BUFFER_WRITE,
	WAIT_EVENT_REORDER_LOGICAL_MAPPING_READ,
	WAIT_EVENT_REPLICATION_SLOT_READ,
	WAIT_EVENT_REPLICATION_SLOT_RESTORE_SYNC,
	WAIT_EVENT_REPLICATION_SLOT_SYNC,
	WAIT_EVENT_REPLICATION_SLOT_WRITE,
	WAIT_EVENT_SLRU_FLUSH_SYNC,
	WAIT_EVENT_SLRU_READ,
	WAIT_EVENT_SLRU_SYNC,
	WAIT_EVENT_SLRU_WRITE,
	WAIT_EVENT_SNAPBUILD_READ,
	WAIT_EVENT_SNAPBUILD_SYNC,
	WAIT_EVENT_SNAPBUILD_WRITE,
	WAIT_EVENT_TIMELINE_HISTORY_FILE_SYNC,
	WAIT_EVENT_TIMELINE_HISTORY_FILE_WRITE,
	WAIT_EVENT_TIMELINE_HISTORY_READ,
	WAIT_EVENT_TIMELINE_HISTORY_SYNC,
	WAIT_EVENT_TIMELINE_HISTORY_WRITE,
	WAIT_EVENT_TWOPHASE_FILE_READ,
	WAIT_EVENT_TWOPHASE_FILE_SYNC,
	WAIT_EVENT_TWOPHASE_FILE_WRITE,
	WAIT_EVENT_WALSENDER_TIMELINE_HISTORY_READ,
	WAIT_EVENT_WAL_BOOTSTRAP_SYNC,
	WAIT_EVENT_WAL_BOOTSTRAP_WRITE,
	WAIT_EVENT_WAL_COPY_READ,
	WAIT_EVENT_WAL_COPY_SYNC,
	WAIT_EVENT_WAL_COPY_WRITE,
	WAIT_EVENT_WAL_INIT_SYNC,
	WAIT_EVENT_WAL_INIT_WRITE,
	WAIT_EVENT_WAL_READ,
	WAIT_EVENT_WAL_SYNC_METHOD_ASSIGN,
	WAIT_EVENT_WAL_WRITE
} WaitEventIO;

/* ----------
 * Command type for progress reporting purposes
 * ----------
 */
typedef enum ProgressCommandType
{
	PROGRESS_COMMAND_INVALID,
	PROGRESS_COMMAND_VACUUM
} ProgressCommandType;

#define UXSTAT_NUM_PROGRESS_PARAM	10

/* ----------
 * Shared-memory data structures
 * ----------
 */


/*
 * UxBackendSSLStatus
 *
 * For each backend, we keep the SSL status in a separate struct, that
 * is only filled in if SSL is enabled.
 */
typedef struct UxBackendSSLStatus
{
	/* Information about SSL connection */
	int			ssl_bits;
	bool		ssl_compression;
	char		ssl_version[NAMEDATALEN];	/* MUST be null-terminated */
	char		ssl_cipher[NAMEDATALEN];	/* MUST be null-terminated */
	char		ssl_clientdn[NAMEDATALEN];	/* MUST be null-terminated */
} UxBackendSSLStatus;


/* ----------
 * UxBackendStatus
 *
 * Each live backend maintains a UxBackendStatus struct in shared memory
 * showing its current activity.  (The structs are allocated according to
 * BackendId, but that is not critical.)  Note that the collector process
 * has no involvement in, or even access to, these structs.
 *
 * Each auxiliary process also maintains a UxBackendStatus struct in shared
 * memory.
 * ----------
 */
typedef struct UxBackendStatus
{
	/*
	 * To avoid locking overhead, we use the following protocol: a backend
	 * increments st_changecount before modifying its entry, and again after
	 * finishing a modification.  A would-be reader should note the value of
	 * st_changecount, copy the entry into private memory, then check
	 * st_changecount again.  If the value hasn't changed, and if it's even,
	 * the copy is valid; otherwise start over.  This makes updates cheap
	 * while reads are potentially expensive, but that's the tradeoff we want.
	 *
	 * The above protocol needs the memory barriers to ensure that the
	 * apparent order of execution is as it desires. Otherwise, for example,
	 * the CPU might rearrange the code so that st_changecount is incremented
	 * twice before the modification on a machine with weak memory ordering.
	 * This surprising result can lead to bugs.
	 */
	int			st_changecount;

	/* The entry is valid iff st_procpid > 0, unused if st_procpid == 0 */
	int			st_procpid;

	/* Type of backends */
	BackendType st_backendType;

	/* Times when current backend, transaction, and activity started */
	TimestampTz st_proc_start_timestamp;
	TimestampTz st_xact_start_timestamp;
	TimestampTz st_activity_start_timestamp;
	TimestampTz st_state_start_timestamp;

	/* Database OID, owning user's OID, connection client address */
	Oid			st_databaseid;
	Oid			st_userid;
	SockAddr	st_clientaddr;
	char	   *st_clienthostname;	/* MUST be null-terminated */

	/* Information about SSL connection */
	bool		st_ssl;
	UxBackendSSLStatus *st_sslstatus;

	/* current state */
	BackendState st_state;

	/* application name; MUST be null-terminated */
	char	   *st_appname;

	/* current command string; MUST be null-terminated */
	char	   *st_activity;

	/*
	 * Command progress reporting.  Any command which wishes can advertise
	 * that it is running by setting st_progress_command,
	 * st_progress_command_target, and st_progress_param[].
	 * st_progress_command_target should be the OID of the relation which the
	 * command targets (we assume there's just one, as this is meant for
	 * utility commands), but the meaning of each element in the
	 * st_progress_param array is command-specific.
	 */
	ProgressCommandType st_progress_command;
	Oid			st_progress_command_target;
	int64		st_progress_param[UXSTAT_NUM_PROGRESS_PARAM];
} UxBackendStatus;

/*
 * Macros to load and store st_changecount with the memory barriers.
 *
 * uxstat_increment_changecount_before() and
 * uxstat_increment_changecount_after() need to be called before and after
 * UxBackendStatus entries are modified, respectively. This makes sure that
 * st_changecount is incremented around the modification.
 *
 * Also uxstat_save_changecount_before() and uxstat_save_changecount_after()
 * need to be called before and after UxBackendStatus entries are copied into
 * private memory, respectively.
 */
#define uxstat_increment_changecount_before(beentry)	\
	do {	\
		beentry->st_changecount++;	\
		ux_write_barrier(); \
	} while (0)

#define uxstat_increment_changecount_after(beentry) \
	do {	\
		ux_write_barrier(); \
		beentry->st_changecount++;	\
		Assert((beentry->st_changecount & 1) == 0); \
	} while (0)

#define uxstat_save_changecount_before(beentry, save_changecount)	\
	do {	\
		save_changecount = beentry->st_changecount; \
		ux_read_barrier();	\
	} while (0)

#define uxstat_save_changecount_after(beentry, save_changecount)	\
	do {	\
		ux_read_barrier();	\
		save_changecount = beentry->st_changecount; \
	} while (0)

/* ----------
 * LocalUxBackendStatus
 *
 * When we build the backend status array, we use LocalUxBackendStatus to be
 * able to add new values to the struct when needed without adding new fields
 * to the shared memory. It contains the backend status as a first member.
 * ----------
 */
typedef struct LocalUxBackendStatus
{
	/*
	 * Local version of the backend status entry.
	 */
	UxBackendStatus backendStatus;

	/*
	 * The xid of the current transaction if available, InvalidTransactionId
	 * if not.
	 */
	TransactionId backend_xid;

	/*
	 * The xmin of the current session if available, InvalidTransactionId if
	 * not.
	 */
	TransactionId backend_xmin;
} LocalUxBackendStatus;

/*
 * Working state needed to accumulate per-function-call timing statistics.
 */
typedef struct UxStat_FunctionCallUsage
{
	/* Link to function's hashtable entry (must still be there at exit!) */
	/* NULL means we are not tracking the current function call */
	UxStat_FunctionCounts *fs;
	/* Total time previously charged to function, as of function start */
	instr_time	save_f_total_time;
	/* Backend-wide total time as of function start */
	instr_time	save_total;
	/* system clock as of function start */
	instr_time	f_start;
} UxStat_FunctionCallUsage;


/* ----------
 * GUC parameters
 * ----------
 */
extern bool uxstat_track_activities;
extern bool uxstat_track_counts;
extern int	uxstat_track_functions;
extern UXDLLIMPORT int uxstat_track_activity_query_size;
extern char *uxstat_stat_directory;
extern char *uxstat_stat_tmpname;
extern char *uxstat_stat_filename;

/*
 * BgWriter statistics counters are updated directly by bgwriter and bufmgr
 */
extern UxStat_MsgBgWriter BgWriterStats;

/*
 * Updated by uxstat_count_buffer_*_time macros
 */
extern UxStat_Counter uxStatBlockReadTime;
extern UxStat_Counter uxStatBlockWriteTime;

/* ----------
 * Functions called from uxmaster
 * ----------
 */
extern Size BackendStatusShmemSize(void);
extern void CreateSharedBackendStatus(void);

extern void uxstat_init(void);
extern int	uxstat_start(void);
extern void uxstat_reset_all(void);
extern void allow_immediate_uxstat_restart(void);

#ifdef EXEC_BACKEND
extern void UxstatCollectorMain(int argc, char *argv[]) ux_attribute_noreturn();
#endif


/* ----------
 * Functions called from backends
 * ----------
 */
extern void uxstat_ping(void);

extern void uxstat_report_stat(bool force);
extern void uxstat_vacuum_stat(void);
extern void uxstat_drop_database(Oid databaseid);

extern void uxstat_clear_snapshot(void);
extern void uxstat_reset_counters(void);
extern void uxstat_reset_shared_counters(const char *);
extern void uxstat_reset_single_counter(Oid objectid, UxStat_Single_Reset_Type type);

extern void uxstat_report_autovac(Oid dboid);
extern void uxstat_report_vacuum(Oid tableoid, bool shared,
					 UxStat_Counter livetuples, UxStat_Counter deadtuples);
extern void uxstat_report_analyze(Relation rel,
					  UxStat_Counter livetuples, UxStat_Counter deadtuples,
					  bool resetcounter);

extern void uxstat_report_recovery_conflict(int reason);
extern void uxstat_report_deadlock(void);

extern void uxstat_initialize(void);
extern void uxstat_bestart(void);

extern void uxstat_report_activity(BackendState state, const char *cmd_str);
extern void uxstat_report_tempfile(size_t filesize);
extern void uxstat_report_appname(const char *appname);
extern void uxstat_report_xact_timestamp(TimestampTz tstamp);
extern const char *uxstat_get_wait_event(uint32 wait_event_info);
extern const char *uxstat_get_wait_event_type(uint32 wait_event_info);
extern const char *uxstat_get_backend_current_activity(int pid, bool checkUser);
extern const char *uxstat_get_crashed_backend_activity(int pid, char *buffer,
									int buflen);
extern const char *uxstat_get_backend_desc(BackendType backendType);

extern void uxstat_progress_start_command(ProgressCommandType cmdtype,
							  Oid relid);
extern void uxstat_progress_update_param(int index, int64 val);
extern void uxstat_progress_update_multi_param(int nparam, const int *index,
								   const int64 *val);
extern void uxstat_progress_end_command(void);

extern UxStat_TableStatus *find_tabstat_entry(Oid rel_id);
extern UxStat_BackendFunctionEntry *find_funcstat_entry(Oid func_id);

extern void uxstat_initstats(Relation rel);

/* ----------
 * uxstat_report_wait_start() -
 *
 *	Called from places where server process needs to wait.  This is called
 *	to report wait event information.  The wait information is stored
 *	as 4-bytes where first byte represents the wait event class (type of
 *	wait, for different types of wait, refer WaitClass) and the next
 *	3-bytes represent the actual wait event.  Currently 2-bytes are used
 *	for wait event which is sufficient for current usage, 1-byte is
 *	reserved for future usage.
 *
 * NB: this *must* be able to survive being called before MyProc has been
 * initialized.
 * ----------
 */
static inline void
uxstat_report_wait_start(uint32 wait_event_info)
{
	volatile UXPROC *proc = MyProc;

	if (!uxstat_track_activities || !proc)
		return;

	/*
	 * Since this is a four-byte field which is always read and written as
	 * four-bytes, updates are atomic.
	 */
	proc->wait_event_info = wait_event_info;
}

/* ----------
 * uxstat_report_wait_end() -
 *
 *	Called to report end of a wait.
 *
 * NB: this *must* be able to survive being called before MyProc has been
 * initialized.
 * ----------
 */
static inline void
uxstat_report_wait_end(void)
{
	volatile UXPROC *proc = MyProc;

	if (!uxstat_track_activities || !proc)
		return;

	/*
	 * Since this is a four-byte field which is always read and written as
	 * four-bytes, updates are atomic.
	 */
	proc->wait_event_info = 0;
}

/* nontransactional event counts are simple enough to inline */

#define uxstat_count_heap_scan(rel)									\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_numscans++;				\
	} while (0)
#define uxstat_count_heap_getnext(rel)								\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_tuples_returned++;		\
	} while (0)
#define uxstat_count_heap_fetch(rel)								\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_tuples_fetched++;		\
	} while (0)
#define uxstat_count_index_scan(rel)								\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_numscans++;				\
	} while (0)
#define uxstat_count_index_tuples(rel, n)							\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_tuples_returned += (n);	\
	} while (0)
#define uxstat_count_buffer_read(rel)								\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_blocks_fetched++;		\
	} while (0)
#define uxstat_count_buffer_hit(rel)								\
	do {															\
		if ((rel)->uxstat_info != NULL)								\
			(rel)->uxstat_info->t_counts.t_blocks_hit++;			\
	} while (0)
#define uxstat_count_buffer_read_time(n)							\
	(uxStatBlockReadTime += (n))
#define uxstat_count_buffer_write_time(n)							\
	(uxStatBlockWriteTime += (n))

extern void uxstat_count_heap_insert(Relation rel, UxStat_Counter n);
extern void uxstat_count_heap_update(Relation rel, bool hot);
extern void uxstat_count_heap_delete(Relation rel);
extern void uxstat_count_truncate(Relation rel);
extern void uxstat_update_heap_dead_tuples(Relation rel, int delta);

extern void uxstat_init_function_usage(FunctionCallInfoData *fcinfo,
						   UxStat_FunctionCallUsage *fcu);
extern void uxstat_end_function_usage(UxStat_FunctionCallUsage *fcu,
						  bool finalize);

extern void AtEOXact_UxStat(bool isCommit);
extern void AtEOSubXact_UxStat(bool isCommit, int nestDepth);

extern void AtPrepare_UxStat(void);
extern void PostPrepare_UxStat(void);

extern void uxstat_twophase_postcommit(TransactionId xid, uint16 info,
						   void *recdata, uint32 len);
extern void uxstat_twophase_postabort(TransactionId xid, uint16 info,
						  void *recdata, uint32 len);

extern void uxstat_send_archiver(const char *xlog, bool failed);
extern void uxstat_send_bgwriter(void);

/* ----------
 * Support functions for the SQL-callable functions to
 * generate the uxstat* views.
 * ----------
 */
extern UxStat_StatDBEntry *uxstat_fetch_stat_dbentry(Oid dbid);
extern UxStat_StatTabEntry *uxstat_fetch_stat_tabentry(Oid relid);
extern UxBackendStatus *uxstat_fetch_stat_beentry(int beid);
extern LocalUxBackendStatus *uxstat_fetch_stat_local_beentry(int beid);
extern UxStat_StatFuncEntry *uxstat_fetch_stat_funcentry(Oid funcid);
extern int	uxstat_fetch_stat_numbackends(void);
extern UxStat_ArchiverStats *uxstat_fetch_stat_archiver(void);
extern UxStat_GlobalStats *uxstat_fetch_global(void);

#endif							/* UXSTAT_H */
