/*-------------------------------------------------------------------------
 *
 * ux_proc.h
 *	  definition of the system "procedure" relation (ux_proc)
 *	  along with the relation's initial contents.
 *
 * Portions Copyright (c) 1996-2017, UXsinoDB Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * src/include/catalog/ux_proc.h
 *
 * NOTES
 *	  The script catalog/genbki.pl reads this file and generates .bki
 *	  information from the DATA() statements.  utils/Gen_fmgrtab.pl
 *	  generates fmgroids.h and fmgrtab.c the same way.
 *
 *	  XXX do NOT break up DATA() statements into multiple lines!
 *		  the scripts are not as smart as you might think...
 *	  XXX (eg. #if 0 #endif won't do what you think)
 *
 *-------------------------------------------------------------------------
 */
#ifndef UX_PROC_H
#define UX_PROC_H

#include "catalog/genbki.h"

/* ----------------
 *		ux_proc definition.  cpp turns this into
 *		typedef struct FormData_ux_proc
 * ----------------
 */
#define ProcedureRelationId  1255
#define ProcedureRelation_Rowtype_Id  81

CATALOG(ux_proc,1255) BKI_BOOTSTRAP BKI_ROWTYPE_OID(81) BKI_SCHEMA_MACRO
{
	NameData	proname;		/* procedure name */
	Oid			pronamespace;	/* OID of namespace containing this proc */
	Oid			proowner;		/* procedure owner */
	Oid			prolang;		/* OID of ux_language entry */
	float4		procost;		/* estimated execution cost */
	float4		prorows;		/* estimated # of rows out (if proretset) */
	Oid			provariadic;	/* element type of variadic array, or 0 */
	regproc		protransform;	/* transforms calls to it during planning */
	bool		proisagg;		/* is it an aggregate? */
	bool		proiswindow;	/* is it a window function? */
	bool		prosecdef;		/* security definer */
	bool		proleakproof;	/* is it a leak-proof function? */
	bool		proisstrict;	/* strict with respect to NULLs? */
	bool		proretset;		/* returns a set? */
	char		provolatile;	/* see PROVOLATILE_ categories below */
	char		proparallel;	/* see PROPARALLEL_ categories below */
	int16		pronargs;		/* number of arguments */
	int16		pronargdefaults;	/* number of arguments with defaults */
	Oid			prorettype;		/* OID of result type */

	/*
	 * variable-length fields start here, but we allow direct access to
	 * proargtypes
	 */
	oidvector	proargtypes;	/* parameter types (excludes OUT params) */

#ifdef CATALOG_VARLEN
	Oid			proallargtypes[1];	/* all param types (NULL if IN only) */
	char		proargmodes[1]; /* parameter modes (NULL if IN only) */
	text		proargnames[1]; /* parameter names (NULL if no names) */
	ux_node_tree proargdefaults;	/* list of expression trees for argument
									 * defaults (NULL if none) */
	Oid			protrftypes[1]; /* types for which to apply transforms */
	text		prosrc BKI_FORCE_NOT_NULL;	/* procedure source text */
	text		probin;			/* secondary procedure info (can be NULL) */
	text		proconfig[1];	/* procedure-local GUC settings */
	aclitem		proacl[1];		/* access permissions */
#endif
} FormData_ux_proc;

/* ----------------
 *		Form_ux_proc corresponds to a pointer to a tuple with
 *		the format of ux_proc relation.
 * ----------------
 */
typedef FormData_ux_proc *Form_ux_proc;

/* ----------------
 *		compiler constants for ux_proc
 * ----------------
 */
#define Natts_ux_proc					29
#define Anum_ux_proc_proname			1
#define Anum_ux_proc_pronamespace		2
#define Anum_ux_proc_proowner			3
#define Anum_ux_proc_prolang			4
#define Anum_ux_proc_procost			5
#define Anum_ux_proc_prorows			6
#define Anum_ux_proc_provariadic		7
#define Anum_ux_proc_protransform		8
#define Anum_ux_proc_proisagg			9
#define Anum_ux_proc_proiswindow		10
#define Anum_ux_proc_prosecdef			11
#define Anum_ux_proc_proleakproof		12
#define Anum_ux_proc_proisstrict		13
#define Anum_ux_proc_proretset			14
#define Anum_ux_proc_provolatile		15
#define Anum_ux_proc_proparallel		16
#define Anum_ux_proc_pronargs			17
#define Anum_ux_proc_pronargdefaults	18
#define Anum_ux_proc_prorettype			19
#define Anum_ux_proc_proargtypes		20
#define Anum_ux_proc_proallargtypes		21
#define Anum_ux_proc_proargmodes		22
#define Anum_ux_proc_proargnames		23
#define Anum_ux_proc_proargdefaults		24
#define Anum_ux_proc_protrftypes		25
#define Anum_ux_proc_prosrc				26
#define Anum_ux_proc_probin				27
#define Anum_ux_proc_proconfig			28
#define Anum_ux_proc_proacl				29

/* ----------------
 *		initial contents of ux_proc
 * ----------------
 */

/*
 * Note: every entry in ux_proc.h is expected to have a DESCR() comment,
 * except for functions that implement ux_operator.h operators and don't
 * have a good reason to be called directly rather than via the operator.
 * (If you do expect such a function to be used directly, you should
 * duplicate the operator's comment.)  initdb will supply suitable default
 * comments for functions referenced by ux_operator.
 *
 * Try to follow the style of existing functions' comments.
 * Some recommended conventions:
 *		"I/O" for typinput, typoutput, typreceive, typsend functions
 *		"I/O typmod" for typmodin, typmodout functions
 *		"aggregate transition function" for aggtransfn functions, unless
 *					they are reasonably useful in their own right
 *		"aggregate final function" for aggfinalfn functions (likewise)
 *		"convert srctypename to desttypename" for cast functions
 *		"less-equal-greater" for B-tree comparison functions
 */

/* keep the following ordered by OID so that later changes can be made easier */

/* OIDS 1 - 99 */

DATA(insert OID = 1242 (  boolin		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "2275" _null_ _null_ _null_ _null_ _null_ boolin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1243 (  boolout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "16" _null_ _null_ _null_ _null_ _null_ boolout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1244 (  byteain		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2275" _null_ _null_ _null_ _null_ _null_ byteain _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  31 (  byteaout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "17" _null_ _null_ _null_ _null_ _null_ byteaout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1245 (  charin		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 18 "2275" _null_ _null_ _null_ _null_ _null_ charin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  33 (  charout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "18" _null_ _null_ _null_ _null_ _null_ charout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  34 (  namein			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 19 "2275" _null_ _null_ _null_ _null_ _null_ namein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  35 (  nameout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "19" _null_ _null_ _null_ _null_ _null_ nameout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  38 (  int2in			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "2275" _null_ _null_ _null_ _null_ _null_ int2in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  39 (  int2out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "21" _null_ _null_ _null_ _null_ _null_ int2out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  40 (  int2vectorin	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 22 "2275" _null_ _null_ _null_ _null_ _null_ int2vectorin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  41 (  int2vectorout	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "22" _null_ _null_ _null_ _null_ _null_ int2vectorout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  42 (  int4in			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2275" _null_ _null_ _null_ _null_ _null_ int4in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  43 (  int4out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ int4out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  44 (  regprocin		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 24 "2275" _null_ _null_ _null_ _null_ _null_ regprocin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  45 (  regprocout		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "24" _null_ _null_ _null_ _null_ _null_ regprocout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3494 (  to_regproc		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 24 "25" _null_ _null_ _null_ _null_ _null_ to_regproc _null_ _null_ _null_ ));
DESCR("convert proname to regproc");
DATA(insert OID = 3479 (  to_regprocedure	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2202 "25" _null_ _null_ _null_ _null_ _null_ to_regprocedure _null_ _null_ _null_ ));
DESCR("convert proname to regprocedure");
DATA(insert OID =  46 (  textin			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "2275" _null_ _null_ _null_ _null_ _null_ textin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  47 (  textout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "25" _null_ _null_ _null_ _null_ _null_ textout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  48 (  tidin			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 27 "2275" _null_ _null_ _null_ _null_ _null_ tidin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  49 (  tidout			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "27" _null_ _null_ _null_ _null_ _null_ tidout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  50 (  xidin			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 28 "2275" _null_ _null_ _null_ _null_ _null_ xidin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  51 (  xidout			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "28" _null_ _null_ _null_ _null_ _null_ xidout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  52 (  cidin			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 29 "2275" _null_ _null_ _null_ _null_ _null_ cidin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  53 (  cidout			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "29" _null_ _null_ _null_ _null_ _null_ cidout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  54 (  oidvectorin	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 30 "2275" _null_ _null_ _null_ _null_ _null_ oidvectorin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  55 (  oidvectorout	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "30" _null_ _null_ _null_ _null_ _null_ oidvectorout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  56 (  boollt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boollt _null_ _null_ _null_ ));
DATA(insert OID =  57 (  boolgt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boolgt _null_ _null_ _null_ ));
DATA(insert OID =  60 (  booleq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ booleq _null_ _null_ _null_ ));
DATA(insert OID =  61 (  chareq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ chareq _null_ _null_ _null_ ));
DATA(insert OID =  62 (  nameeq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ nameeq _null_ _null_ _null_ ));
DATA(insert OID =  63 (  int2eq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2eq _null_ _null_ _null_ ));
DATA(insert OID =  64 (  int2lt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2lt _null_ _null_ _null_ ));
DATA(insert OID =  65 (  int4eq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4eq _null_ _null_ _null_ ));
DATA(insert OID =  66 (  int4lt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4lt _null_ _null_ _null_ ));
DATA(insert OID =  67 (  texteq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ texteq _null_ _null_ _null_ ));
DATA(insert OID =  68 (  xideq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "28 28" _null_ _null_ _null_ _null_ _null_ xideq _null_ _null_ _null_ ));
DATA(insert OID = 3308 (  xidneq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "28 28" _null_ _null_ _null_ _null_ _null_ xidneq _null_ _null_ _null_ ));
DATA(insert OID =  69 (  cideq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "29 29" _null_ _null_ _null_ _null_ _null_ cideq _null_ _null_ _null_ ));
DATA(insert OID =  70 (  charne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ charne _null_ _null_ _null_ ));
DATA(insert OID = 1246 (  charlt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ charlt _null_ _null_ _null_ ));
DATA(insert OID =  72 (  charle			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ charle _null_ _null_ _null_ ));
DATA(insert OID =  73 (  chargt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ chargt _null_ _null_ _null_ ));
DATA(insert OID =  74 (  charge			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "18 18" _null_ _null_ _null_ _null_ _null_ charge _null_ _null_ _null_ ));
DATA(insert OID =  77 (  int4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23	"18" _null_ _null_ _null_ _null_ _null_ chartoi4 _null_ _null_ _null_ ));
DESCR("convert char to int4");
DATA(insert OID =  78 (  char			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 18	"23" _null_ _null_ _null_ _null_ _null_ i4tochar _null_ _null_ _null_ ));
DESCR("convert int4 to char");

DATA(insert OID =  79 (  nameregexeq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1252 (  nameregexne	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameregexne _null_ _null_ _null_ ));
DATA(insert OID = 1254 (  textregexeq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1256 (  textregexne	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textregexne _null_ _null_ _null_ ));
DATA(insert OID = 1257 (  textlen		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ textlen _null_ _null_ _null_ ));
DESCR("length");
DATA(insert OID = 1258 (  textcat		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ textcat _null_ _null_ _null_ ));

DATA(insert OID =  84 (  boolne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boolne _null_ _null_ _null_ ));
DATA(insert OID =  89 (  version		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 25 "" _null_ _null_ _null_ _null_ _null_ uxsql_version _null_ _null_ _null_ ));
DESCR("UXsinoDB version string");

DATA(insert OID = 86  (  ux_ddl_command_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 32 "2275" _null_ _null_ _null_ _null_ _null_ ux_ddl_command_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 87  (  ux_ddl_command_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "32" _null_ _null_ _null_ _null_ _null_ ux_ddl_command_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 88  (  ux_ddl_command_recv	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 32 "2281" _null_ _null_ _null_ _null_ _null_ ux_ddl_command_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 90  (  ux_ddl_command_send	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "32" _null_ _null_ _null_ _null_ _null_ ux_ddl_command_send _null_ _null_ _null_ ));
DESCR("I/O");

/* OIDS 100 - 199 */

DATA(insert OID = 101 (  eqsel			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	eqsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of = and related operators");
DATA(insert OID = 102 (  neqsel			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	neqsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of <> and related operators");
DATA(insert OID = 103 (  scalarltsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	scalarltsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of < and related operators on scalar datatypes");
DATA(insert OID = 104 (  scalargtsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	scalargtsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of > and related operators on scalar datatypes");
DATA(insert OID = 105 (  eqjoinsel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ eqjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of = and related operators");
DATA(insert OID = 106 (  neqjoinsel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ neqjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of <> and related operators");
DATA(insert OID = 107 (  scalarltjoinsel   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ scalarltjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of < and related operators on scalar datatypes");
DATA(insert OID = 108 (  scalargtjoinsel   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ scalargtjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of > and related operators on scalar datatypes");

DATA(insert OID =  109 (  unknownin		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 705 "2275" _null_ _null_ _null_ _null_ _null_ unknownin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  110 (  unknownout	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "705" _null_ _null_ _null_ _null_ _null_ unknownout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 111 (  numeric_fac	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_	numeric_fac _null_ _null_ _null_ ));

DATA(insert OID = 115 (  box_above_eq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_above_eq _null_ _null_ _null_ ));
DATA(insert OID = 116 (  box_below_eq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_below_eq _null_ _null_ _null_ ));

DATA(insert OID = 117 (  point_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "2275" _null_ _null_ _null_ _null_ _null_ point_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 118 (  point_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "600" _null_ _null_ _null_ _null_ _null_ point_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 119 (  lseg_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 601 "2275" _null_ _null_ _null_ _null_ _null_ lseg_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 120 (  lseg_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "601" _null_ _null_ _null_ _null_ _null_ lseg_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 121 (  path_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 602 "2275" _null_ _null_ _null_ _null_ _null_ path_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 122 (  path_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "602" _null_ _null_ _null_ _null_ _null_ path_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 123 (  box_in			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 603 "2275" _null_ _null_ _null_ _null_ _null_ box_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 124 (  box_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "603" _null_ _null_ _null_ _null_ _null_ box_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 125 (  box_overlap	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_overlap _null_ _null_ _null_ ));
DATA(insert OID = 126 (  box_ge			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_ge _null_ _null_ _null_ ));
DATA(insert OID = 127 (  box_gt			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_gt _null_ _null_ _null_ ));
DATA(insert OID = 128 (  box_eq			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_eq _null_ _null_ _null_ ));
DATA(insert OID = 129 (  box_lt			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_lt _null_ _null_ _null_ ));
DATA(insert OID = 130 (  box_le			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_le _null_ _null_ _null_ ));
DATA(insert OID = 131 (  point_above	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_above _null_ _null_ _null_ ));
DATA(insert OID = 132 (  point_left		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_left _null_ _null_ _null_ ));
DATA(insert OID = 133 (  point_right	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_right _null_ _null_ _null_ ));
DATA(insert OID = 134 (  point_below	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_below _null_ _null_ _null_ ));
DATA(insert OID = 135 (  point_eq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_eq _null_ _null_ _null_ ));
DATA(insert OID = 136 (  on_pb			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 603" _null_ _null_ _null_ _null_ _null_ on_pb _null_ _null_ _null_ ));
DATA(insert OID = 137 (  on_ppath		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 602" _null_ _null_ _null_ _null_ _null_ on_ppath _null_ _null_ _null_ ));
DATA(insert OID = 138 (  box_center		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "603" _null_ _null_ _null_ _null_ _null_	box_center _null_ _null_ _null_ ));
DATA(insert OID = 139 (  areasel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	areasel _null_ _null_ _null_ ));
DESCR("restriction selectivity for area-comparison operators");
DATA(insert OID = 140 (  areajoinsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ areajoinsel _null_ _null_ _null_ ));
DESCR("join selectivity for area-comparison operators");
DATA(insert OID = 141 (  int4mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4mul _null_ _null_ _null_ ));
DATA(insert OID = 144 (  int4ne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4ne _null_ _null_ _null_ ));
DATA(insert OID = 145 (  int2ne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2ne _null_ _null_ _null_ ));
DATA(insert OID = 146 (  int2gt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2gt _null_ _null_ _null_ ));
DATA(insert OID = 147 (  int4gt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4gt _null_ _null_ _null_ ));
DATA(insert OID = 148 (  int2le			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2le _null_ _null_ _null_ ));
DATA(insert OID = 149 (  int4le			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4le _null_ _null_ _null_ ));
DATA(insert OID = 150 (  int4ge			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ int4ge _null_ _null_ _null_ ));
DATA(insert OID = 151 (  int2ge			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 21" _null_ _null_ _null_ _null_ _null_ int2ge _null_ _null_ _null_ ));
DATA(insert OID = 152 (  int2mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2mul _null_ _null_ _null_ ));
DATA(insert OID = 153 (  int2div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2div _null_ _null_ _null_ ));
DATA(insert OID = 154 (  int4div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4div _null_ _null_ _null_ ));
DATA(insert OID = 155 (  int2mod		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2mod _null_ _null_ _null_ ));
DATA(insert OID = 156 (  int4mod		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4mod _null_ _null_ _null_ ));
DATA(insert OID = 157 (  textne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textne _null_ _null_ _null_ ));
DATA(insert OID = 158 (  int24eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24eq _null_ _null_ _null_ ));
DATA(insert OID = 159 (  int42eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42eq _null_ _null_ _null_ ));
DATA(insert OID = 160 (  int24lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24lt _null_ _null_ _null_ ));
DATA(insert OID = 161 (  int42lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42lt _null_ _null_ _null_ ));
DATA(insert OID = 162 (  int24gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24gt _null_ _null_ _null_ ));
DATA(insert OID = 163 (  int42gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42gt _null_ _null_ _null_ ));
DATA(insert OID = 164 (  int24ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24ne _null_ _null_ _null_ ));
DATA(insert OID = 165 (  int42ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42ne _null_ _null_ _null_ ));
DATA(insert OID = 166 (  int24le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24le _null_ _null_ _null_ ));
DATA(insert OID = 167 (  int42le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42le _null_ _null_ _null_ ));
DATA(insert OID = 168 (  int24ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 23" _null_ _null_ _null_ _null_ _null_ int24ge _null_ _null_ _null_ ));
DATA(insert OID = 169 (  int42ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 21" _null_ _null_ _null_ _null_ _null_ int42ge _null_ _null_ _null_ ));
DATA(insert OID = 170 (  int24mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 23" _null_ _null_ _null_ _null_ _null_ int24mul _null_ _null_ _null_ ));
DATA(insert OID = 171 (  int42mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 21" _null_ _null_ _null_ _null_ _null_ int42mul _null_ _null_ _null_ ));
DATA(insert OID = 172 (  int24div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 23" _null_ _null_ _null_ _null_ _null_ int24div _null_ _null_ _null_ ));
DATA(insert OID = 173 (  int42div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 21" _null_ _null_ _null_ _null_ _null_ int42div _null_ _null_ _null_ ));
DATA(insert OID = 176 (  int2pl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2pl _null_ _null_ _null_ ));
DATA(insert OID = 177 (  int4pl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4pl _null_ _null_ _null_ ));
DATA(insert OID = 178 (  int24pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 23" _null_ _null_ _null_ _null_ _null_ int24pl _null_ _null_ _null_ ));
DATA(insert OID = 179 (  int42pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 21" _null_ _null_ _null_ _null_ _null_ int42pl _null_ _null_ _null_ ));
DATA(insert OID = 180 (  int2mi			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2mi _null_ _null_ _null_ ));
DATA(insert OID = 181 (  int4mi			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4mi _null_ _null_ _null_ ));
DATA(insert OID = 182 (  int24mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 23" _null_ _null_ _null_ _null_ _null_ int24mi _null_ _null_ _null_ ));
DATA(insert OID = 183 (  int42mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 21" _null_ _null_ _null_ _null_ _null_ int42mi _null_ _null_ _null_ ));
DATA(insert OID = 184 (  oideq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oideq _null_ _null_ _null_ ));
DATA(insert OID = 185 (  oidne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oidne _null_ _null_ _null_ ));
DATA(insert OID = 186 (  box_same		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_same _null_ _null_ _null_ ));
DATA(insert OID = 187 (  box_contain	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_contain _null_ _null_ _null_ ));
DATA(insert OID = 188 (  box_left		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_left _null_ _null_ _null_ ));
DATA(insert OID = 189 (  box_overleft	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_overleft _null_ _null_ _null_ ));
DATA(insert OID = 190 (  box_overright	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_overright _null_ _null_ _null_ ));
DATA(insert OID = 191 (  box_right		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_right _null_ _null_ _null_ ));
DATA(insert OID = 192 (  box_contained	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_contained _null_ _null_ _null_ ));
DATA(insert OID = 193 (  box_contain_pt    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 600" _null_ _null_ _null_ _null_ _null_ box_contain_pt _null_ _null_ _null_ ));

DATA(insert OID = 195 (  ux_node_tree_in	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 194 "2275" _null_ _null_ _null_ _null_ _null_ ux_node_tree_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 196 (  ux_node_tree_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "194" _null_ _null_ _null_ _null_ _null_ ux_node_tree_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 197 (  ux_node_tree_recv	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 194 "2281" _null_ _null_ _null_ _null_ _null_ ux_node_tree_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 198 (  ux_node_tree_send	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "194" _null_ _null_ _null_ _null_ _null_	ux_node_tree_send _null_ _null_ _null_ ));
DESCR("I/O");

/* OIDS 200 - 299 */

DATA(insert OID = 200 (  float4in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "2275" _null_ _null_ _null_ _null_ _null_ float4in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 201 (  float4out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "700" _null_ _null_ _null_ _null_ _null_ float4out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 202 (  float4mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4mul _null_ _null_ _null_ ));
DATA(insert OID = 203 (  float4div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4div _null_ _null_ _null_ ));
DATA(insert OID = 204 (  float4pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4pl _null_ _null_ _null_ ));
DATA(insert OID = 205 (  float4mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4mi _null_ _null_ _null_ ));
DATA(insert OID = 206 (  float4um		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_	float4um _null_ _null_ _null_ ));
DATA(insert OID = 207 (  float4abs		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_	float4abs _null_ _null_ _null_ ));
DATA(insert OID = 208 (  float4_accum	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1022 "1022 700" _null_ _null_ _null_ _null_ _null_ float4_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 209 (  float4larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 211 (  float4smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "700 700" _null_ _null_ _null_ _null_ _null_	float4smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 212 (  int4um			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4um _null_ _null_ _null_ ));
DATA(insert OID = 213 (  int2um			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ int2um _null_ _null_ _null_ ));

DATA(insert OID = 214 (  float8in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "2275" _null_ _null_ _null_ _null_ _null_ float8in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 215 (  float8out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "701" _null_ _null_ _null_ _null_ _null_ float8out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 216 (  float8mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8mul _null_ _null_ _null_ ));
DATA(insert OID = 217 (  float8div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8div _null_ _null_ _null_ ));
DATA(insert OID = 218 (  float8pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8pl _null_ _null_ _null_ ));
DATA(insert OID = 219 (  float8mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8mi _null_ _null_ _null_ ));
DATA(insert OID = 220 (  float8um		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	float8um _null_ _null_ _null_ ));
DATA(insert OID = 221 (  float8abs		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	float8abs _null_ _null_ _null_ ));
DATA(insert OID = 222 (  float8_accum	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1022 "1022 701" _null_ _null_ _null_ _null_ _null_ float8_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 276 (  float8_combine		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1022 "1022 1022" _null_ _null_ _null_ _null_ _null_ float8_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 223 (  float8larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 224 (  float8smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	float8smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 225 (  lseg_center	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "601" _null_ _null_ _null_ _null_ _null_	lseg_center _null_ _null_ _null_ ));
DATA(insert OID = 226 (  path_center	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "602" _null_ _null_ _null_ _null_ _null_	path_center _null_ _null_ _null_ ));
DATA(insert OID = 227 (  poly_center	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "604" _null_ _null_ _null_ _null_ _null_	poly_center _null_ _null_ _null_ ));

DATA(insert OID = 228 (  dround			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dround _null_ _null_ _null_ ));
DESCR("round to nearest integer");
DATA(insert OID = 229 (  dtrunc			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dtrunc _null_ _null_ _null_ ));
DESCR("truncate to integer");
DATA(insert OID = 2308 ( ceil			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dceil _null_ _null_ _null_ ));
DESCR("nearest integer >= value");
DATA(insert OID = 2320 ( ceiling		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dceil _null_ _null_ _null_ ));
DESCR("nearest integer >= value");
DATA(insert OID = 2309 ( floor			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dfloor _null_ _null_ _null_ ));
DESCR("nearest integer <= value");
DATA(insert OID = 2310 ( sign			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dsign _null_ _null_ _null_ ));
DESCR("sign of value");
DATA(insert OID = 230 (  dsqrt			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dsqrt _null_ _null_ _null_ ));
DATA(insert OID = 231 (  dcbrt			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dcbrt _null_ _null_ _null_ ));
DATA(insert OID = 232 (  dpow			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_	dpow _null_ _null_ _null_ ));
DATA(insert OID = 233 (  dexp			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dexp _null_ _null_ _null_ ));
DESCR("natural exponential (e^x)");
DATA(insert OID = 234 (  dlog1			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	dlog1 _null_ _null_ _null_ ));
DESCR("natural logarithm");
DATA(insert OID = 235 (  float8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "21" _null_ _null_ _null_ _null_ _null_ i2tod _null_ _null_ _null_ ));
DESCR("convert int2 to float8");
DATA(insert OID = 236 (  float4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "21" _null_ _null_ _null_ _null_ _null_ i2tof _null_ _null_ _null_ ));
DESCR("convert int2 to float4");
DATA(insert OID = 237 (  int2			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "701" _null_ _null_ _null_ _null_ _null_ dtoi2 _null_ _null_ _null_ ));
DESCR("convert float8 to int2");
DATA(insert OID = 238 (  int2			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "700" _null_ _null_ _null_ _null_ _null_ ftoi2 _null_ _null_ _null_ ));
DESCR("convert float4 to int2");
DATA(insert OID = 239 (  line_distance	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "628 628" _null_ _null_ _null_ _null_ _null_	line_distance _null_ _null_ _null_ ));

DATA(insert OID = 240 (  abstimein		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 702 "2275" _null_ _null_ _null_ _null_ _null_ abstimein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 241 (  abstimeout		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "702" _null_ _null_ _null_ _null_ _null_ abstimeout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 242 (  reltimein		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 703 "2275" _null_ _null_ _null_ _null_ _null_ reltimein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 243 (  reltimeout		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "703" _null_ _null_ _null_ _null_ _null_ reltimeout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 244 (  timepl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 702 "702 703" _null_ _null_ _null_ _null_ _null_	timepl _null_ _null_ _null_ ));
DATA(insert OID = 245 (  timemi			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 702 "702 703" _null_ _null_ _null_ _null_ _null_	timemi _null_ _null_ _null_ ));
DATA(insert OID = 246 (  tintervalin	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 704 "2275" _null_ _null_ _null_ _null_ _null_ tintervalin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 247 (  tintervalout	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "704" _null_ _null_ _null_ _null_ _null_ tintervalout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 248 (  intinterval	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "702 704" _null_ _null_ _null_ _null_ _null_ intinterval _null_ _null_ _null_ ));
DATA(insert OID = 249 (  tintervalrel	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 703 "704" _null_ _null_ _null_ _null_ _null_	tintervalrel _null_ _null_ _null_ ));
DESCR("tinterval to reltime");
DATA(insert OID = 250 (  timenow		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 702 "" _null_ _null_ _null_ _null_ _null_ timenow _null_ _null_ _null_ ));
DESCR("current date and time (abstime)");
DATA(insert OID = 251 (  abstimeeq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimeeq _null_ _null_ _null_ ));
DATA(insert OID = 252 (  abstimene		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimene _null_ _null_ _null_ ));
DATA(insert OID = 253 (  abstimelt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimelt _null_ _null_ _null_ ));
DATA(insert OID = 254 (  abstimegt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimegt _null_ _null_ _null_ ));
DATA(insert OID = 255 (  abstimele		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimele _null_ _null_ _null_ ));
DATA(insert OID = 256 (  abstimege		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "702 702" _null_ _null_ _null_ _null_ _null_ abstimege _null_ _null_ _null_ ));
DATA(insert OID = 257 (  reltimeeq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimeeq _null_ _null_ _null_ ));
DATA(insert OID = 258 (  reltimene		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimene _null_ _null_ _null_ ));
DATA(insert OID = 259 (  reltimelt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimelt _null_ _null_ _null_ ));
DATA(insert OID = 260 (  reltimegt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimegt _null_ _null_ _null_ ));
DATA(insert OID = 261 (  reltimele		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimele _null_ _null_ _null_ ));
DATA(insert OID = 262 (  reltimege		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "703 703" _null_ _null_ _null_ _null_ _null_ reltimege _null_ _null_ _null_ ));
DATA(insert OID = 263 (  tintervalsame	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalsame _null_ _null_ _null_ ));
DATA(insert OID = 264 (  tintervalct	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalct _null_ _null_ _null_ ));
DATA(insert OID = 265 (  tintervalov	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalov _null_ _null_ _null_ ));
DATA(insert OID = 266 (  tintervalleneq    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervalleneq _null_ _null_ _null_ ));
DATA(insert OID = 267 (  tintervallenne    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervallenne _null_ _null_ _null_ ));
DATA(insert OID = 268 (  tintervallenlt    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervallenlt _null_ _null_ _null_ ));
DATA(insert OID = 269 (  tintervallengt    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervallengt _null_ _null_ _null_ ));
DATA(insert OID = 270 (  tintervallenle    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervallenle _null_ _null_ _null_ ));
DATA(insert OID = 271 (  tintervallenge    UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 703" _null_ _null_ _null_ _null_ _null_ tintervallenge _null_ _null_ _null_ ));
DATA(insert OID = 272 (  tintervalstart    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 702 "704" _null_ _null_ _null_ _null_ _null_	tintervalstart _null_ _null_ _null_ ));
DATA(insert OID = 273 (  tintervalend	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 702 "704" _null_ _null_ _null_ _null_ _null_	tintervalend _null_ _null_ _null_ ));
DESCR("end of interval");
DATA(insert OID = 274 (  timeofday		   UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 25 "" _null_ _null_ _null_ _null_ _null_ timeofday _null_ _null_ _null_ ));
DESCR("current date and time - increments during transactions");
DATA(insert OID = 275 (  isfinite		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "702" _null_ _null_ _null_ _null_ _null_ abstime_finite _null_ _null_ _null_ ));
DESCR("finite abstime?");

DATA(insert OID = 277 (  inter_sl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 628" _null_ _null_ _null_ _null_ _null_ inter_sl _null_ _null_ _null_ ));
DATA(insert OID = 278 (  inter_lb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 603" _null_ _null_ _null_ _null_ _null_ inter_lb _null_ _null_ _null_ ));

DATA(insert OID = 279 (  float48mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "700 701" _null_ _null_ _null_ _null_ _null_	float48mul _null_ _null_ _null_ ));
DATA(insert OID = 280 (  float48div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "700 701" _null_ _null_ _null_ _null_ _null_	float48div _null_ _null_ _null_ ));
DATA(insert OID = 281 (  float48pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "700 701" _null_ _null_ _null_ _null_ _null_	float48pl _null_ _null_ _null_ ));
DATA(insert OID = 282 (  float48mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "700 701" _null_ _null_ _null_ _null_ _null_	float48mi _null_ _null_ _null_ ));
DATA(insert OID = 283 (  float84mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 700" _null_ _null_ _null_ _null_ _null_	float84mul _null_ _null_ _null_ ));
DATA(insert OID = 284 (  float84div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 700" _null_ _null_ _null_ _null_ _null_	float84div _null_ _null_ _null_ ));
DATA(insert OID = 285 (  float84pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 700" _null_ _null_ _null_ _null_ _null_	float84pl _null_ _null_ _null_ ));
DATA(insert OID = 286 (  float84mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 700" _null_ _null_ _null_ _null_ _null_	float84mi _null_ _null_ _null_ ));

DATA(insert OID = 287 (  float4eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4eq _null_ _null_ _null_ ));
DATA(insert OID = 288 (  float4ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4ne _null_ _null_ _null_ ));
DATA(insert OID = 289 (  float4lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4lt _null_ _null_ _null_ ));
DATA(insert OID = 290 (  float4le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4le _null_ _null_ _null_ ));
DATA(insert OID = 291 (  float4gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4gt _null_ _null_ _null_ ));
DATA(insert OID = 292 (  float4ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 700" _null_ _null_ _null_ _null_ _null_ float4ge _null_ _null_ _null_ ));

DATA(insert OID = 293 (  float8eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8eq _null_ _null_ _null_ ));
DATA(insert OID = 294 (  float8ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8ne _null_ _null_ _null_ ));
DATA(insert OID = 295 (  float8lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8lt _null_ _null_ _null_ ));
DATA(insert OID = 296 (  float8le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8le _null_ _null_ _null_ ));
DATA(insert OID = 297 (  float8gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8gt _null_ _null_ _null_ ));
DATA(insert OID = 298 (  float8ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 701" _null_ _null_ _null_ _null_ _null_ float8ge _null_ _null_ _null_ ));

DATA(insert OID = 299 (  float48eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48eq _null_ _null_ _null_ ));

/* OIDS 300 - 399 */

DATA(insert OID = 300 (  float48ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48ne _null_ _null_ _null_ ));
DATA(insert OID = 301 (  float48lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48lt _null_ _null_ _null_ ));
DATA(insert OID = 302 (  float48le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48le _null_ _null_ _null_ ));
DATA(insert OID = 303 (  float48gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48gt _null_ _null_ _null_ ));
DATA(insert OID = 304 (  float48ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "700 701" _null_ _null_ _null_ _null_ _null_ float48ge _null_ _null_ _null_ ));
DATA(insert OID = 305 (  float84eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84eq _null_ _null_ _null_ ));
DATA(insert OID = 306 (  float84ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84ne _null_ _null_ _null_ ));
DATA(insert OID = 307 (  float84lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84lt _null_ _null_ _null_ ));
DATA(insert OID = 308 (  float84le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84le _null_ _null_ _null_ ));
DATA(insert OID = 309 (  float84gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84gt _null_ _null_ _null_ ));
DATA(insert OID = 310 (  float84ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "701 700" _null_ _null_ _null_ _null_ _null_ float84ge _null_ _null_ _null_ ));
DATA(insert OID = 320 ( width_bucket	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 23 "701 701 701 23" _null_ _null_ _null_ _null_ _null_ width_bucket_float8 _null_ _null_ _null_ ));
DESCR("bucket number of operand in equal-width histogram");

DATA(insert OID = 311 (  float8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_	ftod _null_ _null_ _null_ ));
DESCR("convert float4 to float8");
DATA(insert OID = 312 (  float4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "701" _null_ _null_ _null_ _null_ _null_	dtof _null_ _null_ _null_ ));
DESCR("convert float8 to float4");
DATA(insert OID = 313 (  int4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23	"21" _null_ _null_ _null_ _null_ _null_ i2toi4 _null_ _null_ _null_ ));
DESCR("convert int2 to int4");
DATA(insert OID = 314 (  int2			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21	"23" _null_ _null_ _null_ _null_ _null_ i4toi2 _null_ _null_ _null_ ));
DESCR("convert int4 to int2");
DATA(insert OID = 316 (  float8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701  "23" _null_ _null_ _null_ _null_ _null_	i4tod _null_ _null_ _null_ ));
DESCR("convert int4 to float8");
DATA(insert OID = 317 (  int4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "701" _null_ _null_ _null_ _null_ _null_ dtoi4 _null_ _null_ _null_ ));
DESCR("convert float8 to int4");
DATA(insert OID = 318 (  float4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700  "23" _null_ _null_ _null_ _null_ _null_	i4tof _null_ _null_ _null_ ));
DESCR("convert int4 to float4");
DATA(insert OID = 319 (  int4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1  0 23 "700" _null_ _null_ _null_ _null_ _null_	ftoi4 _null_ _null_ _null_ ));
DESCR("convert float4 to int4");

/* Index access method handlers */
DATA(insert OID = 330 (  bthandler		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	bthandler _null_ _null_ _null_ ));
DESCR("btree index access method handler");
DATA(insert OID = 331 (  hashhandler	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	hashhandler _null_ _null_ _null_ ));
DESCR("hash index access method handler");
DATA(insert OID = 332 (  gisthandler	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	gisthandler _null_ _null_ _null_ ));
DESCR("gist index access method handler");
DATA(insert OID = 333 (  ginhandler		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	ginhandler _null_ _null_ _null_ ));
DESCR("gin index access method handler");
DATA(insert OID = 334 (  spghandler		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	spghandler _null_ _null_ _null_ ));
DESCR("spgist index access method handler");
DATA(insert OID = 335 (  brinhandler	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 325 "2281" _null_ _null_ _null_ _null_ _null_	brinhandler _null_ _null_ _null_ ));
DESCR("brin index access method handler");
DATA(insert OID = 3952 (  brin_summarize_new_values UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 23 "2205" _null_ _null_ _null_ _null_ _null_ brin_summarize_new_values _null_ _null_ _null_ ));
DESCR("brin: standalone scan new table pages");
DATA(insert OID = 3999 (  brin_summarize_range UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 23 "2205 20" _null_ _null_ _null_ _null_ _null_ brin_summarize_range _null_ _null_ _null_ ));
DESCR("brin: standalone scan new table pages");
DATA(insert OID = 4014 (  brin_desummarize_range UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 2278 "2205 20" _null_ _null_ _null_ _null_ _null_ brin_desummarize_range _null_ _null_ _null_ ));
DESCR("brin: desummarize page range");

DATA(insert OID = 338 (  amvalidate		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_	amvalidate _null_ _null_ _null_ ));
DESCR("validate an operator class");

DATA(insert OID = 636 (  ux_indexam_has_property		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_	ux_indexam_has_property _null_ _null_ _null_ ));
DESCR("test property of an index access method");
DATA(insert OID = 637 (  ux_index_has_property			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "2205 25" _null_ _null_ _null_ _null_ _null_	ux_index_has_property _null_ _null_ _null_ ));
DESCR("test property of an index");
DATA(insert OID = 638 (  ux_index_column_has_property	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "2205 23 25" _null_ _null_ _null_ _null_ _null_ ux_index_column_has_property _null_ _null_ _null_ ));
DESCR("test property of an index column");

DATA(insert OID = 339 (  poly_same		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_same _null_ _null_ _null_ ));
DATA(insert OID = 340 (  poly_contain	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_contain _null_ _null_ _null_ ));
DATA(insert OID = 341 (  poly_left		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_left _null_ _null_ _null_ ));
DATA(insert OID = 342 (  poly_overleft	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_overleft _null_ _null_ _null_ ));
DATA(insert OID = 343 (  poly_overright    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_overright _null_ _null_ _null_ ));
DATA(insert OID = 344 (  poly_right		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_right _null_ _null_ _null_ ));
DATA(insert OID = 345 (  poly_contained    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_contained _null_ _null_ _null_ ));
DATA(insert OID = 346 (  poly_overlap	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_overlap _null_ _null_ _null_ ));
DATA(insert OID = 347 (  poly_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 604 "2275" _null_ _null_ _null_ _null_ _null_ poly_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 348 (  poly_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "604" _null_ _null_ _null_ _null_ _null_ poly_out _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 350 (  btint2cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 21" _null_ _null_ _null_ _null_ _null_ btint2cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3129 ( btint2sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btint2sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 351 (  btint4cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ btint4cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3130 ( btint4sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btint4sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 842 (  btint8cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "20 20" _null_ _null_ _null_ _null_ _null_ btint8cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3131 ( btint8sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btint8sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 354 (  btfloat4cmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "700 700" _null_ _null_ _null_ _null_ _null_ btfloat4cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3132 ( btfloat4sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btfloat4sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 355 (  btfloat8cmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "701 701" _null_ _null_ _null_ _null_ _null_ btfloat8cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3133 ( btfloat8sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btfloat8sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 356 (  btoidcmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "26 26" _null_ _null_ _null_ _null_ _null_ btoidcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3134 ( btoidsortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btoidsortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 404 (  btoidvectorcmp    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "30 30" _null_ _null_ _null_ _null_ _null_ btoidvectorcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 357 (  btabstimecmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "702 702" _null_ _null_ _null_ _null_ _null_ btabstimecmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 358 (  btcharcmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "18 18" _null_ _null_ _null_ _null_ _null_ btcharcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 359 (  btnamecmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "19 19" _null_ _null_ _null_ _null_ _null_ btnamecmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3135 ( btnamesortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btnamesortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 360 (  bttextcmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ bttextcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3255 ( bttextsortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ bttextsortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 377 (  cash_cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "790 790" _null_ _null_ _null_ _null_ _null_ cash_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 380 (  btreltimecmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "703 703" _null_ _null_ _null_ _null_ _null_ btreltimecmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 381 (  bttintervalcmp    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "704 704" _null_ _null_ _null_ _null_ _null_ bttintervalcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 382 (  btarraycmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2277 2277" _null_ _null_ _null_ _null_ _null_ btarraycmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 361 (  lseg_distance	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "601 601" _null_ _null_ _null_ _null_ _null_	lseg_distance _null_ _null_ _null_ ));
DATA(insert OID = 362 (  lseg_interpt	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "601 601" _null_ _null_ _null_ _null_ _null_	lseg_interpt _null_ _null_ _null_ ));
DATA(insert OID = 363 (  dist_ps		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 601" _null_ _null_ _null_ _null_ _null_	dist_ps _null_ _null_ _null_ ));
DATA(insert OID = 364 (  dist_pb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 603" _null_ _null_ _null_ _null_ _null_	dist_pb _null_ _null_ _null_ ));
DATA(insert OID = 365 (  dist_sb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "601 603" _null_ _null_ _null_ _null_ _null_	dist_sb _null_ _null_ _null_ ));
DATA(insert OID = 366 (  close_ps		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 601" _null_ _null_ _null_ _null_ _null_	close_ps _null_ _null_ _null_ ));
DATA(insert OID = 367 (  close_pb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 603" _null_ _null_ _null_ _null_ _null_	close_pb _null_ _null_ _null_ ));
DATA(insert OID = 368 (  close_sb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "601 603" _null_ _null_ _null_ _null_ _null_	close_sb _null_ _null_ _null_ ));
DATA(insert OID = 369 (  on_ps			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 601" _null_ _null_ _null_ _null_ _null_ on_ps _null_ _null_ _null_ ));
DATA(insert OID = 370 (  path_distance	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "602 602" _null_ _null_ _null_ _null_ _null_	path_distance _null_ _null_ _null_ ));
DATA(insert OID = 371 (  dist_ppath		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 602" _null_ _null_ _null_ _null_ _null_	dist_ppath _null_ _null_ _null_ ));
DATA(insert OID = 372 (  on_sb			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 603" _null_ _null_ _null_ _null_ _null_ on_sb _null_ _null_ _null_ ));
DATA(insert OID = 373 (  inter_sb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 603" _null_ _null_ _null_ _null_ _null_ inter_sb _null_ _null_ _null_ ));

/* OIDS 400 - 499 */

DATA(insert OID =  401 (  text			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "1042" _null_ _null_ _null_ _null_ _null_	rtrim1 _null_ _null_ _null_ ));
DESCR("convert char(n) to text");
DATA(insert OID =  406 (  text			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "19" _null_ _null_ _null_ _null_ _null_ name_text _null_ _null_ _null_ ));
DESCR("convert name to text");
DATA(insert OID =  407 (  name			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 19 "25" _null_ _null_ _null_ _null_ _null_ text_name _null_ _null_ _null_ ));
DESCR("convert text to name");
DATA(insert OID =  408 (  bpchar		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1042 "19" _null_ _null_ _null_ _null_ _null_ name_bpchar _null_ _null_ _null_ ));
DESCR("convert name to char(n)");
DATA(insert OID =  409 (  name			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 19 "1042" _null_ _null_ _null_ _null_ _null_	bpchar_name _null_ _null_ _null_ ));
DESCR("convert char(n) to name");

DATA(insert OID = 449 (  hashint2		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "21" _null_ _null_ _null_ _null_ _null_ hashint2 _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 450 (  hashint4		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ hashint4 _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 949 (  hashint8		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "20" _null_ _null_ _null_ _null_ _null_ hashint8 _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 451 (  hashfloat4		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "700" _null_ _null_ _null_ _null_ _null_ hashfloat4 _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 452 (  hashfloat8		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "701" _null_ _null_ _null_ _null_ _null_ hashfloat8 _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 453 (  hashoid		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "26" _null_ _null_ _null_ _null_ _null_ hashoid _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 454 (  hashchar		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "18" _null_ _null_ _null_ _null_ _null_ hashchar _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 455 (  hashname		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "19" _null_ _null_ _null_ _null_ _null_ hashname _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 400 (  hashtext		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ hashtext _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 456 (  hashvarlena	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2281" _null_ _null_ _null_ _null_ _null_ hashvarlena _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 457 (  hashoidvector	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "30" _null_ _null_ _null_ _null_ _null_ hashoidvector _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 329 (  hash_aclitem	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1033" _null_ _null_ _null_ _null_ _null_	hash_aclitem _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 399 (  hashmacaddr	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "829" _null_ _null_ _null_ _null_ _null_ hashmacaddr _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 422 (  hashinet		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "869" _null_ _null_ _null_ _null_ _null_ hashinet _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 432 (  hash_numeric	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1700" _null_ _null_ _null_ _null_ _null_ hash_numeric _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 328 (  hashmacaddr8	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "774" _null_ _null_ _null_ _null_ _null_ hashmacaddr8 _null_ _null_ _null_ ));
DESCR("hash");

DATA(insert OID = 438 (  num_nulls		   UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 1 0 23 "2276" "{2276}" "{v}" _null_ _null_ _null_ ux_num_nulls _null_ _null_ _null_ ));
DESCR("count the number of NULL arguments");
DATA(insert OID = 440 (  num_nonnulls	   UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 1 0 23 "2276" "{2276}" "{v}" _null_ _null_ _null_ ux_num_nonnulls _null_ _null_ _null_ ));
DESCR("count the number of non-NULL arguments");

DATA(insert OID = 458 (  text_larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ text_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 459 (  text_smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ text_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 460 (  int8in			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "2275" _null_ _null_ _null_ _null_ _null_ int8in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 461 (  int8out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "20" _null_ _null_ _null_ _null_ _null_ int8out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 462 (  int8um			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8um _null_ _null_ _null_ ));
DATA(insert OID = 463 (  int8pl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8pl _null_ _null_ _null_ ));
DATA(insert OID = 464 (  int8mi			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8mi _null_ _null_ _null_ ));
DATA(insert OID = 465 (  int8mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8mul _null_ _null_ _null_ ));
DATA(insert OID = 466 (  int8div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8div _null_ _null_ _null_ ));
DATA(insert OID = 467 (  int8eq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8eq _null_ _null_ _null_ ));
DATA(insert OID = 468 (  int8ne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8ne _null_ _null_ _null_ ));
DATA(insert OID = 469 (  int8lt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8lt _null_ _null_ _null_ ));
DATA(insert OID = 470 (  int8gt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8gt _null_ _null_ _null_ ));
DATA(insert OID = 471 (  int8le			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8le _null_ _null_ _null_ ));
DATA(insert OID = 472 (  int8ge			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 20" _null_ _null_ _null_ _null_ _null_ int8ge _null_ _null_ _null_ ));

DATA(insert OID = 474 (  int84eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84eq _null_ _null_ _null_ ));
DATA(insert OID = 475 (  int84ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84ne _null_ _null_ _null_ ));
DATA(insert OID = 476 (  int84lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84lt _null_ _null_ _null_ ));
DATA(insert OID = 477 (  int84gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84gt _null_ _null_ _null_ ));
DATA(insert OID = 478 (  int84le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84le _null_ _null_ _null_ ));
DATA(insert OID = 479 (  int84ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 23" _null_ _null_ _null_ _null_ _null_ int84ge _null_ _null_ _null_ ));

DATA(insert OID = 480 (  int4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "20" _null_ _null_ _null_ _null_ _null_ int84 _null_ _null_ _null_ ));
DESCR("convert int8 to int4");
DATA(insert OID = 481 (  int8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "23" _null_ _null_ _null_ _null_ _null_ int48 _null_ _null_ _null_ ));
DESCR("convert int4 to int8");
DATA(insert OID = 482 (  float8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "20" _null_ _null_ _null_ _null_ _null_ i8tod _null_ _null_ _null_ ));
DESCR("convert int8 to float8");
DATA(insert OID = 483 (  int8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "701" _null_ _null_ _null_ _null_ _null_ dtoi8 _null_ _null_ _null_ ));
DESCR("convert float8 to int8");

/* OIDS 500 - 599 */

/* OIDS 600 - 699 */

DATA(insert OID = 626 (  hash_array		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2277" _null_ _null_ _null_ _null_ _null_ hash_array _null_ _null_ _null_ ));
DESCR("hash");

DATA(insert OID = 652 (  float4			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "20" _null_ _null_ _null_ _null_ _null_ i8tof _null_ _null_ _null_ ));
DESCR("convert int8 to float4");
DATA(insert OID = 653 (  int8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "700" _null_ _null_ _null_ _null_ _null_ ftoi8 _null_ _null_ _null_ ));
DESCR("convert float4 to int8");

DATA(insert OID = 714 (  int2			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "20" _null_ _null_ _null_ _null_ _null_ int82 _null_ _null_ _null_ ));
DESCR("convert int8 to int2");
DATA(insert OID = 754 (  int8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "21" _null_ _null_ _null_ _null_ _null_ int28 _null_ _null_ _null_ ));
DESCR("convert int2 to int8");

DATA(insert OID = 655 (  namelt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ namelt _null_ _null_ _null_ ));
DATA(insert OID = 656 (  namele			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ namele _null_ _null_ _null_ ));
DATA(insert OID = 657 (  namegt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ namegt _null_ _null_ _null_ ));
DATA(insert OID = 658 (  namege			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ namege _null_ _null_ _null_ ));
DATA(insert OID = 659 (  namene			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "19 19" _null_ _null_ _null_ _null_ _null_ namene _null_ _null_ _null_ ));

DATA(insert OID = 668 (  bpchar			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1042 "1042 23 16" _null_ _null_ _null_ _null_ _null_ bpchar _null_ _null_ _null_ ));
DESCR("adjust char() to typmod length");
DATA(insert OID = 3097 ( varchar_transform UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ varchar_transform _null_ _null_ _null_ ));
DESCR("transform a varchar length coercion");
DATA(insert OID = 669 (  varchar		   UXNSP UXUID 12 1 0 0 varchar_transform f f f f t f i s 3 0 1043 "1043 23 16" _null_ _null_ _null_ _null_ _null_ varchar _null_ _null_ _null_ ));
DESCR("adjust varchar() to typmod length");

DATA(insert OID = 676 (  mktinterval	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 704 "702 702" _null_ _null_ _null_ _null_ _null_ mktinterval _null_ _null_ _null_ ));

DATA(insert OID = 619 (  oidvectorne	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectorne _null_ _null_ _null_ ));
DATA(insert OID = 677 (  oidvectorlt	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectorlt _null_ _null_ _null_ ));
DATA(insert OID = 678 (  oidvectorle	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectorle _null_ _null_ _null_ ));
DATA(insert OID = 679 (  oidvectoreq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectoreq _null_ _null_ _null_ ));
DATA(insert OID = 680 (  oidvectorge	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectorge _null_ _null_ _null_ ));
DATA(insert OID = 681 (  oidvectorgt	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "30 30" _null_ _null_ _null_ _null_ _null_ oidvectorgt _null_ _null_ _null_ ));

/* OIDS 700 - 799 */
DATA(insert OID = 710 (  getuxusername	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ current_user _null_ _null_ _null_ ));
DESCR("deprecated, use current_user instead");
DATA(insert OID = 716 (  oidlt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oidlt _null_ _null_ _null_ ));
DATA(insert OID = 717 (  oidle			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oidle _null_ _null_ _null_ ));

DATA(insert OID = 720 (  octet_length	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "17" _null_ _null_ _null_ _null_ _null_ byteaoctetlen _null_ _null_ _null_ ));
DESCR("octet length");
DATA(insert OID = 721 (  get_byte		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "17 23" _null_ _null_ _null_ _null_ _null_ byteaGetByte _null_ _null_ _null_ ));
DESCR("get byte");
DATA(insert OID = 722 (  set_byte		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 17 "17 23 23" _null_ _null_ _null_ _null_ _null_	byteaSetByte _null_ _null_ _null_ ));
DESCR("set byte");
DATA(insert OID = 723 (  get_bit		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "17 23" _null_ _null_ _null_ _null_ _null_ byteaGetBit _null_ _null_ _null_ ));
DESCR("get bit");
DATA(insert OID = 724 (  set_bit		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 17 "17 23 23" _null_ _null_ _null_ _null_ _null_	byteaSetBit _null_ _null_ _null_ ));
DESCR("set bit");
DATA(insert OID = 749 (  overlay		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 17 "17 17 23 23" _null_ _null_ _null_ _null_ _null_ byteaoverlay _null_ _null_ _null_ ));
DESCR("substitute portion of string");
DATA(insert OID = 752 (  overlay		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 17 "17 17 23" _null_ _null_ _null_ _null_ _null_	byteaoverlay_no_len _null_ _null_ _null_ ));
DESCR("substitute portion of string");

DATA(insert OID = 725 (  dist_pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 628" _null_ _null_ _null_ _null_ _null_	dist_pl _null_ _null_ _null_ ));
DATA(insert OID = 726 (  dist_lb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "628 603" _null_ _null_ _null_ _null_ _null_	dist_lb _null_ _null_ _null_ ));
DATA(insert OID = 727 (  dist_sl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "601 628" _null_ _null_ _null_ _null_ _null_	dist_sl _null_ _null_ _null_ ));
DATA(insert OID = 728 (  dist_cpoly		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "718 604" _null_ _null_ _null_ _null_ _null_	dist_cpoly _null_ _null_ _null_ ));
DATA(insert OID = 729 (  poly_distance	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "604 604" _null_ _null_ _null_ _null_ _null_	poly_distance _null_ _null_ _null_ ));
DATA(insert OID = 3275 (  dist_ppoly	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 604" _null_ _null_ _null_ _null_ _null_	dist_ppoly _null_ _null_ _null_ ));
DATA(insert OID = 3292 (  dist_polyp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "604 600" _null_ _null_ _null_ _null_ _null_	dist_polyp _null_ _null_ _null_ ));
DATA(insert OID = 3290 (  dist_cpoint	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "718 600" _null_ _null_ _null_ _null_ _null_	dist_cpoint _null_ _null_ _null_ ));

DATA(insert OID = 740 (  text_lt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_lt _null_ _null_ _null_ ));
DATA(insert OID = 741 (  text_le		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_le _null_ _null_ _null_ ));
DATA(insert OID = 742 (  text_gt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_gt _null_ _null_ _null_ ));
DATA(insert OID = 743 (  text_ge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_ge _null_ _null_ _null_ ));

DATA(insert OID = 745 (  current_user	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ current_user _null_ _null_ _null_ ));
DESCR("current user name");
DATA(insert OID = 746 (  session_user	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ session_user _null_ _null_ _null_ ));
DESCR("session user name");

DATA(insert OID = 744 (  array_eq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_eq _null_ _null_ _null_ ));
DATA(insert OID = 390 (  array_ne		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_ne _null_ _null_ _null_ ));
DATA(insert OID = 391 (  array_lt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_lt _null_ _null_ _null_ ));
DATA(insert OID = 392 (  array_gt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_gt _null_ _null_ _null_ ));
DATA(insert OID = 393 (  array_le		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_le _null_ _null_ _null_ ));
DATA(insert OID = 396 (  array_ge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_ge _null_ _null_ _null_ ));
DATA(insert OID = 747 (  array_dims		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "2277" _null_ _null_ _null_ _null_ _null_ array_dims _null_ _null_ _null_ ));
DESCR("array dimensions");
DATA(insert OID = 748 (  array_ndims	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2277" _null_ _null_ _null_ _null_ _null_ array_ndims _null_ _null_ _null_ ));
DESCR("number of array dimensions");
DATA(insert OID = 750 (  array_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2277 "2275 26 23" _null_ _null_ _null_ _null_ _null_	array_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 751 (  array_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2277" _null_ _null_ _null_ _null_ _null_ array_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2091 (  array_lower	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2277 23" _null_ _null_ _null_ _null_ _null_ array_lower _null_ _null_ _null_ ));
DESCR("array lower dimension");
DATA(insert OID = 2092 (  array_upper	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2277 23" _null_ _null_ _null_ _null_ _null_ array_upper _null_ _null_ _null_ ));
DESCR("array upper dimension");
DATA(insert OID = 2176 (  array_length	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2277 23" _null_ _null_ _null_ _null_ _null_ array_length _null_ _null_ _null_ ));
DESCR("array length");
DATA(insert OID = 3179 (  cardinality	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2277" _null_ _null_ _null_ _null_ _null_ array_cardinality _null_ _null_ _null_ ));
DESCR("array cardinality");
DATA(insert OID = 378 (  array_append	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2277 2283" _null_ _null_ _null_ _null_ _null_ array_append _null_ _null_ _null_ ));
DESCR("append element onto end of array");
DATA(insert OID = 379 (  array_prepend	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2283 2277" _null_ _null_ _null_ _null_ _null_ array_prepend _null_ _null_ _null_ ));
DESCR("prepend element onto front of array");
DATA(insert OID = 383 (  array_cat		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_cat _null_ _null_ _null_ ));
DATA(insert OID = 394 (  string_to_array   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1009 "25 25" _null_ _null_ _null_ _null_ _null_ text_to_array _null_ _null_ _null_ ));
DESCR("split delimited text into text[]");
DATA(insert OID = 395 (  array_to_string   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "2277 25" _null_ _null_ _null_ _null_ _null_ array_to_text _null_ _null_ _null_ ));
DESCR("concatenate array elements, using delimiter, into text");
DATA(insert OID = 376 (  string_to_array   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 1009 "25 25 25" _null_ _null_ _null_ _null_ _null_ text_to_array_null _null_ _null_ _null_ ));
DESCR("split delimited text into text[], with null string");
DATA(insert OID = 384 (  array_to_string   UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 25 "2277 25 25" _null_ _null_ _null_ _null_ _null_ array_to_text_null _null_ _null_ _null_ ));
DESCR("concatenate array elements, using delimiter and null string, into text");
DATA(insert OID = 515 (  array_larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2277 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 516 (  array_smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2277 "2277 2277" _null_ _null_ _null_ _null_ _null_ array_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 3277 (  array_position		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 23 "2277 2283" _null_ _null_ _null_ _null_ _null_ array_position _null_ _null_ _null_ ));
DESCR("returns an offset of value in array");
DATA(insert OID = 3278 (  array_position		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 23 "2277 2283 23" _null_ _null_ _null_ _null_ _null_ array_position_start _null_ _null_ _null_ ));
DESCR("returns an offset of value in array with start index");
DATA(insert OID = 3279 (  array_positions		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1007 "2277 2283" _null_ _null_ _null_ _null_ _null_ array_positions _null_ _null_ _null_ ));
DESCR("returns an array of offsets of some value in array");
DATA(insert OID = 1191 (  generate_subscripts UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 23 "2277 23 16" _null_ _null_ _null_ _null_ _null_ generate_subscripts _null_ _null_ _null_ ));
DESCR("array subscripts generator");
DATA(insert OID = 1192 (  generate_subscripts UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 23 "2277 23" _null_ _null_ _null_ _null_ _null_ generate_subscripts_nodir _null_ _null_ _null_ ));
DESCR("array subscripts generator");
DATA(insert OID = 1193 (  array_fill UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2283 1007" _null_ _null_ _null_ _null_ _null_ array_fill _null_ _null_ _null_ ));
DESCR("array constructor with value");
DATA(insert OID = 1286 (  array_fill UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2277 "2283 1007 1007" _null_ _null_ _null_ _null_ _null_ array_fill_with_lower_bounds _null_ _null_ _null_ ));
DESCR("array constructor with value");
DATA(insert OID = 2331 (  unnest		   UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 2283 "2277" _null_ _null_ _null_ _null_ _null_ array_unnest _null_ _null_ _null_ ));
DESCR("expand array to set of rows");
DATA(insert OID = 3167 (  array_remove	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2277 2283" _null_ _null_ _null_ _null_ _null_ array_remove _null_ _null_ _null_ ));
DESCR("remove any occurrences of an element from an array");
DATA(insert OID = 3168 (  array_replace    UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2277 "2277 2283 2283" _null_ _null_ _null_ _null_ _null_ array_replace _null_ _null_ _null_ ));
DESCR("replace any occurrences of an element in an array");
DATA(insert OID = 2333 (  array_agg_transfn   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2776" _null_ _null_ _null_ _null_ _null_ array_agg_transfn _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 2334 (  array_agg_finalfn   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2281 2776" _null_ _null_ _null_ _null_ _null_ array_agg_finalfn _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2335 (  array_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 2277 "2776" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("concatenate aggregate input into an array");
DATA(insert OID = 4051 (  array_agg_array_transfn	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2277" _null_ _null_ _null_ _null_ _null_ array_agg_array_transfn _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 4052 (  array_agg_array_finalfn	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2277 "2281 2277" _null_ _null_ _null_ _null_ _null_ array_agg_array_finalfn _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 4053 (  array_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 2277 "2277" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("concatenate aggregate input into an array");
DATA(insert OID = 3218 ( width_bucket	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2283 2277" _null_ _null_ _null_ _null_ _null_ width_bucket_array _null_ _null_ _null_ ));
DESCR("bucket number of operand given a sorted array of bucket lower bounds");
DATA(insert OID = 3816 (  array_typanalyze UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_ array_typanalyze _null_ _null_ _null_ ));
DESCR("array typanalyze");
DATA(insert OID = 3817 (  arraycontsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ arraycontsel _null_ _null_ _null_ ));
DESCR("restriction selectivity for array-containment operators");
DATA(insert OID = 3818 (  arraycontjoinsel UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ arraycontjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity for array-containment operators");

DATA(insert OID = 760 (  smgrin			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 210 "2275" _null_ _null_ _null_ _null_ _null_ smgrin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 761 (  smgrout		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "210" _null_ _null_ _null_ _null_ _null_ smgrout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 762 (  smgreq			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "210 210" _null_ _null_ _null_ _null_ _null_ smgreq _null_ _null_ _null_ ));
DESCR("storage manager");
DATA(insert OID = 763 (  smgrne			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "210 210" _null_ _null_ _null_ _null_ _null_ smgrne _null_ _null_ _null_ ));
DESCR("storage manager");

DATA(insert OID = 764 (  lo_import		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 26 "25" _null_ _null_ _null_ _null_ _null_ be_lo_import _null_ _null_ _null_ ));
DESCR("large object import");
DATA(insert OID = 767 (  lo_import		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 26 "25 26" _null_ _null_ _null_ _null_ _null_ be_lo_import_with_oid _null_ _null_ _null_ ));
DESCR("large object import");
DATA(insert OID = 765 (  lo_export		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 23 "26 25" _null_ _null_ _null_ _null_ _null_ be_lo_export _null_ _null_ _null_ ));
DESCR("large object export");

DATA(insert OID = 766 (  int4inc		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4inc _null_ _null_ _null_ ));
DESCR("increment");
DATA(insert OID = 768 (  int4larger		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 769 (  int4smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 770 (  int2larger		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 771 (  int2smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 784 (  tintervaleq	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervaleq _null_ _null_ _null_ ));
DATA(insert OID = 785 (  tintervalne	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalne _null_ _null_ _null_ ));
DATA(insert OID = 786 (  tintervallt	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervallt _null_ _null_ _null_ ));
DATA(insert OID = 787 (  tintervalgt	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalgt _null_ _null_ _null_ ));
DATA(insert OID = 788 (  tintervalle	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalle _null_ _null_ _null_ ));
DATA(insert OID = 789 (  tintervalge	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "704 704" _null_ _null_ _null_ _null_ _null_ tintervalge _null_ _null_ _null_ ));

/* OIDS 800 - 899 */

DATA(insert OID =  846 (  cash_mul_flt4    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 700" _null_ _null_ _null_ _null_ _null_	cash_mul_flt4 _null_ _null_ _null_ ));
DATA(insert OID =  847 (  cash_div_flt4    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 700" _null_ _null_ _null_ _null_ _null_	cash_div_flt4 _null_ _null_ _null_ ));
DATA(insert OID =  848 (  flt4_mul_cash    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "700 790" _null_ _null_ _null_ _null_ _null_	flt4_mul_cash _null_ _null_ _null_ ));

DATA(insert OID =  849 (  position		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ textpos _null_ _null_ _null_ ));
DESCR("position of substring");
DATA(insert OID =  850 (  textlike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textlike _null_ _null_ _null_ ));
DATA(insert OID =  851 (  textnlike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textnlike _null_ _null_ _null_ ));

DATA(insert OID =  852 (  int48eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48eq _null_ _null_ _null_ ));
DATA(insert OID =  853 (  int48ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48ne _null_ _null_ _null_ ));
DATA(insert OID =  854 (  int48lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48lt _null_ _null_ _null_ ));
DATA(insert OID =  855 (  int48gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48gt _null_ _null_ _null_ ));
DATA(insert OID =  856 (  int48le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48le _null_ _null_ _null_ ));
DATA(insert OID =  857 (  int48ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "23 20" _null_ _null_ _null_ _null_ _null_ int48ge _null_ _null_ _null_ ));

DATA(insert OID =  858 (  namelike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ namelike _null_ _null_ _null_ ));
DATA(insert OID =  859 (  namenlike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ namenlike _null_ _null_ _null_ ));

DATA(insert OID =  860 (  bpchar		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1042 "18" _null_ _null_ _null_ _null_ _null_	char_bpchar _null_ _null_ _null_ ));
DESCR("convert char to char(n)");

DATA(insert OID = 861 ( current_database	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ current_database _null_ _null_ _null_ ));
DESCR("name of the current database");
DATA(insert OID = 817 (  current_query		  UXNSP UXUID 12 1 0 0 0 f f f f f f v r 0 0 25 "" _null_ _null_ _null_ _null_	_null_ current_query _null_ _null_ _null_ ));
DESCR("get the currently executing query");

DATA(insert OID = 3399 (  int8_mul_cash		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "20 790" _null_ _null_ _null_ _null_ _null_ int8_mul_cash _null_ _null_ _null_ ));
DATA(insert OID =  862 (  int4_mul_cash		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "23 790" _null_ _null_ _null_ _null_ _null_ int4_mul_cash _null_ _null_ _null_ ));
DATA(insert OID =  863 (  int2_mul_cash		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "21 790" _null_ _null_ _null_ _null_ _null_ int2_mul_cash _null_ _null_ _null_ ));
DATA(insert OID = 3344 (  cash_mul_int8		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 20" _null_ _null_ _null_ _null_ _null_ cash_mul_int8 _null_ _null_ _null_ ));
DATA(insert OID = 3345 (  cash_div_int8		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 20" _null_ _null_ _null_ _null_ _null_ cash_div_int8 _null_ _null_ _null_ ));
DATA(insert OID =  864 (  cash_mul_int4		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 23" _null_ _null_ _null_ _null_ _null_ cash_mul_int4 _null_ _null_ _null_ ));
DATA(insert OID =  865 (  cash_div_int4		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 23" _null_ _null_ _null_ _null_ _null_ cash_div_int4 _null_ _null_ _null_ ));
DATA(insert OID =  866 (  cash_mul_int2		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 21" _null_ _null_ _null_ _null_ _null_ cash_mul_int2 _null_ _null_ _null_ ));
DATA(insert OID =  867 (  cash_div_int2		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 21" _null_ _null_ _null_ _null_ _null_ cash_div_int2 _null_ _null_ _null_ ));

DATA(insert OID =  886 (  cash_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 790 "2275" _null_ _null_ _null_ _null_ _null_ cash_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  887 (  cash_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "790" _null_ _null_ _null_ _null_ _null_ cash_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  888 (  cash_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_eq _null_ _null_ _null_ ));
DATA(insert OID =  889 (  cash_ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_ne _null_ _null_ _null_ ));
DATA(insert OID =  890 (  cash_lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_lt _null_ _null_ _null_ ));
DATA(insert OID =  891 (  cash_le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_le _null_ _null_ _null_ ));
DATA(insert OID =  892 (  cash_gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_gt _null_ _null_ _null_ ));
DATA(insert OID =  893 (  cash_ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "790 790" _null_ _null_ _null_ _null_ _null_ cash_ge _null_ _null_ _null_ ));
DATA(insert OID =  894 (  cash_pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 790" _null_ _null_ _null_ _null_ _null_	cash_pl _null_ _null_ _null_ ));
DATA(insert OID =  895 (  cash_mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 790" _null_ _null_ _null_ _null_ _null_	cash_mi _null_ _null_ _null_ ));
DATA(insert OID =  896 (  cash_mul_flt8    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 701" _null_ _null_ _null_ _null_ _null_	cash_mul_flt8 _null_ _null_ _null_ ));
DATA(insert OID =  897 (  cash_div_flt8    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 701" _null_ _null_ _null_ _null_ _null_	cash_div_flt8 _null_ _null_ _null_ ));
DATA(insert OID =  898 (  cashlarger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 790" _null_ _null_ _null_ _null_ _null_	cashlarger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID =  899 (  cashsmaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "790 790" _null_ _null_ _null_ _null_ _null_	cashsmaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID =  919 (  flt8_mul_cash    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 790 "701 790" _null_ _null_ _null_ _null_ _null_	flt8_mul_cash _null_ _null_ _null_ ));
DATA(insert OID =  935 (  cash_words	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "790" _null_ _null_ _null_ _null_ _null_ cash_words _null_ _null_ _null_ ));
DESCR("output money amount as words");
DATA(insert OID = 3822 (  cash_div_cash    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "790 790" _null_ _null_ _null_ _null_ _null_	cash_div_cash _null_ _null_ _null_ ));
DATA(insert OID = 3823 (  numeric		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1700 "790" _null_ _null_ _null_ _null_ _null_ cash_numeric _null_ _null_ _null_ ));
DESCR("convert money to numeric");
DATA(insert OID = 3824 (  money			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 790 "1700" _null_ _null_ _null_ _null_ _null_ numeric_cash _null_ _null_ _null_ ));
DESCR("convert numeric to money");
DATA(insert OID = 3811 (  money			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 790 "23" _null_ _null_ _null_ _null_ _null_ int4_cash _null_ _null_ _null_ ));
DESCR("convert int4 to money");
DATA(insert OID = 3812 (  money			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 790 "20" _null_ _null_ _null_ _null_ _null_ int8_cash _null_ _null_ _null_ ));
DESCR("convert int8 to money");

/* OIDS 900 - 999 */

DATA(insert OID = 940 (  mod			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2mod _null_ _null_ _null_ ));
DESCR("modulus");
DATA(insert OID = 941 (  mod			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4mod _null_ _null_ _null_ ));
DESCR("modulus");

DATA(insert OID = 945 (  int8mod		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8mod _null_ _null_ _null_ ));
DATA(insert OID = 947 (  mod			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8mod _null_ _null_ _null_ ));
DESCR("modulus");

DATA(insert OID = 944 (  char			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 18 "25" _null_ _null_ _null_ _null_ _null_ text_char _null_ _null_ _null_ ));
DESCR("convert text to char");
DATA(insert OID = 946 (  text			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "18" _null_ _null_ _null_ _null_ _null_ char_text _null_ _null_ _null_ ));
DESCR("convert char to text");

DATA(insert OID = 952 (  lo_open		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 23 "26 23" _null_ _null_ _null_ _null_ _null_ be_lo_open _null_ _null_ _null_ ));
DESCR("large object open");
DATA(insert OID = 953 (  lo_close		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ be_lo_close _null_ _null_ _null_ ));
DESCR("large object close");
DATA(insert OID = 954 (  loread			   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 17 "23 23" _null_ _null_ _null_ _null_ _null_ be_loread _null_ _null_ _null_ ));
DESCR("large object read");
DATA(insert OID = 955 (  lowrite		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 23 "23 17" _null_ _null_ _null_ _null_ _null_ be_lowrite _null_ _null_ _null_ ));
DESCR("large object write");
DATA(insert OID = 956 (  lo_lseek		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 23 "23 23 23" _null_ _null_ _null_ _null_ _null_	be_lo_lseek _null_ _null_ _null_ ));
DESCR("large object seek");
DATA(insert OID = 3170 (  lo_lseek64	   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 20 "23 20 23" _null_ _null_ _null_ _null_ _null_	be_lo_lseek64 _null_ _null_ _null_ ));
DESCR("large object seek (64 bit)");
DATA(insert OID = 957 (  lo_creat		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 26 "23" _null_ _null_ _null_ _null_ _null_ be_lo_creat _null_ _null_ _null_ ));
DESCR("large object create");
DATA(insert OID = 715 (  lo_create		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 26 "26" _null_ _null_ _null_ _null_ _null_ be_lo_create _null_ _null_ _null_ ));
DESCR("large object create");
DATA(insert OID = 958 (  lo_tell		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ be_lo_tell _null_ _null_ _null_ ));
DESCR("large object position");
DATA(insert OID = 3171 (  lo_tell64		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 20 "23" _null_ _null_ _null_ _null_ _null_ be_lo_tell64 _null_ _null_ _null_ ));
DESCR("large object position (64 bit)");
DATA(insert OID = 1004 (  lo_truncate	   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ be_lo_truncate _null_ _null_ _null_ ));
DESCR("truncate large object");
DATA(insert OID = 3172 (  lo_truncate64    UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 23 "23 20" _null_ _null_ _null_ _null_ _null_ be_lo_truncate64 _null_ _null_ _null_ ));
DESCR("truncate large object (64 bit)");

DATA(insert OID = 3457 (  lo_from_bytea    UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 26 "26 17" _null_ _null_ _null_ _null_ _null_ be_lo_from_bytea _null_ _null_ _null_ ));
DESCR("create new large object with given content");
DATA(insert OID = 3458 (  lo_get		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 17 "26" _null_ _null_ _null_ _null_ _null_ be_lo_get _null_ _null_ _null_ ));
DESCR("read entire large object");
DATA(insert OID = 3459 (  lo_get		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 17 "26 20 23" _null_ _null_ _null_ _null_ _null_ be_lo_get_fragment _null_ _null_ _null_ ));
DESCR("read large object from offset for length");
DATA(insert OID = 3460 (  lo_put		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 2278 "26 20 17" _null_ _null_ _null_ _null_ _null_ be_lo_put _null_ _null_ _null_ ));
DESCR("write data at offset");

DATA(insert OID = 959 (  on_pl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 628" _null_ _null_ _null_ _null_ _null_ on_pl _null_ _null_ _null_ ));
DATA(insert OID = 960 (  on_sl			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 628" _null_ _null_ _null_ _null_ _null_ on_sl _null_ _null_ _null_ ));
DATA(insert OID = 961 (  close_pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 628" _null_ _null_ _null_ _null_ _null_	close_pl _null_ _null_ _null_ ));
DATA(insert OID = 962 (  close_sl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "601 628" _null_ _null_ _null_ _null_ _null_	close_sl _null_ _null_ _null_ ));
DATA(insert OID = 963 (  close_lb		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "628 603" _null_ _null_ _null_ _null_ _null_	close_lb _null_ _null_ _null_ ));

DATA(insert OID = 964 (  lo_unlink		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 23 "26" _null_ _null_ _null_ _null_ _null_ be_lo_unlink _null_ _null_ _null_ ));
DESCR("large object unlink (delete)");

DATA(insert OID = 973 (  path_inter		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_inter _null_ _null_ _null_ ));
DATA(insert OID = 975 (  area			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "603" _null_ _null_ _null_ _null_ _null_	box_area _null_ _null_ _null_ ));
DESCR("box area");
DATA(insert OID = 976 (  width			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "603" _null_ _null_ _null_ _null_ _null_	box_width _null_ _null_ _null_ ));
DESCR("box width");
DATA(insert OID = 977 (  height			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "603" _null_ _null_ _null_ _null_ _null_	box_height _null_ _null_ _null_ ));
DESCR("box height");
DATA(insert OID = 978 (  box_distance	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "603 603" _null_ _null_ _null_ _null_ _null_	box_distance _null_ _null_ _null_ ));
DATA(insert OID = 979 (  area			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "602" _null_ _null_ _null_ _null_ _null_	path_area _null_ _null_ _null_ ));
DESCR("area of a closed path");
DATA(insert OID = 980 (  box_intersect	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 603" _null_ _null_ _null_ _null_ _null_	box_intersect _null_ _null_ _null_ ));
DATA(insert OID = 4067 ( bound_box		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 603" _null_ _null_ _null_ _null_ _null_	boxes_bound_box _null_ _null_ _null_ ));
DESCR("bounding box of two boxes");
DATA(insert OID = 981 (  diagonal		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 601 "603" _null_ _null_ _null_ _null_ _null_	box_diagonal _null_ _null_ _null_ ));
DESCR("box diagonal");
DATA(insert OID = 982 (  path_n_lt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_n_lt _null_ _null_ _null_ ));
DATA(insert OID = 983 (  path_n_gt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_n_gt _null_ _null_ _null_ ));
DATA(insert OID = 984 (  path_n_eq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_n_eq _null_ _null_ _null_ ));
DATA(insert OID = 985 (  path_n_le		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_n_le _null_ _null_ _null_ ));
DATA(insert OID = 986 (  path_n_ge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "602 602" _null_ _null_ _null_ _null_ _null_ path_n_ge _null_ _null_ _null_ ));
DATA(insert OID = 987 (  path_length	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "602" _null_ _null_ _null_ _null_ _null_	path_length _null_ _null_ _null_ ));
DATA(insert OID = 988 (  point_ne		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_ne _null_ _null_ _null_ ));
DATA(insert OID = 989 (  point_vert		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_vert _null_ _null_ _null_ ));
DATA(insert OID = 990 (  point_horiz	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_ _null_ point_horiz _null_ _null_ _null_ ));
DATA(insert OID = 991 (  point_distance    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 600" _null_ _null_ _null_ _null_ _null_	point_distance _null_ _null_ _null_ ));
DATA(insert OID = 992 (  slope			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 600" _null_ _null_ _null_ _null_ _null_	point_slope _null_ _null_ _null_ ));
DESCR("slope between points");
DATA(insert OID = 993 (  lseg			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 601 "600 600" _null_ _null_ _null_ _null_ _null_	lseg_construct _null_ _null_ _null_ ));
DESCR("convert points to line segment");
DATA(insert OID = 994 (  lseg_intersect    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_ _null_ lseg_intersect _null_ _null_ _null_ ));
DATA(insert OID = 995 (  lseg_parallel	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_ _null_ lseg_parallel _null_ _null_ _null_ ));
DATA(insert OID = 996 (  lseg_perp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_ _null_ lseg_perp _null_ _null_ _null_ ));
DATA(insert OID = 997 (  lseg_vertical	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "601" _null_ _null_ _null_ _null_ _null_ lseg_vertical _null_ _null_ _null_ ));
DATA(insert OID = 998 (  lseg_horizontal   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "601" _null_ _null_ _null_ _null_ _null_ lseg_horizontal _null_ _null_ _null_ ));
DATA(insert OID = 999 (  lseg_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_ _null_ lseg_eq _null_ _null_ _null_ ));

/* OIDS 1000 - 1999 */

DATA(insert OID = 3994 (  timestamp_izone_transform UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ timestamp_izone_transform _null_ _null_ _null_ ));
DESCR("transform a time zone adjustment");
DATA(insert OID = 1026 (  timezone		   UXNSP UXUID 12 1 0 0 timestamp_izone_transform f f f f t f i s 2 0 1114 "1186 1184" _null_ _null_ _null_ _null_ _null_ timestamptz_izone _null_ _null_ _null_ ));
DESCR("adjust timestamp to new time zone");

DATA(insert OID = 1031 (  aclitemin		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1033 "2275" _null_ _null_ _null_ _null_ _null_ aclitemin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1032 (  aclitemout	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "1033" _null_ _null_ _null_ _null_ _null_ aclitemout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1035 (  aclinsert		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1034 "1034 1033" _null_ _null_ _null_ _null_ _null_ aclinsert _null_ _null_ _null_ ));
DESCR("add/update ACL item");
DATA(insert OID = 1036 (  aclremove		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1034 "1034 1033" _null_ _null_ _null_ _null_ _null_ aclremove _null_ _null_ _null_ ));
DESCR("remove ACL item");
DATA(insert OID = 1037 (  aclcontains	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1034 1033" _null_ _null_ _null_ _null_ _null_ aclcontains _null_ _null_ _null_ ));
DESCR("contains");
DATA(insert OID = 1062 (  aclitemeq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1033 1033" _null_ _null_ _null_ _null_ _null_ aclitem_eq _null_ _null_ _null_ ));
DATA(insert OID = 1365 (  makeaclitem	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 1033 "26 26 25 16" _null_ _null_ _null_ _null_ _null_ makeaclitem _null_ _null_ _null_ ));
DESCR("make ACL item");
DATA(insert OID = 3943 (  acldefault	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1034 "18 26" _null_ _null_ _null_ _null_  _null_ acldefault_sql _null_ _null_ _null_ ));
DESCR("TODO");
DATA(insert OID = 1689 (  aclexplode	UXNSP UXUID 12 1 10 0 0 f f f f t t s s 1 0 2249 "1034" "{1034,26,26,25,16}" "{i,o,o,o,o}" "{acl,grantor,grantee,privilege_type,is_grantable}" _null_ _null_ aclexplode _null_ _null_ _null_ ));
DESCR("convert ACL item array to table, for use by information schema");
DATA(insert OID = 1044 (  bpcharin		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1042 "2275 26 23" _null_ _null_ _null_ _null_ _null_ bpcharin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1045 (  bpcharout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1042" _null_ _null_ _null_ _null_ _null_ bpcharout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2913 (  bpchartypmodin   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_	bpchartypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2914 (  bpchartypmodout  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_	bpchartypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1046 (  varcharin		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1043 "2275 26 23" _null_ _null_ _null_ _null_ _null_ varcharin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1047 (  varcharout	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1043" _null_ _null_ _null_ _null_ _null_ varcharout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2915 (  varchartypmodin  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_	varchartypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2916 (  varchartypmodout UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_	varchartypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1048 (  bpchareq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchareq _null_ _null_ _null_ ));
DATA(insert OID = 1049 (  bpcharlt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpcharlt _null_ _null_ _null_ ));
DATA(insert OID = 1050 (  bpcharle		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpcharle _null_ _null_ _null_ ));
DATA(insert OID = 1051 (  bpchargt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchargt _null_ _null_ _null_ ));
DATA(insert OID = 1052 (  bpcharge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpcharge _null_ _null_ _null_ ));
DATA(insert OID = 1053 (  bpcharne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpcharne _null_ _null_ _null_ ));
DATA(insert OID = 1063 (  bpchar_larger    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1042 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1064 (  bpchar_smaller   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1042 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1078 (  bpcharcmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpcharcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3328 ( bpchar_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ bpchar_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 1080 (  hashbpchar	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1042" _null_ _null_ _null_ _null_ _null_	hashbpchar _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 1081 (  format_type	   UXNSP UXUID 12 1 0 0 0 f f f f f f s s 2 0 25 "26 23" _null_ _null_ _null_ _null_ _null_ format_type _null_ _null_ _null_ ));
DESCR("format a type oid and atttypmod to canonical SQL");
DATA(insert OID = 1084 (  date_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1082 "2275" _null_ _null_ _null_ _null_ _null_ date_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1085 (  date_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "1082" _null_ _null_ _null_ _null_ _null_ date_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1086 (  date_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_eq _null_ _null_ _null_ ));
DATA(insert OID = 1087 (  date_lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_lt _null_ _null_ _null_ ));
DATA(insert OID = 1088 (  date_le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_le _null_ _null_ _null_ ));
DATA(insert OID = 1089 (  date_gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_gt _null_ _null_ _null_ ));
DATA(insert OID = 1090 (  date_ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_ge _null_ _null_ _null_ ));
DATA(insert OID = 1091 (  date_ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_ne _null_ _null_ _null_ ));
DATA(insert OID = 1092 (  date_cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3136 (  date_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ date_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");

/* OIDS 1100 - 1199 */

DATA(insert OID = 1102 (  time_lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_lt _null_ _null_ _null_ ));
DATA(insert OID = 1103 (  time_le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_le _null_ _null_ _null_ ));
DATA(insert OID = 1104 (  time_gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_gt _null_ _null_ _null_ ));
DATA(insert OID = 1105 (  time_ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_ge _null_ _null_ _null_ ));
DATA(insert OID = 1106 (  time_ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_ne _null_ _null_ _null_ ));
DATA(insert OID = 1107 (  time_cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 1138 (  date_larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1082 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1139 (  date_smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1082 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1140 (  date_mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1082 1082" _null_ _null_ _null_ _null_ _null_ date_mi _null_ _null_ _null_ ));
DATA(insert OID = 1141 (  date_pli		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1082 "1082 23" _null_ _null_ _null_ _null_ _null_ date_pli _null_ _null_ _null_ ));
DATA(insert OID = 1142 (  date_mii		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1082 "1082 23" _null_ _null_ _null_ _null_ _null_ date_mii _null_ _null_ _null_ ));
DATA(insert OID = 1143 (  time_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1083 "2275 26 23" _null_ _null_ _null_ _null_ _null_ time_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1144 (  time_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1083" _null_ _null_ _null_ _null_ _null_ time_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2909 (  timetypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ timetypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2910 (  timetypmodout		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ timetypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1145 (  time_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_eq _null_ _null_ _null_ ));

DATA(insert OID = 1146 (  circle_add_pt    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 718 "718 600" _null_ _null_ _null_ _null_ _null_	circle_add_pt _null_ _null_ _null_ ));
DATA(insert OID = 1147 (  circle_sub_pt    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 718 "718 600" _null_ _null_ _null_ _null_ _null_	circle_sub_pt _null_ _null_ _null_ ));
DATA(insert OID = 1148 (  circle_mul_pt    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 718 "718 600" _null_ _null_ _null_ _null_ _null_	circle_mul_pt _null_ _null_ _null_ ));
DATA(insert OID = 1149 (  circle_div_pt    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 718 "718 600" _null_ _null_ _null_ _null_ _null_	circle_div_pt _null_ _null_ _null_ ));

DATA(insert OID = 1150 (  timestamptz_in   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1184 "2275 26 23" _null_ _null_ _null_ _null_ _null_ timestamptz_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1151 (  timestamptz_out  UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2907 (  timestamptztypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ timestamptztypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2908 (  timestamptztypmodout		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ timestamptztypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1152 (  timestamptz_eq   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_eq _null_ _null_ _null_ ));
DATA(insert OID = 1153 (  timestamptz_ne   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_ne _null_ _null_ _null_ ));
DATA(insert OID = 1154 (  timestamptz_lt   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_lt _null_ _null_ _null_ ));
DATA(insert OID = 1155 (  timestamptz_le   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_le _null_ _null_ _null_ ));
DATA(insert OID = 1156 (  timestamptz_ge   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_ge _null_ _null_ _null_ ));
DATA(insert OID = 1157 (  timestamptz_gt   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_gt _null_ _null_ _null_ ));
DATA(insert OID = 1158 (  to_timestamp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1184 "701" _null_ _null_ _null_ _null_ _null_ float8_timestamptz _null_ _null_ _null_ ));
DESCR("convert UNIX epoch to timestamptz");
DATA(insert OID = 3995 (  timestamp_zone_transform UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ timestamp_zone_transform _null_ _null_ _null_ ));
DESCR("transform a time zone adjustment");
DATA(insert OID = 1159 (  timezone		   UXNSP UXUID 12 1 0 0 timestamp_zone_transform f f f f t f i s 2 0 1114 "25 1184" _null_ _null_ _null_ _null_ _null_	timestamptz_zone _null_ _null_ _null_ ));
DESCR("adjust timestamp to new time zone");

DATA(insert OID = 1160 (  interval_in	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1186 "2275 26 23" _null_ _null_ _null_ _null_ _null_ interval_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1161 (  interval_out	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1186" _null_ _null_ _null_ _null_ _null_ interval_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2903 (  intervaltypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ intervaltypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2904 (  intervaltypmodout		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ intervaltypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1162 (  interval_eq	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_eq _null_ _null_ _null_ ));
DATA(insert OID = 1163 (  interval_ne	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_ne _null_ _null_ _null_ ));
DATA(insert OID = 1164 (  interval_lt	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_lt _null_ _null_ _null_ ));
DATA(insert OID = 1165 (  interval_le	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_le _null_ _null_ _null_ ));
DATA(insert OID = 1166 (  interval_ge	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_ge _null_ _null_ _null_ ));
DATA(insert OID = 1167 (  interval_gt	   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_gt _null_ _null_ _null_ ));
DATA(insert OID = 1168 (  interval_um	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ interval_um _null_ _null_ _null_ ));
DATA(insert OID = 1169 (  interval_pl	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_pl _null_ _null_ _null_ ));
DATA(insert OID = 1170 (  interval_mi	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_mi _null_ _null_ _null_ ));
DATA(insert OID = 1171 (  date_part		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 701 "25 1184" _null_ _null_ _null_ _null_ _null_ timestamptz_part _null_ _null_ _null_ ));
DESCR("extract field from timestamp with time zone");
DATA(insert OID = 1172 (  date_part		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "25 1186" _null_ _null_ _null_ _null_ _null_ interval_part _null_ _null_ _null_ ));
DESCR("extract field from interval");
DATA(insert OID = 1173 (  timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1184 "702" _null_ _null_ _null_ _null_ _null_ abstime_timestamptz _null_ _null_ _null_ ));
DESCR("convert abstime to timestamp with time zone");
DATA(insert OID = 1174 (  timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1184 "1082" _null_ _null_ _null_ _null_ _null_ date_timestamptz _null_ _null_ _null_ ));
DESCR("convert date to timestamp with time zone");
DATA(insert OID = 2711 (  justify_interval UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ interval_justify_interval _null_ _null_ _null_ ));
DESCR("promote groups of 24 hours to numbers of days and promote groups of 30 days to numbers of months");
DATA(insert OID = 1175 (  justify_hours    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ interval_justify_hours _null_ _null_ _null_ ));
DESCR("promote groups of 24 hours to numbers of days");
DATA(insert OID = 1295 (  justify_days	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ interval_justify_days _null_ _null_ _null_ ));
DESCR("promote groups of 30 days to numbers of months");
DATA(insert OID = 1176 (  timestamptz	   UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 1184 "1082 1083" _null_ _null_ _null_ _null_ _null_ "select cast(($1 + $2) as timestamp with time zone)" _null_ _null_ _null_ ));
DESCR("convert date and time to timestamp with time zone");
DATA(insert OID = 1177 (  interval		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "703" _null_ _null_ _null_ _null_ _null_ reltime_interval _null_ _null_ _null_ ));
DESCR("convert reltime to interval");
DATA(insert OID = 1178 (  date			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1082 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_date _null_ _null_ _null_ ));
DESCR("convert timestamp with time zone to date");
DATA(insert OID = 1179 (  date			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1082 "702" _null_ _null_ _null_ _null_ _null_ abstime_date _null_ _null_ _null_ ));
DESCR("convert abstime to date");
DATA(insert OID = 1180 (  abstime		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 702 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_abstime _null_ _null_ _null_ ));
DESCR("convert timestamp with time zone to abstime");
DATA(insert OID = 1181 (  age			   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 23 "28" _null_ _null_ _null_ _null_ _null_ xid_age _null_ _null_ _null_ ));
DESCR("age of a transaction ID, in transactions before current transaction");
DATA(insert OID = 3939 (  mxid_age		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 23 "28" _null_ _null_ _null_ _null_ _null_ mxid_age _null_ _null_ _null_ ));
DESCR("age of a multi-transaction ID, in multi-transactions before current multi-transaction");

DATA(insert OID = 1188 (  timestamptz_mi   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_mi _null_ _null_ _null_ ));
DATA(insert OID = 1189 (  timestamptz_pl_interval UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1184 "1184 1186" _null_ _null_ _null_ _null_ _null_ timestamptz_pl_interval _null_ _null_ _null_ ));
DATA(insert OID = 1190 (  timestamptz_mi_interval UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1184 "1184 1186" _null_ _null_ _null_ _null_ _null_ timestamptz_mi_interval _null_ _null_ _null_ ));
DATA(insert OID = 1194 (  reltime			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 703 "1186" _null_ _null_ _null_ _null_ _null_ interval_reltime _null_ _null_ _null_ ));
DESCR("convert interval to reltime");
DATA(insert OID = 1195 (  timestamptz_smaller UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1184 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1196 (  timestamptz_larger  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1184 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1197 (  interval_smaller	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 1186" _null_ _null_ _null_ _null_ _null_	interval_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1198 (  interval_larger	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 1186" _null_ _null_ _null_ _null_ _null_	interval_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1199 (  age				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1184 1184" _null_ _null_ _null_ _null_ _null_	timestamptz_age _null_ _null_ _null_ ));
DESCR("date difference preserving months and years");

/* OIDS 1200 - 1299 */

DATA(insert OID = 3918 (  interval_transform UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ interval_transform _null_ _null_ _null_ ));
DESCR("transform an interval length coercion");
DATA(insert OID = 1200 (  interval			UXNSP UXUID 12 1 0 0 interval_transform f f f f t f i s 2 0 1186 "1186 23" _null_ _null_ _null_ _null_ _null_ interval_scale _null_ _null_ _null_ ));
DESCR("adjust interval precision");

DATA(insert OID = 1215 (  obj_description	UXNSP UXUID 14 100 0 0 0 f f f f t f s s 2 0 25 "26 19" _null_ _null_ _null_ _null_ _null_ "select description from ux_catalog.ux_description where objoid = $1 and classoid = (select oid from ux_catalog.ux_class where relname = $2 and relnamespace = UXNSP) and objsubid = 0" _null_ _null_ _null_ ));
DESCR("get description for object id and catalog name");
DATA(insert OID = 1216 (  col_description	UXNSP UXUID 14 100 0 0 0 f f f f t f s s 2 0 25 "26 23" _null_ _null_ _null_ _null_ _null_ "select description from ux_catalog.ux_description where objoid = $1 and classoid = ''ux_catalog.ux_class''::ux_catalog.regclass and objsubid = $2" _null_ _null_ _null_ ));
DESCR("get description for table column");
DATA(insert OID = 1993 ( shobj_description	UXNSP UXUID 14 100 0 0 0 f f f f t f s s 2 0 25 "26 19" _null_ _null_ _null_ _null_ _null_ "select description from ux_catalog.ux_shdescription where objoid = $1 and classoid = (select oid from ux_catalog.ux_class where relname = $2 and relnamespace = UXNSP)" _null_ _null_ _null_ ));
DESCR("get description for object id and shared catalog name");

DATA(insert OID = 1217 (  date_trunc	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1184 "25 1184" _null_ _null_ _null_ _null_ _null_ timestamptz_trunc _null_ _null_ _null_ ));
DESCR("truncate timestamp with time zone to specified units");
DATA(insert OID = 1218 (  date_trunc	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "25 1186" _null_ _null_ _null_ _null_ _null_ interval_trunc _null_ _null_ _null_ ));
DESCR("truncate interval to specified units");

DATA(insert OID = 1219 (  int8inc		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8inc _null_ _null_ _null_ ));
DESCR("increment");
DATA(insert OID = 3546 (  int8dec		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8dec _null_ _null_ _null_ ));
DESCR("decrement");
DATA(insert OID = 2804 (  int8inc_any	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 2276" _null_ _null_ _null_ _null_ _null_ int8inc_any _null_ _null_ _null_ ));
DESCR("increment, ignores second argument");
DATA(insert OID = 3547 (  int8dec_any	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 2276" _null_ _null_ _null_ _null_ _null_ int8dec_any _null_ _null_ _null_ ));
DESCR("decrement, ignores second argument");
DATA(insert OID = 1230 (  int8abs		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8abs _null_ _null_ _null_ ));

DATA(insert OID = 1236 (  int8larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1237 (  int8smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 1238 (  texticregexeq    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ texticregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1239 (  texticregexne    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ texticregexne _null_ _null_ _null_ ));
DATA(insert OID = 1240 (  nameicregexeq    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameicregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1241 (  nameicregexne    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameicregexne _null_ _null_ _null_ ));

DATA(insert OID = 1251 (  int4abs		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4abs _null_ _null_ _null_ ));
DATA(insert OID = 1253 (  int2abs		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ int2abs _null_ _null_ _null_ ));

DATA(insert OID = 1271 (  overlaps		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 4 0 16 "1266 1266 1266 1266" _null_ _null_ _null_ _null_ _null_ overlaps_timetz _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1272 (  datetime_pl	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1082 1083" _null_ _null_ _null_ _null_ _null_ datetime_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 1273 (  date_part		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "25 1266" _null_ _null_ _null_ _null_ _null_ timetz_part _null_ _null_ _null_ ));
DESCR("extract field from time with time zone");
DATA(insert OID = 1274 (  int84pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int84pl _null_ _null_ _null_ ));
DATA(insert OID = 1275 (  int84mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int84mi _null_ _null_ _null_ ));
DATA(insert OID = 1276 (  int84mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int84mul _null_ _null_ _null_ ));
DATA(insert OID = 1277 (  int84div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int84div _null_ _null_ _null_ ));
DATA(insert OID = 1278 (  int48pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "23 20" _null_ _null_ _null_ _null_ _null_ int48pl _null_ _null_ _null_ ));
DATA(insert OID = 1279 (  int48mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "23 20" _null_ _null_ _null_ _null_ _null_ int48mi _null_ _null_ _null_ ));
DATA(insert OID = 1280 (  int48mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "23 20" _null_ _null_ _null_ _null_ _null_ int48mul _null_ _null_ _null_ ));
DATA(insert OID = 1281 (  int48div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "23 20" _null_ _null_ _null_ _null_ _null_ int48div _null_ _null_ _null_ ));

DATA(insert OID =  837 (  int82pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 21" _null_ _null_ _null_ _null_ _null_ int82pl _null_ _null_ _null_ ));
DATA(insert OID =  838 (  int82mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 21" _null_ _null_ _null_ _null_ _null_ int82mi _null_ _null_ _null_ ));
DATA(insert OID =  839 (  int82mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 21" _null_ _null_ _null_ _null_ _null_ int82mul _null_ _null_ _null_ ));
DATA(insert OID =  840 (  int82div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 21" _null_ _null_ _null_ _null_ _null_ int82div _null_ _null_ _null_ ));
DATA(insert OID =  841 (  int28pl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "21 20" _null_ _null_ _null_ _null_ _null_ int28pl _null_ _null_ _null_ ));
DATA(insert OID =  942 (  int28mi		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "21 20" _null_ _null_ _null_ _null_ _null_ int28mi _null_ _null_ _null_ ));
DATA(insert OID =  943 (  int28mul		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "21 20" _null_ _null_ _null_ _null_ _null_ int28mul _null_ _null_ _null_ ));
DATA(insert OID =  948 (  int28div		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "21 20" _null_ _null_ _null_ _null_ _null_ int28div _null_ _null_ _null_ ));

DATA(insert OID = 1287 (  oid			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 26 "20" _null_ _null_ _null_ _null_ _null_ i8tooid _null_ _null_ _null_ ));
DESCR("convert int8 to oid");
DATA(insert OID = 1288 (  int8			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ oidtoi8 _null_ _null_ _null_ ));
DESCR("convert oid to int8");

DATA(insert OID = 1291 (  suppress_redundant_updates_trigger	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ suppress_redundant_updates_trigger _null_ _null_ _null_ ));
DESCR("trigger to suppress updates when new and old records match");

DATA(insert OID = 1292 ( tideq			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tideq _null_ _null_ _null_ ));
DATA(insert OID = 1293 ( currtid		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 27 "26 27" _null_ _null_ _null_ _null_ _null_ currtid_byreloid _null_ _null_ _null_ ));
DESCR("latest tid of a tuple");
DATA(insert OID = 1294 ( currtid2		   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 27 "25 27" _null_ _null_ _null_ _null_ _null_ currtid_byrelname _null_ _null_ _null_ ));
DESCR("latest tid of a tuple");
DATA(insert OID = 1265 ( tidne			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tidne _null_ _null_ _null_ ));
DATA(insert OID = 2790 ( tidgt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tidgt _null_ _null_ _null_ ));
DATA(insert OID = 2791 ( tidlt			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tidlt _null_ _null_ _null_ ));
DATA(insert OID = 2792 ( tidge			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tidge _null_ _null_ _null_ ));
DATA(insert OID = 2793 ( tidle			   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "27 27" _null_ _null_ _null_ _null_ _null_ tidle _null_ _null_ _null_ ));
DATA(insert OID = 2794 ( bttidcmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "27 27" _null_ _null_ _null_ _null_ _null_ bttidcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2795 ( tidlarger		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 27 "27 27" _null_ _null_ _null_ _null_ _null_ tidlarger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 2796 ( tidsmaller		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 27 "27 27" _null_ _null_ _null_ _null_ _null_ tidsmaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 1296 (  timedate_pl	   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1114 "1083 1082" _null_ _null_ _null_ _null_ _null_ "select ($2 + $1)" _null_ _null_ _null_ ));
DATA(insert OID = 1297 (  datetimetz_pl    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1184 "1082 1266" _null_ _null_ _null_ _null_ _null_ datetimetz_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 1298 (  timetzdate_pl    UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1184 "1266 1082" _null_ _null_ _null_ _null_ _null_ "select ($2 + $1)" _null_ _null_ _null_ ));
DATA(insert OID = 1299 (  now			   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ now _null_ _null_ _null_ ));
DESCR("current transaction time");
DATA(insert OID = 2647 (  transaction_timestamp UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ now _null_ _null_ _null_ ));
DESCR("current transaction time");
DATA(insert OID = 2648 (  statement_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ statement_timestamp _null_ _null_ _null_ ));
DESCR("current statement time");
DATA(insert OID = 2649 (  clock_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ clock_timestamp _null_ _null_ _null_ ));
DESCR("current clock time");

/* OIDS 1300 - 1399 */

DATA(insert OID = 1300 (  positionsel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	positionsel _null_ _null_ _null_ ));
DESCR("restriction selectivity for position-comparison operators");
DATA(insert OID = 1301 (  positionjoinsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ positionjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity for position-comparison operators");
DATA(insert OID = 1302 (  contsel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	contsel _null_ _null_ _null_ ));
DESCR("restriction selectivity for containment comparison operators");
DATA(insert OID = 1303 (  contjoinsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ contjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity for containment comparison operators");

DATA(insert OID = 1304 ( overlaps			 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 4 0 16 "1184 1184 1184 1184" _null_ _null_ _null_ _null_ _null_ overlaps_timestamp _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1305 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f s s 4 0 16 "1184 1186 1184 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1306 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f s s 4 0 16 "1184 1184 1184 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, $2) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1307 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f s s 4 0 16 "1184 1186 1184 1184" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, $4)" _null_ _null_ _null_ ));
DESCR("intervals overlap?");

DATA(insert OID = 1308 ( overlaps			 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 4 0 16 "1083 1083 1083 1083" _null_ _null_ _null_ _null_ _null_ overlaps_time _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1309 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1083 1186 1083 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1310 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1083 1083 1083 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, $2) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 1311 ( overlaps			 UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1083 1186 1083 1083" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, $4)" _null_ _null_ _null_ ));
DESCR("intervals overlap?");

DATA(insert OID = 1312 (  timestamp_in		 UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1114 "2275 26 23" _null_ _null_ _null_ _null_ _null_ timestamp_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1313 (  timestamp_out		 UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2905 (  timestamptypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ timestamptypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2906 (  timestamptypmodout	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ timestamptypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1314 (  timestamptz_cmp	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1184 1184" _null_ _null_ _null_ _null_ _null_ timestamp_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 1315 (  interval_cmp		 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1186 1186" _null_ _null_ _null_ _null_ _null_ interval_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 1316 (  time				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1083 "1114" _null_ _null_ _null_ _null_ _null_	timestamp_time _null_ _null_ _null_ ));
DESCR("convert timestamp to time");

DATA(insert OID = 1317 (  length			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_	textlen _null_ _null_ _null_ ));
DESCR("length");
DATA(insert OID = 1318 (  length			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1042" _null_ _null_ _null_ _null_ _null_ bpcharlen _null_ _null_ _null_ ));
DESCR("character length");

DATA(insert OID = 1319 (  xideqint4			 UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "28 23" _null_ _null_ _null_ _null_ _null_ xideq _null_ _null_ _null_ ));
DATA(insert OID = 3309 (  xidneqint4		 UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "28 23" _null_ _null_ _null_ _null_ _null_ xidneq _null_ _null_ _null_ ));

DATA(insert OID = 1326 (  interval_div		 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 701" _null_ _null_ _null_ _null_ _null_	interval_div _null_ _null_ _null_ ));

DATA(insert OID = 1339 (  dlog10			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dlog10 _null_ _null_ _null_ ));
DESCR("base 10 logarithm");
DATA(insert OID = 1340 (  log				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dlog10 _null_ _null_ _null_ ));
DESCR("base 10 logarithm");
DATA(insert OID = 1341 (  ln				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dlog1 _null_ _null_ _null_ ));
DESCR("natural logarithm");
DATA(insert OID = 1342 (  round				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dround _null_ _null_ _null_ ));
DESCR("round to nearest integer");
DATA(insert OID = 1343 (  trunc				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dtrunc _null_ _null_ _null_ ));
DESCR("truncate to integer");
DATA(insert OID = 1344 (  sqrt				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dsqrt _null_ _null_ _null_ ));
DESCR("square root");
DATA(insert OID = 1345 (  cbrt				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dcbrt _null_ _null_ _null_ ));
DESCR("cube root");
DATA(insert OID = 1346 (  pow				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ dpow _null_ _null_ _null_ ));
DESCR("exponentiation");
DATA(insert OID = 1368 (  power				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ dpow _null_ _null_ _null_ ));
DESCR("exponentiation");
DATA(insert OID = 1347 (  exp				 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dexp _null_ _null_ _null_ ));
DESCR("natural exponential (e^x)");

/*
 * This form of obj_description is now deprecated, since it will fail if
 * OIDs are not unique across system catalogs.  Use the other form instead.
 */
DATA(insert OID = 1348 (  obj_description	 UXNSP UXUID 14 100 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ "select description from ux_catalog.ux_description where objoid = $1 and objsubid = 0" _null_ _null_ _null_ ));
DESCR("deprecated, use two-argument form instead");
DATA(insert OID = 1349 (  oidvectortypes	 UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "30" _null_ _null_ _null_ _null_ _null_	oidvectortypes _null_ _null_ _null_ ));
DESCR("print type names of oidvector field");


DATA(insert OID = 1350 (  timetz_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1266 "2275 26 23" _null_ _null_ _null_ _null_ _null_ timetz_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1351 (  timetz_out	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1266" _null_ _null_ _null_ _null_ _null_ timetz_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2911 (  timetztypmodin	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ timetztypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2912 (  timetztypmodout	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ timetztypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 1352 (  timetz_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_eq _null_ _null_ _null_ ));
DATA(insert OID = 1353 (  timetz_ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_ne _null_ _null_ _null_ ));
DATA(insert OID = 1354 (  timetz_lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_lt _null_ _null_ _null_ ));
DATA(insert OID = 1355 (  timetz_le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_le _null_ _null_ _null_ ));
DATA(insert OID = 1356 (  timetz_ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_ge _null_ _null_ _null_ ));
DATA(insert OID = 1357 (  timetz_gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_gt _null_ _null_ _null_ ));
DATA(insert OID = 1358 (  timetz_cmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 1359 (  timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1184 "1082 1266" _null_ _null_ _null_ _null_ _null_ datetimetz_timestamptz _null_ _null_ _null_ ));
DESCR("convert date and time with time zone to timestamp with time zone");

DATA(insert OID = 1364 (  time			   UXNSP UXUID 14 1 0 0 0 f f f f t f s s 1 0 1083 "702" _null_ _null_ _null_ _null_ _null_ "select cast(cast($1 as timestamp without time zone) as ux_catalog.time)" _null_ _null_ _null_ ));
DESCR("convert abstime to time");

DATA(insert OID = 1367 (  character_length	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1042" _null_ _null_ _null_ _null_ _null_ bpcharlen _null_ _null_ _null_ ));
DESCR("character length");
DATA(insert OID = 1369 (  character_length	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ textlen _null_ _null_ _null_ ));
DESCR("character length");

DATA(insert OID = 1370 (  interval			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1083" _null_ _null_ _null_ _null_ _null_	time_interval _null_ _null_ _null_ ));
DESCR("convert time to interval");
DATA(insert OID = 1372 (  char_length		 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1042" _null_ _null_ _null_ _null_ _null_ bpcharlen _null_ _null_ _null_ ));
DESCR("character length");
DATA(insert OID = 1374 (  octet_length			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_	textoctetlen _null_ _null_ _null_ ));
DESCR("octet length");
DATA(insert OID = 1375 (  octet_length			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1042" _null_ _null_ _null_ _null_ _null_ bpcharoctetlen _null_ _null_ _null_ ));
DESCR("octet length");

DATA(insert OID = 1377 (  time_larger	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1083 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1378 (  time_smaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1083 "1083 1083" _null_ _null_ _null_ _null_ _null_ time_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1379 (  timetz_larger    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1266 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1380 (  timetz_smaller   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1266 "1266 1266" _null_ _null_ _null_ _null_ _null_ timetz_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 1381 (  char_length	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ textlen _null_ _null_ _null_ ));
DESCR("character length");

DATA(insert OID = 1382 (  date_part    UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 701 "25 702" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.date_part($1, cast($2 as timestamp with time zone))" _null_ _null_ _null_ ));
DESCR("extract field from abstime");
DATA(insert OID = 1383 (  date_part    UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 701 "25 703" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.date_part($1, cast($2 as ux_catalog.interval))" _null_ _null_ _null_ ));
DESCR("extract field from reltime");
DATA(insert OID = 1384 (  date_part    UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 701 "25 1082" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.date_part($1, cast($2 as timestamp without time zone))" _null_ _null_ _null_ ));
DESCR("extract field from date");
DATA(insert OID = 1385 (  date_part    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "25 1083" _null_ _null_ _null_ _null_  _null_ time_part _null_ _null_ _null_ ));
DESCR("extract field from time");
DATA(insert OID = 1386 (  age		   UXNSP UXUID 14 1 0 0 0 f f f f t f s s 1 0 1186 "1184" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.age(cast(current_date as timestamp with time zone), $1)" _null_ _null_ _null_ ));
DESCR("date difference from today preserving months and years");

DATA(insert OID = 1388 (  timetz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1266 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_timetz _null_ _null_ _null_ ));
DESCR("convert timestamp with time zone to time with time zone");

DATA(insert OID = 1373 (  isfinite	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "1082" _null_ _null_ _null_ _null_ _null_	date_finite _null_ _null_ _null_ ));
DESCR("finite date?");
DATA(insert OID = 1389 (  isfinite	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "1184" _null_ _null_ _null_ _null_ _null_	timestamp_finite _null_ _null_ _null_ ));
DESCR("finite timestamp?");
DATA(insert OID = 1390 (  isfinite	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "1186" _null_ _null_ _null_ _null_ _null_	interval_finite _null_ _null_ _null_ ));
DESCR("finite interval?");


DATA(insert OID = 1376 (  factorial		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_	numeric_fac _null_ _null_ _null_ ));
DESCR("factorial");
DATA(insert OID = 1394 (  abs			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_	float4abs _null_ _null_ _null_ ));
DESCR("absolute value");
DATA(insert OID = 1395 (  abs			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_	float8abs _null_ _null_ _null_ ));
DESCR("absolute value");
DATA(insert OID = 1396 (  abs			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8abs _null_ _null_ _null_ ));
DESCR("absolute value");
DATA(insert OID = 1397 (  abs			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4abs _null_ _null_ _null_ ));
DESCR("absolute value");
DATA(insert OID = 1398 (  abs			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ int2abs _null_ _null_ _null_ ));
DESCR("absolute value");

/* OIDS 1400 - 1499 */

DATA(insert OID = 1400 (  name		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 19 "1043" _null_ _null_ _null_ _null_ _null_	text_name _null_ _null_ _null_ ));
DESCR("convert varchar to name");
DATA(insert OID = 1401 (  varchar	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1043 "19" _null_ _null_ _null_ _null_ _null_	name_text _null_ _null_ _null_ ));
DESCR("convert name to varchar");

DATA(insert OID = 1402 (  current_schema	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ current_schema _null_ _null_ _null_ ));
DESCR("current schema name");
DATA(insert OID = 1403 (  current_schemas	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1003 "16" _null_ _null_ _null_ _null_ _null_ current_schemas _null_ _null_ _null_ ));
DESCR("current schema search list");

DATA(insert OID = 1404 (  overlay			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 25 "25 25 23 23" _null_ _null_ _null_ _null_ _null_	textoverlay _null_ _null_ _null_ ));
DESCR("substitute portion of string");
DATA(insert OID = 1405 (  overlay			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 25 23" _null_ _null_ _null_ _null_ _null_ textoverlay_no_len _null_ _null_ _null_ ));
DESCR("substitute portion of string");

DATA(insert OID = 1406 (  isvertical		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_  _null_ point_vert _null_ _null_ _null_ ));
DESCR("vertically aligned");
DATA(insert OID = 1407 (  ishorizontal		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 600" _null_ _null_ _null_ _null_  _null_ point_horiz _null_ _null_ _null_ ));
DESCR("horizontally aligned");
DATA(insert OID = 1408 (  isparallel		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_parallel _null_ _null_ _null_ ));
DESCR("parallel");
DATA(insert OID = 1409 (  isperp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_perp _null_ _null_ _null_ ));
DESCR("perpendicular");
DATA(insert OID = 1410 (  isvertical		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "601" _null_ _null_ _null_ _null_  _null_ lseg_vertical _null_ _null_ _null_ ));
DESCR("vertical");
DATA(insert OID = 1411 (  ishorizontal		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "601" _null_ _null_ _null_ _null_  _null_ lseg_horizontal _null_ _null_ _null_ ));
DESCR("horizontal");
DATA(insert OID = 1412 (  isparallel		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_  _null_ line_parallel _null_ _null_ _null_ ));
DESCR("parallel");
DATA(insert OID = 1413 (  isperp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_  _null_ line_perp _null_ _null_ _null_ ));
DESCR("perpendicular");
DATA(insert OID = 1414 (  isvertical		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "628" _null_ _null_ _null_ _null_  _null_ line_vertical _null_ _null_ _null_ ));
DESCR("vertical");
DATA(insert OID = 1415 (  ishorizontal		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "628" _null_ _null_ _null_ _null_  _null_ line_horizontal _null_ _null_ _null_ ));
DESCR("horizontal");
DATA(insert OID = 1416 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "718" _null_ _null_ _null_ _null_ _null_ circle_center _null_ _null_ _null_ ));
DESCR("center of");

DATA(insert OID = 1419 (  time				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1083 "1186" _null_ _null_ _null_ _null_ _null_ interval_time _null_ _null_ _null_ ));
DESCR("convert interval to time");

DATA(insert OID = 1421 (  box				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "600 600" _null_ _null_ _null_ _null_ _null_ points_box _null_ _null_ _null_ ));
DESCR("convert points to box");
DATA(insert OID = 1422 (  box_add			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 600" _null_ _null_ _null_ _null_ _null_ box_add _null_ _null_ _null_ ));
DATA(insert OID = 1423 (  box_sub			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 600" _null_ _null_ _null_ _null_ _null_ box_sub _null_ _null_ _null_ ));
DATA(insert OID = 1424 (  box_mul			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 600" _null_ _null_ _null_ _null_ _null_ box_mul _null_ _null_ _null_ ));
DATA(insert OID = 1425 (  box_div			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "603 600" _null_ _null_ _null_ _null_ _null_ box_div _null_ _null_ _null_ ));
DATA(insert OID = 1426 (  path_contain_pt	UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 16 "602 600" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.on_ppath($2, $1)" _null_ _null_ _null_ ));
DATA(insert OID = 1428 (  poly_contain_pt	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 600" _null_ _null_ _null_ _null_  _null_ poly_contain_pt _null_ _null_ _null_ ));
DATA(insert OID = 1429 (  pt_contained_poly UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 604" _null_ _null_ _null_ _null_  _null_ pt_contained_poly _null_ _null_ _null_ ));

DATA(insert OID = 1430 (  isclosed			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "602" _null_ _null_ _null_ _null_  _null_ path_isclosed _null_ _null_ _null_ ));
DESCR("path closed?");
DATA(insert OID = 1431 (  isopen			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "602" _null_ _null_ _null_ _null_  _null_ path_isopen _null_ _null_ _null_ ));
DESCR("path open?");
DATA(insert OID = 1432 (  path_npoints		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "602" _null_ _null_ _null_ _null_  _null_ path_npoints _null_ _null_ _null_ ));

/* pclose and popen might better be named close and open, but that crashes initdb.
 * - thomas 97/04/20
 */

DATA(insert OID = 1433 (  pclose			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 602 "602" _null_ _null_ _null_ _null_ _null_ path_close _null_ _null_ _null_ ));
DESCR("close path");
DATA(insert OID = 1434 (  popen				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 602 "602" _null_ _null_ _null_ _null_ _null_ path_open _null_ _null_ _null_ ));
DESCR("open path");
DATA(insert OID = 1435 (  path_add			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 602 "602 602" _null_ _null_ _null_ _null_ _null_ path_add _null_ _null_ _null_ ));
DATA(insert OID = 1436 (  path_add_pt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 602 "602 600" _null_ _null_ _null_ _null_ _null_ path_add_pt _null_ _null_ _null_ ));
DATA(insert OID = 1437 (  path_sub_pt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 602 "602 600" _null_ _null_ _null_ _null_ _null_ path_sub_pt _null_ _null_ _null_ ));
DATA(insert OID = 1438 (  path_mul_pt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 602 "602 600" _null_ _null_ _null_ _null_ _null_ path_mul_pt _null_ _null_ _null_ ));
DATA(insert OID = 1439 (  path_div_pt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 602 "602 600" _null_ _null_ _null_ _null_ _null_ path_div_pt _null_ _null_ _null_ ));

DATA(insert OID = 1440 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "701 701" _null_ _null_ _null_ _null_ _null_ construct_point _null_ _null_ _null_ ));
DESCR("convert x, y to point");
DATA(insert OID = 1441 (  point_add			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 600" _null_ _null_ _null_ _null_ _null_ point_add _null_ _null_ _null_ ));
DATA(insert OID = 1442 (  point_sub			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 600" _null_ _null_ _null_ _null_ _null_ point_sub _null_ _null_ _null_ ));
DATA(insert OID = 1443 (  point_mul			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 600" _null_ _null_ _null_ _null_ _null_ point_mul _null_ _null_ _null_ ));
DATA(insert OID = 1444 (  point_div			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "600 600" _null_ _null_ _null_ _null_ _null_ point_div _null_ _null_ _null_ ));

DATA(insert OID = 1445 (  poly_npoints		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "604" _null_ _null_ _null_ _null_  _null_ poly_npoints _null_ _null_ _null_ ));
DATA(insert OID = 1446 (  box				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 603 "604" _null_ _null_ _null_ _null_ _null_ poly_box _null_ _null_ _null_ ));
DESCR("convert polygon to bounding box");
DATA(insert OID = 1447 (  path				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 602 "604" _null_ _null_ _null_ _null_ _null_ poly_path _null_ _null_ _null_ ));
DESCR("convert polygon to path");
DATA(insert OID = 1448 (  polygon			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 604 "603" _null_ _null_ _null_ _null_ _null_ box_poly _null_ _null_ _null_ ));
DESCR("convert box to polygon");
DATA(insert OID = 1449 (  polygon			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 604 "602" _null_ _null_ _null_ _null_ _null_ path_poly _null_ _null_ _null_ ));
DESCR("convert path to polygon");

DATA(insert OID = 1450 (  circle_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 718 "2275" _null_ _null_ _null_ _null_ _null_ circle_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1451 (  circle_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "718" _null_ _null_ _null_ _null_ _null_ circle_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1452 (  circle_same		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_same _null_ _null_ _null_ ));
DATA(insert OID = 1453 (  circle_contain	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_contain _null_ _null_ _null_ ));
DATA(insert OID = 1454 (  circle_left		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_left _null_ _null_ _null_ ));
DATA(insert OID = 1455 (  circle_overleft	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_overleft _null_ _null_ _null_ ));
DATA(insert OID = 1456 (  circle_overright	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_overright _null_ _null_ _null_ ));
DATA(insert OID = 1457 (  circle_right		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_right _null_ _null_ _null_ ));
DATA(insert OID = 1458 (  circle_contained	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_contained _null_ _null_ _null_ ));
DATA(insert OID = 1459 (  circle_overlap	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_overlap _null_ _null_ _null_ ));
DATA(insert OID = 1460 (  circle_below		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_below _null_ _null_ _null_ ));
DATA(insert OID = 1461 (  circle_above		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_above _null_ _null_ _null_ ));
DATA(insert OID = 1462 (  circle_eq			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_eq _null_ _null_ _null_ ));
DATA(insert OID = 1463 (  circle_ne			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_ne _null_ _null_ _null_ ));
DATA(insert OID = 1464 (  circle_lt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_lt _null_ _null_ _null_ ));
DATA(insert OID = 1465 (  circle_gt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_gt _null_ _null_ _null_ ));
DATA(insert OID = 1466 (  circle_le			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_le _null_ _null_ _null_ ));
DATA(insert OID = 1467 (  circle_ge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_ge _null_ _null_ _null_ ));
DATA(insert OID = 1468 (  area				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "718" _null_ _null_ _null_ _null_ _null_ circle_area _null_ _null_ _null_ ));
DESCR("area of circle");
DATA(insert OID = 1469 (  diameter			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "718" _null_ _null_ _null_ _null_ _null_ circle_diameter _null_ _null_ _null_ ));
DESCR("diameter of circle");
DATA(insert OID = 1470 (  radius			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "718" _null_ _null_ _null_ _null_ _null_ circle_radius _null_ _null_ _null_ ));
DESCR("radius of circle");
DATA(insert OID = 1471 (  circle_distance	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "718 718" _null_ _null_ _null_ _null_ _null_ circle_distance _null_ _null_ _null_ ));
DATA(insert OID = 1472 (  circle_center		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "718" _null_ _null_ _null_ _null_ _null_ circle_center _null_ _null_ _null_ ));
DATA(insert OID = 1473 (  circle			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 718 "600 701" _null_ _null_ _null_ _null_ _null_ cr_circle _null_ _null_ _null_ ));
DESCR("convert point and radius to circle");
DATA(insert OID = 1474 (  circle			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 718 "604" _null_ _null_ _null_ _null_ _null_ poly_circle _null_ _null_ _null_ ));
DESCR("convert polygon to circle");
DATA(insert OID = 1475 (  polygon			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 604 "23 718" _null_ _null_ _null_ _null_ _null_	circle_poly _null_ _null_ _null_ ));
DESCR("convert vertex count and circle to polygon");
DATA(insert OID = 1476 (  dist_pc			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "600 718" _null_ _null_ _null_ _null_ _null_ dist_pc _null_ _null_ _null_ ));
DATA(insert OID = 1477 (  circle_contain_pt UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 600" _null_ _null_ _null_ _null_  _null_ circle_contain_pt _null_ _null_ _null_ ));
DATA(insert OID = 1478 (  pt_contained_circle	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "600 718" _null_ _null_ _null_ _null_  _null_ pt_contained_circle _null_ _null_ _null_ ));
DATA(insert OID = 4091 (  box				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 603 "600" _null_ _null_ _null_ _null_ _null_ point_box _null_ _null_ _null_ ));
DESCR("convert point to empty box");
DATA(insert OID = 1479 (  circle			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 718 "603" _null_ _null_ _null_ _null_ _null_ box_circle _null_ _null_ _null_ ));
DESCR("convert box to circle");
DATA(insert OID = 1480 (  box				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 603 "718" _null_ _null_ _null_ _null_ _null_ circle_box _null_ _null_ _null_ ));
DESCR("convert circle to box");
DATA(insert OID = 1481 (  tinterval			 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 704 "702 702" _null_ _null_ _null_ _null_ _null_ mktinterval _null_ _null_ _null_ ));
DESCR("convert to tinterval");

DATA(insert OID = 1482 (  lseg_ne			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_ne _null_ _null_ _null_ ));
DATA(insert OID = 1483 (  lseg_lt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_lt _null_ _null_ _null_ ));
DATA(insert OID = 1484 (  lseg_le			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_le _null_ _null_ _null_ ));
DATA(insert OID = 1485 (  lseg_gt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_gt _null_ _null_ _null_ ));
DATA(insert OID = 1486 (  lseg_ge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "601 601" _null_ _null_ _null_ _null_  _null_ lseg_ge _null_ _null_ _null_ ));
DATA(insert OID = 1487 (  lseg_length		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "601" _null_ _null_ _null_ _null_ _null_ lseg_length _null_ _null_ _null_ ));
DATA(insert OID = 1488 (  close_ls			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "628 601" _null_ _null_ _null_ _null_ _null_ close_ls _null_ _null_ _null_ ));
DATA(insert OID = 1489 (  close_lseg		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "601 601" _null_ _null_ _null_ _null_ _null_ close_lseg _null_ _null_ _null_ ));

DATA(insert OID = 1490 (  line_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 628 "2275" _null_ _null_ _null_ _null_ _null_ line_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1491 (  line_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "628" _null_ _null_ _null_ _null_ _null_ line_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1492 (  line_eq			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_ _null_ line_eq _null_ _null_ _null_ ));
DATA(insert OID = 1493 (  line				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 628 "600 600" _null_ _null_ _null_ _null_ _null_ line_construct_pp _null_ _null_ _null_ ));
DESCR("construct line from points");
DATA(insert OID = 1494 (  line_interpt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 600 "628 628" _null_ _null_ _null_ _null_ _null_ line_interpt _null_ _null_ _null_ ));
DATA(insert OID = 1495 (  line_intersect	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_  _null_ line_intersect _null_ _null_ _null_ ));
DATA(insert OID = 1496 (  line_parallel		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_  _null_ line_parallel _null_ _null_ _null_ ));
DATA(insert OID = 1497 (  line_perp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "628 628" _null_ _null_ _null_ _null_  _null_ line_perp _null_ _null_ _null_ ));
DATA(insert OID = 1498 (  line_vertical		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "628" _null_ _null_ _null_ _null_  _null_ line_vertical _null_ _null_ _null_ ));
DATA(insert OID = 1499 (  line_horizontal	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "628" _null_ _null_ _null_ _null_  _null_ line_horizontal _null_ _null_ _null_ ));

/* OIDS 1500 - 1599 */

DATA(insert OID = 1530 (  length			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "601" _null_ _null_ _null_ _null_ _null_ lseg_length _null_ _null_ _null_ ));
DESCR("distance between endpoints");
DATA(insert OID = 1531 (  length			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "602" _null_ _null_ _null_ _null_ _null_ path_length _null_ _null_ _null_ ));
DESCR("sum of path segments");


DATA(insert OID = 1532 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "601" _null_ _null_ _null_ _null_ _null_ lseg_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1533 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "602" _null_ _null_ _null_ _null_ _null_ path_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1534 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "603" _null_ _null_ _null_ _null_ _null_ box_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1540 (  point				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "604" _null_ _null_ _null_ _null_ _null_ poly_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1541 (  lseg				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 601 "603" _null_ _null_ _null_ _null_ _null_ box_diagonal _null_ _null_ _null_ ));
DESCR("diagonal of");
DATA(insert OID = 1542 (  center			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "603" _null_ _null_ _null_ _null_ _null_ box_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1543 (  center			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "718" _null_ _null_ _null_ _null_ _null_ circle_center _null_ _null_ _null_ ));
DESCR("center of");
DATA(insert OID = 1544 (  polygon			UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 604 "718" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.polygon(12, $1)" _null_ _null_ _null_ ));
DESCR("convert circle to 12-vertex polygon");
DATA(insert OID = 1545 (  npoints			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "602" _null_ _null_ _null_ _null_  _null_ path_npoints _null_ _null_ _null_ ));
DESCR("number of points");
DATA(insert OID = 1556 (  npoints			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "604" _null_ _null_ _null_ _null_  _null_ poly_npoints _null_ _null_ _null_ ));
DESCR("number of points");

DATA(insert OID = 1564 (  bit_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "2275 26 23" _null_ _null_ _null_ _null_ _null_ bit_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1565 (  bit_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1560" _null_ _null_ _null_ _null_ _null_ bit_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2919 (  bittypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ bittypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2920 (  bittypmodout		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ bittypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");

DATA(insert OID = 1569 (  like				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textlike _null_ _null_ _null_ ));
DESCR("matches LIKE expression");
DATA(insert OID = 1570 (  notlike			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ textnlike _null_ _null_ _null_ ));
DESCR("does not match LIKE expression");
DATA(insert OID = 1571 (  like				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ namelike _null_ _null_ _null_ ));
DESCR("matches LIKE expression");
DATA(insert OID = 1572 (  notlike			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ namenlike _null_ _null_ _null_ ));
DESCR("does not match LIKE expression");


/* SEQUENCE functions */
DATA(insert OID = 1574 (  nextval			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ nextval_oid _null_ _null_ _null_ ));
DESCR("sequence next value");
DATA(insert OID = 1575 (  currval			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ currval_oid _null_ _null_ _null_ ));
DESCR("sequence current value");
DATA(insert OID = 1576 (  setval			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 20 "2205 20" _null_ _null_ _null_ _null_  _null_ setval_oid _null_ _null_ _null_ ));
DESCR("set sequence value");
DATA(insert OID = 1765 (  setval			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 20 "2205 20 16" _null_ _null_ _null_ _null_ _null_ setval3_oid _null_ _null_ _null_ ));
DESCR("set sequence value and is_called status");
DATA(insert OID = 3078 (  ux_sequence_parameters	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2249 "26" "{26,20,20,20,20,16,20,26}" "{i,o,o,o,o,o,o,o}" "{sequence_oid,start_value,minimum_value,maximum_value,increment,cycle_option,cache_size,data_type}" _null_ _null_ ux_sequence_parameters _null_ _null_ _null_));
DESCR("sequence parameters, for use by information schema");
DATA(insert OID = 4032 ( ux_sequence_last_value		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ ux_sequence_last_value _null_ _null_ _null_ ));
DESCR("sequence last value");

DATA(insert OID = 1579 (  varbit_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1562 "2275 26 23" _null_ _null_ _null_ _null_ _null_ varbit_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1580 (  varbit_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1562" _null_ _null_ _null_ _null_ _null_ varbit_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2902 (  varbittypmodin	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ varbittypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2921 (  varbittypmodout	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ varbittypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");

DATA(insert OID = 1581 (  biteq				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ biteq _null_ _null_ _null_ ));
DATA(insert OID = 1582 (  bitne				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitne _null_ _null_ _null_ ));
DATA(insert OID = 1592 (  bitge				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitge _null_ _null_ _null_ ));
DATA(insert OID = 1593 (  bitgt				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitgt _null_ _null_ _null_ ));
DATA(insert OID = 1594 (  bitle				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitle _null_ _null_ _null_ ));
DATA(insert OID = 1595 (  bitlt				UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitlt _null_ _null_ _null_ ));
DATA(insert OID = 1596 (  bitcmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 1598 (  random			UXNSP UXUID 12 1 0 0 0 f f f f t f v r 0 0 701 "" _null_ _null_ _null_ _null_ _null_ drandom _null_ _null_ _null_ ));
DESCR("random value");
DATA(insert OID = 1599 (  setseed			UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 2278 "701" _null_ _null_ _null_ _null_ _null_ setseed _null_ _null_ _null_ ));
DESCR("set random seed");

/* OIDS 1600 - 1699 */

DATA(insert OID = 1600 (  asin				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dasin _null_ _null_ _null_ ));
DESCR("arcsine");
DATA(insert OID = 1601 (  acos				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dacos _null_ _null_ _null_ ));
DESCR("arccosine");
DATA(insert OID = 1602 (  atan				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ datan _null_ _null_ _null_ ));
DESCR("arctangent");
DATA(insert OID = 1603 (  atan2				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ datan2 _null_ _null_ _null_ ));
DESCR("arctangent, two arguments");
DATA(insert OID = 1604 (  sin				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dsin _null_ _null_ _null_ ));
DESCR("sine");
DATA(insert OID = 1605 (  cos				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dcos _null_ _null_ _null_ ));
DESCR("cosine");
DATA(insert OID = 1606 (  tan				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dtan _null_ _null_ _null_ ));
DESCR("tangent");
DATA(insert OID = 1607 (  cot				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dcot _null_ _null_ _null_ ));
DESCR("cotangent");

DATA(insert OID = 2731 (  asind				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dasind _null_ _null_ _null_ ));
DESCR("arcsine, degrees");
DATA(insert OID = 2732 (  acosd				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dacosd _null_ _null_ _null_ ));
DESCR("arccosine, degrees");
DATA(insert OID = 2733 (  atand				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ datand _null_ _null_ _null_ ));
DESCR("arctangent, degrees");
DATA(insert OID = 2734 (  atan2d			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ datan2d _null_ _null_ _null_ ));
DESCR("arctangent, two arguments, degrees");
DATA(insert OID = 2735 (  sind				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dsind _null_ _null_ _null_ ));
DESCR("sine, degrees");
DATA(insert OID = 2736 (  cosd				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dcosd _null_ _null_ _null_ ));
DESCR("cosine, degrees");
DATA(insert OID = 2737 (  tand				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dtand _null_ _null_ _null_ ));
DESCR("tangent, degrees");
DATA(insert OID = 2738 (  cotd				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ dcotd _null_ _null_ _null_ ));
DESCR("cotangent, degrees");

DATA(insert OID = 1608 (  degrees			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ degrees _null_ _null_ _null_ ));
DESCR("radians to degrees");
DATA(insert OID = 1609 (  radians			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ radians _null_ _null_ _null_ ));
DESCR("degrees to radians");
DATA(insert OID = 1610 (  pi				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 0 0 701 "" _null_ _null_ _null_ _null_ _null_ dpi _null_ _null_ _null_ ));
DESCR("PI");

DATA(insert OID = 1618 (  interval_mul		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1186 701" _null_ _null_ _null_ _null_ _null_ interval_mul _null_ _null_ _null_ ));

DATA(insert OID = 1620 (  ascii				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ ascii _null_ _null_ _null_ ));
DESCR("convert first char to int4");
DATA(insert OID = 1621 (  chr				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "23" _null_ _null_ _null_ _null_ _null_ chr _null_ _null_ _null_ ));
DESCR("convert int4 to char");
DATA(insert OID = 1622 (  repeat			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ repeat _null_ _null_ _null_ ));
DESCR("replicate string n times");

DATA(insert OID = 1623 (  similar_escape	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ similar_escape _null_ _null_ _null_ ));
DESCR("convert SQL99 regexp pattern to POSIX style");

DATA(insert OID = 1624 (  mul_d_interval	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "701 1186" _null_ _null_ _null_ _null_ _null_ mul_d_interval _null_ _null_ _null_ ));

DATA(insert OID = 1631 (  bpcharlike	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ textlike _null_ _null_ _null_ ));
DATA(insert OID = 1632 (  bpcharnlike	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ textnlike _null_ _null_ _null_ ));

DATA(insert OID = 1633 (  texticlike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ texticlike _null_ _null_ _null_ ));
DATA(insert OID = 1634 (  texticnlike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ texticnlike _null_ _null_ _null_ ));
DATA(insert OID = 1635 (  nameiclike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameiclike _null_ _null_ _null_ ));
DATA(insert OID = 1636 (  nameicnlike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ nameicnlike _null_ _null_ _null_ ));
DATA(insert OID = 1637 (  like_escape		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ like_escape _null_ _null_ _null_ ));
DESCR("convert LIKE pattern to use backslash escapes");

DATA(insert OID = 1656 (  bpcharicregexeq	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ texticregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1657 (  bpcharicregexne	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ texticregexne _null_ _null_ _null_ ));
DATA(insert OID = 1658 (  bpcharregexeq    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ textregexeq _null_ _null_ _null_ ));
DATA(insert OID = 1659 (  bpcharregexne    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ textregexne _null_ _null_ _null_ ));
DATA(insert OID = 1660 (  bpchariclike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ texticlike _null_ _null_ _null_ ));
DATA(insert OID = 1661 (  bpcharicnlike		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 25" _null_ _null_ _null_ _null_ _null_ texticnlike _null_ _null_ _null_ ));

/* Oracle Compatibility Related Functions - By Edmund Mergl <E.Mergl@bawue.de> */
DATA(insert OID =  868 (  strpos	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ textpos _null_ _null_ _null_ ));
DESCR("position of substring");
DATA(insert OID =  870 (  lower		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ lower _null_ _null_ _null_ ));
DESCR("lowercase");
DATA(insert OID =  871 (  upper		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ upper _null_ _null_ _null_ ));
DESCR("uppercase");
DATA(insert OID =  872 (  initcap	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ initcap _null_ _null_ _null_ ));
DESCR("capitalize each word");
DATA(insert OID =  873 (  lpad		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 23 25" _null_ _null_ _null_ _null_ _null_	lpad _null_ _null_ _null_ ));
DESCR("left-pad string to length");
DATA(insert OID =  874 (  rpad		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 23 25" _null_ _null_ _null_ _null_ _null_	rpad _null_ _null_ _null_ ));
DESCR("right-pad string to length");
DATA(insert OID =  875 (  ltrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ ltrim _null_ _null_ _null_ ));
DESCR("trim selected characters from left end of string");
DATA(insert OID =  876 (  rtrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ rtrim _null_ _null_ _null_ ));
DESCR("trim selected characters from right end of string");
DATA(insert OID =  877 (  substr	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 23 23" _null_ _null_ _null_ _null_ _null_	text_substr _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID =  878 (  translate    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 25 25" _null_ _null_ _null_ _null_ _null_	translate _null_ _null_ _null_ ));
DESCR("map a set of characters appearing in string");
DATA(insert OID =  879 (  lpad		   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.lpad($1, $2, '' '')" _null_ _null_ _null_ ));
DESCR("left-pad string to length");
DATA(insert OID =  880 (  rpad		   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.rpad($1, $2, '' '')" _null_ _null_ _null_ ));
DESCR("right-pad string to length");
DATA(insert OID =  881 (  ltrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ ltrim1 _null_ _null_ _null_ ));
DESCR("trim spaces from left end of string");
DATA(insert OID =  882 (  rtrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ rtrim1 _null_ _null_ _null_ ));
DESCR("trim spaces from right end of string");
DATA(insert OID =  883 (  substr	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ text_substr_no_len _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID =  884 (  btrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ btrim _null_ _null_ _null_ ));
DESCR("trim selected characters from both ends of string");
DATA(insert OID =  885 (  btrim		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ btrim1 _null_ _null_ _null_ ));
DESCR("trim spaces from both ends of string");

DATA(insert OID =  936 (  substring    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 23 23" _null_ _null_ _null_ _null_ _null_	text_substr _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID =  937 (  substring    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ text_substr_no_len _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID =  2087 ( replace	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 25 25" _null_ _null_ _null_ _null_ _null_	replace_text _null_ _null_ _null_ ));
DESCR("replace all occurrences in string of old_substr with new_substr");
DATA(insert OID =  2284 ( regexp_replace	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 25 25" _null_ _null_ _null_ _null_ _null_	textregexreplace_noopt _null_ _null_ _null_ ));
DESCR("replace text using regexp");
DATA(insert OID =  2285 ( regexp_replace	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 25 "25 25 25 25" _null_ _null_ _null_ _null_ _null_ textregexreplace _null_ _null_ _null_ ));
DESCR("replace text using regexp");
DATA(insert OID =  3396 ( regexp_match	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1009 "25 25" _null_ _null_ _null_ _null_ _null_ regexp_match_no_flags _null_ _null_ _null_ ));
DESCR("find first match for regexp");
DATA(insert OID =  3397 ( regexp_match	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1009 "25 25 25" _null_ _null_ _null_ _null_ _null_ regexp_match _null_ _null_ _null_ ));
DESCR("find first match for regexp");
DATA(insert OID =  2763 ( regexp_matches   UXNSP UXUID 12 1 1 0 0 f f f f t t i s 2 0 1009 "25 25" _null_ _null_ _null_ _null_ _null_ regexp_matches_no_flags _null_ _null_ _null_ ));
DESCR("find match(es) for regexp");
DATA(insert OID =  2764 ( regexp_matches   UXNSP UXUID 12 1 10 0 0 f f f f t t i s 3 0 1009 "25 25 25" _null_ _null_ _null_ _null_ _null_ regexp_matches _null_ _null_ _null_ ));
DESCR("find match(es) for regexp");
DATA(insert OID =  2088 ( split_part   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 25 "25 25 23" _null_ _null_ _null_ _null_ _null_	split_text _null_ _null_ _null_ ));
DESCR("split string by field_sep and return field_num");
DATA(insert OID =  2765 ( regexp_split_to_table UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ regexp_split_to_table_no_flags _null_ _null_ _null_ ));
DESCR("split string by pattern");
DATA(insert OID =  2766 ( regexp_split_to_table UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 25 "25 25 25" _null_ _null_ _null_ _null_ _null_	regexp_split_to_table _null_ _null_ _null_ ));
DESCR("split string by pattern");
DATA(insert OID =  2767 ( regexp_split_to_array UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1009 "25 25" _null_ _null_ _null_ _null_ _null_	regexp_split_to_array_no_flags _null_ _null_ _null_ ));
DESCR("split string by pattern");
DATA(insert OID =  2768 ( regexp_split_to_array UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1009 "25 25 25" _null_ _null_ _null_ _null_ _null_ regexp_split_to_array _null_ _null_ _null_ ));
DESCR("split string by pattern");
DATA(insert OID =  2089 ( to_hex	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "23" _null_ _null_ _null_ _null_ _null_ to_hex32 _null_ _null_ _null_ ));
DESCR("convert int4 number to hex");
DATA(insert OID =  2090 ( to_hex	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "20" _null_ _null_ _null_ _null_ _null_ to_hex64 _null_ _null_ _null_ ));
DESCR("convert int8 number to hex");

/* for character set encoding support */

/* return database encoding name */
DATA(insert OID = 1039 (  getdatabaseencoding	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ getdatabaseencoding _null_ _null_ _null_ ));
DESCR("encoding name of current database");

/* return client encoding name i.e. session encoding */
DATA(insert OID = 810 (  ux_client_encoding    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 19 "" _null_ _null_ _null_ _null_ _null_ ux_client_encoding _null_ _null_ _null_ ));
DESCR("encoding name of current database");

DATA(insert OID = 1713 (  length		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 23 "17 19" _null_ _null_ _null_ _null_ _null_ length_in_encoding _null_ _null_ _null_ ));
DESCR("length of string in specified encoding");

DATA(insert OID = 1714 (  convert_from		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "17 19" _null_ _null_ _null_ _null_ _null_ ux_convert_from _null_ _null_ _null_ ));
DESCR("convert string with specified source encoding name");

DATA(insert OID = 1717 (  convert_to		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 17 "25 19" _null_ _null_ _null_ _null_ _null_ ux_convert_to _null_ _null_ _null_ ));
DESCR("convert string with specified destination encoding name");

DATA(insert OID = 1813 (  convert		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 17 "17 19 19" _null_ _null_ _null_ _null_ _null_ ux_convert _null_ _null_ _null_ ));
DESCR("convert string with specified encoding names");

DATA(insert OID = 1264 (  ux_char_to_encoding	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 23 "19" _null_ _null_ _null_ _null_ _null_ UX_char_to_encoding _null_ _null_ _null_ ));
DESCR("convert encoding name to encoding id");

DATA(insert OID = 1597 (  ux_encoding_to_char	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 19 "23" _null_ _null_ _null_ _null_ _null_ UX_encoding_to_char _null_ _null_ _null_ ));
DESCR("convert encoding id to encoding name");

DATA(insert OID = 2319 (  ux_encoding_max_length   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ ux_encoding_max_length_sql _null_ _null_ _null_ ));
DESCR("maximum octet length of a character in given encoding");

DATA(insert OID = 1638 (  oidgt				   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oidgt _null_ _null_ _null_ ));
DATA(insert OID = 1639 (  oidge				   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "26 26" _null_ _null_ _null_ _null_ _null_ oidge _null_ _null_ _null_ ));

/* System-view support functions */
DATA(insert OID = 1573 (  ux_get_ruledef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_ruledef _null_ _null_ _null_ ));
DESCR("source text of a rule");
DATA(insert OID = 1640 (  ux_get_viewdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ ux_get_viewdef_name _null_ _null_ _null_ ));
DESCR("select statement of a view");
DATA(insert OID = 1641 (  ux_get_viewdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_viewdef _null_ _null_ _null_ ));
DESCR("select statement of a view");
DATA(insert OID = 1642 (  ux_get_userbyid	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 19 "26" _null_ _null_ _null_ _null_ _null_ ux_get_userbyid _null_ _null_ _null_ ));
DESCR("role name by OID (with fallback)");
DATA(insert OID = 1643 (  ux_get_indexdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_indexdef _null_ _null_ _null_ ));
DESCR("index description");
DATA(insert OID = 3415 (  ux_get_statisticsobjdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_statisticsobjdef _null_ _null_ _null_ ));
DESCR("extended statistics object description");
DATA(insert OID = 3352 (  ux_get_partkeydef    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_partkeydef _null_ _null_ _null_ ));
DESCR("partition key description");
DATA(insert OID = 3408 (  ux_get_partition_constraintdef	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_partition_constraintdef _null_ _null_ _null_ ));
DESCR("partition constraint description");
DATA(insert OID = 1662 (  ux_get_triggerdef    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_triggerdef _null_ _null_ _null_ ));
DESCR("trigger description");
DATA(insert OID = 1387 (  ux_get_constraintdef UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_constraintdef _null_ _null_ _null_ ));
DESCR("constraint description");
DATA(insert OID = 1716 (  ux_get_expr		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "194 26" _null_ _null_ _null_ _null_ _null_ ux_get_expr _null_ _null_ _null_ ));
DESCR("deparse an encoded expression");
DATA(insert OID = 1665 (  ux_get_serial_sequence	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ ux_get_serial_sequence _null_ _null_ _null_ ));
DESCR("name of sequence for a serial column");
DATA(insert OID = 2098 (  ux_get_functiondef	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_functiondef _null_ _null_ _null_ ));
DESCR("definition of a function");
DATA(insert OID = 2162 (  ux_get_function_arguments    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_function_arguments _null_ _null_ _null_ ));
DESCR("argument list of a function");
DATA(insert OID = 2232 (  ux_get_function_identity_arguments	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_function_identity_arguments _null_ _null_ _null_ ));
DESCR("identity argument list of a function");
DATA(insert OID = 2165 (  ux_get_function_result	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_get_function_result _null_ _null_ _null_ ));
DESCR("result type of a function");
DATA(insert OID = 3808 (  ux_get_function_arg_default	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "26 23" _null_ _null_ _null_ _null_ _null_ ux_get_function_arg_default _null_ _null_ _null_ ));
DESCR("function argument default");

DATA(insert OID = 1686 (  ux_get_keywords		UXNSP UXUID 12 10 400 0 0 f f f f t t s s 0 0 2249 "" "{25,18,25}" "{o,o,o}" "{word,catcode,catdesc}" _null_ _null_ ux_get_keywords _null_ _null_ _null_ ));
DESCR("list of SQL keywords");

DATA(insert OID = 2289 (  ux_options_to_table		UXNSP UXUID 12 1 3 0 0 f f f f t t s s 1 0 2249 "1009" "{1009,25,25}" "{i,o,o}" "{options_array,option_name,option_value}" _null_ _null_ ux_options_to_table _null_ _null_ _null_ ));
DESCR("convert generic options array to name/value table");

DATA(insert OID = 1619 (  ux_typeof				UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 2206 "2276" _null_ _null_ _null_ _null_	_null_ ux_typeof _null_ _null_ _null_ ));
DESCR("type of the argument");
DATA(insert OID = 3162 (  ux_collation_for		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0	 25 "2276" _null_ _null_ _null_ _null_	_null_ ux_collation_for _null_ _null_ _null_ ));
DESCR("collation of the argument; implementation of the COLLATION FOR expression");

DATA(insert OID = 3842 (  ux_relation_is_updatable	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 2 0 23 "2205 16" _null_ _null_ _null_ _null_ _null_ ux_relation_is_updatable _null_ _null_ _null_ ));
DESCR("is a relation insertable/updatable/deletable");
DATA(insert OID = 3843 (  ux_column_is_updatable	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 3 0 16 "2205 21 16" _null_ _null_ _null_ _null_ _null_ ux_column_is_updatable _null_ _null_ _null_ ));
DESCR("is a column updatable");

DATA(insert OID = 6120 (  ux_get_replica_identity_index UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 2205 "2205" _null_ _null_ _null_ _null_ _null_ ux_get_replica_identity_index _null_ _null_ _null_ ));
DESCR("oid of replica identity index if any");

/* Deferrable unique constraint trigger */
DATA(insert OID = 1250 (  unique_key_recheck	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ unique_key_recheck _null_ _null_ _null_ ));
DESCR("deferred UNIQUE constraint check");

/* Generic referential integrity constraint triggers */
DATA(insert OID = 1644 (  RI_FKey_check_ins		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_check_ins _null_ _null_ _null_ ));
DESCR("referential integrity FOREIGN KEY ... REFERENCES");
DATA(insert OID = 1645 (  RI_FKey_check_upd		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_check_upd _null_ _null_ _null_ ));
DESCR("referential integrity FOREIGN KEY ... REFERENCES");
DATA(insert OID = 1646 (  RI_FKey_cascade_del	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_cascade_del _null_ _null_ _null_ ));
DESCR("referential integrity ON DELETE CASCADE");
DATA(insert OID = 1647 (  RI_FKey_cascade_upd	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_cascade_upd _null_ _null_ _null_ ));
DESCR("referential integrity ON UPDATE CASCADE");
DATA(insert OID = 1648 (  RI_FKey_restrict_del	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_restrict_del _null_ _null_ _null_ ));
DESCR("referential integrity ON DELETE RESTRICT");
DATA(insert OID = 1649 (  RI_FKey_restrict_upd	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_restrict_upd _null_ _null_ _null_ ));
DESCR("referential integrity ON UPDATE RESTRICT");
DATA(insert OID = 1650 (  RI_FKey_setnull_del	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_setnull_del _null_ _null_ _null_ ));
DESCR("referential integrity ON DELETE SET NULL");
DATA(insert OID = 1651 (  RI_FKey_setnull_upd	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_setnull_upd _null_ _null_ _null_ ));
DESCR("referential integrity ON UPDATE SET NULL");
DATA(insert OID = 1652 (  RI_FKey_setdefault_del UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_setdefault_del _null_ _null_ _null_ ));
DESCR("referential integrity ON DELETE SET DEFAULT");
DATA(insert OID = 1653 (  RI_FKey_setdefault_upd UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_setdefault_upd _null_ _null_ _null_ ));
DESCR("referential integrity ON UPDATE SET DEFAULT");
DATA(insert OID = 1654 (  RI_FKey_noaction_del UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_noaction_del _null_ _null_ _null_ ));
DESCR("referential integrity ON DELETE NO ACTION");
DATA(insert OID = 1655 (  RI_FKey_noaction_upd UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ RI_FKey_noaction_upd _null_ _null_ _null_ ));
DESCR("referential integrity ON UPDATE NO ACTION");

DATA(insert OID = 1666 (  varbiteq			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ biteq _null_ _null_ _null_ ));
DATA(insert OID = 1667 (  varbitne			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitne _null_ _null_ _null_ ));
DATA(insert OID = 1668 (  varbitge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitge _null_ _null_ _null_ ));
DATA(insert OID = 1669 (  varbitgt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitgt _null_ _null_ _null_ ));
DATA(insert OID = 1670 (  varbitle			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitle _null_ _null_ _null_ ));
DATA(insert OID = 1671 (  varbitlt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitlt _null_ _null_ _null_ ));
DATA(insert OID = 1672 (  varbitcmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1562 1562" _null_ _null_ _null_ _null_ _null_ bitcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

/* avoid the C names bitand and bitor, since they are C++ keywords */
DATA(insert OID = 1673 (  bitand			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 1560" _null_ _null_ _null_ _null_ _null_	bit_and _null_ _null_ _null_ ));
DATA(insert OID = 1674 (  bitor				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 1560" _null_ _null_ _null_ _null_ _null_	bit_or _null_ _null_ _null_ ));
DATA(insert OID = 1675 (  bitxor			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 1560" _null_ _null_ _null_ _null_ _null_	bitxor _null_ _null_ _null_ ));
DATA(insert OID = 1676 (  bitnot			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1560 "1560" _null_ _null_ _null_ _null_ _null_ bitnot _null_ _null_ _null_ ));
DATA(insert OID = 1677 (  bitshiftleft		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 23" _null_ _null_ _null_ _null_ _null_ bitshiftleft _null_ _null_ _null_ ));
DATA(insert OID = 1678 (  bitshiftright		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 23" _null_ _null_ _null_ _null_ _null_ bitshiftright _null_ _null_ _null_ ));
DATA(insert OID = 1679 (  bitcat			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1562 "1562 1562" _null_ _null_ _null_ _null_ _null_	bitcat _null_ _null_ _null_ ));
DATA(insert OID = 1680 (  substring			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "1560 23 23" _null_ _null_ _null_ _null_ _null_ bitsubstr _null_ _null_ _null_ ));
DESCR("extract portion of bitstring");
DATA(insert OID = 1681 (  length			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1560" _null_ _null_ _null_ _null_ _null_ bitlength _null_ _null_ _null_ ));
DESCR("bitstring length");
DATA(insert OID = 1682 (  octet_length		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1560" _null_ _null_ _null_ _null_ _null_ bitoctetlength _null_ _null_ _null_ ));
DESCR("octet length");
DATA(insert OID = 1683 (  bit				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "23 23" _null_ _null_ _null_ _null_ _null_	bitfromint4 _null_ _null_ _null_ ));
DESCR("convert int4 to bitstring");
DATA(insert OID = 1684 (  int4				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1560" _null_ _null_ _null_ _null_ _null_ bittoint4 _null_ _null_ _null_ ));
DESCR("convert bitstring to int4");

DATA(insert OID = 1685 (  bit			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "1560 23 16" _null_ _null_ _null_ _null_ _null_ bit _null_ _null_ _null_ ));
DESCR("adjust bit() to typmod length");
DATA(insert OID = 3158 ( varbit_transform  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ varbit_transform _null_ _null_ _null_ ));
DESCR("transform a varbit length coercion");
DATA(insert OID = 1687 (  varbit		   UXNSP UXUID 12 1 0 0 varbit_transform f f f f t f i s 3 0 1562 "1562 23 16" _null_ _null_ _null_ _null_ _null_ varbit _null_ _null_ _null_ ));
DESCR("adjust varbit() to typmod length");

DATA(insert OID = 1698 (  position		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1560 1560" _null_ _null_ _null_ _null_ _null_ bitposition _null_ _null_ _null_ ));
DESCR("position of sub-bitstring");
DATA(insert OID = 1699 (  substring		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "1560 23" _null_ _null_ _null_ _null_ _null_ bitsubstr_no_len _null_ _null_ _null_ ));
DESCR("extract portion of bitstring");

DATA(insert OID = 3030 (  overlay		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 1560 "1560 1560 23 23" _null_ _null_ _null_ _null_ _null_ bitoverlay _null_ _null_ _null_ ));
DESCR("substitute portion of bitstring");
DATA(insert OID = 3031 (  overlay		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "1560 1560 23" _null_ _null_ _null_ _null_ _null_ bitoverlay_no_len _null_ _null_ _null_ ));
DESCR("substitute portion of bitstring");
DATA(insert OID = 3032 (  get_bit		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1560 23" _null_ _null_ _null_ _null_ _null_ bitgetbit _null_ _null_ _null_ ));
DESCR("get bit");
DATA(insert OID = 3033 (  set_bit		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "1560 23 23" _null_ _null_ _null_ _null_ _null_ bitsetbit _null_ _null_ _null_ ));
DESCR("set bit");

/* for macaddr type support */
DATA(insert OID = 436 (  macaddr_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 829 "2275" _null_ _null_ _null_ _null_ _null_ macaddr_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 437 (  macaddr_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "829" _null_ _null_ _null_ _null_ _null_ macaddr_out _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 753 (  trunc				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 829 "829" _null_ _null_ _null_ _null_ _null_ macaddr_trunc _null_ _null_ _null_ ));
DESCR("MACADDR manufacturer fields");

DATA(insert OID = 830 (  macaddr_eq			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_eq _null_ _null_ _null_ ));
DATA(insert OID = 831 (  macaddr_lt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_lt _null_ _null_ _null_ ));
DATA(insert OID = 832 (  macaddr_le			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_le _null_ _null_ _null_ ));
DATA(insert OID = 833 (  macaddr_gt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_gt _null_ _null_ _null_ ));
DATA(insert OID = 834 (  macaddr_ge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_ge _null_ _null_ _null_ ));
DATA(insert OID = 835 (  macaddr_ne			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_ne _null_ _null_ _null_ ));
DATA(insert OID = 836 (  macaddr_cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "829 829" _null_ _null_ _null_ _null_ _null_	macaddr_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3144 (  macaddr_not		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 829 "829" _null_ _null_ _null_ _null_ _null_ macaddr_not _null_ _null_ _null_ ));
DATA(insert OID = 3145 (  macaddr_and		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 829 "829 829" _null_ _null_ _null_ _null_ _null_ macaddr_and _null_ _null_ _null_ ));
DATA(insert OID = 3146 (  macaddr_or		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 829 "829 829" _null_ _null_ _null_ _null_ _null_ macaddr_or _null_ _null_ _null_ ));
DATA(insert OID = 3359 (  macaddr_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ macaddr_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");

/* for macaddr8 type support */
DATA(insert OID = 4110 (  macaddr8_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "2275" _null_ _null_ _null_ _null_ _null_ macaddr8_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4111 (  macaddr8_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "774" _null_ _null_ _null_ _null_ _null_ macaddr8_out _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 4112 (  trunc				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "774" _null_ _null_ _null_ _null_ _null_ macaddr8_trunc _null_ _null_ _null_ ));
DESCR("MACADDR8 manufacturer fields");

DATA(insert OID = 4113 (  macaddr8_eq		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_eq _null_ _null_ _null_ ));
DATA(insert OID = 4114 (  macaddr8_lt		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_lt _null_ _null_ _null_ ));
DATA(insert OID = 4115 (  macaddr8_le		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_le _null_ _null_ _null_ ));
DATA(insert OID = 4116 (  macaddr8_gt		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_gt _null_ _null_ _null_ ));
DATA(insert OID = 4117 (  macaddr8_ge		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_ge _null_ _null_ _null_ ));
DATA(insert OID = 4118 (  macaddr8_ne		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_ne _null_ _null_ _null_ ));
DATA(insert OID = 4119 (  macaddr8_cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "774 774" _null_ _null_ _null_ _null_ _null_	macaddr8_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 4120 (  macaddr8_not		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "774" _null_ _null_ _null_ _null_ _null_ macaddr8_not _null_ _null_ _null_ ));
DATA(insert OID = 4121 (  macaddr8_and		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 774 "774 774" _null_ _null_ _null_ _null_ _null_ macaddr8_and _null_ _null_ _null_ ));
DATA(insert OID = 4122 (  macaddr8_or		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 774 "774 774" _null_ _null_ _null_ _null_ _null_ macaddr8_or _null_ _null_ _null_ ));
DATA(insert OID = 4123 (  macaddr8			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "829" _null_ _null_ _null_ _null_ _null_ macaddrtomacaddr8 _null_ _null_ _null_ ));
DESCR("convert macaddr to macaddr8");
DATA(insert OID = 4124 (  macaddr			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 829 "774" _null_ _null_ _null_ _null_ _null_ macaddr8tomacaddr _null_ _null_ _null_ ));
DESCR("convert macaddr8 to macaddr");
DATA(insert OID = 4125 (  macaddr8_set7bit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "774" _null_ _null_ _null_ _null_ _null_ macaddr8_set7bit _null_ _null_ _null_ ));
DESCR("set 7th bit in macaddr8");

/* for inet type support */
DATA(insert OID = 910 (  inet_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "2275" _null_ _null_ _null_ _null_ _null_ inet_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 911 (  inet_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "869" _null_ _null_ _null_ _null_ _null_ inet_out _null_ _null_ _null_ ));
DESCR("I/O");

/* for cidr type support */
DATA(insert OID = 1267 (  cidr_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 650 "2275" _null_ _null_ _null_ _null_ _null_ cidr_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1427 (  cidr_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "650" _null_ _null_ _null_ _null_ _null_ cidr_out _null_ _null_ _null_ ));
DESCR("I/O");

/* these are used for both inet and cidr */
DATA(insert OID = 920 (  network_eq			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_eq _null_ _null_ _null_ ));
DATA(insert OID = 921 (  network_lt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_lt _null_ _null_ _null_ ));
DATA(insert OID = 922 (  network_le			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_le _null_ _null_ _null_ ));
DATA(insert OID = 923 (  network_gt			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_gt _null_ _null_ _null_ ));
DATA(insert OID = 924 (  network_ge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_ge _null_ _null_ _null_ ));
DATA(insert OID = 925 (  network_ne			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_ne _null_ _null_ _null_ ));
DATA(insert OID = 3562 (  network_larger	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 869" _null_ _null_ _null_ _null_ _null_ network_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 3563 (  network_smaller	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 869" _null_ _null_ _null_ _null_ _null_ network_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 926 (  network_cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "869 869" _null_ _null_ _null_ _null_ _null_	network_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 927 (  network_sub		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_sub _null_ _null_ _null_ ));
DATA(insert OID = 928 (  network_subeq		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_subeq _null_ _null_ _null_ ));
DATA(insert OID = 929 (  network_sup		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_sup _null_ _null_ _null_ ));
DATA(insert OID = 930 (  network_supeq		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_supeq _null_ _null_ _null_ ));
DATA(insert OID = 3551 (  network_overlap	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_	network_overlap _null_ _null_ _null_ ));

/* inet/cidr functions */
DATA(insert OID = 598 (  abbrev				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "869" _null_ _null_ _null_ _null_ _null_	inet_abbrev _null_ _null_ _null_ ));
DESCR("abbreviated display of inet value");
DATA(insert OID = 599 (  abbrev				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "650" _null_ _null_ _null_ _null_ _null_	cidr_abbrev _null_ _null_ _null_ ));
DESCR("abbreviated display of cidr value");
DATA(insert OID = 605 (  set_masklen		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 23" _null_ _null_ _null_ _null_ _null_	inet_set_masklen _null_ _null_ _null_ ));
DESCR("change netmask of inet");
DATA(insert OID = 635 (  set_masklen		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 650 "650 23" _null_ _null_ _null_ _null_ _null_	cidr_set_masklen _null_ _null_ _null_ ));
DESCR("change netmask of cidr");
DATA(insert OID = 711 (  family				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "869" _null_ _null_ _null_ _null_ _null_	network_family _null_ _null_ _null_ ));
DESCR("address family (4 for IPv4, 6 for IPv6)");
DATA(insert OID = 683 (  network			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 650 "869" _null_ _null_ _null_ _null_ _null_ network_network _null_ _null_ _null_ ));
DESCR("network part of address");
DATA(insert OID = 696 (  netmask			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ network_netmask _null_ _null_ _null_ ));
DESCR("netmask of address");
DATA(insert OID = 697 (  masklen			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "869" _null_ _null_ _null_ _null_ _null_	network_masklen _null_ _null_ _null_ ));
DESCR("netmask length");
DATA(insert OID = 698 (  broadcast			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ network_broadcast _null_ _null_ _null_ ));
DESCR("broadcast address of network");
DATA(insert OID = 699 (  host				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "869" _null_ _null_ _null_ _null_ _null_	network_host _null_ _null_ _null_ ));
DESCR("show address octets only");
DATA(insert OID = 730 (  text				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "869" _null_ _null_ _null_ _null_ _null_	network_show _null_ _null_ _null_ ));
DESCR("show all parts of inet/cidr value");
DATA(insert OID = 1362 (  hostmask			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ network_hostmask _null_ _null_ _null_ ));
DESCR("hostmask of address");
DATA(insert OID = 1715 (  cidr				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 650 "869" _null_ _null_ _null_ _null_ _null_ inet_to_cidr _null_ _null_ _null_ ));
DESCR("convert inet to cidr");

DATA(insert OID = 2196 (  inet_client_addr		UXNSP UXUID 12 1 0 0 0 f f f f f f s r 0 0 869 "" _null_ _null_ _null_ _null_ _null_ inet_client_addr _null_ _null_ _null_ ));
DESCR("inet address of the client");
DATA(insert OID = 2197 (  inet_client_port		UXNSP UXUID 12 1 0 0 0 f f f f f f s r 0 0 23 "" _null_ _null_ _null_ _null_ _null_ inet_client_port _null_ _null_ _null_ ));
DESCR("client's port number for this connection");
DATA(insert OID = 2198 (  inet_server_addr		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 869 "" _null_ _null_ _null_ _null_ _null_ inet_server_addr _null_ _null_ _null_ ));
DESCR("inet address of the server");
DATA(insert OID = 2199 (  inet_server_port		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 23 "" _null_ _null_ _null_ _null_ _null_ inet_server_port _null_ _null_ _null_ ));
DESCR("server's port number for this connection");

DATA(insert OID = 2627 (  inetnot			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ inetnot _null_ _null_ _null_ ));
DATA(insert OID = 2628 (  inetand			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 869" _null_ _null_ _null_ _null_ _null_ inetand _null_ _null_ _null_ ));
DATA(insert OID = 2629 (  inetor			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 869" _null_ _null_ _null_ _null_ _null_ inetor _null_ _null_ _null_ ));
DATA(insert OID = 2630 (  inetpl			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 20" _null_ _null_ _null_ _null_ _null_	inetpl _null_ _null_ _null_ ));
DATA(insert OID = 2631 (  int8pl_inet		UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 869 "20 869" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));
DATA(insert OID = 2632 (  inetmi_int8		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "869 20" _null_ _null_ _null_ _null_ _null_	inetmi_int8 _null_ _null_ _null_ ));
DATA(insert OID = 2633 (  inetmi			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "869 869" _null_ _null_ _null_ _null_ _null_	inetmi _null_ _null_ _null_ ));
DATA(insert OID = 4071 (  inet_same_family	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "869 869" _null_ _null_ _null_ _null_ _null_ inet_same_family _null_ _null_ _null_ ));
DESCR("are the addresses from the same family?");
DATA(insert OID = 4063 (  inet_merge		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 650 "869 869" _null_ _null_ _null_ _null_ _null_ inet_merge _null_ _null_ _null_ ));
DESCR("the smallest network which includes both of the given networks");

/* GiST support for inet and cidr */
DATA(insert OID = 3553 (  inet_gist_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 869 21 26 2281" _null_ _null_ _null_ _null_ _null_ inet_gist_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3554 (  inet_gist_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 869 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_gist_union _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3555 (  inet_gist_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ inet_gist_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3556 (  inet_gist_decompress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ inet_gist_decompress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3573 (  inet_gist_fetch		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ inet_gist_fetch _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3557 (  inet_gist_penalty		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ inet_gist_penalty _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3558 (  inet_gist_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_gist_picksplit _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3559 (  inet_gist_same		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "869 869 2281" _null_ _null_ _null_ _null_ _null_ inet_gist_same _null_ _null_ _null_ ));
DESCR("GiST support");

/* SP-GiST support for inet and cidr */
DATA(insert OID = 3795 (  inet_spg_config		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_spg_config _null_ _null_ _null_ ));
DESCR("SP-GiST support");
DATA(insert OID = 3796 (  inet_spg_choose		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_spg_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support");
DATA(insert OID = 3797 (  inet_spg_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_spg_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support");
DATA(insert OID = 3798 (  inet_spg_inner_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_spg_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support");
DATA(insert OID = 3799 (  inet_spg_leaf_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2281 2281" _null_ _null_ _null_ _null_ _null_ inet_spg_leaf_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support");

/* Selectivity estimation for inet and cidr */
DATA(insert OID = 3560 (  networksel		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_	networksel _null_ _null_ _null_ ));
DESCR("restriction selectivity for network operators");
DATA(insert OID = 3561 (  networkjoinsel	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ networkjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity for network operators");

DATA(insert OID = 1690 ( time_mi_time		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1083 1083" _null_ _null_ _null_ _null_ _null_	time_mi_time _null_ _null_ _null_ ));

DATA(insert OID = 1691 (  boolle			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boolle _null_ _null_ _null_ ));
DATA(insert OID = 1692 (  boolge			UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boolge _null_ _null_ _null_ ));
DATA(insert OID = 1693 (  btboolcmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "16 16" _null_ _null_ _null_ _null_ _null_ btboolcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 1688 (  time_hash			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1083" _null_ _null_ _null_ _null_ _null_ time_hash _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 1696 (  timetz_hash		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1266" _null_ _null_ _null_ _null_ _null_ timetz_hash _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 1697 (  interval_hash		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1186" _null_ _null_ _null_ _null_ _null_ interval_hash _null_ _null_ _null_ ));
DESCR("hash");


/* OID's 1700 - 1799 NUMERIC data type */
DATA(insert OID = 1701 ( numeric_in				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1700 "2275 26 23" _null_ _null_ _null_ _null_ _null_ numeric_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1702 ( numeric_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "1700" _null_ _null_ _null_ _null_ _null_ numeric_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2917 (  numerictypmodin		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1263" _null_ _null_ _null_ _null_ _null_ numerictypmodin _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 2918 (  numerictypmodout		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "23" _null_ _null_ _null_ _null_ _null_ numerictypmodout _null_ _null_ _null_ ));
DESCR("I/O typmod");
DATA(insert OID = 3157 ( numeric_transform		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ numeric_transform _null_ _null_ _null_ ));
DESCR("transform a numeric length coercion");
DATA(insert OID = 1703 ( numeric				UXNSP UXUID 12 1 0 0 numeric_transform f f f f t f i s 2 0 1700 "1700 23" _null_ _null_ _null_ _null_ _null_ numeric _null_ _null_ _null_ ));
DESCR("adjust numeric to typmod precision/scale");
DATA(insert OID = 1704 ( numeric_abs			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
DATA(insert OID = 1705 ( abs					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
DESCR("absolute value");
DATA(insert OID = 1706 ( sign					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_sign _null_ _null_ _null_ ));
DESCR("sign of value");
DATA(insert OID = 1707 ( round					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 23" _null_ _null_ _null_ _null_ _null_ numeric_round _null_ _null_ _null_ ));
DESCR("value rounded to 'scale'");
DATA(insert OID = 1708 ( round					UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.round($1,0)" _null_ _null_ _null_ ));
DESCR("value rounded to 'scale' of zero");
DATA(insert OID = 1709 ( trunc					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 23" _null_ _null_ _null_ _null_ _null_ numeric_trunc _null_ _null_ _null_ ));
DESCR("value truncated to 'scale'");
DATA(insert OID = 1710 ( trunc					UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.trunc($1,0)" _null_ _null_ _null_ ));
DESCR("value truncated to 'scale' of zero");
DATA(insert OID = 1711 ( ceil					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_ceil _null_ _null_ _null_ ));
DESCR("nearest integer >= value");
DATA(insert OID = 2167 ( ceiling				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_ceil _null_ _null_ _null_ ));
DESCR("nearest integer >= value");
DATA(insert OID = 1712 ( floor					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_floor _null_ _null_ _null_ ));
DESCR("nearest integer <= value");
DATA(insert OID = 1718 ( numeric_eq				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_eq _null_ _null_ _null_ ));
DATA(insert OID = 1719 ( numeric_ne				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_ne _null_ _null_ _null_ ));
DATA(insert OID = 1720 ( numeric_gt				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_gt _null_ _null_ _null_ ));
DATA(insert OID = 1721 ( numeric_ge				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_ge _null_ _null_ _null_ ));
DATA(insert OID = 1722 ( numeric_lt				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_lt _null_ _null_ _null_ ));
DATA(insert OID = 1723 ( numeric_le				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_le _null_ _null_ _null_ ));
DATA(insert OID = 1724 ( numeric_add			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_add _null_ _null_ _null_ ));
DATA(insert OID = 1725 ( numeric_sub			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_sub _null_ _null_ _null_ ));
DATA(insert OID = 1726 ( numeric_mul			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_mul _null_ _null_ _null_ ));
DATA(insert OID = 1727 ( numeric_div			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_div _null_ _null_ _null_ ));
DATA(insert OID = 1728 ( mod					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_mod _null_ _null_ _null_ ));
DESCR("modulus");
DATA(insert OID = 1729 ( numeric_mod			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_mod _null_ _null_ _null_ ));
DATA(insert OID = 1730 ( sqrt					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_sqrt _null_ _null_ _null_ ));
DESCR("square root");
DATA(insert OID = 1731 ( numeric_sqrt			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_sqrt _null_ _null_ _null_ ));
DESCR("square root");
DATA(insert OID = 1732 ( exp					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_exp _null_ _null_ _null_ ));
DESCR("natural exponential (e^x)");
DATA(insert OID = 1733 ( numeric_exp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_exp _null_ _null_ _null_ ));
DESCR("natural exponential (e^x)");
DATA(insert OID = 1734 ( ln						UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_ln _null_ _null_ _null_ ));
DESCR("natural logarithm");
DATA(insert OID = 1735 ( numeric_ln				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_ln _null_ _null_ _null_ ));
DESCR("natural logarithm");
DATA(insert OID = 1736 ( log					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_log _null_ _null_ _null_ ));
DESCR("logarithm base m of n");
DATA(insert OID = 1737 ( numeric_log			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_log _null_ _null_ _null_ ));
DESCR("logarithm base m of n");
DATA(insert OID = 1738 ( pow					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_power _null_ _null_ _null_ ));
DESCR("exponentiation");
DATA(insert OID = 2169 ( power					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_power _null_ _null_ _null_ ));
DESCR("exponentiation");
DATA(insert OID = 1739 ( numeric_power			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_power _null_ _null_ _null_ ));
DATA(insert OID = 8888 ( scale					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1700" _null_ _null_ _null_ _null_ _null_ numeric_scale _null_ _null_ _null_ ));
DESCR("number of decimal digits in the fractional part");
DATA(insert OID = 1740 ( numeric				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ int4_numeric _null_ _null_ _null_ ));
DESCR("convert int4 to numeric");
DATA(insert OID = 1741 ( log					UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.log(10, $1)" _null_ _null_ _null_ ));
DESCR("base 10 logarithm");
DATA(insert OID = 1742 ( numeric				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "700" _null_ _null_ _null_ _null_ _null_ float4_numeric _null_ _null_ _null_ ));
DESCR("convert float4 to numeric");
DATA(insert OID = 1743 ( numeric				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "701" _null_ _null_ _null_ _null_ _null_ float8_numeric _null_ _null_ _null_ ));
DESCR("convert float8 to numeric");
DATA(insert OID = 1744 ( int4					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1700" _null_ _null_ _null_ _null_ _null_ numeric_int4 _null_ _null_ _null_ ));
DESCR("convert numeric to int4");
DATA(insert OID = 1745 ( float4					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_float4 _null_ _null_ _null_ ));
DESCR("convert numeric to float4");
DATA(insert OID = 1746 ( float8					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1700" _null_ _null_ _null_ _null_ _null_ numeric_float8 _null_ _null_ _null_ ));
DESCR("convert numeric to float8");
DATA(insert OID = 1973 ( div					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_div_trunc _null_ _null_ _null_ ));
DESCR("trunc(x/y)");
DATA(insert OID = 1980 ( numeric_div_trunc		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_div_trunc _null_ _null_ _null_ ));
DESCR("trunc(x/y)");
DATA(insert OID = 2170 ( width_bucket			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 23 "1700 1700 1700 23" _null_ _null_ _null_ _null_ _null_ width_bucket_numeric _null_ _null_ _null_ ));
DESCR("bucket number of operand in equal-width histogram");

DATA(insert OID = 1747 ( time_pl_interval		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1083 "1083 1186" _null_ _null_ _null_ _null_ _null_	time_pl_interval _null_ _null_ _null_ ));
DATA(insert OID = 1748 ( time_mi_interval		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1083 "1083 1186" _null_ _null_ _null_ _null_ _null_	time_mi_interval _null_ _null_ _null_ ));
DATA(insert OID = 1749 ( timetz_pl_interval		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1266 "1266 1186" _null_ _null_ _null_ _null_ _null_	timetz_pl_interval _null_ _null_ _null_ ));
DATA(insert OID = 1750 ( timetz_mi_interval		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1266 "1266 1186" _null_ _null_ _null_ _null_ _null_	timetz_mi_interval _null_ _null_ _null_ ));

DATA(insert OID = 1764 ( numeric_inc			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_inc _null_ _null_ _null_ ));
DESCR("increment by one");
DATA(insert OID = 1766 ( numeric_smaller		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 1767 ( numeric_larger			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_	numeric_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1769 ( numeric_cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1700 1700" _null_ _null_ _null_ _null_ _null_ numeric_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3283 ( numeric_sortsupport	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ numeric_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 1771 ( numeric_uminus			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_uminus _null_ _null_ _null_ ));
DATA(insert OID = 1779 ( int8					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "1700" _null_ _null_ _null_ _null_ _null_ numeric_int8 _null_ _null_ _null_ ));
DESCR("convert numeric to int8");
DATA(insert OID = 1781 ( numeric				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ int8_numeric _null_ _null_ _null_ ));
DESCR("convert int8 to numeric");
DATA(insert OID = 1782 ( numeric				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ int2_numeric _null_ _null_ _null_ ));
DESCR("convert int2 to numeric");
DATA(insert OID = 1783 ( int2					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "1700" _null_ _null_ _null_ _null_ _null_ numeric_int2 _null_ _null_ _null_ ));
DESCR("convert numeric to int2");

/* formatting */
DATA(insert OID = 1770 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "1184 25" _null_ _null_ _null_ _null_  _null_ timestamptz_to_char _null_ _null_ _null_ ));
DESCR("format timestamp with time zone to text");
DATA(insert OID = 1772 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "1700 25" _null_ _null_ _null_ _null_  _null_ numeric_to_char _null_ _null_ _null_ ));
DESCR("format numeric to text");
DATA(insert OID = 1773 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "23 25" _null_ _null_ _null_ _null_ _null_ int4_to_char _null_ _null_ _null_ ));
DESCR("format int4 to text");
DATA(insert OID = 1774 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "20 25" _null_ _null_ _null_ _null_ _null_ int8_to_char _null_ _null_ _null_ ));
DESCR("format int8 to text");
DATA(insert OID = 1775 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "700 25" _null_ _null_ _null_ _null_ _null_ float4_to_char _null_ _null_ _null_ ));
DESCR("format float4 to text");
DATA(insert OID = 1776 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "701 25" _null_ _null_ _null_ _null_ _null_ float8_to_char _null_ _null_ _null_ ));
DESCR("format float8 to text");
DATA(insert OID = 1777 ( to_number			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1700 "25 25" _null_ _null_ _null_ _null_  _null_ numeric_to_number _null_ _null_ _null_ ));
DESCR("convert text to numeric");
DATA(insert OID = 1778 ( to_timestamp		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1184 "25 25" _null_ _null_ _null_ _null_  _null_ to_timestamp _null_ _null_ _null_ ));
DESCR("convert text to timestamp with time zone");
DATA(insert OID = 1780 ( to_date			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 1082 "25 25" _null_ _null_ _null_ _null_  _null_ to_date _null_ _null_ _null_ ));
DESCR("convert text to date");
DATA(insert OID = 1768 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "1186 25" _null_ _null_ _null_ _null_  _null_ interval_to_char _null_ _null_ _null_ ));
DESCR("format interval to text");

DATA(insert OID =  1282 ( quote_ident	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ quote_ident _null_ _null_ _null_ ));
DESCR("quote an identifier for usage in a querystring");
DATA(insert OID =  1283 ( quote_literal    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ quote_literal _null_ _null_ _null_ ));
DESCR("quote a literal for usage in a querystring");
DATA(insert OID =  1285 ( quote_literal    UXNSP UXUID 14 1 0 0 0 f f f f t f s s 1 0 25 "2283" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.quote_literal($1::ux_catalog.text)" _null_ _null_ _null_ ));
DESCR("quote a data value for usage in a querystring");
DATA(insert OID =  1289 ( quote_nullable   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ quote_nullable _null_ _null_ _null_ ));
DESCR("quote a possibly-null literal for usage in a querystring");
DATA(insert OID =  1290 ( quote_nullable   UXNSP UXUID 14 1 0 0 0 f f f f f f s s 1 0 25 "2283" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.quote_nullable($1::ux_catalog.text)" _null_ _null_ _null_ ));
DESCR("quote a possibly-null data value for usage in a querystring");

DATA(insert OID = 1798 (  oidin			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 26 "2275" _null_ _null_ _null_ _null_ _null_ oidin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 1799 (  oidout		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "26" _null_ _null_ _null_ _null_ _null_ oidout _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 3058 ( concat		UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 1 0 25 "2276" "{2276}" "{v}" _null_ _null_ _null_ text_concat _null_ _null_ _null_ ));
DESCR("concatenate values");
DATA(insert OID = 3059 ( concat_ws	UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 2 0 25 "25 2276" "{25,2276}" "{i,v}" _null_ _null_ _null_ text_concat_ws _null_ _null_ _null_ ));
DESCR("concatenate values with separators");
DATA(insert OID = 3060 ( left		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_  _null_ text_left _null_ _null_ _null_ ));
DESCR("extract the first n characters");
DATA(insert OID = 3061 ( right		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_  _null_ text_right _null_ _null_ _null_ ));
DESCR("extract the last n characters");
DATA(insert OID = 3062 ( reverse	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_  text_reverse _null_ _null_ _null_ ));
DESCR("reverse text");
DATA(insert OID = 3539 ( format		UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 2 0 25 "25 2276" "{25,2276}" "{i,v}" _null_ _null_ _null_ text_format _null_ _null_ _null_ ));
DESCR("format text message");
DATA(insert OID = 3540 ( format		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 25 "25" _null_ _null_ _null_ _null_	_null_ text_format_nv _null_ _null_ _null_ ));
DESCR("format text message");

DATA(insert OID = 1810 (  bit_length	   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 23 "17" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.octet_length($1) * 8" _null_ _null_ _null_ ));
DESCR("length in bits");
DATA(insert OID = 1811 (  bit_length	   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 23 "25" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.octet_length($1) * 8" _null_ _null_ _null_ ));
DESCR("length in bits");
DATA(insert OID = 1812 (  bit_length	   UXNSP UXUID 14 1 0 0 0 f f f f t f i s 1 0 23 "1560" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.length($1)" _null_ _null_ _null_ ));
DESCR("length in bits");

/* Selectivity estimators for LIKE and related operators */
DATA(insert OID = 1814 ( iclikesel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ iclikesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of ILIKE");
DATA(insert OID = 1815 ( icnlikesel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ icnlikesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of NOT ILIKE");
DATA(insert OID = 1816 ( iclikejoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ iclikejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of ILIKE");
DATA(insert OID = 1817 ( icnlikejoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ icnlikejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of NOT ILIKE");
DATA(insert OID = 1818 ( regexeqsel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ regexeqsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of regex match");
DATA(insert OID = 1819 ( likesel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ likesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of LIKE");
DATA(insert OID = 1820 ( icregexeqsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ icregexeqsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of case-insensitive regex match");
DATA(insert OID = 1821 ( regexnesel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ regexnesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of regex non-match");
DATA(insert OID = 1822 ( nlikesel			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ nlikesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of NOT LIKE");
DATA(insert OID = 1823 ( icregexnesel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ icregexnesel _null_ _null_ _null_ ));
DESCR("restriction selectivity of case-insensitive regex non-match");
DATA(insert OID = 1824 ( regexeqjoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ regexeqjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of regex match");
DATA(insert OID = 1825 ( likejoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ likejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of LIKE");
DATA(insert OID = 1826 ( icregexeqjoinsel	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ icregexeqjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of case-insensitive regex match");
DATA(insert OID = 1827 ( regexnejoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ regexnejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of regex non-match");
DATA(insert OID = 1828 ( nlikejoinsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ nlikejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of NOT LIKE");
DATA(insert OID = 1829 ( icregexnejoinsel	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_  _null_ icregexnejoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of case-insensitive regex non-match");

/* Aggregate-related functions */
DATA(insert OID = 1830 (  float8_avg	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_avg _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2512 (  float8_var_pop   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_var_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1831 (  float8_var_samp  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_var_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2513 (  float8_stddev_pop UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_stddev_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1832 (  float8_stddev_samp	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_stddev_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1833 (  numeric_accum    UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 1700" _null_ _null_ _null_ _null_ _null_ numeric_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3341 (  numeric_combine	 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ numeric_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 2858 (  numeric_avg_accum    UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 1700" _null_ _null_ _null_ _null_ _null_ numeric_avg_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3337 (  numeric_avg_combine	 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ numeric_avg_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 2740 (  numeric_avg_serialize    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_ numeric_avg_serialize _null_ _null_ _null_ ));
DESCR("aggregate serial function");
DATA(insert OID = 2741 (  numeric_avg_deserialize	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "17 2281" _null_ _null_ _null_ _null_ _null_ numeric_avg_deserialize _null_ _null_ _null_ ));
DESCR("aggregate deserial function");
DATA(insert OID = 3335 (  numeric_serialize    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_ numeric_serialize _null_ _null_ _null_ ));
DESCR("aggregate serial function");
DATA(insert OID = 3336 (  numeric_deserialize	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "17 2281" _null_ _null_ _null_ _null_ _null_ numeric_deserialize _null_ _null_ _null_ ));
DESCR("aggregate deserial function");
DATA(insert OID = 3548 (  numeric_accum_inv    UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 1700" _null_ _null_ _null_ _null_ _null_ numeric_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1834 (  int2_accum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 21" _null_ _null_ _null_ _null_ _null_ int2_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1835 (  int4_accum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 23" _null_ _null_ _null_ _null_ _null_ int4_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1836 (  int8_accum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 20" _null_ _null_ _null_ _null_ _null_ int8_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3338 (  numeric_poly_combine	  UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 3339 (  numeric_poly_serialize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_serialize _null_ _null_ _null_ ));
DESCR("aggregate serial function");
DATA(insert OID = 3340 (  numeric_poly_deserialize	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "17 2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_deserialize _null_ _null_ _null_ ));
DESCR("aggregate deserial function");
DATA(insert OID = 2746 (  int8_avg_accum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 20" _null_ _null_ _null_ _null_ _null_ int8_avg_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3567 (  int2_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 21" _null_ _null_ _null_ _null_ _null_ int2_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3568 (  int4_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 23" _null_ _null_ _null_ _null_ _null_ int4_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3569 (  int8_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 20" _null_ _null_ _null_ _null_ _null_ int8_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3387 (  int8_avg_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 20" _null_ _null_ _null_ _null_ _null_ int8_avg_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 2785 (  int8_avg_combine	  UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ int8_avg_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 2786 (  int8_avg_serialize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_ int8_avg_serialize _null_ _null_ _null_ ));
DESCR("aggregate serial function");
DATA(insert OID = 2787 (  int8_avg_deserialize	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "17 2281" _null_ _null_ _null_ _null_ _null_ int8_avg_deserialize _null_ _null_ _null_ ));
DESCR("aggregate deserial function");
DATA(insert OID = 3324 (  int4_avg_combine	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1016 "1016 1016" _null_ _null_ _null_ _null_ _null_ int4_avg_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 3178 (  numeric_sum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_sum _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1837 (  numeric_avg	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_avg _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2514 (  numeric_var_pop  UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_var_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1838 (  numeric_var_samp UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_var_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2596 (  numeric_stddev_pop UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_	numeric_stddev_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1839 (  numeric_stddev_samp	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_stddev_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1840 (  int2_sum		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 20 "20 21" _null_ _null_ _null_ _null_ _null_ int2_sum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1841 (  int4_sum		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int4_sum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1842 (  int8_sum		   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1700 "1700 20" _null_ _null_ _null_ _null_ _null_ int8_sum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3388 (  numeric_poly_sum	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_sum _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3389 (  numeric_poly_avg	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_avg _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3390 (  numeric_poly_var_pop	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_var_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3391 (  numeric_poly_var_samp UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_var_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3392 (  numeric_poly_stddev_pop UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_stddev_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3393 (  numeric_poly_stddev_samp	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 1700 "2281" _null_ _null_ _null_ _null_ _null_ numeric_poly_stddev_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");

DATA(insert OID = 1843 (  interval_accum   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1187 "1187 1186" _null_ _null_ _null_ _null_ _null_ interval_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3325 (  interval_combine	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1187 "1187 1187" _null_ _null_ _null_ _null_ _null_ interval_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 3549 (  interval_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1187 "1187 1186" _null_ _null_ _null_ _null_ _null_ interval_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1844 (  interval_avg	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1186 "1187" _null_ _null_ _null_ _null_ _null_ interval_avg _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 1962 (  int2_avg_accum   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1016 "1016 21" _null_ _null_ _null_ _null_ _null_ int2_avg_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1963 (  int4_avg_accum   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1016 "1016 23" _null_ _null_ _null_ _null_ _null_ int4_avg_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3570 (  int2_avg_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1016 "1016 21" _null_ _null_ _null_ _null_ _null_ int2_avg_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3571 (  int4_avg_accum_inv   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1016 "1016 23" _null_ _null_ _null_ _null_ _null_ int4_avg_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 1964 (  int8_avg		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1016" _null_ _null_ _null_ _null_ _null_ int8_avg _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3572 (  int2int4_sum	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "1016" _null_ _null_ _null_ _null_ _null_ int2int4_sum _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2805 (  int8inc_float8_float8		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 20 "20 701 701" _null_ _null_ _null_ _null_ _null_ int8inc_float8_float8 _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 2806 (  float8_regr_accum			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1022 "1022 701 701" _null_ _null_ _null_ _null_ _null_ float8_regr_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3342 (  float8_regr_combine		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1022 "1022 1022" _null_ _null_ _null_ _null_ _null_ float8_regr_combine _null_ _null_ _null_ ));
DESCR("aggregate combine function");
DATA(insert OID = 2807 (  float8_regr_sxx			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_sxx _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2808 (  float8_regr_syy			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_syy _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2809 (  float8_regr_sxy			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_sxy _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2810 (  float8_regr_avgx			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_avgx _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2811 (  float8_regr_avgy			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_avgy _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2812 (  float8_regr_r2			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_r2 _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2813 (  float8_regr_slope			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_slope _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2814 (  float8_regr_intercept		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_regr_intercept _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2815 (  float8_covar_pop			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_covar_pop _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2816 (  float8_covar_samp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_covar_samp _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2817 (  float8_corr				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "1022" _null_ _null_ _null_ _null_ _null_ float8_corr _null_ _null_ _null_ ));
DESCR("aggregate final function");

DATA(insert OID = 3535 (  string_agg_transfn		UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2281 "2281 25 25" _null_ _null_ _null_ _null_ _null_ string_agg_transfn _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3536 (  string_agg_finalfn		UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 25 "2281" _null_ _null_ _null_ _null_ _null_ string_agg_finalfn _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3538 (  string_agg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("concatenate aggregate input into a string");
DATA(insert OID = 3543 (  bytea_string_agg_transfn	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2281 "2281 17 17" _null_ _null_ _null_ _null_ _null_ bytea_string_agg_transfn _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3544 (  bytea_string_agg_finalfn	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_ bytea_string_agg_finalfn _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3545 (  string_agg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 17 "17 17" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("concatenate aggregate input into a bytea");

/* To ASCII conversion */
DATA(insert OID = 1845 ( to_ascii	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ to_ascii_default _null_ _null_ _null_ ));
DESCR("encode text from DB encoding to ASCII text");
DATA(insert OID = 1846 ( to_ascii	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 23" _null_ _null_ _null_ _null_ _null_ to_ascii_enc _null_ _null_ _null_ ));
DESCR("encode text from encoding to ASCII text");
DATA(insert OID = 1847 ( to_ascii	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 19" _null_ _null_ _null_ _null_ _null_ to_ascii_encname _null_ _null_ _null_ ));
DESCR("encode text from encoding to ASCII text");

DATA(insert OID = 1848 ( interval_pl_time	UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1083 "1186 1083" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));

DATA(insert OID = 1850 (  int28eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28eq _null_ _null_ _null_ ));
DATA(insert OID = 1851 (  int28ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28ne _null_ _null_ _null_ ));
DATA(insert OID = 1852 (  int28lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28lt _null_ _null_ _null_ ));
DATA(insert OID = 1853 (  int28gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28gt _null_ _null_ _null_ ));
DATA(insert OID = 1854 (  int28le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28le _null_ _null_ _null_ ));
DATA(insert OID = 1855 (  int28ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "21 20" _null_ _null_ _null_ _null_ _null_ int28ge _null_ _null_ _null_ ));

DATA(insert OID = 1856 (  int82eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82eq _null_ _null_ _null_ ));
DATA(insert OID = 1857 (  int82ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82ne _null_ _null_ _null_ ));
DATA(insert OID = 1858 (  int82lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82lt _null_ _null_ _null_ ));
DATA(insert OID = 1859 (  int82gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82gt _null_ _null_ _null_ ));
DATA(insert OID = 1860 (  int82le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82le _null_ _null_ _null_ ));
DATA(insert OID = 1861 (  int82ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "20 21" _null_ _null_ _null_ _null_ _null_ int82ge _null_ _null_ _null_ ));

DATA(insert OID = 1892 (  int2and		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2and _null_ _null_ _null_ ));
DATA(insert OID = 1893 (  int2or		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2or _null_ _null_ _null_ ));
DATA(insert OID = 1894 (  int2xor		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 21" _null_ _null_ _null_ _null_ _null_ int2xor _null_ _null_ _null_ ));
DATA(insert OID = 1895 (  int2not		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ int2not _null_ _null_ _null_ ));
DATA(insert OID = 1896 (  int2shl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 23" _null_ _null_ _null_ _null_ _null_ int2shl _null_ _null_ _null_ ));
DATA(insert OID = 1897 (  int2shr		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 21 "21 23" _null_ _null_ _null_ _null_ _null_ int2shr _null_ _null_ _null_ ));

DATA(insert OID = 1898 (  int4and		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4and _null_ _null_ _null_ ));
DATA(insert OID = 1899 (  int4or		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4or _null_ _null_ _null_ ));
DATA(insert OID = 1900 (  int4xor		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4xor _null_ _null_ _null_ ));
DATA(insert OID = 1901 (  int4not		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4not _null_ _null_ _null_ ));
DATA(insert OID = 1902 (  int4shl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4shl _null_ _null_ _null_ ));
DATA(insert OID = 1903 (  int4shr		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ int4shr _null_ _null_ _null_ ));

DATA(insert OID = 1904 (  int8and		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8and _null_ _null_ _null_ ));
DATA(insert OID = 1905 (  int8or		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8or _null_ _null_ _null_ ));
DATA(insert OID = 1906 (  int8xor		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ int8xor _null_ _null_ _null_ ));
DATA(insert OID = 1907 (  int8not		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8not _null_ _null_ _null_ ));
DATA(insert OID = 1908 (  int8shl		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int8shl _null_ _null_ _null_ ));
DATA(insert OID = 1909 (  int8shr		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "20 23" _null_ _null_ _null_ _null_ _null_ int8shr _null_ _null_ _null_ ));

DATA(insert OID = 1910 (  int8up		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ int8up _null_ _null_ _null_ ));
DATA(insert OID = 1911 (  int2up		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ int2up _null_ _null_ _null_ ));
DATA(insert OID = 1912 (  int4up		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ int4up _null_ _null_ _null_ ));
DATA(insert OID = 1913 (  float4up		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_ float4up _null_ _null_ _null_ ));
DATA(insert OID = 1914 (  float8up		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ float8up _null_ _null_ _null_ ));
DATA(insert OID = 1915 (  numeric_uplus    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ numeric_uplus _null_ _null_ _null_ ));

DATA(insert OID = 1922 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_table_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on relation by username, rel name");
DATA(insert OID = 1923 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_table_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on relation by username, rel oid");
DATA(insert OID = 1924 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_table_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on relation by user oid, rel name");
DATA(insert OID = 1925 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_table_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on relation by user oid, rel oid");
DATA(insert OID = 1926 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_table_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on relation by rel name");
DATA(insert OID = 1927 (  has_table_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_table_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on relation by rel oid");

DATA(insert OID = 2181 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_sequence_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on sequence by username, seq name");
DATA(insert OID = 2182 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_sequence_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on sequence by username, seq oid");
DATA(insert OID = 2183 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_sequence_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on sequence by user oid, seq name");
DATA(insert OID = 2184 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_sequence_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on sequence by user oid, seq oid");
DATA(insert OID = 2185 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_sequence_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on sequence by seq name");
DATA(insert OID = 2186 (  has_sequence_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_sequence_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on sequence by seq oid");

DATA(insert OID = 3012 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "19 25 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_name_name _null_ _null_ _null_ ));
DESCR("user privilege on column by username, rel name, col name");
DATA(insert OID = 3013 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "19 25 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_name_attnum _null_ _null_ _null_ ));
DESCR("user privilege on column by username, rel name, col attnum");
DATA(insert OID = 3014 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "19 26 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_id_name _null_ _null_ _null_ ));
DESCR("user privilege on column by username, rel oid, col name");
DATA(insert OID = 3015 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "19 26 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_id_attnum _null_ _null_ _null_ ));
DESCR("user privilege on column by username, rel oid, col attnum");
DATA(insert OID = 3016 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "26 25 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_name_name _null_ _null_ _null_ ));
DESCR("user privilege on column by user oid, rel name, col name");
DATA(insert OID = 3017 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "26 25 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_name_attnum _null_ _null_ _null_ ));
DESCR("user privilege on column by user oid, rel name, col attnum");
DATA(insert OID = 3018 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "26 26 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_id_name _null_ _null_ _null_ ));
DESCR("user privilege on column by user oid, rel oid, col name");
DATA(insert OID = 3019 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 16 "26 26 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_id_attnum _null_ _null_ _null_ ));
DESCR("user privilege on column by user oid, rel oid, col attnum");
DATA(insert OID = 3020 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "25 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_name _null_ _null_ _null_ ));
DESCR("current user privilege on column by rel name, col name");
DATA(insert OID = 3021 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "25 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_name_attnum _null_ _null_ _null_ ));
DESCR("current user privilege on column by rel name, col attnum");
DATA(insert OID = 3022 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_name _null_ _null_ _null_ ));
DESCR("current user privilege on column by rel oid, col name");
DATA(insert OID = 3023 (  has_column_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 21 25" _null_ _null_ _null_ _null_ _null_ has_column_privilege_id_attnum _null_ _null_ _null_ ));
DESCR("current user privilege on column by rel oid, col attnum");

DATA(insert OID = 3024 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on any column by username, rel name");
DATA(insert OID = 3025 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on any column by username, rel oid");
DATA(insert OID = 3026 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on any column by user oid, rel name");
DATA(insert OID = 3027 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on any column by user oid, rel oid");
DATA(insert OID = 3028 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on any column by rel name");
DATA(insert OID = 3029 (  has_any_column_privilege	   UXNSP UXUID 12 10 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_any_column_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on any column by rel oid");

DATA(insert OID = 3355 (  ux_ndistinct_in	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3361 "2275" _null_ _null_ _null_ _null_ _null_ ux_ndistinct_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3356 (  ux_ndistinct_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3361" _null_ _null_ _null_ _null_ _null_ ux_ndistinct_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3357 (  ux_ndistinct_recv UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 3361 "2281" _null_ _null_ _null_ _null_ _null_ ux_ndistinct_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3358 (  ux_ndistinct_send UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "3361" _null_ _null_ _null_ _null_ _null_ ux_ndistinct_send _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 3404 (  ux_dependencies_in	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3402 "2275" _null_ _null_ _null_ _null_ _null_ ux_dependencies_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3405 (  ux_dependencies_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3402" _null_ _null_ _null_ _null_ _null_ ux_dependencies_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3406 (  ux_dependencies_recv	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 3402 "2281" _null_ _null_ _null_ _null_ _null_ ux_dependencies_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3407 (  ux_dependencies_send	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "3402" _null_ _null_ _null_ _null_ _null_ ux_dependencies_send _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 1928 (  ux_stat_get_numscans			UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_numscans _null_ _null_ _null_ ));
DESCR("statistics: number of scans done for table/index");
DATA(insert OID = 1929 (  ux_stat_get_tuples_returned	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_returned _null_ _null_ _null_ ));
DESCR("statistics: number of tuples read by seqscan");
DATA(insert OID = 1930 (  ux_stat_get_tuples_fetched	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_fetched _null_ _null_ _null_ ));
DESCR("statistics: number of tuples fetched by idxscan");
DATA(insert OID = 1931 (  ux_stat_get_tuples_inserted	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_inserted _null_ _null_ _null_ ));
DESCR("statistics: number of tuples inserted");
DATA(insert OID = 1932 (  ux_stat_get_tuples_updated	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_updated _null_ _null_ _null_ ));
DESCR("statistics: number of tuples updated");
DATA(insert OID = 1933 (  ux_stat_get_tuples_deleted	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_deleted _null_ _null_ _null_ ));
DESCR("statistics: number of tuples deleted");
DATA(insert OID = 1972 (  ux_stat_get_tuples_hot_updated UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_tuples_hot_updated _null_ _null_ _null_ ));
DESCR("statistics: number of tuples hot updated");
DATA(insert OID = 2878 (  ux_stat_get_live_tuples	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_live_tuples _null_ _null_ _null_ ));
DESCR("statistics: number of live tuples");
DATA(insert OID = 2879 (  ux_stat_get_dead_tuples	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_dead_tuples _null_ _null_ _null_ ));
DESCR("statistics: number of dead tuples");
DATA(insert OID = 3177 (  ux_stat_get_mod_since_analyze UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_mod_since_analyze _null_ _null_ _null_ ));
DESCR("statistics: number of tuples changed since last analyze");
DATA(insert OID = 1934 (  ux_stat_get_blocks_fetched	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_blocks_fetched _null_ _null_ _null_ ));
DESCR("statistics: number of blocks fetched");
DATA(insert OID = 1935 (  ux_stat_get_blocks_hit		UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_blocks_hit _null_ _null_ _null_ ));
DESCR("statistics: number of blocks found in cache");
DATA(insert OID = 2781 (  ux_stat_get_last_vacuum_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "26" _null_ _null_ _null_ _null_ _null_	ux_stat_get_last_vacuum_time _null_ _null_ _null_ ));
DESCR("statistics: last manual vacuum time for a table");
DATA(insert OID = 2782 (  ux_stat_get_last_autovacuum_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "26" _null_ _null_ _null_ _null_ _null_	ux_stat_get_last_autovacuum_time _null_ _null_ _null_ ));
DESCR("statistics: last auto vacuum time for a table");
DATA(insert OID = 2783 (  ux_stat_get_last_analyze_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_last_analyze_time _null_ _null_ _null_ ));
DESCR("statistics: last manual analyze time for a table");
DATA(insert OID = 2784 (  ux_stat_get_last_autoanalyze_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_last_autoanalyze_time _null_ _null_ _null_ ));
DESCR("statistics: last auto analyze time for a table");
DATA(insert OID = 3054 ( ux_stat_get_vacuum_count UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_vacuum_count _null_ _null_ _null_ ));
DESCR("statistics: number of manual vacuums for a table");
DATA(insert OID = 3055 ( ux_stat_get_autovacuum_count UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_autovacuum_count _null_ _null_ _null_ ));
DESCR("statistics: number of auto vacuums for a table");
DATA(insert OID = 3056 ( ux_stat_get_analyze_count UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_analyze_count _null_ _null_ _null_ ));
DESCR("statistics: number of manual analyzes for a table");
DATA(insert OID = 3057 ( ux_stat_get_autoanalyze_count UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_autoanalyze_count _null_ _null_ _null_ ));
DESCR("statistics: number of auto analyzes for a table");
DATA(insert OID = 1936 (  ux_stat_get_backend_idset		UXNSP UXUID 12 1 100 0 0 f f f f t t s r 0 0 23 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_idset _null_ _null_ _null_ ));
DESCR("statistics: currently active backend IDs");
DATA(insert OID = 2022 (  ux_stat_get_activity			UXNSP UXUID 12 1 100 0 0 f f f f f t s r 1 0 2249 "23" "{23,26,23,26,25,25,25,25,25,1184,1184,1184,1184,869,25,23,28,28,25,16,25,25,23,16,25}" "{i,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o}" "{pid,datid,pid,usesysid,application_name,state,query,wait_event_type,wait_event,xact_start,query_start,backend_start,state_change,client_addr,client_hostname,client_port,backend_xid,backend_xmin,backend_type,ssl,sslversion,sslcipher,sslbits,sslcompression,sslclientdn}" _null_ _null_ ux_stat_get_activity _null_ _null_ _null_ ));
DESCR("statistics: information about currently active backends");
DATA(insert OID = 3318 (  ux_stat_get_progress_info			  UXNSP UXUID 12 1 100 0 0 f f f f t t s r 1 0 2249 "25" "{25,23,26,26,20,20,20,20,20,20,20,20,20,20}" "{i,o,o,o,o,o,o,o,o,o,o,o,o,o}" "{cmdtype,pid,datid,relid,param1,param2,param3,param4,param5,param6,param7,param8,param9,param10}" _null_ _null_ ux_stat_get_progress_info _null_ _null_ _null_ ));
DESCR("statistics: information about progress of backends running maintenance command");
DATA(insert OID = 3099 (  ux_stat_get_wal_senders	UXNSP UXUID 12 1 10 0 0 f f f f f t s r 0 0 2249 "" "{23,25,3220,3220,3220,3220,1186,1186,1186,23,25}" "{o,o,o,o,o,o,o,o,o,o,o}" "{pid,state,sent_lsn,write_lsn,flush_lsn,replay_lsn,write_lag,flush_lag,replay_lag,sync_priority,sync_state}" _null_ _null_ ux_stat_get_wal_senders _null_ _null_ _null_ ));
DESCR("statistics: information about currently active replication");
DATA(insert OID = 3317 (  ux_stat_get_wal_receiver	UXNSP UXUID 12 1 0 0 0 f f f f f f s r 0 0 2249 "" "{23,25,3220,23,3220,23,1184,1184,3220,1184,25,25}" "{o,o,o,o,o,o,o,o,o,o,o,o}" "{pid,status,receive_start_lsn,receive_start_tli,received_lsn,received_tli,last_msg_send_time,last_msg_receipt_time,latest_end_lsn,latest_end_time,slot_name,conninfo}" _null_ _null_ ux_stat_get_wal_receiver _null_ _null_ _null_ ));
DESCR("statistics: information about WAL receiver");
DATA(insert OID = 6118 (  ux_stat_get_subscription	UXNSP UXUID 12 1 0 0 0 f f f f f f s r 1 0 2249 "26" "{26,26,26,23,3220,1184,1184,3220,1184}" "{i,o,o,o,o,o,o,o,o}" "{subid,subid,relid,pid,received_lsn,last_msg_send_time,last_msg_receipt_time,latest_end_lsn,latest_end_time}" _null_ _null_ ux_stat_get_subscription _null_ _null_ _null_ ));
DESCR("statistics: information about subscription");
DATA(insert OID = 2026 (  ux_backend_pid				UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 23 "" _null_ _null_ _null_ _null_ _null_ ux_backend_pid _null_ _null_ _null_ ));
DESCR("statistics: current backend PID");
DATA(insert OID = 1937 (  ux_stat_get_backend_pid		UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_pid _null_ _null_ _null_ ));
DESCR("statistics: PID of backend");
DATA(insert OID = 1938 (  ux_stat_get_backend_dbid		UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 26 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_dbid _null_ _null_ _null_ ));
DESCR("statistics: database ID of backend");
DATA(insert OID = 1939 (  ux_stat_get_backend_userid	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 26 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_userid _null_ _null_ _null_ ));
DESCR("statistics: user ID of backend");
DATA(insert OID = 1940 (  ux_stat_get_backend_activity	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 25 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_activity _null_ _null_ _null_ ));
DESCR("statistics: current query of backend");
DATA(insert OID = 2788 (  ux_stat_get_backend_wait_event_type	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 25 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_wait_event_type _null_ _null_ _null_ ));
DESCR("statistics: wait event type on which backend is currently waiting");
DATA(insert OID = 2853 (  ux_stat_get_backend_wait_event	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 25 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_wait_event _null_ _null_ _null_ ));
DESCR("statistics: wait event on which backend is currently waiting");
DATA(insert OID = 2094 (  ux_stat_get_backend_activity_start UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_activity_start _null_ _null_ _null_ ));
DESCR("statistics: start time for current query of backend");
DATA(insert OID = 2857 (  ux_stat_get_backend_xact_start UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_xact_start _null_ _null_ _null_ ));
DESCR("statistics: start time for backend's current transaction");
DATA(insert OID = 1391 ( ux_stat_get_backend_start UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_start _null_ _null_ _null_ ));
DESCR("statistics: start time for current backend session");
DATA(insert OID = 1392 ( ux_stat_get_backend_client_addr UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 869 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_client_addr _null_ _null_ _null_ ));
DESCR("statistics: address of client connected to backend");
DATA(insert OID = 1393 ( ux_stat_get_backend_client_port UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ ux_stat_get_backend_client_port _null_ _null_ _null_ ));
DESCR("statistics: port number of client connected to backend");
DATA(insert OID = 1941 (  ux_stat_get_db_numbackends	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 23 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_numbackends _null_ _null_ _null_ ));
DESCR("statistics: number of backends in database");
DATA(insert OID = 1942 (  ux_stat_get_db_xact_commit	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_xact_commit _null_ _null_ _null_ ));
DESCR("statistics: transactions committed");
DATA(insert OID = 1943 (  ux_stat_get_db_xact_rollback	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_xact_rollback _null_ _null_ _null_ ));
DESCR("statistics: transactions rolled back");
DATA(insert OID = 1944 (  ux_stat_get_db_blocks_fetched UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_blocks_fetched _null_ _null_ _null_ ));
DESCR("statistics: blocks fetched for database");
DATA(insert OID = 1945 (  ux_stat_get_db_blocks_hit		UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_blocks_hit _null_ _null_ _null_ ));
DESCR("statistics: blocks found in cache for database");
DATA(insert OID = 2758 (  ux_stat_get_db_tuples_returned UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_tuples_returned _null_ _null_ _null_ ));
DESCR("statistics: tuples returned for database");
DATA(insert OID = 2759 (  ux_stat_get_db_tuples_fetched UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_tuples_fetched _null_ _null_ _null_ ));
DESCR("statistics: tuples fetched for database");
DATA(insert OID = 2760 (  ux_stat_get_db_tuples_inserted UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_tuples_inserted _null_ _null_ _null_ ));
DESCR("statistics: tuples inserted in database");
DATA(insert OID = 2761 (  ux_stat_get_db_tuples_updated UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_tuples_updated _null_ _null_ _null_ ));
DESCR("statistics: tuples updated in database");
DATA(insert OID = 2762 (  ux_stat_get_db_tuples_deleted UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_tuples_deleted _null_ _null_ _null_ ));
DESCR("statistics: tuples deleted in database");
DATA(insert OID = 3065 (  ux_stat_get_db_conflict_tablespace UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_tablespace _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database caused by drop tablespace");
DATA(insert OID = 3066 (  ux_stat_get_db_conflict_lock UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_lock _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database caused by relation lock");
DATA(insert OID = 3067 (  ux_stat_get_db_conflict_snapshot UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_snapshot _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database caused by snapshot expiry");
DATA(insert OID = 3068 (  ux_stat_get_db_conflict_bufferpin UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_bufferpin _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database caused by shared buffer pin");
DATA(insert OID = 3069 (  ux_stat_get_db_conflict_startup_deadlock UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_startup_deadlock _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database caused by buffer deadlock");
DATA(insert OID = 3070 (  ux_stat_get_db_conflict_all UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_conflict_all _null_ _null_ _null_ ));
DESCR("statistics: recovery conflicts in database");
DATA(insert OID = 3152 (  ux_stat_get_db_deadlocks UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_deadlocks _null_ _null_ _null_ ));
DESCR("statistics: deadlocks detected in database");
DATA(insert OID = 3074 (  ux_stat_get_db_stat_reset_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 1184 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_stat_reset_time _null_ _null_ _null_ ));
DESCR("statistics: last reset for a database");
DATA(insert OID = 3150 (  ux_stat_get_db_temp_files UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_temp_files _null_ _null_ _null_ ));
DESCR("statistics: number of temporary files written");
DATA(insert OID = 3151 (  ux_stat_get_db_temp_bytes UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_temp_bytes _null_ _null_ _null_ ));
DESCR("statistics: number of bytes in temporary files written");
DATA(insert OID = 2844 (  ux_stat_get_db_blk_read_time	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_blk_read_time _null_ _null_ _null_ ));
DESCR("statistics: block read time, in milliseconds");
DATA(insert OID = 2845 (  ux_stat_get_db_blk_write_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_db_blk_write_time _null_ _null_ _null_ ));
DESCR("statistics: block write time, in milliseconds");
DATA(insert OID = 3195 (  ux_stat_get_archiver		UXNSP UXUID 12 1 0 0 0 f f f f f f s r 0 0 2249 "" "{20,25,1184,20,25,1184,1184}" "{o,o,o,o,o,o,o}" "{archived_count,last_archived_wal,last_archived_time,failed_count,last_failed_wal,last_failed_time,stats_reset}" _null_ _null_ ux_stat_get_archiver _null_ _null_ _null_ ));
DESCR("statistics: information about WAL archiver");
DATA(insert OID = 2769 ( ux_stat_get_bgwriter_timed_checkpoints UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_timed_checkpoints _null_ _null_ _null_ ));
DESCR("statistics: number of timed checkpoints started by the bgwriter");
DATA(insert OID = 2770 ( ux_stat_get_bgwriter_requested_checkpoints UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_requested_checkpoints _null_ _null_ _null_ ));
DESCR("statistics: number of backend requested checkpoints started by the bgwriter");
DATA(insert OID = 2771 ( ux_stat_get_bgwriter_buf_written_checkpoints UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_buf_written_checkpoints _null_ _null_ _null_ ));
DESCR("statistics: number of buffers written by the bgwriter during checkpoints");
DATA(insert OID = 2772 ( ux_stat_get_bgwriter_buf_written_clean UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_buf_written_clean _null_ _null_ _null_ ));
DESCR("statistics: number of buffers written by the bgwriter for cleaning dirty buffers");
DATA(insert OID = 2773 ( ux_stat_get_bgwriter_maxwritten_clean UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_maxwritten_clean _null_ _null_ _null_ ));
DESCR("statistics: number of times the bgwriter stopped processing when it had written too many buffers while cleaning");
DATA(insert OID = 3075 ( ux_stat_get_bgwriter_stat_reset_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_bgwriter_stat_reset_time _null_ _null_ _null_ ));
DESCR("statistics: last reset for the bgwriter");
DATA(insert OID = 3160 ( ux_stat_get_checkpoint_write_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 701 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_checkpoint_write_time _null_ _null_ _null_ ));
DESCR("statistics: checkpoint time spent writing buffers to disk, in milliseconds");
DATA(insert OID = 3161 ( ux_stat_get_checkpoint_sync_time UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 701 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_checkpoint_sync_time _null_ _null_ _null_ ));
DESCR("statistics: checkpoint time spent synchronizing buffers to disk, in milliseconds");
DATA(insert OID = 2775 ( ux_stat_get_buf_written_backend UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_buf_written_backend _null_ _null_ _null_ ));
DESCR("statistics: number of buffers written by backends");
DATA(insert OID = 3063 ( ux_stat_get_buf_fsync_backend UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_buf_fsync_backend _null_ _null_ _null_ ));
DESCR("statistics: number of backend buffer writes that did their own fsync");
DATA(insert OID = 2859 ( ux_stat_get_buf_alloc			UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 20 "" _null_ _null_ _null_ _null_ _null_ ux_stat_get_buf_alloc _null_ _null_ _null_ ));
DESCR("statistics: number of buffer allocations");

DATA(insert OID = 2978 (  ux_stat_get_function_calls		UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_function_calls _null_ _null_ _null_ ));
DESCR("statistics: number of function calls");
DATA(insert OID = 2979 (  ux_stat_get_function_total_time	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_function_total_time _null_ _null_ _null_ ));
DESCR("statistics: total execution time of function, in milliseconds");
DATA(insert OID = 2980 (  ux_stat_get_function_self_time	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_function_self_time _null_ _null_ _null_ ));
DESCR("statistics: self execution time of function, in milliseconds");

DATA(insert OID = 3037 (  ux_stat_get_xact_numscans				UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_numscans _null_ _null_ _null_ ));
DESCR("statistics: number of scans done for table/index in current transaction");
DATA(insert OID = 3038 (  ux_stat_get_xact_tuples_returned		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_returned _null_ _null_ _null_ ));
DESCR("statistics: number of tuples read by seqscan in current transaction");
DATA(insert OID = 3039 (  ux_stat_get_xact_tuples_fetched		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_fetched _null_ _null_ _null_ ));
DESCR("statistics: number of tuples fetched by idxscan in current transaction");
DATA(insert OID = 3040 (  ux_stat_get_xact_tuples_inserted		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_inserted _null_ _null_ _null_ ));
DESCR("statistics: number of tuples inserted in current transaction");
DATA(insert OID = 3041 (  ux_stat_get_xact_tuples_updated		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_updated _null_ _null_ _null_ ));
DESCR("statistics: number of tuples updated in current transaction");
DATA(insert OID = 3042 (  ux_stat_get_xact_tuples_deleted		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_deleted _null_ _null_ _null_ ));
DESCR("statistics: number of tuples deleted in current transaction");
DATA(insert OID = 3043 (  ux_stat_get_xact_tuples_hot_updated	UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_tuples_hot_updated _null_ _null_ _null_ ));
DESCR("statistics: number of tuples hot updated in current transaction");
DATA(insert OID = 3044 (  ux_stat_get_xact_blocks_fetched		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_blocks_fetched _null_ _null_ _null_ ));
DESCR("statistics: number of blocks fetched in current transaction");
DATA(insert OID = 3045 (  ux_stat_get_xact_blocks_hit			UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_blocks_hit _null_ _null_ _null_ ));
DESCR("statistics: number of blocks found in cache in current transaction");
DATA(insert OID = 3046 (  ux_stat_get_xact_function_calls		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_function_calls _null_ _null_ _null_ ));
DESCR("statistics: number of function calls in current transaction");
DATA(insert OID = 3047 (  ux_stat_get_xact_function_total_time	UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_function_total_time _null_ _null_ _null_ ));
DESCR("statistics: total execution time of function in current transaction, in milliseconds");
DATA(insert OID = 3048 (  ux_stat_get_xact_function_self_time	UXNSP UXUID 12 1 0 0 0 f f f f t f v r 1 0 701 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_get_xact_function_self_time _null_ _null_ _null_ ));
DESCR("statistics: self execution time of function in current transaction, in milliseconds");

DATA(insert OID = 3788 (  ux_stat_get_snapshot_timestamp UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 1184 "" _null_ _null_ _null_ _null_ _null_	ux_stat_get_snapshot_timestamp _null_ _null_ _null_ ));
DESCR("statistics: timestamp of the current statistics snapshot");
DATA(insert OID = 2230 (  ux_stat_clear_snapshot		UXNSP UXUID 12 1 0 0 0 f f f f f f v r 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_stat_clear_snapshot _null_ _null_ _null_ ));
DESCR("statistics: discard current transaction's statistics snapshot");
DATA(insert OID = 2274 (  ux_stat_reset					UXNSP UXUID 12 1 0 0 0 f f f f f f v s 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_stat_reset _null_ _null_ _null_ ));
DESCR("statistics: reset collected statistics for current database");
DATA(insert OID = 3775 (  ux_stat_reset_shared			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 2278 "25" _null_ _null_ _null_ _null_ _null_ ux_stat_reset_shared _null_ _null_ _null_ ));
DESCR("statistics: reset collected statistics shared across the cluster");
DATA(insert OID = 3776 (  ux_stat_reset_single_table_counters	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_reset_single_table_counters _null_ _null_ _null_ ));
DESCR("statistics: reset collected statistics for a single table or index in the current database");
DATA(insert OID = 3777 (  ux_stat_reset_single_function_counters	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ ux_stat_reset_single_function_counters _null_ _null_ _null_ ));
DESCR("statistics: reset collected statistics for a single function in the current database");

DATA(insert OID = 3163 (  ux_trigger_depth				UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 23 "" _null_ _null_ _null_ _null_ _null_ ux_trigger_depth _null_ _null_ _null_ ));
DESCR("current trigger depth");

DATA(insert OID = 3778 ( ux_tablespace_location UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_tablespace_location _null_ _null_ _null_ ));
DESCR("tablespace location");

DATA(insert OID = 1946 (  encode						UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "17 25" _null_ _null_ _null_ _null_ _null_ binary_encode _null_ _null_ _null_ ));
DESCR("convert bytea value into some ascii-only text string");
DATA(insert OID = 1947 (  decode						UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "25 25" _null_ _null_ _null_ _null_ _null_ binary_decode _null_ _null_ _null_ ));
DESCR("convert ascii-encoded text string into bytea value");

DATA(insert OID = 1948 (  byteaeq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteaeq _null_ _null_ _null_ ));
DATA(insert OID = 1949 (  bytealt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ bytealt _null_ _null_ _null_ ));
DATA(insert OID = 1950 (  byteale		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteale _null_ _null_ _null_ ));
DATA(insert OID = 1951 (  byteagt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteagt _null_ _null_ _null_ ));
DATA(insert OID = 1952 (  byteage		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteage _null_ _null_ _null_ ));
DATA(insert OID = 1953 (  byteane		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteane _null_ _null_ _null_ ));
DATA(insert OID = 1954 (  byteacmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "17 17" _null_ _null_ _null_ _null_ _null_ byteacmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3331 (  bytea_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ bytea_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");

DATA(insert OID = 3917 (  timestamp_transform UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ timestamp_transform _null_ _null_ _null_ ));
DESCR("transform a timestamp length coercion");
DATA(insert OID = 3944 (  time_transform   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ time_transform _null_ _null_ _null_ ));
DESCR("transform a time length coercion");

DATA(insert OID = 1961 (  timestamp		   UXNSP UXUID 12 1 0 0 timestamp_transform f f f f t f i s 2 0 1114 "1114 23" _null_ _null_ _null_ _null_ _null_ timestamp_scale _null_ _null_ _null_ ));
DESCR("adjust timestamp precision");

DATA(insert OID = 1965 (  oidlarger		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 26 "26 26" _null_ _null_ _null_ _null_ _null_ oidlarger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 1966 (  oidsmaller	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 26 "26 26" _null_ _null_ _null_ _null_ _null_ oidsmaller _null_ _null_ _null_ ));
DESCR("smaller of two");

DATA(insert OID = 1967 (  timestamptz	   UXNSP UXUID 12 1 0 0 timestamp_transform f f f f t f i s 2 0 1184 "1184 23" _null_ _null_ _null_ _null_ _null_ timestamptz_scale _null_ _null_ _null_ ));
DESCR("adjust timestamptz precision");
DATA(insert OID = 1968 (  time			   UXNSP UXUID 12 1 0 0 time_transform f f f f t f i s 2 0 1083 "1083 23" _null_ _null_ _null_ _null_ _null_ time_scale _null_ _null_ _null_ ));
DESCR("adjust time precision");
DATA(insert OID = 1969 (  timetz		   UXNSP UXUID 12 1 0 0 time_transform f f f f t f i s 2 0 1266 "1266 23" _null_ _null_ _null_ _null_ _null_ timetz_scale _null_ _null_ _null_ ));
DESCR("adjust time with time zone precision");

DATA(insert OID = 2003 (  textanycat	   UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 25 "25 2776" _null_ _null_ _null_ _null_ _null_ "select $1 || $2::ux_catalog.text" _null_ _null_ _null_ ));
DATA(insert OID = 2004 (  anytextcat	   UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 25 "2776 25" _null_ _null_ _null_ _null_ _null_ "select $1::ux_catalog.text || $2" _null_ _null_ _null_ ));

DATA(insert OID = 2005 (  bytealike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ bytealike _null_ _null_ _null_ ));
DATA(insert OID = 2006 (  byteanlike	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteanlike _null_ _null_ _null_ ));
DATA(insert OID = 2007 (  like			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ bytealike _null_ _null_ _null_ ));
DESCR("matches LIKE expression");
DATA(insert OID = 2008 (  notlike		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "17 17" _null_ _null_ _null_ _null_ _null_ byteanlike _null_ _null_ _null_ ));
DESCR("does not match LIKE expression");
DATA(insert OID = 2009 (  like_escape	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "17 17" _null_ _null_ _null_ _null_ _null_ like_escape_bytea _null_ _null_ _null_ ));
DESCR("convert LIKE pattern to use backslash escapes");
DATA(insert OID = 2010 (  length		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "17" _null_ _null_ _null_ _null_ _null_ byteaoctetlen _null_ _null_ _null_ ));
DESCR("octet length");
DATA(insert OID = 2011 (  byteacat		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "17 17" _null_ _null_ _null_ _null_ _null_ byteacat _null_ _null_ _null_ ));
DATA(insert OID = 2012 (  substring		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 17 "17 23 23" _null_ _null_ _null_ _null_ _null_	bytea_substr _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID = 2013 (  substring		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "17 23" _null_ _null_ _null_ _null_ _null_ bytea_substr_no_len _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID = 2085 (  substr		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 17 "17 23 23" _null_ _null_ _null_ _null_ _null_	bytea_substr _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID = 2086 (  substr		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "17 23" _null_ _null_ _null_ _null_ _null_ bytea_substr_no_len _null_ _null_ _null_ ));
DESCR("extract portion of string");
DATA(insert OID = 2014 (  position		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "17 17" _null_ _null_ _null_ _null_ _null_ byteapos _null_ _null_ _null_ ));
DESCR("position of substring");
DATA(insert OID = 2015 (  btrim			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 17 "17 17" _null_ _null_ _null_ _null_ _null_ byteatrim _null_ _null_ _null_ ));
DESCR("trim both ends of string");

DATA(insert OID = 2019 (  time				UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1083 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_time _null_ _null_ _null_ ));
DESCR("convert timestamp with time zone to time");
DATA(insert OID = 2020 (  date_trunc		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "25 1114" _null_ _null_ _null_ _null_ _null_ timestamp_trunc _null_ _null_ _null_ ));
DESCR("truncate timestamp to specified units");
DATA(insert OID = 2021 (  date_part			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "25 1114" _null_ _null_ _null_ _null_ _null_ timestamp_part _null_ _null_ _null_ ));
DESCR("extract field from timestamp");
DATA(insert OID = 2023 (  timestamp			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1114 "702" _null_ _null_ _null_ _null_ _null_ abstime_timestamp _null_ _null_ _null_ ));
DESCR("convert abstime to timestamp");
DATA(insert OID = 2024 (  timestamp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1114 "1082" _null_ _null_ _null_ _null_ _null_ date_timestamp _null_ _null_ _null_ ));
DESCR("convert date to timestamp");
DATA(insert OID = 2025 (  timestamp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1082 1083" _null_ _null_ _null_ _null_ _null_	datetime_timestamp _null_ _null_ _null_ ));
DESCR("convert date and time to timestamp");
DATA(insert OID = 2027 (  timestamp			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1114 "1184" _null_ _null_ _null_ _null_ _null_ timestamptz_timestamp _null_ _null_ _null_ ));
DESCR("convert timestamp with time zone to timestamp");
DATA(insert OID = 2028 (  timestamptz		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1184 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_timestamptz _null_ _null_ _null_ ));
DESCR("convert timestamp to timestamp with time zone");
DATA(insert OID = 2029 (  date				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1082 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_date _null_ _null_ _null_ ));
DESCR("convert timestamp to date");
DATA(insert OID = 2030 (  abstime			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 702 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_abstime _null_ _null_ _null_ ));
DESCR("convert timestamp to abstime");
DATA(insert OID = 2031 (  timestamp_mi		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1114 1114" _null_ _null_ _null_ _null_ _null_	timestamp_mi _null_ _null_ _null_ ));
DATA(insert OID = 2032 (  timestamp_pl_interval UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1114 1186" _null_ _null_ _null_ _null_ _null_	timestamp_pl_interval _null_ _null_ _null_ ));
DATA(insert OID = 2033 (  timestamp_mi_interval UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1114 1186" _null_ _null_ _null_ _null_ _null_	timestamp_mi_interval _null_ _null_ _null_ ));
DATA(insert OID = 2035 (  timestamp_smaller UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1114 1114" _null_ _null_ _null_ _null_ _null_	timestamp_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 2036 (  timestamp_larger	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1114 1114" _null_ _null_ _null_ _null_ _null_	timestamp_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 2037 (  timezone			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 1266 "25 1266" _null_ _null_ _null_ _null_ _null_ timetz_zone _null_ _null_ _null_ ));
DESCR("adjust time with time zone to new zone");
DATA(insert OID = 2038 (  timezone			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1266 "1186 1266" _null_ _null_ _null_ _null_ _null_	timetz_izone _null_ _null_ _null_ ));
DESCR("adjust time with time zone to new zone");
DATA(insert OID = 2039 (  timestamp_hash	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_hash _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 2041 ( overlaps			UXNSP UXUID 12 1 0 0 0 f f f f f f i s 4 0 16 "1114 1114 1114 1114" _null_ _null_ _null_ _null_ _null_	overlaps_timestamp _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 2042 ( overlaps			UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1114 1186 1114 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 2043 ( overlaps			UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1114 1114 1114 1186" _null_ _null_ _null_ _null_ _null_ "select ($1, $2) overlaps ($3, ($3 + $4))" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 2044 ( overlaps			UXNSP UXUID 14 1 0 0 0 f f f f f f i s 4 0 16 "1114 1186 1114 1114" _null_ _null_ _null_ _null_ _null_ "select ($1, ($1 + $2)) overlaps ($3, $4)" _null_ _null_ _null_ ));
DESCR("intervals overlap?");
DATA(insert OID = 2045 (  timestamp_cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3137 (  timestamp_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ timestamp_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 2046 (  time				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1083 "1266" _null_ _null_ _null_ _null_ _null_ timetz_time _null_ _null_ _null_ ));
DESCR("convert time with time zone to time");
DATA(insert OID = 2047 (  timetz			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 1266 "1083" _null_ _null_ _null_ _null_ _null_ time_timetz _null_ _null_ _null_ ));
DESCR("convert time to time with time zone");
DATA(insert OID = 2048 (  isfinite			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "1114" _null_ _null_ _null_ _null_ _null_ timestamp_finite _null_ _null_ _null_ ));
DESCR("finite timestamp?");
DATA(insert OID = 2049 ( to_char			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "1114 25" _null_ _null_ _null_ _null_  _null_ timestamp_to_char _null_ _null_ _null_ ));
DESCR("format timestamp to text");
DATA(insert OID = 2052 (  timestamp_eq		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_eq _null_ _null_ _null_ ));
DATA(insert OID = 2053 (  timestamp_ne		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_ne _null_ _null_ _null_ ));
DATA(insert OID = 2054 (  timestamp_lt		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_lt _null_ _null_ _null_ ));
DATA(insert OID = 2055 (  timestamp_le		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_le _null_ _null_ _null_ ));
DATA(insert OID = 2056 (  timestamp_ge		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_ge _null_ _null_ _null_ ));
DATA(insert OID = 2057 (  timestamp_gt		UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "1114 1114" _null_ _null_ _null_ _null_ _null_ timestamp_gt _null_ _null_ _null_ ));
DATA(insert OID = 2058 (  age				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1186 "1114 1114" _null_ _null_ _null_ _null_ _null_	timestamp_age _null_ _null_ _null_ ));
DESCR("date difference preserving months and years");
DATA(insert OID = 2059 (  age				UXNSP UXUID 14 1 0 0 0 f f f f t f s s 1 0 1186 "1114" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.age(cast(current_date as timestamp without time zone), $1)" _null_ _null_ _null_ ));
DESCR("date difference from today preserving months and years");

DATA(insert OID = 2069 (  timezone			UXNSP UXUID 12 1 0 0 timestamp_zone_transform f f f f t f i s 2 0 1184 "25 1114" _null_ _null_ _null_ _null_ _null_ timestamp_zone _null_ _null_ _null_ ));
DESCR("adjust timestamp to new time zone");
DATA(insert OID = 2070 (  timezone			UXNSP UXUID 12 1 0 0 timestamp_izone_transform f f f f t f i s 2 0 1184 "1186 1114" _null_ _null_ _null_ _null_ _null_	timestamp_izone _null_ _null_ _null_ ));
DESCR("adjust timestamp to new time zone");
DATA(insert OID = 2071 (  date_pl_interval	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1082 1186" _null_ _null_ _null_ _null_ _null_	date_pl_interval _null_ _null_ _null_ ));
DATA(insert OID = 2072 (  date_mi_interval	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1114 "1082 1186" _null_ _null_ _null_ _null_ _null_	date_mi_interval _null_ _null_ _null_ ));

DATA(insert OID = 2073 (  substring			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25 "25 25" _null_ _null_ _null_ _null_ _null_ textregexsubstr _null_ _null_ _null_ ));
DESCR("extract text matching regular expression");
DATA(insert OID = 2074 (  substring			UXNSP UXUID 14 1 0 0 0 f f f f t f i s 3 0 25 "25 25 25" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.substring($1, ux_catalog.similar_escape($2, $3))" _null_ _null_ _null_ ));
DESCR("extract text matching SQL99 regular expression");

DATA(insert OID = 2075 (  bit				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1560 "20 23" _null_ _null_ _null_ _null_ _null_	bitfromint8 _null_ _null_ _null_ ));
DESCR("convert int8 to bitstring");
DATA(insert OID = 2076 (  int8				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "1560" _null_ _null_ _null_ _null_ _null_ bittoint8 _null_ _null_ _null_ ));
DESCR("convert bitstring to int8");

DATA(insert OID = 2077 (  current_setting	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ show_config_by_name _null_ _null_ _null_ ));
DESCR("SHOW X as a function");
DATA(insert OID = 3294 (  current_setting	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "25 16" _null_ _null_ _null_ _null_ _null_ show_config_by_name_missing_ok _null_ _null_ _null_ ));
DESCR("SHOW X as a function, optionally no error for missing variable");
DATA(insert OID = 2078 (  set_config		UXNSP UXUID 12 1 0 0 0 f f f f f f v u 3 0 25 "25 25 16" _null_ _null_ _null_ _null_ _null_ set_config_by_name _null_ _null_ _null_ ));
DESCR("SET X as a function");
DATA(insert OID = 2084 (  ux_show_all_settings	UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 0 0 2249 "" "{25,25,25,25,25,25,25,25,25,25,25,1009,25,25,25,23,16}" "{o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o}" "{name,setting,unit,category,short_desc,extra_desc,context,vartype,source,min_val,max_val,enumvals,boot_val,reset_val,sourcefile,sourceline,pending_restart}" _null_ _null_ show_all_settings _null_ _null_ _null_ ));
DESCR("SHOW ALL as a function");
DATA(insert OID = 3329 (  ux_show_all_file_settings UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 0 0 2249 "" "{25,23,23,25,25,16,25}" "{o,o,o,o,o,o,o}" "{sourcefile,sourceline,seqno,name,setting,applied,error}" _null_ _null_ show_all_file_settings _null_ _null_ _null_ ));
DESCR("show config file settings");
DATA(insert OID = 3401 (  ux_hba_file_rules UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 0 0 2249 "" "{23,25,1009,1009,25,25,25,1009,25}" "{o,o,o,o,o,o,o,o,o}" "{line_number,type,database,user_name,address,netmask,auth_method,options,error}" _null_ _null_ ux_hba_file_rules _null_ _null_ _null_ ));
DESCR("show ux_hba.conf rules");
DATA(insert OID = 1371 (  ux_lock_status   UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 0 0 2249 "" "{25,26,26,23,21,25,28,26,26,21,25,23,25,16,16}" "{o,o,o,o,o,o,o,o,o,o,o,o,o,o,o}" "{locktype,database,relation,page,tuple,virtualxid,transactionid,classid,objid,objsubid,virtualtransaction,pid,mode,granted,fastpath}" _null_ _null_ ux_lock_status _null_ _null_ _null_ ));
DESCR("view system lock information");
DATA(insert OID = 2561 (  ux_blocking_pids UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 1007 "23" _null_ _null_ _null_ _null_ _null_ ux_blocking_pids _null_ _null_ _null_ ));
DESCR("get array of PIDs of sessions blocking specified backend PID from acquiring a heavyweight lock");
DATA(insert OID = 3376 (  ux_safe_snapshot_blocking_pids UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 1007 "23" _null_ _null_ _null_ _null_ _null_ ux_safe_snapshot_blocking_pids _null_ _null_ _null_ ));
DESCR("get array of PIDs of sessions blocking specified backend PID from acquiring a safe snapshot");
DATA(insert OID = 3378 (  ux_isolation_test_session_is_blocked UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 16 "23 1007" _null_ _null_ _null_ _null_ _null_ ux_isolation_test_session_is_blocked _null_ _null_ _null_ ));
DESCR("isolationtester support function");
DATA(insert OID = 1065 (  ux_prepared_xact UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 0 0 2249 "" "{28,25,1184,26,26}" "{o,o,o,o,o}" "{transaction,gid,prepared,ownerid,dbid}" _null_ _null_ ux_prepared_xact _null_ _null_ _null_ ));
DESCR("view two-phase transactions");
DATA(insert OID = 3819 (  ux_get_multixact_members UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 1 0 2249 "28" "{28,28,25}" "{i,o,o}" "{multixid,xid,mode}" _null_ _null_ ux_get_multixact_members _null_ _null_ _null_ ));
DESCR("view members of a multixactid");

DATA(insert OID = 3581 ( ux_xact_commit_timestamp UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 1184 "28" _null_ _null_ _null_ _null_ _null_ ux_xact_commit_timestamp _null_ _null_ _null_ ));
DESCR("get commit timestamp of a transaction");

DATA(insert OID = 3583 ( ux_last_committed_xact UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2249 "" "{28,1184}" "{o,o}" "{xid,timestamp}" _null_ _null_ ux_last_committed_xact _null_ _null_ _null_ ));
DESCR("get transaction Id and commit timestamp of latest transaction commit");

DATA(insert OID = 3537 (  ux_describe_object		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 25 "26 26 23" _null_ _null_ _null_ _null_ _null_ ux_describe_object _null_ _null_ _null_ ));
DESCR("get identification of SQL object");

DATA(insert OID = 3839 (  ux_identify_object		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2249 "26 26 23" "{26,26,23,25,25,25,25}" "{i,i,i,o,o,o,o}" "{classid,objid,objsubid,type,schema,name,identity}" _null_ _null_ ux_identify_object _null_ _null_ _null_ ));
DESCR("get machine-parseable identification of SQL object");

DATA(insert OID = 3382 (  ux_identify_object_as_address UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2249 "26 26 23" "{26,26,23,25,1009,1009}" "{i,i,i,o,o,o}" "{classid,objid,objsubid,type,object_names,object_args}" _null_ _null_ ux_identify_object_as_address _null_ _null_ _null_ ));
DESCR("get identification of SQL object for ux_get_object_address()");

DATA(insert OID = 3954 (  ux_get_object_address    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2249 "25 1009 1009" "{25,1009,1009,26,26,23}" "{i,i,i,o,o,o}" "{type,name,args,classid,objid,objsubid}" _null_ _null_ ux_get_object_address _null_ _null_ _null_ ));
DESCR("get OID-based object address from name/args arrays");

DATA(insert OID = 2079 (  ux_table_is_visible		UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_table_is_visible _null_ _null_ _null_ ));
DESCR("is table visible in search path?");
DATA(insert OID = 2080 (  ux_type_is_visible		UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_type_is_visible _null_ _null_ _null_ ));
DESCR("is type visible in search path?");
DATA(insert OID = 2081 (  ux_function_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_function_is_visible _null_ _null_ _null_ ));
DESCR("is function visible in search path?");
DATA(insert OID = 2082 (  ux_operator_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_operator_is_visible _null_ _null_ _null_ ));
DESCR("is operator visible in search path?");
DATA(insert OID = 2083 (  ux_opclass_is_visible		UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_opclass_is_visible _null_ _null_ _null_ ));
DESCR("is opclass visible in search path?");
DATA(insert OID = 3829 (  ux_opfamily_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_opfamily_is_visible _null_ _null_ _null_ ));
DESCR("is opfamily visible in search path?");
DATA(insert OID = 2093 (  ux_conversion_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_conversion_is_visible _null_ _null_ _null_ ));
DESCR("is conversion visible in search path?");
DATA(insert OID = 3403 (  ux_statistics_obj_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_statistics_obj_is_visible _null_ _null_ _null_ ));
DESCR("is statistics object visible in search path?");
DATA(insert OID = 3756 (  ux_ts_parser_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_ts_parser_is_visible _null_ _null_ _null_ ));
DESCR("is text search parser visible in search path?");
DATA(insert OID = 3757 (  ux_ts_dict_is_visible		UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_ts_dict_is_visible _null_ _null_ _null_ ));
DESCR("is text search dictionary visible in search path?");
DATA(insert OID = 3768 (  ux_ts_template_is_visible UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_ts_template_is_visible _null_ _null_ _null_ ));
DESCR("is text search template visible in search path?");
DATA(insert OID = 3758 (  ux_ts_config_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_ts_config_is_visible _null_ _null_ _null_ ));
DESCR("is text search configuration visible in search path?");
DATA(insert OID = 3815 (  ux_collation_is_visible	UXNSP UXUID 12 10 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_collation_is_visible _null_ _null_ _null_ ));
DESCR("is collation visible in search path?");

DATA(insert OID = 2854 (  ux_my_temp_schema			UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 26 "" _null_ _null_ _null_ _null_ _null_ ux_my_temp_schema _null_ _null_ _null_ ));
DESCR("get OID of current session's temp schema, if any");
DATA(insert OID = 2855 (  ux_is_other_temp_schema	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_ ux_is_other_temp_schema _null_ _null_ _null_ ));
DESCR("is schema another session's temp schema?");

DATA(insert OID = 2171 ( ux_cancel_backend		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 16 "23" _null_ _null_ _null_ _null_ _null_ ux_cancel_backend _null_ _null_ _null_ ));
DESCR("cancel a server process' current query");
DATA(insert OID = 2096 ( ux_terminate_backend		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 16 "23" _null_ _null_ _null_ _null_ _null_ ux_terminate_backend _null_ _null_ _null_ ));
DESCR("terminate a server process");
DATA(insert OID = 2172 ( ux_start_backup		UXNSP UXUID 12 1 0 0 0 f f f f t f v r 3 0 3220 "25 16 16" _null_ _null_ _null_ _null_ _null_ ux_start_backup _null_ _null_ _null_ ));
DESCR("prepare for taking an online backup");
DATA(insert OID = 2173 ( ux_stop_backup			UXNSP UXUID 12 1 0 0 0 f f f f t f v r 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_stop_backup _null_ _null_ _null_ ));
DESCR("finish taking an online backup");
DATA(insert OID = 2739 ( ux_stop_backup			UXNSP UXUID 12 1 1 0 0 f f f f t t v r 2 0 2249 "16 16" "{16,16,3220,25,25}" "{i,i,o,o,o}" "{exclusive,wait_for_archive,lsn,labelfile,spcmapfile}" _null_ _null_ ux_stop_backup_v2 _null_ _null_ _null_ ));
DESCR("finish taking an online backup");
DATA(insert OID = 3813 ( ux_is_in_backup		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_is_in_backup _null_ _null_ _null_ ));
DESCR("true if server is in online backup");
DATA(insert OID = 3814 ( ux_backup_start_time		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ ux_backup_start_time _null_ _null_ _null_ ));
DESCR("start time of an online backup");
DATA(insert OID = 2848 ( ux_switch_wal			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_switch_wal _null_ _null_ _null_ ));
DESCR("switch to new wal file");
DATA(insert OID = 3098 ( ux_create_restore_point	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 3220 "25" _null_ _null_ _null_ _null_ _null_ ux_create_restore_point _null_ _null_ _null_ ));
DESCR("create a named restore point");
DATA(insert OID = 2849 ( ux_current_wal_lsn UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_current_wal_lsn _null_ _null_ _null_ ));
DESCR("current wal write location");
DATA(insert OID = 2852 ( ux_current_wal_insert_lsn UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_current_wal_insert_lsn _null_ _null_ _null_ ));
DESCR("current wal insert location");
DATA(insert OID = 3330 ( ux_current_wal_flush_lsn UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_current_wal_flush_lsn _null_ _null_ _null_ ));
DESCR("current wal flush location");
DATA(insert OID = 2850 ( ux_walfile_name_offset UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2249 "3220" "{3220,25,23}" "{i,o,o}" "{lsn,file_name,file_offset}" _null_ _null_ ux_walfile_name_offset _null_ _null_ _null_ ));
DESCR("wal filename and byte offset, given a wal location");
DATA(insert OID = 2851 ( ux_walfile_name			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "3220" _null_ _null_ _null_ _null_ _null_ ux_walfile_name _null_ _null_ _null_ ));
DESCR("wal filename, given a wal location");

DATA(insert OID = 3165 ( ux_wal_lsn_diff		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_wal_lsn_diff _null_ _null_ _null_ ));
DESCR("difference in bytes, given two wal locations");

DATA(insert OID = 3809 ( ux_export_snapshot		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 0 0 25 "" _null_ _null_ _null_ _null_ _null_ ux_export_snapshot _null_ _null_ _null_ ));
DESCR("export a snapshot");

DATA(insert OID = 3810 (  ux_is_in_recovery		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_is_in_recovery _null_ _null_ _null_ ));
DESCR("true if server is in recovery");

DATA(insert OID = 3820 ( ux_last_wal_receive_lsn	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_last_wal_receive_lsn _null_ _null_ _null_ ));
DESCR("current wal flush location");
DATA(insert OID = 3821 ( ux_last_wal_replay_lsn UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 3220 "" _null_ _null_ _null_ _null_ _null_ ux_last_wal_replay_lsn _null_ _null_ _null_ ));
DESCR("last wal replay location");
DATA(insert OID = 3830 ( ux_last_xact_replay_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ ux_last_xact_replay_timestamp _null_ _null_ _null_ ));
DESCR("timestamp of last replay xact");

DATA(insert OID = 3071 ( ux_wal_replay_pause		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_wal_replay_pause _null_ _null_ _null_ ));
DESCR("pause wal replay");
DATA(insert OID = 3072 ( ux_wal_replay_resume		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_wal_replay_resume _null_ _null_ _null_ ));
DESCR("resume wal replay, if it was paused");
DATA(insert OID = 3073 ( ux_is_wal_replay_paused	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_is_wal_replay_paused _null_ _null_ _null_ ));
DESCR("true if wal replay is paused");

DATA(insert OID = 2621 ( ux_reload_conf			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_reload_conf _null_ _null_ _null_ ));
DESCR("reload configuration files");
DATA(insert OID = 2622 ( ux_rotate_logfile		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_rotate_logfile _null_ _null_ _null_ ));
DESCR("rotate log file");
DATA(insert OID = 3800 ( ux_current_logfile				UXNSP UXUID 12 1 0 0 0 f f f f f f v s 0 0 25 "" _null_ _null_ _null_ _null_ _null_ ux_current_logfile _null_ _null_ _null_ ));
DESCR("current logging collector file location");
DATA(insert OID = 3801 ( ux_current_logfile				UXNSP UXUID 12 1 0 0 0 f f f f f f v s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ ux_current_logfile_1arg _null_ _null_ _null_ ));
DESCR("current logging collector file location");

DATA(insert OID = 2623 ( ux_stat_file		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 2249 "25" "{25,20,1184,1184,1184,1184,16}" "{i,o,o,o,o,o,o}" "{filename,size,access,modification,change,creation,isdir}" _null_ _null_ ux_stat_file_1arg _null_ _null_ _null_ ));
DESCR("get information about file");
DATA(insert OID = 3307 ( ux_stat_file		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 2249 "25 16" "{25,16,20,1184,1184,1184,1184,16}" "{i,i,o,o,o,o,o,o}" "{filename,missing_ok,size,access,modification,change,creation,isdir}" _null_ _null_ ux_stat_file _null_ _null_ _null_ ));
DESCR("get information about file");
DATA(insert OID = 2624 ( ux_read_file		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 3 0 25 "25 20 20" _null_ _null_ _null_ _null_ _null_ ux_read_file_off_len _null_ _null_ _null_ ));
DESCR("read text from a file");
DATA(insert OID = 3293 ( ux_read_file		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 4 0 25 "25 20 20 16" _null_ _null_ _null_ _null_ _null_ ux_read_file _null_ _null_ _null_ ));
DESCR("read text from a file");
DATA(insert OID = 3826 ( ux_read_file		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ ux_read_file_all _null_ _null_ _null_ ));
DESCR("read text from a file");
DATA(insert OID = 3827 ( ux_read_binary_file	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 3 0 17 "25 20 20" _null_ _null_ _null_ _null_ _null_ ux_read_binary_file_off_len _null_ _null_ _null_ ));
DESCR("read bytea from a file");
DATA(insert OID = 3295 ( ux_read_binary_file	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 4 0 17 "25 20 20 16" _null_ _null_ _null_ _null_ _null_ ux_read_binary_file _null_ _null_ _null_ ));
DESCR("read bytea from a file");
DATA(insert OID = 3828 ( ux_read_binary_file	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 17 "25" _null_ _null_ _null_ _null_ _null_ ux_read_binary_file_all _null_ _null_ _null_ ));
DESCR("read bytea from a file");
DATA(insert OID = 2625 ( ux_ls_dir			UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ ux_ls_dir_1arg _null_ _null_ _null_ ));
DESCR("list all files in a directory");
DATA(insert OID = 3297 ( ux_ls_dir			UXNSP UXUID 12 1 1000 0 0 f f f f t t v s 3 0 25 "25 16 16" _null_ _null_ _null_ _null_ _null_ ux_ls_dir _null_ _null_ _null_ ));
DESCR("list all files in a directory");
DATA(insert OID = 2626 ( ux_sleep			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 2278 "701" _null_ _null_ _null_ _null_ _null_ ux_sleep _null_ _null_ _null_ ));
DESCR("sleep for the specified time in seconds");
DATA(insert OID = 3935 ( ux_sleep_for			UXNSP UXUID 14 1 0 0 0 f f f f t f v s 1 0 2278 "1186" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.ux_sleep(extract(epoch from ux_catalog.clock_timestamp() operator(ux_catalog.+) $1) operator(ux_catalog.-) extract(epoch from ux_catalog.clock_timestamp()))" _null_ _null_ _null_ ));
DESCR("sleep for the specified interval");
DATA(insert OID = 3936 ( ux_sleep_until			UXNSP UXUID 14 1 0 0 0 f f f f t f v s 1 0 2278 "1184" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.ux_sleep(extract(epoch from $1) operator(ux_catalog.-) extract(epoch from ux_catalog.clock_timestamp()))" _null_ _null_ _null_ ));
DESCR("sleep until the specified time");

DATA(insert OID = 2971 (  text				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "16" _null_ _null_ _null_ _null_ _null_ booltext _null_ _null_ _null_ ));
DESCR("convert boolean to text");

/* Aggregates (moved here from ux_aggregate for 7.3) */

DATA(insert OID = 2100 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as numeric of all bigint values");
DATA(insert OID = 2101 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as numeric of all integer values");
DATA(insert OID = 2102 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as numeric of all smallint values");
DATA(insert OID = 2103 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as numeric of all numeric values");
DATA(insert OID = 2104 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as float8 of all float4 values");
DATA(insert OID = 2105 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as float8 of all float8 values");
DATA(insert OID = 2106 (  avg				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("the average (arithmetic mean) as interval of all interval values");

DATA(insert OID = 2107 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as numeric across all bigint input values");
DATA(insert OID = 2108 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as bigint across all integer input values");
DATA(insert OID = 2109 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as bigint across all smallint input values");
DATA(insert OID = 2110 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as float4 across all float4 input values");
DATA(insert OID = 2111 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as float8 across all float8 input values");
DATA(insert OID = 2112 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 790 "790" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as money across all money input values");
DATA(insert OID = 2113 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as interval across all interval input values");
DATA(insert OID = 2114 (  sum				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum as numeric across all numeric input values");

DATA(insert OID = 2115 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all bigint input values");
DATA(insert OID = 2116 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all integer input values");
DATA(insert OID = 2117 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all smallint input values");
DATA(insert OID = 2118 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 26 "26" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all oid input values");
DATA(insert OID = 2119 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all float4 input values");
DATA(insert OID = 2120 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all float8 input values");
DATA(insert OID = 2121 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 702 "702" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all abstime input values");
DATA(insert OID = 2122 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1082 "1082" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all date input values");
DATA(insert OID = 2123 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1083 "1083" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all time input values");
DATA(insert OID = 2124 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1266 "1266" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all time with time zone input values");
DATA(insert OID = 2125 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 790 "790" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all money input values");
DATA(insert OID = 2126 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1114 "1114" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all timestamp input values");
DATA(insert OID = 2127 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1184 "1184" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all timestamp with time zone input values");
DATA(insert OID = 2128 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all interval input values");
DATA(insert OID = 2129 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all text input values");
DATA(insert OID = 2130 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all numeric input values");
DATA(insert OID = 2050 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 2277 "2277" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all anyarray input values");
DATA(insert OID = 2244 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1042 "1042" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all bpchar input values");
DATA(insert OID = 2797 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 27 "27" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all tid input values");
DATA(insert OID = 3564 (  max				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all inet input values");

DATA(insert OID = 2131 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all bigint input values");
DATA(insert OID = 2132 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all integer input values");
DATA(insert OID = 2133 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all smallint input values");
DATA(insert OID = 2134 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 26 "26" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all oid input values");
DATA(insert OID = 2135 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 700 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all float4 input values");
DATA(insert OID = 2136 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all float8 input values");
DATA(insert OID = 2137 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 702 "702" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all abstime input values");
DATA(insert OID = 2138 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1082 "1082" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all date input values");
DATA(insert OID = 2139 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1083 "1083" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all time input values");
DATA(insert OID = 2140 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1266 "1266" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all time with time zone input values");
DATA(insert OID = 2141 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 790 "790" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all money input values");
DATA(insert OID = 2142 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1114 "1114" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all timestamp input values");
DATA(insert OID = 2143 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1184 "1184" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all timestamp with time zone input values");
DATA(insert OID = 2144 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1186 "1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all interval input values");
DATA(insert OID = 2145 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all text values");
DATA(insert OID = 2146 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all numeric input values");
DATA(insert OID = 2051 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 2277 "2277" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all anyarray input values");
DATA(insert OID = 2245 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1042 "1042" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all bpchar input values");
DATA(insert OID = 2798 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 27 "27" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all tid input values");
DATA(insert OID = 3565 (  min				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 869 "869" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all inet input values");

/* count has two forms: count(any) and count(*) */
DATA(insert OID = 2147 (  count				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "2276" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("number of input rows for which the input expression is not null");
DATA(insert OID = 2803 (  count				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 0 0 20 "" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("number of input rows");

DATA(insert OID = 2718 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of bigint input values (square of the population standard deviation)");
DATA(insert OID = 2719 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of integer input values (square of the population standard deviation)");
DATA(insert OID = 2720 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of smallint input values (square of the population standard deviation)");
DATA(insert OID = 2721 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of float4 input values (square of the population standard deviation)");
DATA(insert OID = 2722 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of float8 input values (square of the population standard deviation)");
DATA(insert OID = 2723 (  var_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population variance of numeric input values (square of the population standard deviation)");

DATA(insert OID = 2641 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of bigint input values (square of the sample standard deviation)");
DATA(insert OID = 2642 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of integer input values (square of the sample standard deviation)");
DATA(insert OID = 2643 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of smallint input values (square of the sample standard deviation)");
DATA(insert OID = 2644 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of float4 input values (square of the sample standard deviation)");

DATA(insert OID = 2645 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of float8 input values (square of the sample standard deviation)");
DATA(insert OID = 2646 (  var_samp			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample variance of numeric input values (square of the sample standard deviation)");

DATA(insert OID = 2148 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");
DATA(insert OID = 2149 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");
DATA(insert OID = 2150 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");
DATA(insert OID = 2151 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");
DATA(insert OID = 2152 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");
DATA(insert OID = 2153 (  variance			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for var_samp");

DATA(insert OID = 2724 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of bigint input values");
DATA(insert OID = 2725 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of integer input values");
DATA(insert OID = 2726 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of smallint input values");
DATA(insert OID = 2727 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of float4 input values");
DATA(insert OID = 2728 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of float8 input values");
DATA(insert OID = 2729 (  stddev_pop		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population standard deviation of numeric input values");

DATA(insert OID = 2712 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of bigint input values");
DATA(insert OID = 2713 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of integer input values");
DATA(insert OID = 2714 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of smallint input values");
DATA(insert OID = 2715 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of float4 input values");
DATA(insert OID = 2716 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of float8 input values");
DATA(insert OID = 2717 (  stddev_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample standard deviation of numeric input values");

DATA(insert OID = 2154 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");
DATA(insert OID = 2155 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");
DATA(insert OID = 2156 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");
DATA(insert OID = 2157 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");
DATA(insert OID = 2158 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 701 "701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");
DATA(insert OID = 2159 (  stddev			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1700 "1700" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("historical alias for stddev_samp");

DATA(insert OID = 2818 (  regr_count		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 20 "701 701" _null_ _null_ _null_ _null_  _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("number of input rows in which both expressions are not null");
DATA(insert OID = 2819 (  regr_sxx			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum of squares of the independent variable (sum(X^2) - sum(X)^2/N)");
DATA(insert OID = 2820 (  regr_syy			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum of squares of the dependent variable (sum(Y^2) - sum(Y)^2/N)");
DATA(insert OID = 2821 (  regr_sxy			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sum of products of independent times dependent variable (sum(X*Y) - sum(X) * sum(Y)/N)");
DATA(insert OID = 2822 (  regr_avgx			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("average of the independent variable (sum(X)/N)");
DATA(insert OID = 2823 (  regr_avgy			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("average of the dependent variable (sum(Y)/N)");
DATA(insert OID = 2824 (  regr_r2			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("square of the correlation coefficient");
DATA(insert OID = 2825 (  regr_slope		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("slope of the least-squares-fit linear equation determined by the (X, Y) pairs");
DATA(insert OID = 2826 (  regr_intercept	UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("y-intercept of the least-squares-fit linear equation determined by the (X, Y) pairs");

DATA(insert OID = 2827 (  covar_pop			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("population covariance");
DATA(insert OID = 2828 (  covar_samp		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("sample covariance");
DATA(insert OID = 2829 (  corr				UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("correlation coefficient");

DATA(insert OID = 2160 ( text_pattern_lt	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_pattern_lt _null_ _null_ _null_ ));
DATA(insert OID = 2161 ( text_pattern_le	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_pattern_le _null_ _null_ _null_ ));
DATA(insert OID = 2163 ( text_pattern_ge	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_pattern_ge _null_ _null_ _null_ ));
DATA(insert OID = 2164 ( text_pattern_gt	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ text_pattern_gt _null_ _null_ _null_ ));
DATA(insert OID = 2166 ( bttext_pattern_cmp  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ bttext_pattern_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3332 ( bttext_pattern_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ bttext_pattern_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");

DATA(insert OID = 2174 ( bpchar_pattern_lt	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_pattern_lt _null_ _null_ _null_ ));
DATA(insert OID = 2175 ( bpchar_pattern_le	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_pattern_le _null_ _null_ _null_ ));
DATA(insert OID = 2177 ( bpchar_pattern_ge	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_pattern_ge _null_ _null_ _null_ ));
DATA(insert OID = 2178 ( bpchar_pattern_gt	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1042 1042" _null_ _null_ _null_ _null_ _null_ bpchar_pattern_gt _null_ _null_ _null_ ));
DATA(insert OID = 2180 ( btbpchar_pattern_cmp UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1042 1042" _null_ _null_ _null_ _null_ _null_ btbpchar_pattern_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3333 ( btbpchar_pattern_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ btbpchar_pattern_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");

DATA(insert OID = 2188 ( btint48cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 20" _null_ _null_ _null_ _null_ _null_ btint48cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2189 ( btint84cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "20 23" _null_ _null_ _null_ _null_ _null_ btint84cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2190 ( btint24cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 23" _null_ _null_ _null_ _null_ _null_ btint24cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2191 ( btint42cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "23 21" _null_ _null_ _null_ _null_ _null_ btint42cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2192 ( btint28cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "21 20" _null_ _null_ _null_ _null_ _null_ btint28cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2193 ( btint82cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "20 21" _null_ _null_ _null_ _null_ _null_ btint82cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2194 ( btfloat48cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "700 701" _null_ _null_ _null_ _null_ _null_ btfloat48cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 2195 ( btfloat84cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "701 700" _null_ _null_ _null_ _null_ _null_ btfloat84cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 2212 (  regprocedurein	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2202 "2275" _null_ _null_ _null_ _null_ _null_ regprocedurein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2213 (  regprocedureout	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2202" _null_ _null_ _null_ _null_ _null_ regprocedureout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2214 (  regoperin			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2203 "2275" _null_ _null_ _null_ _null_ _null_ regoperin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2215 (  regoperout		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2203" _null_ _null_ _null_ _null_ _null_ regoperout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3492 (  to_regoper		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2203 "25" _null_ _null_ _null_ _null_ _null_ to_regoper _null_ _null_ _null_ ));
DESCR("convert operator name to regoper");
DATA(insert OID = 3476 (  to_regoperator	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2204 "25" _null_ _null_ _null_ _null_ _null_ to_regoperator _null_ _null_ _null_ ));
DESCR("convert operator name to regoperator");
DATA(insert OID = 2216 (  regoperatorin		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2204 "2275" _null_ _null_ _null_ _null_ _null_ regoperatorin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2217 (  regoperatorout	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2204" _null_ _null_ _null_ _null_ _null_ regoperatorout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2218 (  regclassin		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2205 "2275" _null_ _null_ _null_ _null_ _null_ regclassin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2219 (  regclassout		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2205" _null_ _null_ _null_ _null_ _null_ regclassout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3495 (  to_regclass		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2205 "25" _null_ _null_ _null_ _null_ _null_ to_regclass _null_ _null_ _null_ ));
DESCR("convert classname to regclass");
DATA(insert OID = 2220 (  regtypein			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2206 "2275" _null_ _null_ _null_ _null_ _null_ regtypein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2221 (  regtypeout		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2206" _null_ _null_ _null_ _null_ _null_ regtypeout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3493 (  to_regtype		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2206 "25" _null_ _null_ _null_ _null_ _null_ to_regtype _null_ _null_ _null_ ));
DESCR("convert type name to regtype");
DATA(insert OID = 1079 (  regclass			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2205 "25" _null_ _null_ _null_ _null_ _null_ text_regclass _null_ _null_ _null_ ));
DESCR("convert text to regclass");

DATA(insert OID = 4098 (  regrolein			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 4096 "2275" _null_ _null_ _null_ _null_ _null_ regrolein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4092 (  regroleout		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "4096" _null_ _null_ _null_ _null_ _null_ regroleout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4093 (  to_regrole		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 4096 "25" _null_ _null_ _null_ _null_ _null_ to_regrole _null_ _null_ _null_ ));
DESCR("convert role name to regrole");

DATA(insert OID = 4084 (  regnamespacein	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 4089 "2275" _null_ _null_ _null_ _null_ _null_ regnamespacein _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4085 (  regnamespaceout	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "4089" _null_ _null_ _null_ _null_ _null_ regnamespaceout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4086 (  to_regnamespace	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 4089 "25" _null_ _null_ _null_ _null_ _null_ to_regnamespace _null_ _null_ _null_ ));
DESCR("convert namespace name to regnamespace");

DATA(insert OID = 1268 (  parse_ident		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1009 "25 16" _null_ _null_ "{str,strict}" _null_ _null_ parse_ident _null_ _null_ _null_ ));
DESCR("parse qualified identifier to array of identifiers");

DATA(insert OID = 2246 ( fmgr_internal_validator UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ fmgr_internal_validator _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 2247 ( fmgr_c_validator	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ fmgr_c_validator _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 2248 ( fmgr_sql_validator UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ fmgr_sql_validator _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 2250 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_database_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on database by username, database name");
DATA(insert OID = 2251 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_database_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on database by username, database oid");
DATA(insert OID = 2252 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_database_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on database by user oid, database name");
DATA(insert OID = 2253 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_database_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on database by user oid, database oid");
DATA(insert OID = 2254 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_database_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on database by database name");
DATA(insert OID = 2255 (  has_database_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_database_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on database by database oid");

DATA(insert OID = 2256 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_function_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on function by username, function name");
DATA(insert OID = 2257 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_function_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on function by username, function oid");
DATA(insert OID = 2258 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_function_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on function by user oid, function name");
DATA(insert OID = 2259 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_function_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on function by user oid, function oid");
DATA(insert OID = 2260 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_function_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on function by function name");
DATA(insert OID = 2261 (  has_function_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_function_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on function by function oid");

DATA(insert OID = 2262 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_language_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on language by username, language name");
DATA(insert OID = 2263 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_language_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on language by username, language oid");
DATA(insert OID = 2264 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_language_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on language by user oid, language name");
DATA(insert OID = 2265 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_language_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on language by user oid, language oid");
DATA(insert OID = 2266 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_language_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on language by language name");
DATA(insert OID = 2267 (  has_language_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_language_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on language by language oid");

DATA(insert OID = 2268 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_schema_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on schema by username, schema name");
DATA(insert OID = 2269 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_schema_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on schema by username, schema oid");
DATA(insert OID = 2270 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_schema_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on schema by user oid, schema name");
DATA(insert OID = 2271 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_schema_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on schema by user oid, schema oid");
DATA(insert OID = 2272 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_schema_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on schema by schema name");
DATA(insert OID = 2273 (  has_schema_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_schema_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on schema by schema oid");

DATA(insert OID = 2390 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_tablespace_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on tablespace by username, tablespace name");
DATA(insert OID = 2391 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_tablespace_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on tablespace by username, tablespace oid");
DATA(insert OID = 2392 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_tablespace_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on tablespace by user oid, tablespace name");
DATA(insert OID = 2393 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_tablespace_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on tablespace by user oid, tablespace oid");
DATA(insert OID = 2394 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_tablespace_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on tablespace by tablespace name");
DATA(insert OID = 2395 (  has_tablespace_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_tablespace_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on tablespace by tablespace oid");

DATA(insert OID = 3000 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_foreign_data_wrapper_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on foreign data wrapper by username, foreign data wrapper name");
DATA(insert OID = 3001 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_foreign_data_wrapper_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on foreign data wrapper by username, foreign data wrapper oid");
DATA(insert OID = 3002 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_foreign_data_wrapper_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on foreign data wrapper by user oid, foreign data wrapper name");
DATA(insert OID = 3003 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_foreign_data_wrapper_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on foreign data wrapper by user oid, foreign data wrapper oid");
DATA(insert OID = 3004 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_foreign_data_wrapper_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on foreign data wrapper by foreign data wrapper name");
DATA(insert OID = 3005 (  has_foreign_data_wrapper_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_foreign_data_wrapper_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on foreign data wrapper by foreign data wrapper oid");

DATA(insert OID = 3006 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_server_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on server by username, server name");
DATA(insert OID = 3007 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_server_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on server by username, server oid");
DATA(insert OID = 3008 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_server_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on server by user oid, server name");
DATA(insert OID = 3009 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_server_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on server by user oid, server oid");
DATA(insert OID = 3010 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_server_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on server by server name");
DATA(insert OID = 3011 (  has_server_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_server_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on server by server oid");

DATA(insert OID = 3138 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 25 25" _null_ _null_ _null_ _null_ _null_	has_type_privilege_name_name _null_ _null_ _null_ ));
DESCR("user privilege on type by username, type name");
DATA(insert OID = 3139 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_	has_type_privilege_name_id _null_ _null_ _null_ ));
DESCR("user privilege on type by username, type oid");
DATA(insert OID = 3140 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 25 25" _null_ _null_ _null_ _null_ _null_	has_type_privilege_id_name _null_ _null_ _null_ ));
DESCR("user privilege on type by user oid, type name");
DATA(insert OID = 3141 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_	has_type_privilege_id_id _null_ _null_ _null_ ));
DESCR("user privilege on type by user oid, type oid");
DATA(insert OID = 3142 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ has_type_privilege_name _null_ _null_ _null_ ));
DESCR("current user privilege on type by type name");
DATA(insert OID = 3143 (  has_type_privilege		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ has_type_privilege_id _null_ _null_ _null_ ));
DESCR("current user privilege on type by type oid");

DATA(insert OID = 2705 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 19 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_name_name _null_ _null_ _null_ ));
DESCR("user privilege on role by username, role name");
DATA(insert OID = 2706 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "19 26 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_name_id _null_ _null_ _null_ ));
DESCR("user privilege on role by username, role oid");
DATA(insert OID = 2707 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 19 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_id_name _null_ _null_ _null_ ));
DESCR("user privilege on role by user oid, role name");
DATA(insert OID = 2708 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 16 "26 26 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_id_id _null_ _null_ _null_ ));
DESCR("user privilege on role by user oid, role oid");
DATA(insert OID = 2709 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "19 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_name _null_ _null_ _null_ ));
DESCR("current user privilege on role by role name");
DATA(insert OID = 2710 (  ux_has_role		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "26 25" _null_ _null_ _null_ _null_ _null_ ux_has_role_id _null_ _null_ _null_ ));
DESCR("current user privilege on role by role oid");

DATA(insert OID = 1269 (  ux_column_size		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 23 "2276" _null_ _null_ _null_ _null_ _null_ ux_column_size _null_ _null_ _null_ ));
DESCR("bytes required to store the value, perhaps with compression");
DATA(insert OID = 2322 ( ux_tablespace_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_tablespace_size_oid _null_ _null_ _null_ ));
DESCR("total disk space usage for the specified tablespace");
DATA(insert OID = 2323 ( ux_tablespace_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "19" _null_ _null_ _null_ _null_ _null_ ux_tablespace_size_name _null_ _null_ _null_ ));
DESCR("total disk space usage for the specified tablespace");
DATA(insert OID = 2324 ( ux_database_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "26" _null_ _null_ _null_ _null_ _null_ ux_database_size_oid _null_ _null_ _null_ ));
DESCR("total disk space usage for the specified database");
DATA(insert OID = 2168 ( ux_database_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "19" _null_ _null_ _null_ _null_ _null_ ux_database_size_name _null_ _null_ _null_ ));
DESCR("total disk space usage for the specified database");
DATA(insert OID = 2325 ( ux_relation_size		UXNSP UXUID 14 1 0 0 0 f f f f t f v s 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.ux_relation_size($1, ''main'')" _null_ _null_ _null_ ));
DESCR("disk space usage for the main fork of the specified table or index");
DATA(insert OID = 2332 ( ux_relation_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 2 0 20 "2205 25" _null_ _null_ _null_ _null_ _null_ ux_relation_size _null_ _null_ _null_ ));
DESCR("disk space usage for the specified fork of a table or index");
DATA(insert OID = 2286 ( ux_total_relation_size UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ ux_total_relation_size _null_ _null_ _null_ ));
DESCR("total disk space usage for the specified table and associated indexes");
DATA(insert OID = 2288 ( ux_size_pretty			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "20" _null_ _null_ _null_ _null_ _null_ ux_size_pretty _null_ _null_ _null_ ));
DESCR("convert a long int to a human readable text using size units");
DATA(insert OID = 3166 ( ux_size_pretty			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "1700" _null_ _null_ _null_ _null_ _null_ ux_size_pretty_numeric _null_ _null_ _null_ ));
DESCR("convert a numeric to a human readable text using size units");
DATA(insert OID = 3334 ( ux_size_bytes			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "25" _null_ _null_ _null_ _null_ _null_ ux_size_bytes _null_ _null_ _null_ ));
DESCR("convert a size in human-readable format with size units into bytes");
DATA(insert OID = 2997 ( ux_table_size			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ ux_table_size _null_ _null_ _null_ ));
DESCR("disk space usage for the specified table, including TOAST, free space and visibility map");
DATA(insert OID = 2998 ( ux_indexes_size		UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ ux_indexes_size _null_ _null_ _null_ ));
DESCR("disk space usage for all indexes attached to the specified table");
DATA(insert OID = 2999 ( ux_relation_filenode	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 26 "2205" _null_ _null_ _null_ _null_ _null_ ux_relation_filenode _null_ _null_ _null_ ));
DESCR("filenode identifier of relation");
DATA(insert OID = 3454 ( ux_filenode_relation UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 2205 "26 26" _null_ _null_ _null_ _null_ _null_ ux_filenode_relation _null_ _null_ _null_ ));
DESCR("relation OID for filenode and tablespace");
DATA(insert OID = 3034 ( ux_relation_filepath	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "2205" _null_ _null_ _null_ _null_ _null_ ux_relation_filepath _null_ _null_ _null_ ));
DESCR("file path of relation");

DATA(insert OID = 2316 ( uxsinodb_fdw_validator UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1009 26" _null_ _null_ _null_ _null_ _null_ uxsinodb_fdw_validator _null_ _null_ _null_));
DESCR("(internal)");

DATA(insert OID = 2290 (  record_in			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2249 "2275 26 23" _null_ _null_ _null_ _null_ _null_ record_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2291 (  record_out		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2249" _null_ _null_ _null_ _null_ _null_ record_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2292 (  cstring_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2275" _null_ _null_ _null_ _null_ _null_ cstring_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2293 (  cstring_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2275" _null_ _null_ _null_ _null_ _null_ cstring_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2294 (  any_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2276 "2275" _null_ _null_ _null_ _null_ _null_ any_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2295 (  any_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2276" _null_ _null_ _null_ _null_ _null_ any_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2296 (  anyarray_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2277 "2275" _null_ _null_ _null_ _null_ _null_ anyarray_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2297 (  anyarray_out		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2277" _null_ _null_ _null_ _null_ _null_ anyarray_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2298 (  void_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2275" _null_ _null_ _null_ _null_ _null_ void_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2299 (  void_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2278" _null_ _null_ _null_ _null_ _null_ void_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2300 (  trigger_in		UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 2279 "2275" _null_ _null_ _null_ _null_ _null_ trigger_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2301 (  trigger_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2279" _null_ _null_ _null_ _null_ _null_ trigger_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3594 (  event_trigger_in	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 3838 "2275" _null_ _null_ _null_ _null_ _null_ event_trigger_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3595 (  event_trigger_out UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3838" _null_ _null_ _null_ _null_ _null_ event_trigger_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2302 (  language_handler_in	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 2280 "2275" _null_ _null_ _null_ _null_ _null_ language_handler_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2303 (  language_handler_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2280" _null_ _null_ _null_ _null_ _null_ language_handler_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2304 (  internal_in		UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 2281 "2275" _null_ _null_ _null_ _null_ _null_ internal_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2305 (  internal_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2281" _null_ _null_ _null_ _null_ _null_ internal_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2306 (  opaque_in			UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 2282 "2275" _null_ _null_ _null_ _null_ _null_ opaque_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2307 (  opaque_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2282" _null_ _null_ _null_ _null_ _null_ opaque_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2312 (  anyelement_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2283 "2275" _null_ _null_ _null_ _null_ _null_ anyelement_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2313 (  anyelement_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2283" _null_ _null_ _null_ _null_ _null_ anyelement_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2398 (  shell_in			UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 2282 "2275" _null_ _null_ _null_ _null_ _null_ shell_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2399 (  shell_out			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2282" _null_ _null_ _null_ _null_ _null_ shell_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2597 (  domain_in			UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 2276 "2275 26 23" _null_ _null_ _null_ _null_ _null_ domain_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2598 (  domain_recv		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 2276 "2281 26 23" _null_ _null_ _null_ _null_ _null_ domain_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2777 (  anynonarray_in	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2776 "2275" _null_ _null_ _null_ _null_ _null_ anynonarray_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2778 (  anynonarray_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2776" _null_ _null_ _null_ _null_ _null_ anynonarray_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3116 (  fdw_handler_in	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 3115 "2275" _null_ _null_ _null_ _null_ _null_ fdw_handler_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3117 (  fdw_handler_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3115" _null_ _null_ _null_ _null_ _null_ fdw_handler_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 326  (  index_am_handler_in	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 325 "2275" _null_ _null_ _null_ _null_ _null_ index_am_handler_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 327  (  index_am_handler_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "325" _null_ _null_ _null_ _null_ _null_ index_am_handler_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3311 (  tsm_handler_in	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 3310 "2275" _null_ _null_ _null_ _null_ _null_ tsm_handler_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3312 (  tsm_handler_out	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3310" _null_ _null_ _null_ _null_ _null_ tsm_handler_out _null_ _null_ _null_ ));
DESCR("I/O");

/* tablesample method handlers */
DATA(insert OID = 3313 (  bernoulli			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 3310 "2281" _null_ _null_ _null_ _null_ _null_ tsm_bernoulli_handler _null_ _null_ _null_ ));
DESCR("BERNOULLI tablesample method handler");
DATA(insert OID = 3314 (  system			UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 3310 "2281" _null_ _null_ _null_ _null_ _null_ tsm_system_handler _null_ _null_ _null_ ));
DESCR("SYSTEM tablesample method handler");

/* cryptographic */
DATA(insert OID =  2311 (  md5	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "25" _null_ _null_ _null_ _null_ _null_ md5_text _null_ _null_ _null_ ));
DESCR("MD5 hash");
DATA(insert OID =  2321 (  md5	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "17" _null_ _null_ _null_ _null_ _null_ md5_bytea _null_ _null_ _null_ ));
DESCR("MD5 hash");

/* crosstype operations for date vs. timestamp and timestamptz */
DATA(insert OID = 2338 (  date_lt_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_lt_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2339 (  date_le_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_le_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2340 (  date_eq_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_eq_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2341 (  date_gt_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_gt_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2342 (  date_ge_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_ge_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2343 (  date_ne_timestamp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_ne_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2344 (  date_cmp_timestamp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1082 1114" _null_ _null_ _null_ _null_ _null_ date_cmp_timestamp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 2351 (  date_lt_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_lt_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2352 (  date_le_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_le_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2353 (  date_eq_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_eq_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2354 (  date_gt_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_gt_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2355 (  date_ge_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_ge_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2356 (  date_ne_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_ne_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2357 (  date_cmp_timestamptz	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 23 "1082 1184" _null_ _null_ _null_ _null_ _null_ date_cmp_timestamptz _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 2364 (  timestamp_lt_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_lt_date _null_ _null_ _null_ ));
DATA(insert OID = 2365 (  timestamp_le_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_le_date _null_ _null_ _null_ ));
DATA(insert OID = 2366 (  timestamp_eq_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_eq_date _null_ _null_ _null_ ));
DATA(insert OID = 2367 (  timestamp_gt_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_gt_date _null_ _null_ _null_ ));
DATA(insert OID = 2368 (  timestamp_ge_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_ge_date _null_ _null_ _null_ ));
DATA(insert OID = 2369 (  timestamp_ne_date		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_ne_date _null_ _null_ _null_ ));
DATA(insert OID = 2370 (  timestamp_cmp_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "1114 1082" _null_ _null_ _null_ _null_ _null_ timestamp_cmp_date _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 2377 (  timestamptz_lt_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_lt_date _null_ _null_ _null_ ));
DATA(insert OID = 2378 (  timestamptz_le_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_le_date _null_ _null_ _null_ ));
DATA(insert OID = 2379 (  timestamptz_eq_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_eq_date _null_ _null_ _null_ ));
DATA(insert OID = 2380 (  timestamptz_gt_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_gt_date _null_ _null_ _null_ ));
DATA(insert OID = 2381 (  timestamptz_ge_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_ge_date _null_ _null_ _null_ ));
DATA(insert OID = 2382 (  timestamptz_ne_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_ne_date _null_ _null_ _null_ ));
DATA(insert OID = 2383 (  timestamptz_cmp_date	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 23 "1184 1082" _null_ _null_ _null_ _null_ _null_ timestamptz_cmp_date _null_ _null_ _null_ ));
DESCR("less-equal-greater");

/* crosstype operations for timestamp vs. timestamptz */
DATA(insert OID = 2520 (  timestamp_lt_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_lt_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2521 (  timestamp_le_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_le_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2522 (  timestamp_eq_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_eq_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2523 (  timestamp_gt_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_gt_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2524 (  timestamp_ge_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_ge_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2525 (  timestamp_ne_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_ne_timestamptz _null_ _null_ _null_ ));
DATA(insert OID = 2526 (  timestamp_cmp_timestamptz UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 23 "1114 1184" _null_ _null_ _null_ _null_ _null_ timestamp_cmp_timestamptz _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 2527 (  timestamptz_lt_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_lt_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2528 (  timestamptz_le_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_le_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2529 (  timestamptz_eq_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_eq_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2530 (  timestamptz_gt_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_gt_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2531 (  timestamptz_ge_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_ge_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2532 (  timestamptz_ne_timestamp	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 16 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_ne_timestamp _null_ _null_ _null_ ));
DATA(insert OID = 2533 (  timestamptz_cmp_timestamp UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 23 "1184 1114" _null_ _null_ _null_ _null_ _null_ timestamptz_cmp_timestamp _null_ _null_ _null_ ));
DESCR("less-equal-greater");


/* send/receive functions */
DATA(insert OID = 2400 (  array_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2277 "2281 26 23" _null_ _null_ _null_ _null_  _null_ array_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2401 (  array_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "2277" _null_ _null_ _null_ _null_ _null_	array_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2402 (  record_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 2249 "2281 26 23" _null_ _null_ _null_ _null_  _null_ record_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2403 (  record_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "2249" _null_ _null_ _null_ _null_  _null_ record_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2404 (  int2recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 21 "2281" _null_ _null_ _null_ _null_ _null_	int2recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2405 (  int2send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "21" _null_ _null_ _null_ _null_ _null_ int2send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2406 (  int4recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2281" _null_ _null_ _null_ _null_ _null_	int4recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2407 (  int4send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "23" _null_ _null_ _null_ _null_ _null_ int4send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2408 (  int8recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 20 "2281" _null_ _null_ _null_ _null_ _null_	int8recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2409 (  int8send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "20" _null_ _null_ _null_ _null_ _null_ int8send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2410 (  int2vectorrecv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 22 "2281" _null_ _null_ _null_ _null_ _null_	int2vectorrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2411 (  int2vectorsend	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "22" _null_ _null_ _null_ _null_ _null_ int2vectorsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2412 (  bytearecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2281" _null_ _null_ _null_ _null_ _null_	bytearecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2413 (  byteasend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "17" _null_ _null_ _null_ _null_ _null_ byteasend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2414 (  textrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 25 "2281" _null_ _null_ _null_ _null_ _null_	textrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2415 (  textsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "25" _null_ _null_ _null_ _null_ _null_ textsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2416 (  unknownrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 705 "2281" _null_ _null_ _null_ _null_ _null_ unknownrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2417 (  unknownsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "705" _null_ _null_ _null_ _null_ _null_ unknownsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2418 (  oidrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 26 "2281" _null_ _null_ _null_ _null_ _null_	oidrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2419 (  oidsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "26" _null_ _null_ _null_ _null_ _null_ oidsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2420 (  oidvectorrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 30 "2281" _null_ _null_ _null_ _null_ _null_	oidvectorrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2421 (  oidvectorsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "30" _null_ _null_ _null_ _null_ _null_ oidvectorsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2422 (  namerecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 19 "2281" _null_ _null_ _null_ _null_ _null_	namerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2423 (  namesend			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "19" _null_ _null_ _null_ _null_ _null_ namesend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2424 (  float4recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 700 "2281" _null_ _null_ _null_ _null_ _null_ float4recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2425 (  float4send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "700" _null_ _null_ _null_ _null_ _null_ float4send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2426 (  float8recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 701 "2281" _null_ _null_ _null_ _null_ _null_ float8recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2427 (  float8send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "701" _null_ _null_ _null_ _null_ _null_ float8send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2428 (  point_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 600 "2281" _null_ _null_ _null_ _null_ _null_ point_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2429 (  point_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "600" _null_ _null_ _null_ _null_ _null_ point_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2430 (  bpcharrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1042 "2281 26 23" _null_ _null_ _null_ _null_  _null_ bpcharrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2431 (  bpcharsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "1042" _null_ _null_ _null_ _null_ _null_	bpcharsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2432 (  varcharrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 1043 "2281 26 23" _null_ _null_ _null_ _null_  _null_ varcharrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2433 (  varcharsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "1043" _null_ _null_ _null_ _null_ _null_	varcharsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2434 (  charrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 18 "2281" _null_ _null_ _null_ _null_ _null_	charrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2435 (  charsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "18" _null_ _null_ _null_ _null_ _null_ charsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2436 (  boolrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_	boolrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2437 (  boolsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "16" _null_ _null_ _null_ _null_ _null_ boolsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2438 (  tidrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 27 "2281" _null_ _null_ _null_ _null_ _null_	tidrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2439 (  tidsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "27" _null_ _null_ _null_ _null_ _null_ tidsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2440 (  xidrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 28 "2281" _null_ _null_ _null_ _null_ _null_	xidrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2441 (  xidsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "28" _null_ _null_ _null_ _null_ _null_ xidsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2442 (  cidrecv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 29 "2281" _null_ _null_ _null_ _null_ _null_	cidrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2443 (  cidsend			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "29" _null_ _null_ _null_ _null_ _null_ cidsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2444 (  regprocrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 24 "2281" _null_ _null_ _null_ _null_ _null_	regprocrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2445 (  regprocsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "24" _null_ _null_ _null_ _null_ _null_ regprocsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2446 (  regprocedurerecv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2202 "2281" _null_ _null_ _null_ _null_ _null_ regprocedurerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2447 (  regproceduresend	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2202" _null_ _null_ _null_ _null_ _null_	regproceduresend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2448 (  regoperrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2203 "2281" _null_ _null_ _null_ _null_ _null_ regoperrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2449 (  regopersend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2203" _null_ _null_ _null_ _null_ _null_	regopersend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2450 (  regoperatorrecv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2204 "2281" _null_ _null_ _null_ _null_ _null_ regoperatorrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2451 (  regoperatorsend	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2204" _null_ _null_ _null_ _null_ _null_	regoperatorsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2452 (  regclassrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2205 "2281" _null_ _null_ _null_ _null_ _null_ regclassrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2453 (  regclasssend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2205" _null_ _null_ _null_ _null_ _null_	regclasssend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2454 (  regtyperecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2206 "2281" _null_ _null_ _null_ _null_ _null_ regtyperecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2455 (  regtypesend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2206" _null_ _null_ _null_ _null_ _null_	regtypesend _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 4094 (  regrolerecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 4096 "2281" _null_ _null_ _null_ _null_ _null_	regrolerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4095 (  regrolesend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "4096" _null_ _null_ _null_ _null_ _null_	regrolesend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4087 (  regnamespacerecv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 4089 "2281" _null_ _null_ _null_ _null_ _null_ regnamespacerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 4088 (  regnamespacesend	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "4089" _null_ _null_ _null_ _null_ _null_	regnamespacesend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2456 (  bit_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1560 "2281 26 23" _null_ _null_ _null_ _null_  _null_ bit_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2457 (  bit_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1560" _null_ _null_ _null_ _null_ _null_	bit_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2458 (  varbit_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1562 "2281 26 23" _null_ _null_ _null_ _null_  _null_ varbit_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2459 (  varbit_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1562" _null_ _null_ _null_ _null_ _null_	varbit_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2460 (  numeric_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1700 "2281 26 23" _null_ _null_ _null_ _null_  _null_ numeric_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2461 (  numeric_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1700" _null_ _null_ _null_ _null_ _null_	numeric_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2462 (  abstimerecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 702 "2281" _null_ _null_ _null_ _null_ _null_ abstimerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2463 (  abstimesend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "702" _null_ _null_ _null_ _null_ _null_ abstimesend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2464 (  reltimerecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 703 "2281" _null_ _null_ _null_ _null_ _null_ reltimerecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2465 (  reltimesend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "703" _null_ _null_ _null_ _null_ _null_ reltimesend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2466 (  tintervalrecv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 704 "2281" _null_ _null_ _null_ _null_ _null_ tintervalrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2467 (  tintervalsend		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "704" _null_ _null_ _null_ _null_ _null_ tintervalsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2468 (  date_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 1082 "2281" _null_ _null_ _null_ _null_ _null_ date_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2469 (  date_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1082" _null_ _null_ _null_ _null_ _null_	date_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2470 (  time_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1083 "2281 26 23" _null_ _null_ _null_ _null_  _null_ time_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2471 (  time_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1083" _null_ _null_ _null_ _null_ _null_	time_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2472 (  timetz_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1266 "2281 26 23" _null_ _null_ _null_ _null_  _null_ timetz_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2473 (  timetz_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1266" _null_ _null_ _null_ _null_ _null_	timetz_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2474 (  timestamp_recv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1114 "2281 26 23" _null_ _null_ _null_ _null_  _null_ timestamp_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2475 (  timestamp_send	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1114" _null_ _null_ _null_ _null_ _null_	timestamp_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2476 (  timestamptz_recv	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1184 "2281 26 23" _null_ _null_ _null_ _null_  _null_ timestamptz_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2477 (  timestamptz_send	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1184" _null_ _null_ _null_ _null_ _null_	timestamptz_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2478 (  interval_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1186 "2281 26 23" _null_ _null_ _null_ _null_  _null_ interval_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2479 (  interval_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "1186" _null_ _null_ _null_ _null_ _null_	interval_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2480 (  lseg_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 601 "2281" _null_ _null_ _null_ _null_ _null_ lseg_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2481 (  lseg_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "601" _null_ _null_ _null_ _null_ _null_ lseg_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2482 (  path_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 602 "2281" _null_ _null_ _null_ _null_ _null_ path_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2483 (  path_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "602" _null_ _null_ _null_ _null_ _null_ path_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2484 (  box_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 603 "2281" _null_ _null_ _null_ _null_ _null_ box_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2485 (  box_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "603" _null_ _null_ _null_ _null_ _null_ box_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2486 (  poly_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 604 "2281" _null_ _null_ _null_ _null_ _null_ poly_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2487 (  poly_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "604" _null_ _null_ _null_ _null_ _null_ poly_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2488 (  line_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 628 "2281" _null_ _null_ _null_ _null_ _null_ line_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2489 (  line_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "628" _null_ _null_ _null_ _null_ _null_ line_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2490 (  circle_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 718 "2281" _null_ _null_ _null_ _null_ _null_ circle_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2491 (  circle_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "718" _null_ _null_ _null_ _null_ _null_ circle_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2492 (  cash_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 790 "2281" _null_ _null_ _null_ _null_ _null_ cash_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2493 (  cash_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "790" _null_ _null_ _null_ _null_ _null_ cash_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2494 (  macaddr_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 829 "2281" _null_ _null_ _null_ _null_ _null_ macaddr_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2495 (  macaddr_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "829" _null_ _null_ _null_ _null_ _null_ macaddr_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2496 (  inet_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 869 "2281" _null_ _null_ _null_ _null_ _null_ inet_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2497 (  inet_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "869" _null_ _null_ _null_ _null_ _null_ inet_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2498 (  cidr_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 650 "2281" _null_ _null_ _null_ _null_ _null_ cidr_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2499 (  cidr_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "650" _null_ _null_ _null_ _null_ _null_ cidr_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2500 (  cstring_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "2281" _null_ _null_ _null_ _null_ _null_ cstring_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2501 (  cstring_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "2275" _null_ _null_ _null_ _null_ _null_	cstring_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2502 (  anyarray_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2277 "2281" _null_ _null_ _null_ _null_ _null_ anyarray_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2503 (  anyarray_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "2277" _null_ _null_ _null_ _null_ _null_	anyarray_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3120 (  void_recv			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ void_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3121 (  void_send			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2278" _null_ _null_ _null_ _null_ _null_	void_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3446 (  macaddr8_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 774 "2281" _null_ _null_ _null_ _null_ _null_ macaddr8_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3447 (  macaddr8_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "774" _null_ _null_ _null_ _null_ _null_ macaddr8_send _null_ _null_ _null_ ));
DESCR("I/O");

/* System-view support functions with pretty-print option */
DATA(insert OID = 2504 (  ux_get_ruledef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "26 16" _null_ _null_ _null_ _null_ _null_ ux_get_ruledef_ext _null_ _null_ _null_ ));
DESCR("source text of a rule with pretty-print option");
DATA(insert OID = 2505 (  ux_get_viewdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 2 0 25 "25 16" _null_ _null_ _null_ _null_ _null_ ux_get_viewdef_name_ext _null_ _null_ _null_ ));
DESCR("select statement of a view with pretty-print option");
DATA(insert OID = 2506 (  ux_get_viewdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 2 0 25 "26 16" _null_ _null_ _null_ _null_ _null_ ux_get_viewdef_ext _null_ _null_ _null_ ));
DESCR("select statement of a view with pretty-print option");
DATA(insert OID = 3159 (  ux_get_viewdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s r 2 0 25 "26 23" _null_ _null_ _null_ _null_ _null_ ux_get_viewdef_wrap _null_ _null_ _null_ ));
DESCR("select statement of a view with pretty-printing and specified line wrapping");
DATA(insert OID = 2507 (  ux_get_indexdef	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 25 "26 23 16" _null_ _null_ _null_ _null_ _null_	ux_get_indexdef_ext _null_ _null_ _null_ ));
DESCR("index description (full create statement or single expression) with pretty-print option");
DATA(insert OID = 2508 (  ux_get_constraintdef UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "26 16" _null_ _null_ _null_ _null_ _null_ ux_get_constraintdef_ext _null_ _null_ _null_ ));
DESCR("constraint description with pretty-print option");
DATA(insert OID = 2509 (  ux_get_expr		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 25 "194 26 16" _null_ _null_ _null_ _null_ _null_ ux_get_expr_ext _null_ _null_ _null_ ));
DESCR("deparse an encoded expression with pretty-print option");
DATA(insert OID = 2510 (  ux_prepared_statement UXNSP UXUID 12 1 1000 0 0 f f f f t t s r 0 0 2249 "" "{25,25,1184,2211,16}" "{o,o,o,o,o}" "{name,statement,prepare_time,parameter_types,from_sql}" _null_ _null_ ux_prepared_statement _null_ _null_ _null_ ));
DESCR("get the prepared statements for this session");
DATA(insert OID = 2511 (  ux_cursor UXNSP UXUID 12 1 1000 0 0 f f f f t t s r 0 0 2249 "" "{25,25,16,16,16,1184}" "{o,o,o,o,o,o}" "{name,statement,is_holdable,is_binary,is_scrollable,creation_time}" _null_ _null_ ux_cursor _null_ _null_ _null_ ));
DESCR("get the open cursors for this session");
DATA(insert OID = 2599 (  ux_timezone_abbrevs	UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 0 0 2249 "" "{25,1186,16}" "{o,o,o}" "{abbrev,utc_offset,is_dst}" _null_ _null_ ux_timezone_abbrevs _null_ _null_ _null_ ));
DESCR("get the available time zone abbreviations");
DATA(insert OID = 2856 (  ux_timezone_names		UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 0 0 2249 "" "{25,25,1186,16}" "{o,o,o,o}" "{name,abbrev,utc_offset,is_dst}" _null_ _null_ ux_timezone_names _null_ _null_ _null_ ));
DESCR("get the available time zone names");
DATA(insert OID = 2730 (  ux_get_triggerdef		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 25 "26 16" _null_ _null_ _null_ _null_ _null_ ux_get_triggerdef_ext _null_ _null_ _null_ ));
DESCR("trigger description with pretty-print option");

/* asynchronous notifications */
DATA(insert OID = 3035 (  ux_listening_channels UXNSP UXUID 12 1 10 0 0 f f f f t t s r 0 0 25 "" _null_ _null_ _null_ _null_ _null_ ux_listening_channels _null_ _null_ _null_ ));
DESCR("get the channels that the current backend listens to");
DATA(insert OID = 3036 (  ux_notify				UXNSP UXUID 12 1 0 0 0 f f f f f f v r 2 0 2278 "25 25" _null_ _null_ _null_ _null_ _null_ ux_notify _null_ _null_ _null_ ));
DESCR("send a notification event");
DATA(insert OID = 3296 (  ux_notification_queue_usage	UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 701 "" _null_ _null_ _null_ _null_ _null_ ux_notification_queue_usage _null_ _null_ _null_ ));
DESCR("get the fraction of the asynchronous notification queue currently in use");

/* non-persistent series generator */
DATA(insert OID = 1066 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 23 "23 23 23" _null_ _null_ _null_ _null_ _null_ generate_series_step_int4 _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 1067 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 23 "23 23" _null_ _null_ _null_ _null_ _null_ generate_series_int4 _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 1068 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 20 "20 20 20" _null_ _null_ _null_ _null_ _null_ generate_series_step_int8 _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 1069 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 20 "20 20" _null_ _null_ _null_ _null_ _null_ generate_series_int8 _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 3259 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 1700 "1700 1700 1700" _null_ _null_ _null_ _null_ _null_ generate_series_step_numeric _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 3260 (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 1700 "1700 1700" _null_ _null_ _null_ _null_ _null_ generate_series_numeric _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 938  (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 3 0 1114 "1114 1114 1186" _null_ _null_ _null_ _null_ _null_ generate_series_timestamp _null_ _null_ _null_ ));
DESCR("non-persistent series generator");
DATA(insert OID = 939  (  generate_series UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 3 0 1184 "1184 1184 1186" _null_ _null_ _null_ _null_ _null_ generate_series_timestamptz _null_ _null_ _null_ ));
DESCR("non-persistent series generator");

/* boolean aggregates */
DATA(insert OID = 2515 ( booland_statefunc			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ booland_statefunc _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 2516 ( boolor_statefunc			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "16 16" _null_ _null_ _null_ _null_ _null_ boolor_statefunc _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3496 ( bool_accum					   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 16" _null_ _null_ _null_ _null_ _null_ bool_accum _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3497 ( bool_accum_inv				   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 16" _null_ _null_ _null_ _null_ _null_ bool_accum_inv _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3498 ( bool_alltrue				   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_ bool_alltrue _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3499 ( bool_anytrue				   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_ bool_anytrue _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 2517 ( bool_and					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 16 "16" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("boolean-and aggregate");
/* ANY, SOME? These names conflict with subquery operators. See doc. */
DATA(insert OID = 2518 ( bool_or					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 16 "16" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("boolean-or aggregate");
DATA(insert OID = 2519 ( every						   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 16 "16" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("boolean-and aggregate");

/* bitwise integer aggregates */
DATA(insert OID = 2236 ( bit_and					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-and smallint aggregate");
DATA(insert OID = 2237 ( bit_or						   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 21 "21" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-or smallint aggregate");
DATA(insert OID = 2238 ( bit_and					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-and integer aggregate");
DATA(insert OID = 2239 ( bit_or						   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-or integer aggregate");
DATA(insert OID = 2240 ( bit_and					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-and bigint aggregate");
DATA(insert OID = 2241 ( bit_or						   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 20 "20" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-or bigint aggregate");
DATA(insert OID = 2242 ( bit_and					   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1560 "1560" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-and bit aggregate");
DATA(insert OID = 2243 ( bit_or						   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 1560 "1560" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("bitwise-or bit aggregate");

/* formerly-missing interval + datetime operators */
DATA(insert OID = 2546 ( interval_pl_date			UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1114 "1186 1082" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));
DATA(insert OID = 2547 ( interval_pl_timetz			UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1266 "1186 1266" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));
DATA(insert OID = 2548 ( interval_pl_timestamp		UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1114 "1186 1114" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));
DATA(insert OID = 2549 ( interval_pl_timestamptz	UXNSP UXUID 14 1 0 0 0 f f f f t f s s 2 0 1184 "1186 1184" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));
DATA(insert OID = 2550 ( integer_pl_date			UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 1082 "23 1082" _null_ _null_ _null_ _null_ _null_ "select $2 + $1" _null_ _null_ _null_ ));

DATA(insert OID = 2556 ( ux_tablespace_databases	UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 1 0 26 "26" _null_ _null_ _null_ _null_ _null_ ux_tablespace_databases _null_ _null_ _null_ ));
DESCR("get OIDs of databases in a tablespace");

DATA(insert OID = 2557 ( bool				   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "23" _null_ _null_ _null_ _null_ _null_ int4_bool _null_ _null_ _null_ ));
DESCR("convert int4 to boolean");
DATA(insert OID = 2558 ( int4				   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "16" _null_ _null_ _null_ _null_ _null_ bool_int4 _null_ _null_ _null_ ));
DESCR("convert boolean to int4");
DATA(insert OID = 2559 ( lastval			   UXNSP UXUID 12 1 0 0 0 f f f f t f v u 0 0 20 "" _null_ _null_ _null_ _null_ _null_	lastval _null_ _null_ _null_ ));
DESCR("current value from last used sequence");

/* start time function */
DATA(insert OID = 2560 (  ux_uxmaster_start_time	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ ux_uxmaster_start_time _null_ _null_ _null_ ));
DESCR("uxmaster start time");
/* config reload time function */
DATA(insert OID = 2034 (  ux_conf_load_time			UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 1184 "" _null_ _null_ _null_ _null_ _null_ ux_conf_load_time _null_ _null_ _null_ ));
DESCR("configuration load time");

/* new functions for Y-direction rtree opclasses */
DATA(insert OID = 2562 (  box_below		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_below _null_ _null_ _null_ ));
DATA(insert OID = 2563 (  box_overbelow    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_overbelow _null_ _null_ _null_ ));
DATA(insert OID = 2564 (  box_overabove    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_overabove _null_ _null_ _null_ ));
DATA(insert OID = 2565 (  box_above		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "603 603" _null_ _null_ _null_ _null_ _null_ box_above _null_ _null_ _null_ ));
DATA(insert OID = 2566 (  poly_below	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_below _null_ _null_ _null_ ));
DATA(insert OID = 2567 (  poly_overbelow   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_overbelow _null_ _null_ _null_ ));
DATA(insert OID = 2568 (  poly_overabove   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_overabove _null_ _null_ _null_ ));
DATA(insert OID = 2569 (  poly_above	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "604 604" _null_ _null_ _null_ _null_ _null_ poly_above _null_ _null_ _null_ ));
DATA(insert OID = 2587 (  circle_overbelow		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_overbelow _null_ _null_ _null_ ));
DATA(insert OID = 2588 (  circle_overabove		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "718 718" _null_ _null_ _null_ _null_  _null_ circle_overabove _null_ _null_ _null_ ));

/* support functions for GiST r-tree emulation */
DATA(insert OID = 2578 (  gist_box_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 603 21 26 2281" _null_ _null_ _null_ _null_ _null_	gist_box_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2579 (  gist_box_compress		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_box_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2580 (  gist_box_decompress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_box_decompress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3281 (  gist_box_fetch	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_box_fetch _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2581 (  gist_box_penalty		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gist_box_penalty _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2582 (  gist_box_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_	gist_box_picksplit _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2583 (  gist_box_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 603 "2281 2281" _null_ _null_ _null_ _null_ _null_ gist_box_union _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2584 (  gist_box_same			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "603 603 2281" _null_ _null_ _null_ _null_ _null_ gist_box_same _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2585 (  gist_poly_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 604 21 26 2281" _null_ _null_ _null_ _null_ _null_	gist_poly_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2586 (  gist_poly_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_poly_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2591 (  gist_circle_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 718 21 26 2281" _null_ _null_ _null_ _null_ _null_ gist_circle_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2592 (  gist_circle_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_circle_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 1030 (  gist_point_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_point_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3282 (  gist_point_fetch	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gist_point_fetch _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 2179 (  gist_point_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 600 21 26 2281" _null_ _null_ _null_ _null_ _null_	gist_point_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3064 (  gist_point_distance	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 701 "2281 600 21 26 2281" _null_ _null_ _null_ _null_ _null_ gist_point_distance _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3280 (  gist_circle_distance	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 701 "2281 718 21 26 2281" _null_ _null_ _null_ _null_ _null_ gist_circle_distance _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3288 (  gist_poly_distance	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 701 "2281 604 21 26 2281" _null_ _null_ _null_ _null_ _null_ gist_poly_distance _null_ _null_ _null_ ));
DESCR("GiST support");

/* GIN array support */
DATA(insert OID = 2743 (  ginarrayextract	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2277 2281 2281" _null_ _null_ _null_ _null_ _null_ ginarrayextract _null_ _null_ _null_ ));
DESCR("GIN array support");
DATA(insert OID = 2774 (  ginqueryarrayextract	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 2281 "2277 2281 21 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ ginqueryarrayextract _null_ _null_ _null_ ));
DESCR("GIN array support");
DATA(insert OID = 2744 (  ginarrayconsistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 8 0 16 "2281 21 2277 23 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ ginarrayconsistent _null_ _null_ _null_ ));
DESCR("GIN array support");
DATA(insert OID = 3920 (  ginarraytriconsistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 18 "2281 21 2277 23 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ ginarraytriconsistent _null_ _null_ _null_ ));
DESCR("GIN array support");
DATA(insert OID = 3076 (  ginarrayextract	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2277 2281" _null_ _null_ _null_ _null_ _null_ ginarrayextract_2args _null_ _null_ _null_ ));
DESCR("GIN array support (obsolete)");

/* overlap/contains/contained */
DATA(insert OID = 2747 (  arrayoverlap		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ arrayoverlap _null_ _null_ _null_ ));
DATA(insert OID = 2748 (  arraycontains		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ arraycontains _null_ _null_ _null_ ));
DATA(insert OID = 2749 (  arraycontained	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2277 2277" _null_ _null_ _null_ _null_ _null_ arraycontained _null_ _null_ _null_ ));

/* BRIN minmax */
DATA(insert OID = 3383 ( brin_minmax_opcinfo	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ brin_minmax_opcinfo _null_ _null_ _null_ ));
DESCR("BRIN minmax support");
DATA(insert OID = 3384 ( brin_minmax_add_value	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 16 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_minmax_add_value _null_ _null_ _null_ ));
DESCR("BRIN minmax support");
DATA(insert OID = 3385 ( brin_minmax_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 16 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_minmax_consistent _null_ _null_ _null_ ));
DESCR("BRIN minmax support");
DATA(insert OID = 3386 ( brin_minmax_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 16 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_minmax_union _null_ _null_ _null_ ));
DESCR("BRIN minmax support");

/* BRIN inclusion */
DATA(insert OID = 4105 ( brin_inclusion_opcinfo UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ brin_inclusion_opcinfo _null_ _null_ _null_ ));
DESCR("BRIN inclusion support");
DATA(insert OID = 4106 ( brin_inclusion_add_value UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 16 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_inclusion_add_value _null_ _null_ _null_ ));
DESCR("BRIN inclusion support");
DATA(insert OID = 4107 ( brin_inclusion_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 16 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_inclusion_consistent _null_ _null_ _null_ ));
DESCR("BRIN inclusion support");
DATA(insert OID = 4108 ( brin_inclusion_union	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 16 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ brin_inclusion_union _null_ _null_ _null_ ));
DESCR("BRIN inclusion support");

/* userlock replacements */
DATA(insert OID = 2880 (  ux_advisory_lock				UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_lock_int8 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock");
DATA(insert OID = 3089 (  ux_advisory_xact_lock				UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_xact_lock_int8 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock");
DATA(insert OID = 2881 (  ux_advisory_lock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_lock_shared_int8 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock");
DATA(insert OID = 3090 (  ux_advisory_xact_lock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_xact_lock_shared_int8 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock");
DATA(insert OID = 2882 (  ux_try_advisory_lock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_lock_int8 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock if available");
DATA(insert OID = 3091 (  ux_try_advisory_xact_lock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_xact_lock_int8 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock if available");
DATA(insert OID = 2883 (  ux_try_advisory_lock_shared	UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_lock_shared_int8 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock if available");
DATA(insert OID = 3092 (  ux_try_advisory_xact_lock_shared	UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_xact_lock_shared_int8 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock if available");
DATA(insert OID = 2884 (  ux_advisory_unlock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_unlock_int8 _null_ _null_ _null_ ));
DESCR("release exclusive advisory lock");
DATA(insert OID = 2885 (  ux_advisory_unlock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 16 "20" _null_ _null_ _null_ _null_ _null_ ux_advisory_unlock_shared_int8 _null_ _null_ _null_ ));
DESCR("release shared advisory lock");
DATA(insert OID = 2886 (  ux_advisory_lock				UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_lock_int4 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock");
DATA(insert OID = 3093 (  ux_advisory_xact_lock				UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_xact_lock_int4 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock");
DATA(insert OID = 2887 (  ux_advisory_lock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_lock_shared_int4 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock");
DATA(insert OID = 3094 (  ux_advisory_xact_lock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_xact_lock_shared_int4 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock");
DATA(insert OID = 2888 (  ux_try_advisory_lock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_lock_int4 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock if available");
DATA(insert OID = 3095 (  ux_try_advisory_xact_lock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_xact_lock_int4 _null_ _null_ _null_ ));
DESCR("obtain exclusive advisory lock if available");
DATA(insert OID = 2889 (  ux_try_advisory_lock_shared	UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_lock_shared_int4 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock if available");
DATA(insert OID = 3096 (  ux_try_advisory_xact_lock_shared	UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_try_advisory_xact_lock_shared_int4 _null_ _null_ _null_ ));
DESCR("obtain shared advisory lock if available");
DATA(insert OID = 2890 (  ux_advisory_unlock			UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_unlock_int4 _null_ _null_ _null_ ));
DESCR("release exclusive advisory lock");
DATA(insert OID = 2891 (  ux_advisory_unlock_shared		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 16 "23 23" _null_ _null_ _null_ _null_ _null_ ux_advisory_unlock_shared_int4 _null_ _null_ _null_ ));
DESCR("release shared advisory lock");
DATA(insert OID = 2892 (  ux_advisory_unlock_all		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_advisory_unlock_all _null_ _null_ _null_ ));
DESCR("release all advisory locks");

/* XML support */
DATA(insert OID = 2893 (  xml_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 142 "2275" _null_ _null_ _null_ _null_ _null_ xml_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2894 (  xml_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "142" _null_ _null_ _null_ _null_ _null_ xml_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2895 (  xmlcomment	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 142 "25" _null_ _null_ _null_ _null_ _null_ xmlcomment _null_ _null_ _null_ ));
DESCR("generate XML comment");
DATA(insert OID = 2896 (  xml			   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 142 "25" _null_ _null_ _null_ _null_ _null_ texttoxml _null_ _null_ _null_ ));
DESCR("perform a non-validating parse of a character string to produce an XML value");
DATA(insert OID = 2897 (  xmlvalidate	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "142 25" _null_ _null_ _null_ _null_ _null_ xmlvalidate _null_ _null_ _null_ ));
DESCR("validate an XML value");
DATA(insert OID = 2898 (  xml_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 142 "2281" _null_ _null_ _null_ _null_ _null_ xml_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2899 (  xml_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "142" _null_ _null_ _null_ _null_ _null_ xml_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2900 (  xmlconcat2	   UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 142 "142 142" _null_ _null_ _null_ _null_ _null_ xmlconcat2 _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 2901 (  xmlagg		   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 142 "142" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("concatenate XML values");
DATA(insert OID = 2922 (  text			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "142" _null_ _null_ _null_ _null_ _null_ xmltotext _null_ _null_ _null_ ));
DESCR("serialize an XML value to a character string");

DATA(insert OID = 2923 (  table_to_xml				  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "2205 16 16 25" _null_ _null_ "{tbl,nulls,tableforest,targetns}" _null_ _null_ table_to_xml _null_ _null_ _null_ ));
DESCR("map table contents to XML");
DATA(insert OID = 2924 (  query_to_xml				  UXNSP UXUID 12 100 0 0 0 f f f f t f s u 4 0 142 "25 16 16 25" _null_ _null_ "{query,nulls,tableforest,targetns}" _null_ _null_ query_to_xml _null_ _null_ _null_ ));
DESCR("map query result to XML");
DATA(insert OID = 2925 (  cursor_to_xml				  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 5 0 142 "1790 23 16 16 25" _null_ _null_ "{cursor,count,nulls,tableforest,targetns}" _null_ _null_ cursor_to_xml _null_ _null_ _null_ ));
DESCR("map rows from cursor to XML");
DATA(insert OID = 2926 (  table_to_xmlschema		  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "2205 16 16 25" _null_ _null_ "{tbl,nulls,tableforest,targetns}" _null_ _null_ table_to_xmlschema _null_ _null_ _null_ ));
DESCR("map table structure to XML Schema");
DATA(insert OID = 2927 (  query_to_xmlschema		  UXNSP UXUID 12 100 0 0 0 f f f f t f s u 4 0 142 "25 16 16 25" _null_ _null_ "{query,nulls,tableforest,targetns}" _null_ _null_ query_to_xmlschema _null_ _null_ _null_ ));
DESCR("map query result structure to XML Schema");
DATA(insert OID = 2928 (  cursor_to_xmlschema		  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "1790 16 16 25" _null_ _null_ "{cursor,nulls,tableforest,targetns}" _null_ _null_ cursor_to_xmlschema _null_ _null_ _null_ ));
DESCR("map cursor structure to XML Schema");
DATA(insert OID = 2929 (  table_to_xml_and_xmlschema  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "2205 16 16 25" _null_ _null_ "{tbl,nulls,tableforest,targetns}" _null_ _null_ table_to_xml_and_xmlschema _null_ _null_ _null_ ));
DESCR("map table contents and structure to XML and XML Schema");
DATA(insert OID = 2930 (  query_to_xml_and_xmlschema  UXNSP UXUID 12 100 0 0 0 f f f f t f s u 4 0 142 "25 16 16 25" _null_ _null_ "{query,nulls,tableforest,targetns}" _null_ _null_ query_to_xml_and_xmlschema _null_ _null_ _null_ ));
DESCR("map query result and structure to XML and XML Schema");

DATA(insert OID = 2933 (  schema_to_xml				  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "19 16 16 25" _null_ _null_ "{schema,nulls,tableforest,targetns}" _null_ _null_ schema_to_xml _null_ _null_ _null_ ));
DESCR("map schema contents to XML");
DATA(insert OID = 2934 (  schema_to_xmlschema		  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "19 16 16 25" _null_ _null_ "{schema,nulls,tableforest,targetns}" _null_ _null_ schema_to_xmlschema _null_ _null_ _null_ ));
DESCR("map schema structure to XML Schema");
DATA(insert OID = 2935 (  schema_to_xml_and_xmlschema UXNSP UXUID 12 100 0 0 0 f f f f t f s r 4 0 142 "19 16 16 25" _null_ _null_ "{schema,nulls,tableforest,targetns}" _null_ _null_ schema_to_xml_and_xmlschema _null_ _null_ _null_ ));
DESCR("map schema contents and structure to XML and XML Schema");

DATA(insert OID = 2936 (  database_to_xml			  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 3 0 142 "16 16 25" _null_ _null_ "{nulls,tableforest,targetns}" _null_ _null_ database_to_xml _null_ _null_ _null_ ));
DESCR("map database contents to XML");
DATA(insert OID = 2937 (  database_to_xmlschema		  UXNSP UXUID 12 100 0 0 0 f f f f t f s r 3 0 142 "16 16 25" _null_ _null_ "{nulls,tableforest,targetns}" _null_ _null_ database_to_xmlschema _null_ _null_ _null_ ));
DESCR("map database structure to XML Schema");
DATA(insert OID = 2938 (  database_to_xml_and_xmlschema UXNSP UXUID 12 100 0 0 0 f f f f t f s r 3 0 142 "16 16 25" _null_ _null_ "{nulls,tableforest,targetns}" _null_ _null_ database_to_xml_and_xmlschema _null_ _null_ _null_ ));
DESCR("map database contents and structure to XML and XML Schema");

DATA(insert OID = 2931 (  xpath		 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 143 "25 142 1009" _null_ _null_ _null_ _null_ _null_ xpath _null_ _null_ _null_ ));
DESCR("evaluate XPath expression, with namespaces support");
DATA(insert OID = 2932 (  xpath		 UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 143 "25 142" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.xpath($1, $2, ''{}''::ux_catalog.text[])" _null_ _null_ _null_ ));
DESCR("evaluate XPath expression");

DATA(insert OID = 2614 (  xmlexists  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "25 142" _null_ _null_ _null_ _null_ _null_ xmlexists _null_ _null_ _null_ ));
DESCR("test XML value against XPath expression");

DATA(insert OID = 3049 (  xpath_exists	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 16 "25 142 1009" _null_ _null_ _null_ _null_ _null_ xpath_exists _null_ _null_ _null_ ));
DESCR("test XML value against XPath expression, with namespace support");
DATA(insert OID = 3050 (  xpath_exists	 UXNSP UXUID 14 1 0 0 0 f f f f t f i s 2 0 16 "25 142" _null_ _null_ _null_ _null_ _null_ "select ux_catalog.xpath_exists($1, $2, ''{}''::ux_catalog.text[])" _null_ _null_ _null_ ));
DESCR("test XML value against XPath expression");
DATA(insert OID = 3051 (  xml_is_well_formed			 UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "25" _null_ _null_ _null_ _null_ _null_ xml_is_well_formed _null_ _null_ _null_ ));
DESCR("determine if a string is well formed XML");
DATA(insert OID = 3052 (  xml_is_well_formed_document	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "25" _null_ _null_ _null_ _null_ _null_ xml_is_well_formed_document _null_ _null_ _null_ ));
DESCR("determine if a string is well formed XML document");
DATA(insert OID = 3053 (  xml_is_well_formed_content	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "25" _null_ _null_ _null_ _null_ _null_ xml_is_well_formed_content _null_ _null_ _null_ ));
DESCR("determine if a string is well formed XML content");

/* json */
DATA(insert OID = 321 (  json_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 114 "2275" _null_ _null_ _null_ _null_ _null_ json_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 322 (  json_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "114" _null_ _null_ _null_ _null_ _null_ json_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 323 (  json_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 114 "2281" _null_ _null_ _null_ _null_ _null_ json_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 324 (  json_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "114" _null_ _null_ _null_ _null_ _null_ json_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3153 (  array_to_json    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 114 "2277" _null_ _null_ _null_ _null_ _null_ array_to_json _null_ _null_ _null_ ));
DESCR("map array to json");
DATA(insert OID = 3154 (  array_to_json    UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 114 "2277 16" _null_ _null_ _null_ _null_ _null_ array_to_json_pretty _null_ _null_ _null_ ));
DESCR("map array to json with optional pretty printing");
DATA(insert OID = 3155 (  row_to_json	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 114 "2249" _null_ _null_ _null_ _null_ _null_ row_to_json _null_ _null_ _null_ ));
DESCR("map row to json");
DATA(insert OID = 3156 (  row_to_json	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 114 "2249 16" _null_ _null_ _null_ _null_ _null_ row_to_json_pretty _null_ _null_ _null_ ));
DESCR("map row to json with optional pretty printing");
DATA(insert OID = 3173 (  json_agg_transfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f s s 2 0 2281 "2281 2283" _null_ _null_ _null_ _null_ _null_ json_agg_transfn _null_ _null_ _null_ ));
DESCR("json aggregate transition function");
DATA(insert OID = 3174 (  json_agg_finalfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 114 "2281" _null_ _null_ _null_ _null_ _null_ json_agg_finalfn _null_ _null_ _null_ ));
DESCR("json aggregate final function");
DATA(insert OID = 3175 (  json_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f s s 1 0 114 "2283" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("aggregate input into json");
DATA(insert OID = 3180 (  json_object_agg_transfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 2281 "2281 2276 2276" _null_ _null_ _null_ _null_ _null_ json_object_agg_transfn _null_ _null_ _null_ ));
DESCR("json object aggregate transition function");
DATA(insert OID = 3196 (  json_object_agg_finalfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f i s 1 0 114 "2281" _null_ _null_ _null_ _null_ _null_ json_object_agg_finalfn _null_ _null_ _null_ ));
DESCR("json object aggregate final function");
DATA(insert OID = 3197 (  json_object_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f s s 2 0 114 "2276 2276" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("aggregate input into a json object");
DATA(insert OID = 3198 (  json_build_array	   UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 1 0 114 "2276" "{2276}" "{v}" _null_ _null_ _null_ json_build_array _null_ _null_ _null_ ));
DESCR("build a json array from any inputs");
DATA(insert OID = 3199 (  json_build_array	   UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 114  "" _null_ _null_ _null_ _null_ _null_ json_build_array_noargs _null_ _null_ _null_ ));
DESCR("build an empty json array");
DATA(insert OID = 3200 (  json_build_object    UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 1 0 114 "2276" "{2276}" "{v}" _null_ _null_ _null_ json_build_object _null_ _null_ _null_ ));
DESCR("build a json object from pairwise key/value inputs");
DATA(insert OID = 3201 (  json_build_object    UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 114  "" _null_ _null_ _null_ _null_ _null_ json_build_object_noargs _null_ _null_ _null_ ));
DESCR("build an empty json object");
DATA(insert OID = 3202 (  json_object	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 114 "1009" _null_ _null_ _null_ _null_ _null_ json_object _null_ _null_ _null_ ));
DESCR("map text array of key value pairs to json object");
DATA(insert OID = 3203 (  json_object	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 114 "1009 1009" _null_ _null_ _null_ _null_ _null_ json_object_two_arg _null_ _null_ _null_ ));
DESCR("map text arrays of keys and values to json object");
DATA(insert OID = 3176 (  to_json	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 114 "2283" _null_ _null_ _null_ _null_ _null_ to_json _null_ _null_ _null_ ));
DESCR("map input to json");
DATA(insert OID = 3261 (  json_strip_nulls	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 114 "114" _null_ _null_ _null_ _null_ _null_ json_strip_nulls _null_ _null_ _null_ ));
DESCR("remove object fields with null values from json");

DATA(insert OID = 3947 (  json_object_field			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 114 "114 25" _null_ _null_ "{from_json, field_name}" _null_ _null_ json_object_field _null_ _null_ _null_ ));
DATA(insert OID = 3948 (  json_object_field_text	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25  "114 25" _null_ _null_ "{from_json, field_name}" _null_ _null_ json_object_field_text _null_ _null_ _null_ ));
DATA(insert OID = 3949 (  json_array_element		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 114 "114 23" _null_ _null_ "{from_json, element_index}" _null_ _null_ json_array_element _null_ _null_ _null_ ));
DATA(insert OID = 3950 (  json_array_element_text	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25  "114 23" _null_ _null_ "{from_json, element_index}" _null_ _null_ json_array_element_text _null_ _null_ _null_ ));
DATA(insert OID = 3951 (  json_extract_path			UXNSP UXUID 12 1 0 25 0 f f f f t f i s 2 0 114 "114 1009" "{114,1009}" "{i,v}" "{from_json,path_elems}" _null_ _null_ json_extract_path _null_ _null_ _null_ ));
DESCR("get value from json with path elements");
DATA(insert OID = 3953 (  json_extract_path_text	UXNSP UXUID 12 1 0 25 0 f f f f t f i s 2 0 25 "114 1009" "{114,1009}" "{i,v}" "{from_json,path_elems}" _null_ _null_ json_extract_path_text _null_ _null_ _null_ ));
DESCR("get value from json as text with path elements");
DATA(insert OID = 3955 (  json_array_elements		UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 114 "114" "{114,114}" "{i,o}" "{from_json,value}" _null_ _null_ json_array_elements _null_ _null_ _null_ ));
DESCR("key value pairs of a json object");
DATA(insert OID = 3969 (  json_array_elements_text	UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 25 "114" "{114,25}" "{i,o}" "{from_json,value}" _null_ _null_ json_array_elements_text _null_ _null_ _null_ ));
DESCR("elements of json array");
DATA(insert OID = 3956 (  json_array_length			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "114" _null_ _null_ _null_ _null_ _null_ json_array_length _null_ _null_ _null_ ));
DESCR("length of json array");
DATA(insert OID = 3957 (  json_object_keys			UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 25 "114" _null_ _null_ _null_ _null_ _null_ json_object_keys _null_ _null_ _null_ ));
DESCR("get json object keys");
DATA(insert OID = 3958 (  json_each				   UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 2249 "114" "{114,25,114}" "{i,o,o}" "{from_json,key,value}" _null_ _null_ json_each _null_ _null_ _null_ ));
DESCR("key value pairs of a json object");
DATA(insert OID = 3959 (  json_each_text		   UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 2249 "114" "{114,25,25}" "{i,o,o}" "{from_json,key,value}" _null_ _null_ json_each_text _null_ _null_ _null_ ));
DESCR("key value pairs of a json object");
DATA(insert OID = 3960 (  json_populate_record	   UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 2283 "2283 114 16" _null_ _null_ _null_ _null_ _null_ json_populate_record _null_ _null_ _null_ ));
DESCR("get record fields from a json object");
DATA(insert OID = 3961 (  json_populate_recordset  UXNSP UXUID 12 1 100 0 0 f f f f f t s s 3 0 2283 "2283 114 16" _null_ _null_ _null_ _null_ _null_ json_populate_recordset _null_ _null_ _null_ ));
DESCR("get set of records with fields from a json array of objects");
DATA(insert OID = 3204 (  json_to_record		   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2249 "114" _null_ _null_ _null_ _null_ _null_ json_to_record _null_ _null_ _null_ ));
DESCR("get record fields from a json object");
DATA(insert OID = 3205 (  json_to_recordset		   UXNSP UXUID 12 1 100 0 0 f f f f f t s s 1 0 2249 "114" _null_ _null_ _null_ _null_ _null_ json_to_recordset _null_ _null_ _null_ ));
DESCR("get set of records with fields from a json array of objects");
DATA(insert OID = 3968 (  json_typeof			   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "114" _null_ _null_ _null_ _null_ _null_ json_typeof _null_ _null_ _null_ ));
DESCR("get the type of a json value");

/* uuid */
DATA(insert OID = 2952 (  uuid_in		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2950 "2275" _null_ _null_ _null_ _null_ _null_ uuid_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2953 (  uuid_out		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "2950" _null_ _null_ _null_ _null_ _null_ uuid_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2954 (  uuid_lt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_lt _null_ _null_ _null_ ));
DATA(insert OID = 2955 (  uuid_le		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_le _null_ _null_ _null_ ));
DATA(insert OID = 2956 (  uuid_eq		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_eq _null_ _null_ _null_ ));
DATA(insert OID = 2957 (  uuid_ge		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_ge _null_ _null_ _null_ ));
DATA(insert OID = 2958 (  uuid_gt		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_gt _null_ _null_ _null_ ));
DATA(insert OID = 2959 (  uuid_ne		   UXNSP UXUID 12 1 0 0 0 f f f t t f i s 2 0 16 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_ne _null_ _null_ _null_ ));
DATA(insert OID = 2960 (  uuid_cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2950 2950" _null_ _null_ _null_ _null_ _null_ uuid_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3300 (  uuid_sortsupport UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ uuid_sortsupport _null_ _null_ _null_ ));
DESCR("sort support");
DATA(insert OID = 2961 (  uuid_recv		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2950 "2281" _null_ _null_ _null_ _null_ _null_ uuid_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2962 (  uuid_send		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "2950" _null_ _null_ _null_ _null_ _null_ uuid_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2963 (  uuid_hash		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "2950" _null_ _null_ _null_ _null_ _null_ uuid_hash _null_ _null_ _null_ ));
DESCR("hash");

/* ux_lsn */
DATA(insert OID = 3229 (  ux_lsn_in		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3220 "2275" _null_ _null_ _null_ _null_ _null_ ux_lsn_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3230 (  ux_lsn_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3231 (  ux_lsn_lt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_lt _null_ _null_ _null_ ));
DATA(insert OID = 3232 (  ux_lsn_le		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_le _null_ _null_ _null_ ));
DATA(insert OID = 3233 (  ux_lsn_eq		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_eq _null_ _null_ _null_ ));
DATA(insert OID = 3234 (  ux_lsn_ge		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_ge _null_ _null_ _null_ ));
DATA(insert OID = 3235 (  ux_lsn_gt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_gt _null_ _null_ _null_ ));
DATA(insert OID = 3236 (  ux_lsn_ne		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_ne _null_ _null_ _null_ ));
DATA(insert OID = 3237 (  ux_lsn_mi		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1700 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_mi _null_ _null_ _null_ ));
DATA(insert OID = 3238 (  ux_lsn_recv	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3220 "2281" _null_ _null_ _null_ _null_ _null_ ux_lsn_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3239 (  ux_lsn_send	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3251 (  ux_lsn_cmp	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3220 3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3252 (  ux_lsn_hash	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3220" _null_ _null_ _null_ _null_ _null_ ux_lsn_hash _null_ _null_ _null_ ));
DESCR("hash");

/* enum related procs */
DATA(insert OID = 3504 (  anyenum_in	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3500 "2275" _null_ _null_ _null_ _null_ _null_ anyenum_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3505 (  anyenum_out	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3500" _null_ _null_ _null_ _null_ _null_ anyenum_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3506 (  enum_in		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 3500 "2275 26" _null_ _null_ _null_ _null_ _null_ enum_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3507 (  enum_out		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3500" _null_ _null_ _null_ _null_ _null_ enum_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3508 (  enum_eq		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_eq _null_ _null_ _null_ ));
DATA(insert OID = 3509 (  enum_ne		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_ne _null_ _null_ _null_ ));
DATA(insert OID = 3510 (  enum_lt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_lt _null_ _null_ _null_ ));
DATA(insert OID = 3511 (  enum_gt		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_gt _null_ _null_ _null_ ));
DATA(insert OID = 3512 (  enum_le		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_le _null_ _null_ _null_ ));
DATA(insert OID = 3513 (  enum_ge		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_ge _null_ _null_ _null_ ));
DATA(insert OID = 3514 (  enum_cmp		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3515 (  hashenum		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3500" _null_ _null_ _null_ _null_ _null_ hashenum _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 3524 (  enum_smaller	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3500 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_smaller _null_ _null_ _null_ ));
DESCR("smaller of two");
DATA(insert OID = 3525 (  enum_larger	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3500 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_larger _null_ _null_ _null_ ));
DESCR("larger of two");
DATA(insert OID = 3526 (  max			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 3500 "3500" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("maximum value of all enum input values");
DATA(insert OID = 3527 (  min			UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 3500 "3500" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("minimum value of all enum input values");
DATA(insert OID = 3528 (  enum_first	UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 3500 "3500" _null_ _null_ _null_ _null_ _null_ enum_first _null_ _null_ _null_ ));
DESCR("first value of the input enum type");
DATA(insert OID = 3529 (  enum_last		UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 3500 "3500" _null_ _null_ _null_ _null_ _null_ enum_last _null_ _null_ _null_ ));
DESCR("last value of the input enum type");
DATA(insert OID = 3530 (  enum_range	UXNSP UXUID 12 1 0 0 0 f f f f f f s s 2 0 2277 "3500 3500" _null_ _null_ _null_ _null_ _null_ enum_range_bounds _null_ _null_ _null_ ));
DESCR("range between the two given enum values, as an ordered array");
DATA(insert OID = 3531 (  enum_range	UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 2277 "3500" _null_ _null_ _null_ _null_ _null_ enum_range_all _null_ _null_ _null_ ));
DESCR("range of the given enum type, as an ordered array");
DATA(insert OID = 3532 (  enum_recv		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 2 0 3500 "2281 26" _null_ _null_ _null_ _null_ _null_ enum_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3533 (  enum_send		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "3500" _null_ _null_ _null_ _null_ _null_ enum_send _null_ _null_ _null_ ));
DESCR("I/O");

/* text search stuff */
DATA(insert OID =  3610 (  tsvectorin			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3614 "2275" _null_ _null_ _null_ _null_ _null_ tsvectorin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3639 (  tsvectorrecv			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3614 "2281" _null_ _null_ _null_ _null_ _null_ tsvectorrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3611 (  tsvectorout			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3614" _null_ _null_ _null_ _null_ _null_ tsvectorout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3638 (  tsvectorsend			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3614" _null_ _null_ _null_ _null_ _null_ tsvectorsend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3612 (  tsqueryin			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3615 "2275" _null_ _null_ _null_ _null_ _null_ tsqueryin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3641 (  tsqueryrecv			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3615 "2281" _null_ _null_ _null_ _null_ _null_ tsqueryrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3613 (  tsqueryout			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3615" _null_ _null_ _null_ _null_ _null_ tsqueryout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3640 (  tsquerysend			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3615" _null_ _null_ _null_ _null_ _null_ tsquerysend _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3646 (  gtsvectorin			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3642 "2275" _null_ _null_ _null_ _null_ _null_ gtsvectorin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3647 (  gtsvectorout			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3642" _null_ _null_ _null_ _null_ _null_ gtsvectorout _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 3616 (  tsvector_lt			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_lt _null_ _null_ _null_ ));
DATA(insert OID = 3617 (  tsvector_le			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_le _null_ _null_ _null_ ));
DATA(insert OID = 3618 (  tsvector_eq			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_eq _null_ _null_ _null_ ));
DATA(insert OID = 3619 (  tsvector_ne			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_ne _null_ _null_ _null_ ));
DATA(insert OID = 3620 (  tsvector_ge			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_ge _null_ _null_ _null_ ));
DATA(insert OID = 3621 (  tsvector_gt			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_gt _null_ _null_ _null_ ));
DATA(insert OID = 3622 (  tsvector_cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 3711 (  length				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3614" _null_ _null_ _null_ _null_ _null_ tsvector_length _null_ _null_ _null_ ));
DESCR("number of lexemes");
DATA(insert OID = 3623 (  strip					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3614 "3614" _null_ _null_ _null_ _null_ _null_ tsvector_strip _null_ _null_ _null_ ));
DESCR("strip position information");
DATA(insert OID = 3624 (  setweight				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3614 "3614 18" _null_ _null_ _null_ _null_ _null_ tsvector_setweight _null_ _null_ _null_ ));
DESCR("set given weight for whole tsvector");
DATA(insert OID = 3320 (  setweight				UXNSP UXUID 12 1 0	0 0 f f f f t f i s 3 0 3614 "3614 18 1009" _null_ _null_ _null_ _null_ _null_ tsvector_setweight_by_filter _null_ _null_ _null_ ));
DESCR("set given weight for given lexemes");
DATA(insert OID = 3625 (  tsvector_concat		UXNSP UXUID 12 1 0	0 0 f f f f t f i s 2 0 3614 "3614 3614" _null_ _null_ _null_ _null_ _null_ tsvector_concat _null_ _null_ _null_ ));
DATA(insert OID = 3321 (  ts_delete				UXNSP UXUID 12 1 0	0 0 f f f f t f i s 2 0 3614 "3614 25" _null_ _null_ _null_ _null_ _null_ tsvector_delete_str _null_ _null_ _null_ ));
DESCR("delete lexeme");
DATA(insert OID = 3323 (  ts_delete				UXNSP UXUID 12 1 0	0 0 f f f f t f i s 2 0 3614 "3614 1009" _null_ _null_ _null_ _null_ _null_ tsvector_delete_arr _null_ _null_ _null_ ));
DESCR("delete given lexemes");
DATA(insert OID = 3322 (  unnest				UXNSP UXUID 12 1 10 0 0 f f f f t t i s 1 0 2249 "3614" "{3614,25,1005,1009}" "{i,o,o,o}" "{tsvector,lexeme,positions,weights}"  _null_ _null_ tsvector_unnest _null_ _null_ _null_ ));
DESCR("expand tsvector to set of rows");
DATA(insert OID = 3326 (  tsvector_to_array		UXNSP UXUID 12 1 0	0 0 f f f f t f i s 1 0 1009 "3614" _null_ _null_ _null_ _null_ _null_ tsvector_to_array _null_ _null_ _null_ ));
DESCR("convert tsvector to array of lexemes");
DATA(insert OID = 3327 (  array_to_tsvector		UXNSP UXUID 12 1 0	0 0 f f f f t f i s 1 0 3614 "1009" _null_ _null_ _null_ _null_ _null_ array_to_tsvector _null_ _null_ _null_ ));
DESCR("build tsvector from array of lexemes");
DATA(insert OID = 3319 (  ts_filter				UXNSP UXUID 12 1 0	0 0 f f f f t f i s 2 0 3614 "3614 1002" _null_ _null_ _null_ _null_ _null_ tsvector_filter _null_ _null_ _null_ ));
DESCR("delete lexemes that do not have one of the given weights");

DATA(insert OID = 3634 (  ts_match_vq			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3614 3615" _null_ _null_ _null_ _null_ _null_ ts_match_vq _null_ _null_ _null_ ));
DATA(insert OID = 3635 (  ts_match_qv			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3614" _null_ _null_ _null_ _null_ _null_ ts_match_qv _null_ _null_ _null_ ));
DATA(insert OID = 3760 (  ts_match_tt			UXNSP UXUID 12 100 0 0 0 f f f f t f s s 2 0 16 "25 25" _null_ _null_ _null_ _null_ _null_ ts_match_tt _null_ _null_ _null_ ));
DATA(insert OID = 3761 (  ts_match_tq			UXNSP UXUID 12 100 0 0 0 f f f f t f s s 2 0 16 "25 3615" _null_ _null_ _null_ _null_ _null_ ts_match_tq _null_ _null_ _null_ ));

DATA(insert OID = 3648 (  gtsvector_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gtsvector_compress _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3649 (  gtsvector_decompress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gtsvector_decompress _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3650 (  gtsvector_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_picksplit _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3651 (  gtsvector_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3642 "2281 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_union _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3652 (  gtsvector_same		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "3642 3642 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_same _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3653 (  gtsvector_penalty		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_penalty _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3654 (  gtsvector_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 3614 21 26 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_consistent _null_ _null_ _null_ ));
DESCR("GiST tsvector support");
DATA(insert OID = 3790 (  gtsvector_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 3642 23 26 2281" _null_ _null_ _null_ _null_ _null_ gtsvector_consistent_oldsig _null_ _null_ _null_ ));
DESCR("GiST tsvector support (obsolete)");

DATA(insert OID = 3656 (  gin_extract_tsvector	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "3614 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_tsvector _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 3657 (  gin_extract_tsquery	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 2281 "3614 2281 21 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_tsquery _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 3658 (  gin_tsquery_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 8 0 16 "2281 21 3614 23 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_tsquery_consistent _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 3921 (  gin_tsquery_triconsistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 18 "2281 21 3614 23 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_tsquery_triconsistent _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 3724 (  gin_cmp_tslexeme		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ gin_cmp_tslexeme _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 2700 (  gin_cmp_prefix		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 23 "25 25 21 2281" _null_ _null_ _null_ _null_ _null_ gin_cmp_prefix _null_ _null_ _null_ ));
DESCR("GIN tsvector support");
DATA(insert OID = 3077 (  gin_extract_tsvector	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "3614 2281" _null_ _null_ _null_ _null_ _null_	gin_extract_tsvector_2args _null_ _null_ _null_ ));
DESCR("GIN tsvector support (obsolete)");
DATA(insert OID = 3087 (  gin_extract_tsquery	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 2281 "3615 2281 21 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_tsquery_5args _null_ _null_ _null_ ));
DESCR("GIN tsvector support (obsolete)");
DATA(insert OID = 3088 (  gin_tsquery_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 6 0 16 "2281 21 3615 23 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_tsquery_consistent_6args _null_ _null_ _null_ ));
DESCR("GIN tsvector support (obsolete)");
DATA(insert OID = 3791 (  gin_extract_tsquery	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 2281 "3615 2281 21 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_tsquery_oldsig _null_ _null_ _null_ ));
DESCR("GIN tsvector support (obsolete)");
DATA(insert OID = 3792 (  gin_tsquery_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 8 0 16 "2281 21 3615 23 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_tsquery_consistent_oldsig _null_ _null_ _null_ ));
DESCR("GIN tsvector support (obsolete)");

DATA(insert OID = 3789 (  gin_clean_pending_list UXNSP UXUID 12 1 0 0 0 f f f f t f v s 1 0 20 "2205" _null_ _null_ _null_ _null_ _null_ gin_clean_pending_list _null_ _null_ _null_ ));
DESCR("clean up GIN pending list");

DATA(insert OID = 3662 (  tsquery_lt			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_lt _null_ _null_ _null_ ));
DATA(insert OID = 3663 (  tsquery_le			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_le _null_ _null_ _null_ ));
DATA(insert OID = 3664 (  tsquery_eq			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_eq _null_ _null_ _null_ ));
DATA(insert OID = 3665 (  tsquery_ne			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_ne _null_ _null_ _null_ ));
DATA(insert OID = 3666 (  tsquery_ge			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_ge _null_ _null_ _null_ ));
DATA(insert OID = 3667 (  tsquery_gt			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_gt _null_ _null_ _null_ ));
DATA(insert OID = 3668 (  tsquery_cmp			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

DATA(insert OID = 3669 (  tsquery_and		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3615 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_and _null_ _null_ _null_ ));
DATA(insert OID = 3670 (  tsquery_or		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3615 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_or _null_ _null_ _null_ ));
DATA(insert OID = 5003 (  tsquery_phrase	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3615 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_phrase _null_ _null_ _null_ ));
DATA(insert OID = 5004 (  tsquery_phrase	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 3615 "3615 3615 23" _null_ _null_ _null_ _null_ _null_ tsquery_phrase_distance _null_ _null_ _null_ ));
DESCR("phrase-concatenate with distance");
DATA(insert OID = 3671 (  tsquery_not		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3615 "3615" _null_ _null_ _null_ _null_ _null_ tsquery_not _null_ _null_ _null_ ));

DATA(insert OID = 3691 (  tsq_mcontains		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsq_mcontains _null_ _null_ _null_ ));
DATA(insert OID = 3692 (  tsq_mcontained	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3615 3615" _null_ _null_ _null_ _null_ _null_ tsq_mcontained _null_ _null_ _null_ ));

DATA(insert OID = 3672 (  numnode			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3615" _null_ _null_ _null_ _null_ _null_ tsquery_numnode _null_ _null_ _null_ ));
DESCR("number of nodes");
DATA(insert OID = 3673 (  querytree			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "3615" _null_ _null_ _null_ _null_ _null_ tsquerytree _null_ _null_ _null_ ));
DESCR("show real useful query for GiST index");

DATA(insert OID = 3684 (  ts_rewrite		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 3615 "3615 3615 3615" _null_ _null_ _null_ _null_ _null_ tsquery_rewrite _null_ _null_ _null_ ));
DESCR("rewrite tsquery");
DATA(insert OID = 3685 (  ts_rewrite		UXNSP UXUID 12 100 0 0 0 f f f f t f v s 2 0 3615 "3615 25" _null_ _null_ _null_ _null_ _null_ tsquery_rewrite_query _null_ _null_ _null_ ));
DESCR("rewrite tsquery");

DATA(insert OID = 3695 (  gtsquery_compress				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gtsquery_compress _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3696 (  gtsquery_decompress			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ gtsquery_decompress _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3697 (  gtsquery_picksplit			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_picksplit _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3698 (  gtsquery_union				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 20 "2281 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_union _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3699 (  gtsquery_same					UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "20 20 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_same _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3700 (  gtsquery_penalty				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_penalty _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3701 (  gtsquery_consistent			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 3615 21 26 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_consistent _null_ _null_ _null_ ));
DESCR("GiST tsquery support");
DATA(insert OID = 3793 (  gtsquery_consistent			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 2281 23 26 2281" _null_ _null_ _null_ _null_ _null_ gtsquery_consistent_oldsig _null_ _null_ _null_ ));
DESCR("GiST tsquery support (obsolete)");

DATA(insert OID = 3686 (  tsmatchsel		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ tsmatchsel _null_ _null_ _null_ ));
DESCR("restriction selectivity of tsvector @@ tsquery");
DATA(insert OID = 3687 (  tsmatchjoinsel	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 5 0 701 "2281 26 2281 21 2281" _null_ _null_ _null_ _null_ _null_ tsmatchjoinsel _null_ _null_ _null_ ));
DESCR("join selectivity of tsvector @@ tsquery");
DATA(insert OID = 3688 (  ts_typanalyze		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_ ts_typanalyze _null_ _null_ _null_ ));
DESCR("tsvector typanalyze");

DATA(insert OID = 3689 (  ts_stat		UXNSP UXUID 12 10 10000 0 0 f f f f t t v s 1 0 2249 "25" "{25,25,23,23}" "{i,o,o,o}" "{query,word,ndoc,nentry}" _null_ _null_ ts_stat1 _null_ _null_ _null_ ));
DESCR("statistics of tsvector column");
DATA(insert OID = 3690 (  ts_stat		UXNSP UXUID 12 10 10000 0 0 f f f f t t v s 2 0 2249 "25 25" "{25,25,25,23,23}" "{i,i,o,o,o}" "{query,weights,word,ndoc,nentry}" _null_ _null_ ts_stat2 _null_ _null_ _null_ ));
DESCR("statistics of tsvector column");

DATA(insert OID = 3703 (  ts_rank		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 700 "1021 3614 3615 23" _null_ _null_ _null_ _null_ _null_ ts_rank_wttf _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3704 (  ts_rank		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 700 "1021 3614 3615" _null_ _null_ _null_ _null_ _null_ ts_rank_wtt _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3705 (  ts_rank		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 700 "3614 3615 23" _null_ _null_ _null_ _null_ _null_ ts_rank_ttf _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3706 (  ts_rank		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "3614 3615" _null_ _null_ _null_ _null_ _null_ ts_rank_tt _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3707 (  ts_rank_cd	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 700 "1021 3614 3615 23" _null_ _null_ _null_ _null_ _null_ ts_rankcd_wttf _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3708 (  ts_rank_cd	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 700 "1021 3614 3615" _null_ _null_ _null_ _null_ _null_ ts_rankcd_wtt _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3709 (  ts_rank_cd	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 700 "3614 3615 23" _null_ _null_ _null_ _null_ _null_ ts_rankcd_ttf _null_ _null_ _null_ ));
DESCR("relevance");
DATA(insert OID = 3710 (  ts_rank_cd	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 700 "3614 3615" _null_ _null_ _null_ _null_ _null_ ts_rankcd_tt _null_ _null_ _null_ ));
DESCR("relevance");

DATA(insert OID = 3713 (  ts_token_type UXNSP UXUID 12 1 16 0 0 f f f f t t i s 1 0 2249 "26" "{26,23,25,25}" "{i,o,o,o}" "{parser_oid,tokid,alias,description}" _null_ _null_ ts_token_type_byid _null_ _null_ _null_ ));
DESCR("get parser's token types");
DATA(insert OID = 3714 (  ts_token_type UXNSP UXUID 12 1 16 0 0 f f f f t t s s 1 0 2249 "25" "{25,23,25,25}" "{i,o,o,o}" "{parser_name,tokid,alias,description}" _null_ _null_ ts_token_type_byname _null_ _null_ _null_ ));
DESCR("get parser's token types");
DATA(insert OID = 3715 (  ts_parse		UXNSP UXUID 12 1 1000 0 0 f f f f t t i s 2 0 2249 "26 25" "{26,25,23,25}" "{i,i,o,o}" "{parser_oid,txt,tokid,token}" _null_ _null_ ts_parse_byid _null_ _null_ _null_ ));
DESCR("parse text to tokens");
DATA(insert OID = 3716 (  ts_parse		UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 2 0 2249 "25 25" "{25,25,23,25}" "{i,i,o,o}" "{parser_name,txt,tokid,token}" _null_ _null_ ts_parse_byname _null_ _null_ _null_ ));
DESCR("parse text to tokens");

DATA(insert OID = 3717 (  prsd_start		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 23" _null_ _null_ _null_ _null_ _null_ prsd_start _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3718 (  prsd_nexttoken	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ prsd_nexttoken _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3719 (  prsd_end			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2278 "2281" _null_ _null_ _null_ _null_ _null_ prsd_end _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3720 (  prsd_headline		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 3615" _null_ _null_ _null_ _null_ _null_ prsd_headline _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3721 (  prsd_lextype		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ prsd_lextype _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 3723 (  ts_lexize			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 1009 "3769 25" _null_ _null_ _null_ _null_ _null_ ts_lexize _null_ _null_ _null_ ));
DESCR("normalize one word by dictionary");

DATA(insert OID = 3725 (  dsimple_init		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ dsimple_init _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3726 (  dsimple_lexize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 2281 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ dsimple_lexize _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 3728 (  dsynonym_init		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ dsynonym_init _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3729 (  dsynonym_lexize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 2281 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ dsynonym_lexize _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 3731 (  dispell_init		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ dispell_init _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3732 (  dispell_lexize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 2281 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ dispell_lexize _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 3740 (  thesaurus_init	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ thesaurus_init _null_ _null_ _null_ ));
DESCR("(internal)");
DATA(insert OID = 3741 (  thesaurus_lexize	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 2281 "2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ thesaurus_lexize _null_ _null_ _null_ ));
DESCR("(internal)");

DATA(insert OID = 3743 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 4 0 25 "3734 25 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_byid_opt _null_ _null_ _null_ ));
DESCR("generate headline");
DATA(insert OID = 3744 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 3 0 25 "3734 25 3615" _null_ _null_ _null_ _null_ _null_ ts_headline_byid _null_ _null_ _null_ ));
DESCR("generate headline");
DATA(insert OID = 3754 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 3 0 25 "25 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_opt _null_ _null_ _null_ ));
DESCR("generate headline");
DATA(insert OID = 3755 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 2 0 25 "25 3615" _null_ _null_ _null_ _null_ _null_ ts_headline _null_ _null_ _null_ ));
DESCR("generate headline");

DATA(insert OID = 4201 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 4 0 3802 "3734 3802 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_jsonb_byid_opt _null_ _null_ _null_ ));
DESCR("generate headline from jsonb");
DATA(insert OID = 4202 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 3 0 3802 "3734 3802 3615" _null_ _null_ _null_ _null_ _null_ ts_headline_jsonb_byid _null_ _null_ _null_ ));
DESCR("generate headline from jsonb");
DATA(insert OID = 4203 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 3 0 3802 "3802 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_jsonb_opt _null_ _null_ _null_ ));
DESCR("generate headline from jsonb");
DATA(insert OID = 4204 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 2 0 3802 "3802 3615" _null_ _null_ _null_ _null_ _null_ ts_headline_jsonb _null_ _null_ _null_ ));
DESCR("generate headline from jsonb");

DATA(insert OID = 4205 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 4 0 114 "3734 114 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_json_byid_opt _null_ _null_ _null_ ));
DESCR("generate headline from json");
DATA(insert OID = 4206 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 3 0 114 "3734 114 3615" _null_ _null_ _null_ _null_ _null_ ts_headline_json_byid _null_ _null_ _null_ ));
DESCR("generate headline from json");
DATA(insert OID = 4207 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 3 0 114 "114 3615 25" _null_ _null_ _null_ _null_ _null_ ts_headline_json_opt _null_ _null_ _null_ ));
DESCR("generate headline from json");
DATA(insert OID = 4208 (  ts_headline	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 2 0 114 "114 3615" _null_ _null_ _null_ _null_ _null_ ts_headline_json _null_ _null_ _null_ ));
DESCR("generate headline from json");

DATA(insert OID = 3745 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3614 "3734 25" _null_ _null_ _null_ _null_ _null_ to_tsvector_byid _null_ _null_ _null_ ));
DESCR("transform to tsvector");
DATA(insert OID = 3746 (  to_tsquery		UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3615 "3734 25" _null_ _null_ _null_ _null_ _null_ to_tsquery_byid _null_ _null_ _null_ ));
DESCR("make tsquery");
DATA(insert OID = 3747 (  plainto_tsquery	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3615 "3734 25" _null_ _null_ _null_ _null_ _null_ plainto_tsquery_byid _null_ _null_ _null_ ));
DESCR("transform to tsquery");
DATA(insert OID = 5006 (  phraseto_tsquery	UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3615 "3734 25" _null_ _null_ _null_ _null_ _null_ phraseto_tsquery_byid _null_ _null_ _null_ ));
DESCR("transform to tsquery");
DATA(insert OID = 3749 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3614 "25" _null_ _null_ _null_ _null_ _null_ to_tsvector _null_ _null_ _null_ ));
DESCR("transform to tsvector");
DATA(insert OID = 3750 (  to_tsquery		UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3615 "25" _null_ _null_ _null_ _null_ _null_ to_tsquery _null_ _null_ _null_ ));
DESCR("make tsquery");
DATA(insert OID = 3751 (  plainto_tsquery	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3615 "25" _null_ _null_ _null_ _null_ _null_ plainto_tsquery _null_ _null_ _null_ ));
DESCR("transform to tsquery");
DATA(insert OID = 5001 (  phraseto_tsquery	UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3615 "25" _null_ _null_ _null_ _null_ _null_ phraseto_tsquery _null_ _null_ _null_ ));
DESCR("transform to tsquery");
DATA(insert OID = 4209 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3614 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_to_tsvector _null_ _null_ _null_ ));
DESCR("transform jsonb to tsvector");
DATA(insert OID = 4210 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f s s 1 0 3614 "114" _null_ _null_ _null_ _null_ _null_ json_to_tsvector _null_ _null_ _null_ ));
DESCR("transform json to tsvector");
DATA(insert OID = 4211 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3614 "3734 3802" _null_ _null_ _null_ _null_ _null_ jsonb_to_tsvector_byid _null_ _null_ _null_ ));
DESCR("transform jsonb to tsvector");
DATA(insert OID = 4212 (  to_tsvector		UXNSP UXUID 12 100 0 0 0 f f f f t f i s 2 0 3614 "3734 114" _null_ _null_ _null_ _null_ _null_ json_to_tsvector_byid _null_ _null_ _null_ ));
DESCR("transform json to tsvector");

DATA(insert OID = 3752 (  tsvector_update_trigger			UXNSP UXUID 12 1 0 0 0 f f f f f f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ tsvector_update_trigger_byid _null_ _null_ _null_ ));
DESCR("trigger for automatic update of tsvector column");
DATA(insert OID = 3753 (  tsvector_update_trigger_column	UXNSP UXUID 12 1 0 0 0 f f f f f f v s 0 0 2279 "" _null_ _null_ _null_ _null_ _null_ tsvector_update_trigger_bycolumn _null_ _null_ _null_ ));
DESCR("trigger for automatic update of tsvector column");

DATA(insert OID = 3759 (  get_current_ts_config UXNSP UXUID 12 1 0 0 0 f f f f t f s s 0 0 3734 "" _null_ _null_ _null_ _null_ _null_ get_current_ts_config _null_ _null_ _null_ ));
DESCR("get current tsearch configuration");

DATA(insert OID = 3736 (  regconfigin		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 3734 "2275" _null_ _null_ _null_ _null_ _null_ regconfigin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3737 (  regconfigout		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3734" _null_ _null_ _null_ _null_ _null_ regconfigout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3738 (  regconfigrecv		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3734 "2281" _null_ _null_ _null_ _null_ _null_ regconfigrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3739 (  regconfigsend		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3734" _null_ _null_ _null_ _null_ _null_ regconfigsend _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 3771 (  regdictionaryin	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 3769 "2275" _null_ _null_ _null_ _null_ _null_ regdictionaryin _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3772 (  regdictionaryout	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3769" _null_ _null_ _null_ _null_ _null_ regdictionaryout _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3773 (  regdictionaryrecv UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3769 "2281" _null_ _null_ _null_ _null_ _null_ regdictionaryrecv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3774 (  regdictionarysend UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3769" _null_ _null_ _null_ _null_ _null_ regdictionarysend _null_ _null_ _null_ ));
DESCR("I/O");

/* jsonb */
DATA(insert OID =  3806 (  jsonb_in			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3802 "2275" _null_ _null_ _null_ _null_ _null_ jsonb_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3805 (  jsonb_recv		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3802 "2281" _null_ _null_ _null_ _null_ _null_ jsonb_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3804 (  jsonb_out		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2275 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID =  3803 (  jsonb_send		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 17 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_send _null_ _null_ _null_ ));
DESCR("I/O");

DATA(insert OID = 3263 (  jsonb_object	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3802 "1009" _null_ _null_ _null_ _null_ _null_ jsonb_object _null_ _null_ _null_ ));
DESCR("map text array of key value pairs to jsonb object");
DATA(insert OID = 3264 (  jsonb_object	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "1009 1009" _null_ _null_ _null_ _null_ _null_ jsonb_object_two_arg _null_ _null_ _null_ ));
DESCR("map text array of key value pairs to jsonb object");
DATA(insert OID = 3787 (  to_jsonb	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 3802 "2283" _null_ _null_ _null_ _null_ _null_ to_jsonb _null_ _null_ _null_ ));
DESCR("map input to jsonb");
DATA(insert OID = 3265 (  jsonb_agg_transfn  UXNSP UXUID 12 1 0 0 0 f f f f f f s s 2 0 2281 "2281 2283" _null_ _null_ _null_ _null_ _null_ jsonb_agg_transfn _null_ _null_ _null_ ));
DESCR("jsonb aggregate transition function");
DATA(insert OID = 3266 (  jsonb_agg_finalfn  UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 3802 "2281" _null_ _null_ _null_ _null_ _null_ jsonb_agg_finalfn _null_ _null_ _null_ ));
DESCR("jsonb aggregate final function");
DATA(insert OID = 3267 (  jsonb_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f s s 1 0 3802 "2283" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("aggregate input into jsonb");
DATA(insert OID = 3268 (  jsonb_object_agg_transfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f s s 3 0 2281 "2281 2276 2276" _null_ _null_ _null_ _null_ _null_ jsonb_object_agg_transfn _null_ _null_ _null_ ));
DESCR("jsonb object aggregate transition function");
DATA(insert OID = 3269 (  jsonb_object_agg_finalfn	 UXNSP UXUID 12 1 0 0 0 f f f f f f s s 1 0 3802 "2281" _null_ _null_ _null_ _null_ _null_ jsonb_object_agg_finalfn _null_ _null_ _null_ ));
DESCR("jsonb object aggregate final function");
DATA(insert OID = 3270 (  jsonb_object_agg		   UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 3802 "2276 2276" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("aggregate inputs into jsonb object");
DATA(insert OID = 3271 (  jsonb_build_array    UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 1 0 3802 "2276" "{2276}" "{v}" _null_ _null_ _null_ jsonb_build_array _null_ _null_ _null_ ));
DESCR("build a jsonb array from any inputs");
DATA(insert OID = 3272 (  jsonb_build_array    UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 3802	"" _null_ _null_ _null_ _null_ _null_ jsonb_build_array_noargs _null_ _null_ _null_ ));
DESCR("build an empty jsonb array");
DATA(insert OID = 3273 (  jsonb_build_object	UXNSP UXUID 12 1 0 2276 0 f f f f f f s s 1 0 3802 "2276" "{2276}" "{v}" _null_ _null_ _null_ jsonb_build_object _null_ _null_ _null_ ));
DESCR("build a jsonb object from pairwise key/value inputs");
DATA(insert OID = 3274 (  jsonb_build_object	UXNSP UXUID 12 1 0 0 0 f f f f f f s s 0 0 3802  "" _null_ _null_ _null_ _null_ _null_ jsonb_build_object_noargs _null_ _null_ _null_ ));
DESCR("build an empty jsonb object");
DATA(insert OID = 3262 (  jsonb_strip_nulls    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3802 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_strip_nulls _null_ _null_ _null_ ));
DESCR("remove object fields with null values from jsonb");

DATA(insert OID = 3478 (  jsonb_object_field			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 25" _null_ _null_ "{from_json, field_name}" _null_ _null_ jsonb_object_field _null_ _null_ _null_ ));
DATA(insert OID = 3214 (  jsonb_object_field_text	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25  "3802 25" _null_ _null_ "{from_json, field_name}" _null_ _null_ jsonb_object_field_text _null_ _null_ _null_ ));
DATA(insert OID = 3215 (  jsonb_array_element		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 23" _null_ _null_ "{from_json, element_index}" _null_ _null_ jsonb_array_element _null_ _null_ _null_ ));
DATA(insert OID = 3216 (  jsonb_array_element_text	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 25  "3802 23" _null_ _null_ "{from_json, element_index}" _null_ _null_ jsonb_array_element_text _null_ _null_ _null_ ));
DATA(insert OID = 3217 (  jsonb_extract_path			UXNSP UXUID 12 1 0 25 0 f f f f t f i s 2 0 3802 "3802 1009" "{3802,1009}" "{i,v}" "{from_json,path_elems}" _null_ _null_ jsonb_extract_path _null_ _null_ _null_ ));
DESCR("get value from jsonb with path elements");
DATA(insert OID = 3940 (  jsonb_extract_path_text	UXNSP UXUID 12 1 0 25 0 f f f f t f i s 2 0 25 "3802 1009" "{3802,1009}" "{i,v}" "{from_json,path_elems}" _null_ _null_ jsonb_extract_path_text _null_ _null_ _null_ ));
DESCR("get value from jsonb as text with path elements");
DATA(insert OID = 3219 (  jsonb_array_elements		UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 3802 "3802" "{3802,3802}" "{i,o}" "{from_json,value}" _null_ _null_ jsonb_array_elements _null_ _null_ _null_ ));
DESCR("elements of a jsonb array");
DATA(insert OID = 3465 (  jsonb_array_elements_text UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 25 "3802" "{3802,25}" "{i,o}" "{from_json,value}" _null_ _null_ jsonb_array_elements_text _null_ _null_ _null_ ));
DESCR("elements of jsonb array");
DATA(insert OID = 3207 (  jsonb_array_length			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_array_length _null_ _null_ _null_ ));
DESCR("length of jsonb array");
DATA(insert OID = 3931 (  jsonb_object_keys			UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 25 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_object_keys _null_ _null_ _null_ ));
DESCR("get jsonb object keys");
DATA(insert OID = 3208 (  jsonb_each				   UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 2249 "3802" "{3802,25,3802}" "{i,o,o}" "{from_json,key,value}" _null_ _null_ jsonb_each _null_ _null_ _null_ ));
DESCR("key value pairs of a jsonb object");
DATA(insert OID = 3932 (  jsonb_each_text		   UXNSP UXUID 12 1 100 0 0 f f f f t t i s 1 0 2249 "3802" "{3802,25,25}" "{i,o,o}" "{from_json,key,value}" _null_ _null_ jsonb_each_text _null_ _null_ _null_ ));
DESCR("key value pairs of a jsonb object");
DATA(insert OID = 3209 (  jsonb_populate_record    UXNSP UXUID 12 1 0 0 0 f f f f f f s s 2 0 2283 "2283 3802" _null_ _null_ _null_ _null_ _null_ jsonb_populate_record _null_ _null_ _null_ ));
DESCR("get record fields from a jsonb object");
DATA(insert OID = 3475 (  jsonb_populate_recordset	UXNSP UXUID 12 1 100 0 0 f f f f f t s s 2 0 2283 "2283 3802" _null_ _null_ _null_ _null_ _null_ jsonb_populate_recordset _null_ _null_ _null_ ));
DESCR("get set of records with fields from a jsonb array of objects");
DATA(insert OID = 3490 (  jsonb_to_record			UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2249 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_to_record _null_ _null_ _null_ ));
DESCR("get record fields from a jsonb object");
DATA(insert OID = 3491 (  jsonb_to_recordset		UXNSP UXUID 12 1 100 0 0 f f f f f t s s 1 0 2249 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_to_recordset _null_ _null_ _null_ ));
DESCR("get set of records with fields from a jsonb array of objects");
DATA(insert OID = 3210 (  jsonb_typeof				UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_typeof _null_ _null_ _null_ ));
DESCR("get the type of a jsonb value");
DATA(insert OID = 4038 (  jsonb_ne		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_ne _null_ _null_ _null_ ));
DATA(insert OID = 4039 (  jsonb_lt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_lt _null_ _null_ _null_ ));
DATA(insert OID = 4040 (  jsonb_gt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_gt _null_ _null_ _null_ ));
DATA(insert OID = 4041 (  jsonb_le		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_le _null_ _null_ _null_ ));
DATA(insert OID = 4042 (  jsonb_ge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_ge _null_ _null_ _null_ ));
DATA(insert OID = 4043 (  jsonb_eq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_eq _null_ _null_ _null_ ));
DATA(insert OID = 4044 (  jsonb_cmp		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 4045 (  jsonb_hash	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_hash _null_ _null_ _null_ ));
DESCR("hash");
DATA(insert OID = 4046 (  jsonb_contains   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_contains _null_ _null_ _null_ ));
DATA(insert OID = 4047 (  jsonb_exists	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 25" _null_ _null_ _null_ _null_ _null_ jsonb_exists _null_ _null_ _null_ ));
DATA(insert OID = 4048 (  jsonb_exists_any	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 1009" _null_ _null_ _null_ _null_ _null_ jsonb_exists_any _null_ _null_ _null_ ));
DATA(insert OID = 4049 (  jsonb_exists_all	 UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 1009" _null_ _null_ _null_ _null_ _null_ jsonb_exists_all _null_ _null_ _null_ ));
DATA(insert OID = 4050 (  jsonb_contained	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_contained _null_ _null_ _null_ ));
DATA(insert OID = 3480 (  gin_compare_jsonb  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "25 25" _null_ _null_ _null_ _null_ _null_ gin_compare_jsonb _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3482 (  gin_extract_jsonb  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "3802 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_jsonb _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3483 (  gin_extract_jsonb_query  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 2281 "3802 2281 21 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_jsonb_query _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3484 (  gin_consistent_jsonb	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 8 0 16 "2281 21 3802 23 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_consistent_jsonb _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3488 (  gin_triconsistent_jsonb	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 18 "2281 21 3802 23 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_triconsistent_jsonb _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3485 (  gin_extract_jsonb_path  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "3802 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_jsonb_path _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3486 (  gin_extract_jsonb_query_path	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 2281 "3802 2281 21 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_extract_jsonb_query_path _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3487 (  gin_consistent_jsonb_path  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 8 0 16 "2281 21 3802 23 2281 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_consistent_jsonb_path _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3489 (  gin_triconsistent_jsonb_path	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 18 "2281 21 3802 23 2281 2281 2281" _null_ _null_ _null_ _null_ _null_ gin_triconsistent_jsonb_path _null_ _null_ _null_ ));
DESCR("GIN support");
DATA(insert OID = 3301 (  jsonb_concat	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 3802" _null_ _null_ _null_ _null_ _null_ jsonb_concat _null_ _null_ _null_ ));
DATA(insert OID = 3302 (  jsonb_delete	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 25" _null_ _null_ _null_ _null_ _null_ jsonb_delete _null_ _null_ _null_ ));
DATA(insert OID = 3303 (  jsonb_delete	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 23" _null_ _null_ _null_ _null_ _null_ jsonb_delete_idx _null_ _null_ _null_ ));
DATA(insert OID = 3343 ( jsonb_delete	   UXNSP UXUID 12 1 0 25 0 f f f f t f i s 2 0 3802 "3802 1009" "{3802,1009}" "{i,v}" "{from_json,path_elems}" _null_ _null_ jsonb_delete_array _null_ _null_ _null_ ));
DATA(insert OID = 3304 (  jsonb_delete_path    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3802 "3802 1009" _null_ _null_ _null_ _null_ _null_ jsonb_delete_path _null_ _null_ _null_ ));
DATA(insert OID = 3305 (  jsonb_set    UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 3802 "3802 1009 3802 16" _null_ _null_ _null_ _null_ _null_ jsonb_set _null_ _null_ _null_ ));
DESCR("Set part of a jsonb");
DATA(insert OID = 3306 (  jsonb_pretty	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 25 "3802" _null_ _null_ _null_ _null_ _null_ jsonb_pretty _null_ _null_ _null_ ));
DESCR("Indented text from jsonb");
DATA(insert OID = 3579 (  jsonb_insert	  UXNSP UXUID 12 1 0 0 0 f f f f t f i s 4 0 3802 "3802 1009 3802 16" _null_ _null_ _null_ _null_ _null_ jsonb_insert _null_ _null_ _null_ ));
DESCR("Insert value into a jsonb");
/* txid */
DATA(insert OID = 2939 (  txid_snapshot_in			UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 2970 "2275" _null_ _null_ _null_ _null_ _null_ txid_snapshot_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2940 (  txid_snapshot_out			UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 2275 "2970" _null_ _null_ _null_ _null_ _null_ txid_snapshot_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2941 (  txid_snapshot_recv		UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 2970 "2281" _null_ _null_ _null_ _null_ _null_ txid_snapshot_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2942 (  txid_snapshot_send		UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 17 "2970" _null_ _null_ _null_ _null_ _null_ txid_snapshot_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 2943 (  txid_current				UXNSP UXUID 12 1  0 0 0 f f f f t f s u 0 0 20 "" _null_ _null_ _null_ _null_ _null_ txid_current _null_ _null_ _null_ ));
DESCR("get current transaction ID");
DATA(insert OID = 3348 (  txid_current_if_assigned	UXNSP UXUID 12 1  0 0 0 f f f f t f s u 0 0 20 "" _null_ _null_ _null_ _null_ _null_ txid_current_if_assigned _null_ _null_ _null_ ));
DESCR("get current transaction ID");
DATA(insert OID = 2944 (  txid_current_snapshot		UXNSP UXUID 12 1  0 0 0 f f f f t f s s 0 0 2970 "" _null_ _null_ _null_ _null_ _null_ txid_current_snapshot _null_ _null_ _null_ ));
DESCR("get current snapshot");
DATA(insert OID = 2945 (  txid_snapshot_xmin		UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 20 "2970" _null_ _null_ _null_ _null_ _null_ txid_snapshot_xmin _null_ _null_ _null_ ));
DESCR("get xmin of snapshot");
DATA(insert OID = 2946 (  txid_snapshot_xmax		UXNSP UXUID 12 1  0 0 0 f f f f t f i s 1 0 20 "2970" _null_ _null_ _null_ _null_ _null_ txid_snapshot_xmax _null_ _null_ _null_ ));
DESCR("get xmax of snapshot");
DATA(insert OID = 2947 (  txid_snapshot_xip			UXNSP UXUID 12 1 50 0 0 f f f f t t i s 1 0 20 "2970" _null_ _null_ _null_ _null_ _null_ txid_snapshot_xip _null_ _null_ _null_ ));
DESCR("get set of in-progress txids in snapshot");
DATA(insert OID = 2948 (  txid_visible_in_snapshot	UXNSP UXUID 12 1  0 0 0 f f f f t f i s 2 0 16 "20 2970" _null_ _null_ _null_ _null_ _null_ txid_visible_in_snapshot _null_ _null_ _null_ ));
DESCR("is txid visible in snapshot?");
DATA(insert OID = 3360 (  txid_status				UXNSP UXUID 12 1  0 0 0 f f f f t f v s 1 0 25 "20" _null_ _null_ _null_ _null_ _null_ txid_status _null_ _null_ _null_ ));
DESCR("commit status of transaction");

/* record comparison using normal comparison rules */
DATA(insert OID = 2981 (  record_eq		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_eq _null_ _null_ _null_ ));
DATA(insert OID = 2982 (  record_ne		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_ne _null_ _null_ _null_ ));
DATA(insert OID = 2983 (  record_lt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_lt _null_ _null_ _null_ ));
DATA(insert OID = 2984 (  record_gt		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_gt _null_ _null_ _null_ ));
DATA(insert OID = 2985 (  record_le		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_le _null_ _null_ _null_ ));
DATA(insert OID = 2986 (  record_ge		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_ge _null_ _null_ _null_ ));
DATA(insert OID = 2987 (  btrecordcmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2249 2249" _null_ _null_ _null_ _null_ _null_ btrecordcmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");

/* record comparison using raw byte images */
DATA(insert OID = 3181 (  record_image_eq	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_eq _null_ _null_ _null_ ));
DATA(insert OID = 3182 (  record_image_ne	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_ne _null_ _null_ _null_ ));
DATA(insert OID = 3183 (  record_image_lt	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_lt _null_ _null_ _null_ ));
DATA(insert OID = 3184 (  record_image_gt	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_gt _null_ _null_ _null_ ));
DATA(insert OID = 3185 (  record_image_le	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_le _null_ _null_ _null_ ));
DATA(insert OID = 3186 (  record_image_ge	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2249 2249" _null_ _null_ _null_ _null_ _null_ record_image_ge _null_ _null_ _null_ ));
DATA(insert OID = 3187 (  btrecordimagecmp	   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "2249 2249" _null_ _null_ _null_ _null_ _null_ btrecordimagecmp _null_ _null_ _null_ ));
DESCR("less-equal-greater based on byte images");

/* Extensions */
DATA(insert OID = 3082 (  ux_available_extensions		UXNSP UXUID 12 10 100 0 0 f f f f t t s s 0 0 2249 "" "{19,25,25}" "{o,o,o}" "{name,default_version,comment}" _null_ _null_ ux_available_extensions _null_ _null_ _null_ ));
DESCR("list available extensions");
DATA(insert OID = 3083 (  ux_available_extension_versions	UXNSP UXUID 12 10 100 0 0 f f f f t t s s 0 0 2249 "" "{19,25,16,16,19,1003,25}" "{o,o,o,o,o,o,o}" "{name,version,superuser,relocatable,schema,requires,comment}" _null_ _null_ ux_available_extension_versions _null_ _null_ _null_ ));
DESCR("list available extension versions");
DATA(insert OID = 3084 (  ux_extension_update_paths		UXNSP UXUID 12 10 100 0 0 f f f f t t s s 1 0 2249 "19" "{19,25,25,25}" "{i,o,o,o}" "{name,source,target,path}" _null_ _null_ ux_extension_update_paths _null_ _null_ _null_ ));
DESCR("list an extension's version update paths");
DATA(insert OID = 3086 (  ux_extension_config_dump		UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "2205 25" _null_ _null_ _null_ _null_ _null_ ux_extension_config_dump _null_ _null_ _null_ ));
DESCR("flag an extension's table contents to be emitted by ux_dump");

/* SQL-spec window functions */
DATA(insert OID = 3100 (  row_number	UXNSP UXUID 12 1 0 0 0 f t f f f f i s 0 0 20 "" _null_ _null_ _null_ _null_ _null_ window_row_number _null_ _null_ _null_ ));
DESCR("row number within partition");
DATA(insert OID = 3101 (  rank			UXNSP UXUID 12 1 0 0 0 f t f f f f i s 0 0 20 "" _null_ _null_ _null_ _null_ _null_ window_rank _null_ _null_ _null_ ));
DESCR("integer rank with gaps");
DATA(insert OID = 3102 (  dense_rank	UXNSP UXUID 12 1 0 0 0 f t f f f f i s 0 0 20 "" _null_ _null_ _null_ _null_ _null_ window_dense_rank _null_ _null_ _null_ ));
DESCR("integer rank without gaps");
DATA(insert OID = 3103 (  percent_rank	UXNSP UXUID 12 1 0 0 0 f t f f f f i s 0 0 701 "" _null_ _null_ _null_ _null_ _null_ window_percent_rank _null_ _null_ _null_ ));
DESCR("fractional rank within partition");
DATA(insert OID = 3104 (  cume_dist		UXNSP UXUID 12 1 0 0 0 f t f f f f i s 0 0 701 "" _null_ _null_ _null_ _null_ _null_ window_cume_dist _null_ _null_ _null_ ));
DESCR("fractional row number within partition");
DATA(insert OID = 3105 (  ntile			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 1 0 23 "23" _null_ _null_ _null_ _null_ _null_ window_ntile _null_ _null_ _null_ ));
DESCR("split rows into N groups");
DATA(insert OID = 3106 (  lag			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 1 0 2283 "2283" _null_ _null_ _null_ _null_ _null_ window_lag _null_ _null_ _null_ ));
DESCR("fetch the preceding row value");
DATA(insert OID = 3107 (  lag			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 2 0 2283 "2283 23" _null_ _null_ _null_ _null_ _null_ window_lag_with_offset _null_ _null_ _null_ ));
DESCR("fetch the Nth preceding row value");
DATA(insert OID = 3108 (  lag			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 3 0 2283 "2283 23 2283" _null_ _null_ _null_ _null_ _null_ window_lag_with_offset_and_default _null_ _null_ _null_ ));
DESCR("fetch the Nth preceding row value with default");
DATA(insert OID = 3109 (  lead			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 1 0 2283 "2283" _null_ _null_ _null_ _null_ _null_ window_lead _null_ _null_ _null_ ));
DESCR("fetch the following row value");
DATA(insert OID = 3110 (  lead			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 2 0 2283 "2283 23" _null_ _null_ _null_ _null_ _null_ window_lead_with_offset _null_ _null_ _null_ ));
DESCR("fetch the Nth following row value");
DATA(insert OID = 3111 (  lead			UXNSP UXUID 12 1 0 0 0 f t f f t f i s 3 0 2283 "2283 23 2283" _null_ _null_ _null_ _null_ _null_ window_lead_with_offset_and_default _null_ _null_ _null_ ));
DESCR("fetch the Nth following row value with default");
DATA(insert OID = 3112 (  first_value	UXNSP UXUID 12 1 0 0 0 f t f f t f i s 1 0 2283 "2283" _null_ _null_ _null_ _null_ _null_ window_first_value _null_ _null_ _null_ ));
DESCR("fetch the first row value");
DATA(insert OID = 3113 (  last_value	UXNSP UXUID 12 1 0 0 0 f t f f t f i s 1 0 2283 "2283" _null_ _null_ _null_ _null_ _null_ window_last_value _null_ _null_ _null_ ));
DESCR("fetch the last row value");
DATA(insert OID = 3114 (  nth_value		UXNSP UXUID 12 1 0 0 0 f t f f t f i s 2 0 2283 "2283 23" _null_ _null_ _null_ _null_ _null_ window_nth_value _null_ _null_ _null_ ));
DESCR("fetch the Nth row value");

/* functions for range types */
DATA(insert OID = 3832 (  anyrange_in	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 3831 "2275 26 23" _null_ _null_ _null_ _null_ _null_ anyrange_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3833 (  anyrange_out	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3831" _null_ _null_ _null_ _null_ _null_ anyrange_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3834 (  range_in		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 3831 "2275 26 23" _null_ _null_ _null_ _null_ _null_ range_in _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3835 (  range_out		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 2275 "3831" _null_ _null_ _null_ _null_ _null_ range_out _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3836 (  range_recv	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 3 0 3831 "2281 26 23" _null_ _null_ _null_ _null_ _null_ range_recv _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3837 (  range_send	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 17 "3831" _null_ _null_ _null_ _null_ _null_ range_send _null_ _null_ _null_ ));
DESCR("I/O");
DATA(insert OID = 3848 (  lower		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2283 "3831" _null_ _null_ _null_ _null_ _null_ range_lower _null_ _null_ _null_ ));
DESCR("lower bound of range");
DATA(insert OID = 3849 (  upper		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2283 "3831" _null_ _null_ _null_ _null_ _null_ range_upper _null_ _null_ _null_ ));
DESCR("upper bound of range");
DATA(insert OID = 3850 (  isempty	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "3831" _null_ _null_ _null_ _null_ _null_ range_empty _null_ _null_ _null_ ));
DESCR("is the range empty?");
DATA(insert OID = 3851 (  lower_inc UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "3831" _null_ _null_ _null_ _null_ _null_ range_lower_inc _null_ _null_ _null_ ));
DESCR("is the range's lower bound inclusive?");
DATA(insert OID = 3852 (  upper_inc UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "3831" _null_ _null_ _null_ _null_ _null_ range_upper_inc _null_ _null_ _null_ ));
DESCR("is the range's upper bound inclusive?");
DATA(insert OID = 3853 (  lower_inf UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "3831" _null_ _null_ _null_ _null_ _null_ range_lower_inf _null_ _null_ _null_ ));
DESCR("is the range's lower bound infinite?");
DATA(insert OID = 3854 (  upper_inf UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 16 "3831" _null_ _null_ _null_ _null_ _null_ range_upper_inf _null_ _null_ _null_ ));
DESCR("is the range's upper bound infinite?");
DATA(insert OID = 3855 (  range_eq	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_eq _null_ _null_ _null_ ));
DATA(insert OID = 3856 (  range_ne	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_ne _null_ _null_ _null_ ));
DATA(insert OID = 3857 (  range_overlaps		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_overlaps _null_ _null_ _null_ ));
DATA(insert OID = 3858 (  range_contains_elem	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 2283" _null_ _null_ _null_ _null_ _null_ range_contains_elem _null_ _null_ _null_ ));
DATA(insert OID = 3859 (  range_contains		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_contains _null_ _null_ _null_ ));
DATA(insert OID = 3860 (  elem_contained_by_range	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2283 3831" _null_ _null_ _null_ _null_ _null_ elem_contained_by_range _null_ _null_ _null_ ));
DATA(insert OID = 3861 (  range_contained_by	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_contained_by _null_ _null_ _null_ ));
DATA(insert OID = 3862 (  range_adjacent		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_adjacent _null_ _null_ _null_ ));
DATA(insert OID = 3863 (  range_before		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_before _null_ _null_ _null_ ));
DATA(insert OID = 3864 (  range_after		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_after _null_ _null_ _null_ ));
DATA(insert OID = 3865 (  range_overleft	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_overleft _null_ _null_ _null_ ));
DATA(insert OID = 3866 (  range_overright	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_overright _null_ _null_ _null_ ));
DATA(insert OID = 3867 (  range_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3831 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_union _null_ _null_ _null_ ));
DATA(insert OID = 4057 (  range_merge		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3831 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_merge _null_ _null_ _null_ ));
DESCR("the smallest range which includes both of the given ranges");
DATA(insert OID = 3868 (  range_intersect	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3831 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_intersect _null_ _null_ _null_ ));
DATA(insert OID = 3869 (  range_minus		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3831 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_minus _null_ _null_ _null_ ));
DATA(insert OID = 3870 (  range_cmp UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 23 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_cmp _null_ _null_ _null_ ));
DESCR("less-equal-greater");
DATA(insert OID = 3871 (  range_lt	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_lt _null_ _null_ _null_ ));
DATA(insert OID = 3872 (  range_le	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_le _null_ _null_ _null_ ));
DATA(insert OID = 3873 (  range_ge	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_ge _null_ _null_ _null_ ));
DATA(insert OID = 3874 (  range_gt	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "3831 3831" _null_ _null_ _null_ _null_ _null_ range_gt _null_ _null_ _null_ ));
DATA(insert OID = 3875 (  range_gist_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 5 0 16 "2281 3831 21 26 2281" _null_ _null_ _null_ _null_ _null_ range_gist_consistent _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3876 (  range_gist_union		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 3831 "2281 2281" _null_ _null_ _null_ _null_ _null_ range_gist_union _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3877 (  range_gist_compress	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ range_gist_compress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3878 (  range_gist_decompress UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ range_gist_decompress _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3996 (  range_gist_fetch		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 2281 "2281" _null_ _null_ _null_ _null_ _null_ range_gist_fetch _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3879 (  range_gist_penalty	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "2281 2281 2281" _null_ _null_ _null_ _null_ _null_ range_gist_penalty _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3880 (  range_gist_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2281 "2281 2281" _null_ _null_ _null_ _null_ _null_ range_gist_picksplit _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3881 (  range_gist_same		UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 2281 "3831 3831 2281" _null_ _null_ _null_ _null_ _null_ range_gist_same _null_ _null_ _null_ ));
DESCR("GiST support");
DATA(insert OID = 3902 (  hash_range			UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 23 "3831" _null_ _null_ _null_ _null_ _null_ hash_range _null_ _null_ _null_ ));
DESCR("hash a range");
DATA(insert OID = 3916 (  range_typanalyze		UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "2281" _null_ _null_ _null_ _null_ _null_ range_typanalyze _null_ _null_ _null_ ));
DESCR("range typanalyze");
DATA(insert OID = 3169 (  rangesel				UXNSP UXUID 12 1 0 0 0 f f f f t f s s 4 0 701 "2281 26 2281 23" _null_ _null_ _null_ _null_ _null_ rangesel _null_ _null_ _null_ ));
DESCR("restriction selectivity for range operators");

DATA(insert OID = 3914 (  int4range_canonical		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3904 "3904" _null_ _null_ _null_ _null_ _null_ int4range_canonical _null_ _null_ _null_ ));
DESCR("convert an int4 range to canonical form");
DATA(insert OID = 3928 (  int8range_canonical		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3926 "3926" _null_ _null_ _null_ _null_ _null_ int8range_canonical _null_ _null_ _null_ ));
DESCR("convert an int8 range to canonical form");
DATA(insert OID = 3915 (  daterange_canonical		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 1 0 3912 "3912" _null_ _null_ _null_ _null_ _null_ daterange_canonical _null_ _null_ _null_ ));
DESCR("convert a date range to canonical form");
DATA(insert OID = 3922 (  int4range_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "23 23" _null_ _null_ _null_ _null_ _null_ int4range_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two int4 values");
DATA(insert OID = 3923 (  int8range_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "20 20" _null_ _null_ _null_ _null_ _null_ int8range_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two int8 values");
DATA(insert OID = 3924 (  numrange_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "1700 1700" _null_ _null_ _null_ _null_ _null_ numrange_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two numeric values");
DATA(insert OID = 3925 (  daterange_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "1082 1082" _null_ _null_ _null_ _null_ _null_ daterange_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two date values");
DATA(insert OID = 3929 (  tsrange_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "1114 1114" _null_ _null_ _null_ _null_ _null_ tsrange_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two timestamp values");
DATA(insert OID = 3930 (  tstzrange_subdiff		   UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 701 "1184 1184" _null_ _null_ _null_ _null_ _null_ tstzrange_subdiff _null_ _null_ _null_ ));
DESCR("float8 difference of two timestamp with time zone values");

DATA(insert OID = 3840 (  int4range UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3904 "23 23" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("int4range constructor");
DATA(insert OID = 3841 (  int4range UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3904 "23 23 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("int4range constructor");
DATA(insert OID = 3844 (  numrange	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3906 "1700 1700" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("numrange constructor");
DATA(insert OID = 3845 (  numrange	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3906 "1700 1700 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("numrange constructor");
DATA(insert OID = 3933 (  tsrange	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3908 "1114 1114" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("tsrange constructor");
DATA(insert OID = 3934 (  tsrange	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3908 "1114 1114 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("tsrange constructor");
DATA(insert OID = 3937 (  tstzrange UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3910 "1184 1184" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("tstzrange constructor");
DATA(insert OID = 3938 (  tstzrange UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3910 "1184 1184 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("tstzrange constructor");
DATA(insert OID = 3941 (  daterange UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3912 "1082 1082" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("daterange constructor");
DATA(insert OID = 3942 (  daterange UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3912 "1082 1082 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("daterange constructor");
DATA(insert OID = 3945 (  int8range UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 3926 "20 20" _null_ _null_ _null_ _null_ _null_ range_constructor2 _null_ _null_ _null_ ));
DESCR("int8range constructor");
DATA(insert OID = 3946 (  int8range UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 3926 "20 20 25" _null_ _null_ _null_ _null_ _null_ range_constructor3 _null_ _null_ _null_ ));
DESCR("int8range constructor");

/* date, time, timestamp constructors */
DATA(insert OID = 3846 ( make_date	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1082 "23 23 23" _null_ _null_ "{year,month,day}" _null_ _null_ make_date _null_ _null_ _null_ ));
DESCR("construct date");
DATA(insert OID = 3847 ( make_time	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 3 0 1083 "23 23 701" _null_ _null_ "{hour,min,sec}" _null_ _null_ make_time _null_ _null_ _null_ ));
DESCR("construct time");
DATA(insert OID = 3461 ( make_timestamp UXNSP UXUID 12 1 0 0 0 f f f f t f i s 6 0 1114 "23 23 23 23 23 701" _null_ _null_ "{year,month,mday,hour,min,sec}" _null_ _null_ make_timestamp _null_ _null_ _null_ ));
DESCR("construct timestamp");
DATA(insert OID = 3462 ( make_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 6 0 1184 "23 23 23 23 23 701" _null_ _null_ "{year,month,mday,hour,min,sec}" _null_ _null_ make_timestamptz _null_ _null_ _null_ ));
DESCR("construct timestamp with time zone");
DATA(insert OID = 3463 ( make_timestamptz	UXNSP UXUID 12 1 0 0 0 f f f f t f s s 7 0 1184 "23 23 23 23 23 701 25" _null_ _null_ "{year,month,mday,hour,min,sec,timezone}" _null_ _null_ make_timestamptz_at_timezone _null_ _null_ _null_ ));
DESCR("construct timestamp with time zone");
DATA(insert OID = 3464 ( make_interval	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 7 0 1186 "23 23 23 23 23 23 701" _null_ _null_ "{years,months,weeks,days,hours,mins,secs}" _null_ _null_ make_interval _null_ _null_ _null_ ));
DESCR("construct interval");

/* spgist opclasses */
DATA(insert OID = 4018 (  spg_quad_config	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_quad_config _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over point");
DATA(insert OID = 4019 (  spg_quad_choose	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_quad_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over point");
DATA(insert OID = 4020 (  spg_quad_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_quad_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over point");
DATA(insert OID = 4021 (  spg_quad_inner_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_quad_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over point");
DATA(insert OID = 4022 (  spg_quad_leaf_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_quad_leaf_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree and k-d tree over point");

DATA(insert OID = 4023 (  spg_kd_config UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_kd_config _null_ _null_ _null_ ));
DESCR("SP-GiST support for k-d tree over point");
DATA(insert OID = 4024 (  spg_kd_choose UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_kd_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support for k-d tree over point");
DATA(insert OID = 4025 (  spg_kd_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_kd_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support for k-d tree over point");
DATA(insert OID = 4026 (  spg_kd_inner_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_kd_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for k-d tree over point");

DATA(insert OID = 4027 (  spg_text_config	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_text_config _null_ _null_ _null_ ));
DESCR("SP-GiST support for radix tree over text");
DATA(insert OID = 4028 (  spg_text_choose	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_text_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support for radix tree over text");
DATA(insert OID = 4029 (  spg_text_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_text_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support for radix tree over text");
DATA(insert OID = 4030 (  spg_text_inner_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_text_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for radix tree over text");
DATA(insert OID = 4031 (  spg_text_leaf_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_text_leaf_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for radix tree over text");

DATA(insert OID = 3469 (  spg_range_quad_config UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_range_quad_config _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over range");
DATA(insert OID = 3470 (  spg_range_quad_choose UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_range_quad_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over range");
DATA(insert OID = 3471 (  spg_range_quad_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_range_quad_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over range");
DATA(insert OID = 3472 (  spg_range_quad_inner_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_range_quad_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over range");
DATA(insert OID = 3473 (  spg_range_quad_leaf_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_range_quad_leaf_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over range");

DATA(insert OID = 5012 (  spg_box_quad_config UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_box_quad_config _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over box");
DATA(insert OID = 5013 (  spg_box_quad_choose UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_box_quad_choose _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over box");
DATA(insert OID = 5014 (  spg_box_quad_picksplit	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_box_quad_picksplit _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over box");
DATA(insert OID = 5015 (  spg_box_quad_inner_consistent UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 2278 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_box_quad_inner_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over box");
DATA(insert OID = 5016 (  spg_box_quad_leaf_consistent	UXNSP UXUID 12 1 0 0 0 f f f f t f i s 2 0 16 "2281 2281" _null_ _null_ _null_ _null_  _null_ spg_box_quad_leaf_consistent _null_ _null_ _null_ ));
DESCR("SP-GiST support for quad tree over box");

/* replication slots */
DATA(insert OID = 3779 (  ux_create_physical_replication_slot UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 2249 "19 16 16" "{19,16,16,19,3220}" "{i,i,i,o,o}" "{slot_name,immediately_reserve,temporary,slot_name,lsn}" _null_ _null_ ux_create_physical_replication_slot _null_ _null_ _null_ ));
DESCR("create a physical replication slot");
DATA(insert OID = 3780 (  ux_drop_replication_slot UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "19" _null_ _null_ _null_ _null_ _null_ ux_drop_replication_slot _null_ _null_ _null_ ));
DESCR("drop a replication slot");
DATA(insert OID = 3781 (  ux_get_replication_slots	UXNSP UXUID 12 1 10 0 0 f f f f f t s s 0 0 2249 "" "{19,19,25,26,16,16,23,28,28,3220,3220}" "{o,o,o,o,o,o,o,o,o,o,o}" "{slot_name,plugin,slot_type,datoid,temporary,active,active_pid,xmin,catalog_xmin,restart_lsn,confirmed_flush_lsn}" _null_ _null_ ux_get_replication_slots _null_ _null_ _null_ ));
DESCR("information about replication slots currently in use");
DATA(insert OID = 3786 (  ux_create_logical_replication_slot UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 2249 "19 19 16" "{19,19,16,25,3220}" "{i,i,i,o,o}" "{slot_name,plugin,temporary,slot_name,lsn}" _null_ _null_ ux_create_logical_replication_slot _null_ _null_ _null_ ));
DESCR("set up a logical replication slot");
DATA(insert OID = 3782 (  ux_logical_slot_get_changes UXNSP UXUID 12 1000 1000 25 0 f f f f f t v u 4 0 2249 "19 3220 23 1009" "{19,3220,23,1009,3220,28,25}" "{i,i,i,v,o,o,o}" "{slot_name,upto_lsn,upto_nchanges,options,lsn,xid,data}" _null_ _null_ ux_logical_slot_get_changes _null_ _null_ _null_ ));
DESCR("get changes from replication slot");
DATA(insert OID = 3783 (  ux_logical_slot_get_binary_changes UXNSP UXUID 12 1000 1000 25 0 f f f f f t v u 4 0 2249 "19 3220 23 1009" "{19,3220,23,1009,3220,28,17}" "{i,i,i,v,o,o,o}" "{slot_name,upto_lsn,upto_nchanges,options,lsn,xid,data}" _null_ _null_ ux_logical_slot_get_binary_changes _null_ _null_ _null_ ));
DESCR("get binary changes from replication slot");
DATA(insert OID = 3784 (  ux_logical_slot_peek_changes UXNSP UXUID 12 1000 1000 25 0 f f f f f t v u 4 0 2249 "19 3220 23 1009" "{19,3220,23,1009,3220,28,25}" "{i,i,i,v,o,o,o}" "{slot_name,upto_lsn,upto_nchanges,options,lsn,xid,data}" _null_ _null_ ux_logical_slot_peek_changes _null_ _null_ _null_ ));
DESCR("peek at changes from replication slot");
DATA(insert OID = 3785 (  ux_logical_slot_peek_binary_changes UXNSP UXUID 12 1000 1000 25 0 f f f f f t v u 4 0 2249 "19 3220 23 1009" "{19,3220,23,1009,3220,28,17}" "{i,i,i,v,o,o,o}" "{slot_name,upto_lsn,upto_nchanges,options,lsn,xid,data}" _null_ _null_ ux_logical_slot_peek_binary_changes _null_ _null_ _null_ ));
DESCR("peek at binary changes from replication slot");
DATA(insert OID = 3577 (  ux_logical_emit_message UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 3220 "16 25 25" _null_ _null_ _null_ _null_ _null_ ux_logical_emit_message_text _null_ _null_ _null_ ));
DESCR("emit a textual logical decoding message");
DATA(insert OID = 3578 (  ux_logical_emit_message UXNSP UXUID 12 1 0 0 0 f f f f t f v u 3 0 3220 "16 25 17" _null_ _null_ _null_ _null_ _null_ ux_logical_emit_message_bytea _null_ _null_ _null_ ));
DESCR("emit a binary logical decoding message");

/* event triggers */
DATA(insert OID = 3566 (  ux_event_trigger_dropped_objects		UXNSP UXUID 12 10 100 0 0 f f f f t t s r 0 0 2249 "" "{26,26,23,16,16,16,25,25,25,25,1009,1009}" "{o,o,o,o,o,o,o,o,o,o,o,o}" "{classid, objid, objsubid, original, normal, is_temporary, object_type, schema_name, object_name, object_identity, address_names, address_args}" _null_ _null_ ux_event_trigger_dropped_objects _null_ _null_ _null_ ));
DESCR("list objects dropped by the current command");
DATA(insert OID = 4566 (  ux_event_trigger_table_rewrite_oid	UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 26 "" "{26}" "{o}" "{oid}" _null_ _null_ ux_event_trigger_table_rewrite_oid _null_ _null_ _null_ ));
DESCR("return Oid of the table getting rewritten");
DATA(insert OID = 4567 (  ux_event_trigger_table_rewrite_reason UXNSP UXUID 12 1 0 0 0 f f f f t f s r 0 0 23 "" _null_ _null_ _null_ _null_ _null_ ux_event_trigger_table_rewrite_reason _null_ _null_ _null_ ));
DESCR("return reason code for table getting rewritten");
DATA(insert OID = 4568 (  ux_event_trigger_ddl_commands			UXNSP UXUID 12 10 100 0 0 f f f f t t s r 0 0 2249 "" "{26,26,23,25,25,25,25,16,32}" "{o,o,o,o,o,o,o,o,o}" "{classid, objid, objsubid, command_tag, object_type, schema_name, object_identity, in_extension, command}" _null_ _null_ ux_event_trigger_ddl_commands _null_ _null_ _null_ ));
DESCR("list DDL actions being executed by the current command");

/* generic transition functions for ordered-set aggregates */
DATA(insert OID = 3970 ( ordered_set_transition			UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2281 "2281 2276" _null_ _null_ _null_ _null_ _null_ ordered_set_transition _null_ _null_ _null_ ));
DESCR("aggregate transition function");
DATA(insert OID = 3971 ( ordered_set_transition_multi	UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 2 0 2281 "2281 2276" "{2281,2276}" "{i,v}" _null_ _null_ _null_ ordered_set_transition_multi _null_ _null_ _null_ ));
DESCR("aggregate transition function");

/* inverse distribution aggregates (and their support functions) */
DATA(insert OID = 3972 ( percentile_disc		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 2283 "701 2283" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("discrete percentile");
DATA(insert OID = 3973 ( percentile_disc_final	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2283 "2281 701 2283" _null_ _null_ _null_ _null_ _null_ percentile_disc_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3974 ( percentile_cont		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 701 "701 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("continuous distribution percentile");
DATA(insert OID = 3975 ( percentile_cont_float8_final	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 701 "2281 701" _null_ _null_ _null_ _null_ _null_ percentile_cont_float8_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3976 ( percentile_cont		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 1186 "701 1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("continuous distribution percentile");
DATA(insert OID = 3977 ( percentile_cont_interval_final UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1186 "2281 701" _null_ _null_ _null_ _null_ _null_ percentile_cont_interval_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3978 ( percentile_disc		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 2277 "1022 2283" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("multiple discrete percentiles");
DATA(insert OID = 3979 ( percentile_disc_multi_final	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 3 0 2277 "2281 1022 2283" _null_ _null_ _null_ _null_ _null_ percentile_disc_multi_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3980 ( percentile_cont		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 1022 "1022 701" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("multiple continuous percentiles");
DATA(insert OID = 3981 ( percentile_cont_float8_multi_final UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1022 "2281 1022" _null_ _null_ _null_ _null_ _null_ percentile_cont_float8_multi_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3982 ( percentile_cont		UXNSP UXUID 12 1 0 0 0 t f f f f f i s 2 0 1187 "1022 1186" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("multiple continuous percentiles");
DATA(insert OID = 3983 ( percentile_cont_interval_multi_final	UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 1187 "2281 1022" _null_ _null_ _null_ _null_ _null_ percentile_cont_interval_multi_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3984 ( mode					UXNSP UXUID 12 1 0 0 0 t f f f f f i s 1 0 2283 "2283" _null_ _null_ _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("most common value");
DATA(insert OID = 3985 ( mode_final				UXNSP UXUID 12 1 0 0 0 f f f f f f i s 2 0 2283 "2281 2283" _null_ _null_ _null_ _null_ _null_	mode_final _null_ _null_ _null_ ));
DESCR("aggregate final function");

/* hypothetical-set aggregates (and their support functions) */
DATA(insert OID = 3986 ( rank				UXNSP UXUID 12 1 0 2276 0 t f f f f f i s 1 0 20 "2276" "{2276}" "{v}" _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("rank of hypothetical row");
DATA(insert OID = 3987 ( rank_final			UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 2 0 20 "2281 2276" "{2281,2276}" "{i,v}" _null_ _null_ _null_ hypothetical_rank_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3988 ( percent_rank		UXNSP UXUID 12 1 0 2276 0 t f f f f f i s 1 0 701 "2276" "{2276}" "{v}" _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("fractional rank of hypothetical row");
DATA(insert OID = 3989 ( percent_rank_final UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 2 0 701 "2281 2276" "{2281,2276}" "{i,v}" _null_ _null_ _null_ hypothetical_percent_rank_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3990 ( cume_dist			UXNSP UXUID 12 1 0 2276 0 t f f f f f i s 1 0 701 "2276" "{2276}" "{v}" _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("cumulative distribution of hypothetical row");
DATA(insert OID = 3991 ( cume_dist_final	UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 2 0 701 "2281 2276" "{2281,2276}" "{i,v}" _null_ _null_ _null_ hypothetical_cume_dist_final _null_ _null_ _null_ ));
DESCR("aggregate final function");
DATA(insert OID = 3992 ( dense_rank			UXNSP UXUID 12 1 0 2276 0 t f f f f f i s 1 0 20 "2276" "{2276}" "{v}" _null_ _null_ _null_ aggregate_dummy _null_ _null_ _null_ ));
DESCR("rank of hypothetical row without gaps");
DATA(insert OID = 3993 ( dense_rank_final	UXNSP UXUID 12 1 0 2276 0 f f f f f f i s 2 0 20 "2281 2276" "{2281,2276}" "{i,v}" _null_ _null_ _null_ hypothetical_dense_rank_final _null_ _null_ _null_ ));
DESCR("aggregate final function");

/* ux_upgrade support */
DATA(insert OID = 3582 ( binary_upgrade_set_next_ux_type_oid UXNSP UXUID  12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_ux_type_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3584 ( binary_upgrade_set_next_array_ux_type_oid UXNSP UXUID	12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_array_ux_type_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3585 ( binary_upgrade_set_next_toast_ux_type_oid UXNSP UXUID	12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_toast_ux_type_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3586 ( binary_upgrade_set_next_heap_ux_class_oid UXNSP UXUID	12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_heap_ux_class_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3587 ( binary_upgrade_set_next_index_ux_class_oid UXNSP UXUID  12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_index_ux_class_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3588 ( binary_upgrade_set_next_toast_ux_class_oid UXNSP UXUID  12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_toast_ux_class_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3589 ( binary_upgrade_set_next_ux_enum_oid UXNSP UXUID  12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_ux_enum_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3590 ( binary_upgrade_set_next_ux_authid_oid UXNSP UXUID	12 1 0 0 0 f f f f t f v r 1 0 2278 "26" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_next_ux_authid_oid _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 3591 ( binary_upgrade_create_empty_extension UXNSP UXUID	12 1 0 0 0 f f f f f f v r 7 0 2278 "25 25 16 25 1028 1009 1009" _null_ _null_ _null_ _null_ _null_ binary_upgrade_create_empty_extension _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");
DATA(insert OID = 4083 ( binary_upgrade_set_record_init_privs UXNSP UXUID	12 1 0 0 0 f f f f t f v r 1 0 2278 "16" _null_ _null_ _null_ _null_ _null_ binary_upgrade_set_record_init_privs _null_ _null_ _null_ ));
DESCR("for use by ux_upgrade");

/* replication/origin.h */
DATA(insert OID = 6003 ( ux_replication_origin_create UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 26 "25" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_create _null_ _null_ _null_ ));
DESCR("create a replication origin");

DATA(insert OID = 6004 ( ux_replication_origin_drop UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "25" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_drop _null_ _null_ _null_ ));
DESCR("drop replication origin identified by its name");

DATA(insert OID = 6005 ( ux_replication_origin_oid UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 26 "25" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_oid _null_ _null_ _null_ ));
DESCR("translate the replication origin's name to its id");

DATA(insert OID = 6006 ( ux_replication_origin_session_setup UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 2278 "25" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_session_setup _null_ _null_ _null_ ));
DESCR("configure session to maintain replication progress tracking for the passed in origin");

DATA(insert OID = 6007 ( ux_replication_origin_session_reset UXNSP UXUID 12 1 0 0 0 f f f f t f v u 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_session_reset _null_ _null_ _null_ ));
DESCR("teardown configured replication progress tracking");

DATA(insert OID = 6008 ( ux_replication_origin_session_is_setup UXNSP UXUID 12 1 0 0 0 f f f f t f v r 0 0 16 "" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_session_is_setup _null_ _null_ _null_ ));
DESCR("is a replication origin configured in this session");

DATA(insert OID = 6009 ( ux_replication_origin_session_progress UXNSP UXUID 12 1 0 0 0 f f f f t f v u 1 0 3220 "16" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_session_progress _null_ _null_ _null_ ));
DESCR("get the replication progress of the current session");

DATA(insert OID = 6010 ( ux_replication_origin_xact_setup UXNSP UXUID 12 1 0 0 0 f f f f t f v r 2 0 2278 "3220 1184" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_xact_setup _null_ _null_ _null_ ));
DESCR("setup the transaction's origin lsn and timestamp");

DATA(insert OID = 6011 ( ux_replication_origin_xact_reset UXNSP UXUID 12 1 0 0 0 f f f f t f v r 0 0 2278 "" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_xact_reset _null_ _null_ _null_ ));
DESCR("reset the transaction's origin lsn and timestamp");

DATA(insert OID = 6012 ( ux_replication_origin_advance UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 2278 "25 3220" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_advance _null_ _null_ _null_ ));
DESCR("advance replication identifier to specific location");

DATA(insert OID = 6013 ( ux_replication_origin_progress UXNSP UXUID 12 1 0 0 0 f f f f t f v u 2 0 3220 "25 16" _null_ _null_ _null_ _null_ _null_ ux_replication_origin_progress _null_ _null_ _null_ ));
DESCR("get an individual replication origin's replication progress");

DATA(insert OID = 6014 ( ux_show_replication_origin_status UXNSP UXUID 12 1 100 0 0 f f f f f t v r 0 0 2249 "" "{26,25,3220,3220}" "{o,o,o,o}" "{local_id, external_id, remote_lsn, local_lsn}" _null_ _null_ ux_show_replication_origin_status _null_ _null_ _null_ ));
DESCR("get progress for all replication origins");

/* publications */
DATA(insert OID = 6119 ( ux_get_publication_tables	UXNSP UXUID 12 1 1000 0 0 f f f f t t s s 1 0 26 "25" "{25,26}" "{i,o}" "{pubname,relid}" _null_ _null_ ux_get_publication_tables _null_ _null_ _null_ ));
DESCR("get OIDs of tables in a publication");
DATA(insert OID = 6121 ( ux_relation_is_publishable UXNSP UXUID 12 1	0 0 0 f f f f t f s s 1 0 16 "2205" _null_ _null_ _null_ _null_ _null_ ux_relation_is_publishable _null_ _null_ _null_ ));
DESCR("returns whether a relation can be part of a publication");

/* rls */
DATA(insert OID = 3298 (  row_security_active	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "26" _null_ _null_ _null_ _null_ _null_	row_security_active _null_ _null_ _null_ ));
DESCR("row security for current context active on table by table oid");
DATA(insert OID = 3299 (  row_security_active	   UXNSP UXUID 12 1 0 0 0 f f f f t f s s 1 0 16 "25" _null_ _null_ _null_ _null_ _null_	row_security_active_name _null_ _null_ _null_ ));
DESCR("row security for current context active on table by table name");

/* ux_config */
DATA(insert OID = 3400 ( ux_config UXNSP UXUID 12 1 23 0 0 f f f f t t i r 0 0 2249 "" "{25,25}" "{o,o}" "{name,setting}" _null_ _null_ ux_config _null_ _null_ _null_ ));
DESCR("ux_config binary as a function");

/* ux_controldata related functions */
DATA(insert OID = 3441 ( ux_control_system UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2249 "" "{23,23,20,1184}" "{o,o,o,o}" "{ux_control_version,catalog_version_no,system_identifier,ux_control_last_modified}" _null_ _null_ ux_control_system _null_ _null_ _null_ ));
DESCR("ux_controldata general state information as a function");

DATA(insert OID = 3442 ( ux_control_checkpoint UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2249 "" "{3220,3220,3220,25,23,23,16,25,26,28,28,28,26,28,28,26,28,28,1184}" "{o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o,o}" "{checkpoint_lsn,prior_lsn,redo_lsn,redo_wal_file,timeline_id,prev_timeline_id,full_page_writes,next_xid,next_oid,next_multixact_id,next_multi_offset,oldest_xid,oldest_xid_dbid,oldest_active_xid,oldest_multi_xid,oldest_multi_dbid,oldest_commit_ts_xid,newest_commit_ts_xid,checkpoint_time}" _null_ _null_ ux_control_checkpoint _null_ _null_ _null_ ));
DESCR("ux_controldata checkpoint state information as a function");

DATA(insert OID = 3443 ( ux_control_recovery UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2249 "" "{3220,23,3220,3220,16}" "{o,o,o,o,o}" "{min_recovery_end_lsn,min_recovery_end_timeline,backup_start_lsn,backup_end_lsn,end_of_backup_record_required}" _null_ _null_ ux_control_recovery _null_ _null_ _null_ ));
DESCR("ux_controldata recovery state information as a function");

DATA(insert OID = 3444 ( ux_control_init UXNSP UXUID 12 1 0 0 0 f f f f t f v s 0 0 2249 "" "{23,23,23,23,23,23,23,23,23,16,16,23}" "{o,o,o,o,o,o,o,o,o,o,o,o}" "{max_data_alignment,database_block_size,blocks_per_segment,wal_block_size,bytes_per_wal_segment,max_identifier_length,max_index_columns,max_toast_chunk_size,large_object_chunk_size,float4_pass_by_value,float8_pass_by_value,data_page_checksum_version}" _null_ _null_ ux_control_init _null_ _null_ _null_ ));
DESCR("ux_controldata init state information as a function");

/* collation management functions */
DATA(insert OID = 3445 ( ux_import_system_collations UXNSP UXUID 12 100 0 0 0 f f f f t f v r 1 0 23 "4089" _null_ _null_ _null_ _null_ _null_ ux_import_system_collations _null_ _null_ _null_ ));
DESCR("import collations from operating system");

DATA(insert OID = 3448 ( ux_collation_actual_version UXNSP UXUID 12 100 0 0 0 f f f f t f v s 1 0 25 "26" _null_ _null_ _null_ _null_ _null_ ux_collation_actual_version _null_ _null_ _null_ ));
DESCR("get actual version of collation from operating system");

/* system management/monitoring related functions */
DATA(insert OID = 3353 (  ux_ls_logdir				 UXNSP UXUID 12 10 20 0 0 f f f f t t v s 0 0 2249 "" "{25,20,1184}" "{o,o,o}" "{name,size,modification}" _null_ _null_ ux_ls_logdir _null_ _null_ _null_ ));
DESCR("list files in the log directory");
DATA(insert OID = 3354 (  ux_ls_waldir				 UXNSP UXUID 12 10 20 0 0 f f f f t t v s 0 0 2249 "" "{25,20,1184}" "{o,o,o}" "{name,size,modification}" _null_ _null_ ux_ls_waldir _null_ _null_ _null_ ));
DESCR("list of files in the WAL directory");

DATA(insert OID = 3411 ( ux_license UXNSP UXUID 12 1 10 0 0 f f f f t t i r 0 0 2249 "" "{25,25}" "{o,o}" "{name,value}" _null_ _null_ ux_license _null_ _null_ _null_ ));
DESCR("get the license information loaded by the current DB service ");

/*
 * Symbolic values for provolatile column: these indicate whether the result
 * of a function is dependent *only* on the values of its explicit arguments,
 * or can change due to outside factors (such as parameter variables or
 * table contents).  NOTE: functions having side-effects, such as setval(),
 * must be labeled volatile to ensure they will not get optimized away,
 * even if the actual return value is not changeable.
 */
#define PROVOLATILE_IMMUTABLE	'i' /* never changes for given input */
#define PROVOLATILE_STABLE		's' /* does not change within a scan */
#define PROVOLATILE_VOLATILE	'v' /* can change even within a scan */

/*
 * Symbolic values for proparallel column: these indicate whether a function
 * can be safely be run in a parallel backend, during parallelism but
 * necessarily in the master, or only in non-parallel mode.
 */
#define PROPARALLEL_SAFE		's' /* can run in worker or master */
#define PROPARALLEL_RESTRICTED	'r' /* can run in parallel master only */
#define PROPARALLEL_UNSAFE		'u' /* banned while in parallel mode */

/*
 * Symbolic values for proargmodes column.  Note that these must agree with
 * the FunctionParameterMode enum in parsenodes.h; we declare them here to
 * be accessible from either header.
 */
#define PROARGMODE_IN		'i'
#define PROARGMODE_OUT		'o'
#define PROARGMODE_INOUT	'b'
#define PROARGMODE_VARIADIC 'v'
#define PROARGMODE_TABLE	't'

#endif							/* UX_PROC_H */
